/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.commerce.common;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import com.adobe.cq.commerce.api.CommerceException;
import com.adobe.cq.commerce.api.CommerceQuery;
import com.adobe.cq.commerce.api.CommerceResult;
import com.adobe.cq.commerce.api.CommerceService;
import com.adobe.cq.commerce.api.Product;
import com.adobe.cq.commerce.api.promotion.Promotion;
import com.adobe.cq.commerce.api.promotion.PromotionManager;
import com.adobe.cq.commerce.api.promotion.Voucher;
import com.day.cq.wcm.api.Page;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;

/**
 * An abstract base class for implementing {@link CommerceService}s on top of the JCR repository.
 * See GeoCommerceServiceImpl for an example.
 */
public abstract class AbstractJcrCommerceService implements CommerceService {

    private final ServiceContext serviceContext;

    protected Map<String, Object> context;

    protected CommerceSearchProvider searchProvider;

    protected ResourceResolver resolver;

    protected AbstractJcrCommerceService(ServiceContext serviceContext, Resource resource) {
        this.context = new HashMap<String, Object>();
        this.serviceContext = serviceContext;
        this.resolver = resource.getResourceResolver();
    }

    /**
     * Returns the {@link ServiceContext} of this service.
     */
    public ServiceContext serviceContext() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void setContext(Map<String, Object> context) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, Object> getContext() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getServer() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isActivated(Product product) throws CommerceException {
		return false;
	}

    /**
     * Default implementation assumes a JCR-based promotion (in which the path parameter is
     * truly a path).
     */
    @Override
    public Promotion getPromotion(String path) throws CommerceException {
		return null;
	}

    /**
     * Default implementation assumes a JCR-based voucher (in which the path parameter is
     * truly a path).
     */
    @Override
    public Voucher getVoucher(String path) throws CommerceException {
		return null;
	}

    /**
     * Return a vendor record of a placed order for order administration.  The given locale
     * will be used to format prices.
     */
    public VendorJcrPlacedOrder getPlacedOrder(String orderId, Locale locale) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void catalogRolloutHook(Page blueprint, Page catalog) throws CommerceException {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void sectionRolloutHook(Page blueprint, Page section) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void productRolloutHook(Product productData, Page productPage, Product productReference) throws CommerceException {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public CommerceResult search(CommerceQuery query) throws CommerceException {
		return null;
	}

    /**
     * Gets the search provider used in the current e-commerce implementation.  Subclasses may override
     * this method to plug-in a custom search provider implementation into the e-commerce system which
     * is not available via {@code CommerceSearchProviderManger}.
     */
    protected CommerceSearchProvider getSearchProvider() throws CommerceException {
		return null;
	}

    /**
     * Gets the identifier of the search provider used in the current e-commerce implementation.
     * Subclasses should override this method to return the name of the custom search provider service
     * available in {@code CommerceSearchProviderManger}.
     */
    protected String getSearchProviderName() {
		return null;
	}

    /**
     * Returns a list of JCR-based promotions (ie, those known to the {@link PromotionManager}).
     */
    @Override
    public List<Promotion> getAvailablePromotions(ResourceResolver resourceResolver) throws CommerceException {
		return null;
	}


    /*
     * ==================================================================================================
     * Deprecated methods.  For backwards-compatibility only.
     * ==================================================================================================
     */

    /**
     * @deprecated use the {@link #serviceContext} field instead.
     */
    @Deprecated
    public AbstractJcrCommerceServiceFactory.Services services;

    /**
     * @deprecated since 5.6.1; use {@link #AbstractJcrCommerceService(ServiceContext)} instead.
     */
    @Deprecated
    public AbstractJcrCommerceService(AbstractJcrCommerceServiceFactory.Services services) {
        this((ServiceContext) null);
        this.services = services;
    }

    /**
     * @deprecated since 5.6.200; use {@link #AbstractJcrCommerceService(ServiceContext, Resource)}
     * instead.
     */
    @Deprecated
    protected AbstractJcrCommerceService(ServiceContext serviceContext) {
        this.context = new HashMap<String, Object>();
        this.serviceContext = serviceContext;
    }

}
