/*************************************************************************
  *
  * ADOBE CONFIDENTIAL
  * __________________
  *
  *  Copyright 2012 Adobe Systems Incorporated
  *  All Rights Reserved.
  *
  * NOTICE:  All information contained herein is, and remains
  * the property of Adobe Systems Incorporated and its suppliers,
  * if any.  The intellectual and technical concepts contained
  * herein are proprietary to Adobe Systems Incorporated and its
  * suppliers and may be covered by U.S. and Foreign Patents,
  * patents in process, and are protected by trade secret or copyright law.
  * Dissemination of this information or reproduction of this material
  * is strictly forbidden unless prior written permission is obtained
  * from Adobe Systems Incorporated.
  **************************************************************************/
package com.adobe.cq.dms.tagmanager;

/**
 * Represents a TagManager tag type
 */
public enum TagType {
    
    SITECATALYST("product_sc", false),
    
    SITECATALYSTMEDIA("product_sc_media", false),
    
    SITECATALYSTSURVEY("product_sc_survey", false),
    
    SITECATALYSTPARTNER("product_sc_partner", false),
    
    TESTANDTARGET("product_tt", false),
    
    CUSTOM("custom", false),
    
    BEACON("custom_beacon", true),
    
    REMOTEJAVASCRIPT("custom_script_remote", true),
    
    JAVASCRIPT("custom_script_code", false),
    
    IFRAME("custom_iframe", true);
    
    String name;
    
    Boolean hasUrl;
    
    private TagType(String propertyname, boolean hasUrl) {
        this.name = propertyname;
        this.hasUrl = hasUrl;
    }
    
    public String toString() {
        return name;
    }
    
    /**
     * Indicator if TagType uses a script_url property.
     * 
     * @return <code>true</code> if tag uses script_url, <code>false</code>
     *         otherwise.
     */
    public boolean hasUrl() {
        return hasUrl;
    }
    
    /**
     * Returns a TagType for the provided <code>type</code> String.
     * 
     * @param type String representation of tag type (i.e. custom)
     * @return
     */
    public static TagType fromString(String type) {
        for (TagType t : values()) {
            if (t.toString().equals(type)) {
                return t;
            }
        }
        return null;
    }

}
