/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.mcm.salesforce;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.URIException;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.httpclient.util.URIUtil;
import org.apache.commons.io.IOUtils;
import org.apache.sling.commons.json.JSONArray;
import org.apache.sling.commons.json.JSONException;
import org.apache.sling.commons.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * The <code>SalesforceClient</code> uses the Salesforce REST API to authenticate and query Salesforce.
 * */
public class SalesforceClient {

    public static final String INVALID_CLIENT_ID = "invalid_client_id";
    public static final String INVALID_CLIENT = "invalid_client";
    public static final String CLIENT_ID = "client_id";
    public static final String CLIENT_SECRET = "client_secret";
    public static final String REFRESH_ACCESS_TOKEN_ENDPOINT = "/services/oauth2/token";
    public static final String APPLICATION_FORMURL_ENCODED = "application/x-www-form-urlencoded";
    public static final String GRANT_TYPE = "grant_type";
    public static final String REFRESH_TOKEN = "refresh_token";
    final protected int HTTP_CODE_REFRESH_TOKEN_SUCCESS = 200;

    public enum AvailableMethods {GET, POST, PATCH};

    protected String clientId = "";
    protected String clientSecret = "";
    protected String instanceURL = "";
    protected String path = "";
    protected String accessToken = "";
    protected String refreshToken = "";
    protected String contentType = "";
    protected String data = "";

    protected AvailableMethods method;
    protected HashMap<String, String> parameters = new HashMap<String, String>();

    private final Logger logger = LoggerFactory.getLogger(SalesforceClient.class);

    /**
     * Returns the Client Identifier.
     * */
    public String getClientId() {
		return null;
	}

    /**
     * Sets the Client Identifier. Also add into the data if not already present.
     * */
    public void setClientId(String clientId) {
	}

    /**
     * Returns the Client Secret.
     * */
    public String getClientSecret() {
		return null;
	}

    /**
     * Sets the Client Secret. Also add into the data if not already present.
     * */
    public void setClientSecret(String clientSecret) {
	}

    /**
     * Returns the URL encoded parameters used for REST Api calls as a String.
     * */
    public String getData() {
		return null;
	}

    /**
     * Adds a key-value pair to parameters list
     * */
    public void addData(String key, String value) {
	}

    /**
     *  Adds a key-value pair to parameters list with an option not to encode the parameters
     * */
    public void addData(String key, String value, boolean encode) {
	}

    /**
     * Set the URL encoded parameters used for REST Api calls as a Map of key-value pairs.
     * */
    public void setData(Map<String, String> data) {
	}

    /**
     * Set the URL encoded parameters used for REST Api calls as a String.
     * */
    public void setData(String data) {
	}

    /**
     * Returns the Instance URL for the Salesforce configuration
     * */
    public String getInstanceURL() {
		return null;
	}

    /**
     * Sets the Instance URL for the Salesforce configuration
     * */
    public void setInstanceURL(String instanceURL) {
	}

    /**
     *  Returns the current REST API endpoint
     * */
    public String getPath() {
		return null;
	}

    /**
     *  Sets the current REST API endpoint
     * */
    public void setPath(String path) {
	}

    /**
     *  Returns the Access Token
     * */
    public String getAccessToken() {
		return null;
	}

    /**
     * Sets the Access Token
     * */
    public void setAccessToken(String accessToken) {
	}

    /**
     *  Returns the Refresh Token
     * */
    public String getRefreshToken() {
		return null;
	}

    /**
     *  Sets the Refresh Token
     * */
    public void setRefreshToken(String refreshToken) {
	}

    /**
     * Returns the Content-Type of the Request
     * */
    public String getContentType() {
		return null;
	}

    /**
     * Sets the Content-Type of the Request
     * */
    public void setContentType(String contentType) {
	}

    /**
     * Returns the HTTP Method used in the Current Request. Will be one of the {@link AvailableMethods}
     * */
    public AvailableMethods getMethod() {
		return null;
	}

    /**
     *  Sets the HTTP Method used in the Current Request. Will be one of the {@link AvailableMethods}
     * */
    public void setMethod(AvailableMethods method) {
	}

    /**
     * Sets the HTTP Method as a String to be used in Current Request
     * */
    public void setStringMethod(String method) {
	}

    /**
     * Adds a parameter to the param list
     * */
    public void addParameter(String key, String value) {
	}

    /**
     * Returns the list of parameters as a map
     * */
    public HashMap<String, String> getParameters() {
		return null;
	}

    /**
     * Sets the list of parameters.
     * */
    public void setParameters(HashMap<String, String> parameters) {
	}

    /**
     *  Executes the current request. Delegates to appropriate method based on the HTTP method used
     * */
    public SalesforceResponse executeRequest() throws SalesforceException {
		return null;
	}

    /**
     * Executes the Current GET Request and refreshes the access token if expired
     * */
    public SalesforceResponse executeGetRequest() throws SalesforceException {
		return null;
	}

    /**
     * Executes the Current GET Request
     * */
    public SalesforceResponse doExecuteGetRequest() throws HttpException, IOException, SalesforceException {
		return null;
	}

    /**
     *  Executes the Current POST/PATCH Request and refreshes the access token if expired
     * */
    public SalesforceResponse executeDataRequest() throws SalesforceException {
		return null;
	}

    /**
     * Executes the Current POST/PATCH Request
     * */
    public SalesforceResponse doExecuteDataRequest() throws HttpException, IOException, SalesforceException {
		return null;
	}

    /**
     * Refreshes the Access Token using the refresh_token
     * Assumes that the client_id and client_secret are set before invoking this function
     * */
    public SalesforceResponse refreshAccessToken() throws SalesforceException {
		return null;
	}

    /**
     * Checks if the currently set access token has been expired or not
     * */
    public boolean isAccessTokenExpired(SalesforceResponse salesforceResponse) throws SalesforceException {
		return false;
	}
}
