/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.mcm.silverpop;

import com.day.cq.mcm.emailprovider.EmailService;
import com.day.cq.mcm.emailprovider.types.Account;
import com.day.cq.mcm.emailprovider.types.ConnectionParams;
import com.day.cq.mcm.emailprovider.types.Email;
import com.day.cq.mcm.emailprovider.types.EmailServiceActions;
import com.day.cq.mcm.emailprovider.types.ListAttribute;
import com.day.cq.mcm.emailprovider.types.PersonalizationInfo;
import com.day.cq.mcm.emailprovider.types.SendClassification;
import com.day.cq.mcm.emailprovider.types.Subscriber;
import com.day.cq.mcm.emailprovider.types.SubscriptionList;
import com.day.cq.wcm.foundation.forms.FieldDescription;
import com.day.cq.wcm.webservicesupport.Configuration;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface SilverPopService extends EmailService {

    /**
     * This function is used to validate the connection parameters with SilverPop
     * @param params The ConnectionParams for SilverPop. 
     * @throws SilverPopException in case of errors.
     */
    public void checkCredentials(ConnectionParams params) throws SilverPopException {
	}
    
    /**
     * This function is used to return all subscription lists available with the email provider account.
     * @param configuration The email provider configuration from cloud services.
     * @return a List containing SubscriptionList objects for each subscription list available with the email provider.
     * @throws SilverPopException in case of errors.
     */
    public List<SubscriptionList> getLists(Configuration configuration, Map<String, Object> filterMap) throws SilverPopException {
		return null;
	}

    /**
     * This function is used to add a subscriber to a particular subscription list.
     * @param configuration The email provider configuration from cloud services.
     * @param subscriber The Subscriber object containing information about the subscriber.
     * @param lists The lists to which the subscriber will be added. Note the lists should belong to same parent database at SilverPop
     * @param parentDB The parentDB associated with the lists. If lists are null, the subscriber will be added to the parentDB
     * @throws SilverPopException in case of errors.
     */
    public void addSubscriberToList(Configuration configuration, Subscriber subscriber, SubscriptionList[] lists, String parentDB)
            throws SilverPopException {
	}

    /**
     * This function is used to remove a subscriber from a particular mailing list.
     * @param configuration The email provider configuration from cloud services.
     * @param subscriber The Subscriber object containing information about the subscriber.
     * @param lists The lists from which the subscriber will be removed. The lists must share the same parent database at SilverPop.
     * If lists are null, the subscriber will be removed from the parentDB.
     * @param parentDB The parent database id associated with the lists
     * @throws SilverPopException in case of errors.
     */
    public void deleteSubscriberFromList(Configuration configuration, Subscriber subscriber, SubscriptionList[] lists, String parentDB)
            throws SilverPopException {
	}

    /**
     * This function is used to publish an email to an external email provider.
     * @param configuration The email provider configuration from cloud services.
     * @param email An object implementing Email containing details about email to be published
     * @return the published email with new properties saved on external email provider 
     * @throws SilverPopException in case of errors.
     */
    public Email publishEmail(Configuration configuration, Email email) throws SilverPopException {
		return null;
	}
    
    /**
     * This function is used to update a published email on an external email provider
     * @param configuration The email provider configuration from cloud services
     * @param email An object implementing Email containing details about email to be updated
     * @throws SilverPopException
     */
    public void updateEmail(Configuration configuration, Email email) throws SilverPopException {
	}

    /**
     * This function returns all the accounts associated with the email provider configuration. If an email provider
     * supports multiple accounts for a connection then those accounts can be fetched by implementing this API.
     * @param params The connection parameters to connect to the email provider.
     * @param filterMap A map containing filtering criteria if any for account filtering. This will be implementation specific.
     * @return A list of all Accounts associated with this connection.
     * @throws SilverPopException in case of errors.
     */
    public List<Account> getAccounts(ConnectionParams params, Map<String, Object> filterMap)
            throws SilverPopException {
		return null;
	}
    
    /**
     * This function fetches the personalisation elements from SilverPop
     * @param configuration SilverPop Configuration from cloud services
     * @param listId Contact list existing on SilverPop 
     * @return PersonalizationInfo associated with the list
     * @throws SilverPopException in case of errors.
     */    
    public PersonalizationInfo describeSubscriber(Configuration configuration,String listId) throws SilverPopException {
		return null;
	}  

    /**
     * @deprecated since 5.6.1
     * @see {@link SilverPopService#getListAttributes(Configuration, String) getListAttributes}
     * This function returns the form input fields need to execute corresponding formAction on list listID.
     * @param configuration SilverPop Configuration from cloud services
     * @param listId Contact list existing on Silverpop
     * @param formAction EmailServiceActions{ADD_SUBSCRIBER, DELETE_SUBSCRIBER, SEND_AUTO_RESPONDER}
     * @return A list of FieldDescription needed to execute the given formAction on associated list.
     * @throws SilverPopException
     */
    @Deprecated
    public List<FieldDescription> getFormFields(Configuration configuration, String listId, EmailServiceActions formAction) throws SilverPopException {
		return null;
	}
    /**
     * 
     * @param listId The list or database whose attributes are to be fetched
     * @return The list of {@link ListAttribute attributes} in the list 
     * @throws SilverpopException
     */
    public List<ListAttribute> getListAttributes(Configuration configuration, String listId) throws SilverPopException {
		return null;
	}

    public List<MailingTemplate> getMailingTemplates(Configuration configuration, GetMailingTemplateParams params) throws SilverPopException {
		return null;
	}
    
    public void scheduleMailing(Configuration configuration, ScheduleMailingParams params) throws SilverPopException {
	}
}
