/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.searchcollections.lucene;

import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * The <code>DecimalField</code> class is a utility to convert
 * <code>java.math.BigDecimal</code> values to <code>String</code> 
 * values that are lexicographically sortable according to the decimal value.
 * <p>
 * The string format uses the characters '0' to '9' and consists of:
 * <pre>
 * { value signum +2 }
 * { exponent signum +2 }
 * { exponent length -1 }
 * { exponent value }
 * { value (-1 if inverted) }
 * </pre>
 * Only the signum is encoded if the value is zero. The exponent is not
 * encoded if zero. Negative values are "inverted" character by character
 * ('0' -> 9, '1' -> '8', and so on). The same applies to the exponent.
 * <p>
 * Examples: 
 * 0 => "2"
 * 2 => "322" (signum 1; exponent 0; value 2)
 * 120 => "330212" (signum 1; exponent signum 1, length 1, value 2; value 12).
 * -1 => "179" (signum -1, rest inverted; exponent 0; value 1 (-1, inverted).
 * <p>
 * Values between BigDecimal(BigInteger.ONE, Integer.MIN_VALUE) and 
 * BigDecimal(BigInteger.ONE, Integer.MAX_VALUE) are supported.
 */
public class DecimalField {
    
    /**
     * Convert a BigDecimal to a String.
     * 
     * @param value the BigDecimal
     * @return the String
     */
    public static String decimalToString(BigDecimal value) {
		return null;
	}
    
    /**
     * Convert a String to a BigDecimal.
     * 
     * @param value the String
     * @return the BigDecimal
     */
    public static BigDecimal stringToDecimal(String value) {
		return null;
	}
    
    private static String positiveDecimalToString(BigDecimal value) {
		return null;
	}

    /**
     * "Invert" a number digit by digit (0 becomes 9, 9 becomes 0, and so on).
     * 
     * @param s the original string
     * @param incLast how much to increment the last character
     * @return the negated string
     */
    private static String invert(String s, int incLast) {
		return null;
	}

}
