/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.searchcollections.lucene;

import static com.adobe.cq.searchcollections.lucene.IndexerUtil.escapeQuery;
import static javax.jcr.query.Query.JCR_JQOM;
import static javax.jcr.query.Query.JCR_SQL2;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.jcr.query.qom.QueryObjectModelFactory;

import org.apache.jackrabbit.commons.query.sql2.SQL2QOMBuilder;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.KeywordAnalyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.facet.index.CategoryDocumentBuilder;
import org.apache.lucene.facet.index.params.DefaultFacetIndexingParams;
import org.apache.lucene.facet.index.params.FacetIndexingParams;
import org.apache.lucene.facet.search.FacetsCollector;
import org.apache.lucene.facet.search.params.CountFacetRequest;
import org.apache.lucene.facet.search.params.FacetSearchParams;
import org.apache.lucene.facet.search.results.FacetResult;
import org.apache.lucene.facet.search.results.FacetResultNode;
import org.apache.lucene.facet.taxonomy.CategoryPath;
import org.apache.lucene.facet.taxonomy.TaxonomyReader;
import org.apache.lucene.facet.taxonomy.TaxonomyWriter;
import org.apache.lucene.facet.taxonomy.directory.DirectoryTaxonomyReader;
import org.apache.lucene.facet.taxonomy.directory.DirectoryTaxonomyWriter;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriter.MaxFieldLength;
import org.apache.lucene.index.IndexWriterConfig.OpenMode;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanClause.Occur;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.MultiCollector;
import org.apache.lucene.search.QueryWrapperFilter;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.TopScoreDocCollector;
import org.apache.lucene.search.similar.MoreLikeThis;
import org.apache.lucene.store.SingleInstanceLockFactory;
import org.apache.lucene.util.Version;
import org.apache.sling.api.SlingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.searchcollections.api.SearchCollection;

public class LuceneSearchCollection implements SearchCollection {

    private static final Logger log = LoggerFactory
            .getLogger(LuceneSearchCollection.class);
    
    private final Session session;

    private final String path;

    private final NodeIndexer indexer;

    private final JCRDirectory directory;
    private JCRDirectory taxoDirectory;

    private final Analyzer analyzer = new StandardAnalyzer(Version.LUCENE_36);

    private final Analyzer keywordAnalyzer = new KeywordAnalyzer();

    private final LuceneSearchCollectionConfig indexConfig;

    private volatile boolean running = true;

    private final Collection<String> queue = new LinkedHashSet<String>();
    
    private Thread wThread = null;
    private Object execSync = new Object();
    private Date exec = null;
    
    //To prevent synchronization issues, use this global lock to only do one 
    //update at a time.  We should really be using JCR locks, but that doesn't
    //seem to be working even though mix:lockable is used.
    private static Object writeSync = new Object();

    class WorkerThread implements Runnable {
		private Date lastExec = null;
    	
        public void run() {
		}
	}



    public LuceneSearchCollection(Node directory, Node taxoDirectory, NodeIndexer indexer, boolean useJCRLocks) throws RepositoryException {
        this.session = directory.getSession();
        this.path = directory.getPath();
        this.indexer = indexer;
        this.directory = new JCRDirectory(directory);
        this.taxoDirectory = null;
        if (null != taxoDirectory) {
            this.taxoDirectory = new JCRDirectory(taxoDirectory);
        }

        try {
            if (useJCRLocks) {
                // path HAS to be mix:lockable if you want to use the
                // JCRLockFactory
                this.directory.setLockFactory(new JCRLockFactory(this.session,
                        directory.getPath()));
                if (null != taxoDirectory) {
                    this.taxoDirectory.setLockFactory(new JCRLockFactory(this.session,
                        directory.getPath()));
                }
            } else {
                this.directory.setLockFactory(new SingleInstanceLockFactory());
                if (null != taxoDirectory) {
                    this.taxoDirectory.setLockFactory(new SingleInstanceLockFactory());
                }
            }
        } catch (Exception e) {
            //TODO
        }

        indexConfig = new LuceneSearchCollectionConfig(directory);
        
        wThread = new Thread(new WorkerThread());
		wThread.start();

    }

    public LuceneSearchCollection(Node directory) throws RepositoryException {
        this(directory, null, new DefaultNodeIndexer(), false);
    }

    public LuceneSearchCollection(Node directory, NodeIndexer indexer)
            throws RepositoryException {
        this(directory, null, indexer, false);
    }
    
    public LuceneSearchCollection(Node directory,Node taxoDirectory, NodeIndexer indexer)
            throws RepositoryException {
        this(directory, taxoDirectory, indexer, false);
    }
    
    public Map<String, Integer> findFacets(String fieldName, String resourceTypeFilter, String componentFilter, int count) throws RepositoryException{
		return null;
	}

    public List<String> findMLT(String target, String resourceTypeFilter, 
    	String componentFilter, String[] mltFields, int maxResults, 
    	int minTermFreq, int minDocFreq) 
    throws RepositoryException {
		return null;
	}

    public Query createQuery(Session session, String statement, String language)
            throws RepositoryException {
				return null;
			}



    public synchronized void update(Set<String> paths) {
	}

    public synchronized void run() {
	}

    public synchronized void stop() {
	}



    private void doUpdate(Collection<String> paths) {
	}

    private synchronized Collection<String> getPendingPaths() {
		return null;
	}
}

