/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.blog;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;

import com.adobe.cq.social.commons.Comment;
import com.day.cq.tagging.Tag;
import com.day.cq.wcm.api.Page;

/**
 * {@linkplain BlogEntry} represents a blog entry.
 */
public interface BlogEntry {

    public static final String DEFAULT_TEXT = "...";

    public static final String PROP_ID = "id";
    public static final String PROP_TITLE = "jcr:title";
    public static final String PROP_TEXT = "text";
    public static final String PROP_TAGS = "cq:tags";
    public static final String PROP_AUTHOR = "author";
    public static final String PROP_CREATED = "jcr:created";
    public static final String PROP_LASTMODIFIED = "cq:lastModified";
    public static final String PROP_LASTMODIFIEDBY = "cq:lastModifiedBy";
    public static final String PROP_PUBLISHED = "published";
    public static final String PROP_LASTREPLICATED = "cq:lastReplicated";
    public static final String PROP_LASTREPLICATEDBY = "cq:lastReplicatedBy";

    public static final String NODE_TEXT = "par/entry";
    public static final String NODE_COMMENTS = "alt/comments";
    public static final String NODE_TRACKBACKS = "alt/trackback";

    public static final String TYPE_TITLE = "social/blog/components/entrytitle";
    public static final String TYPE_TEXT = "social/blog/components/entrytext";
    public static final String TYPE_COMMENTS = "social/commons/components/comments";
    public static final String TYPE_TRACKBACKS = "social/blog/components/trackback";
    public static final String TYPE_PARSYS = "foundation/components/parsys";

    public static final String DEFAULT_PAR_NAME = "entry";
    public static final String FIELD_INDEX_MODIFIED_DATE = "modifiedDate";
    public static final String FIELD_INDEX_CATEGORY_DATE = "categoryDate";
    public static final String FIELD_INDEX_CATEGORY_YEAR = "categoryYear";

    /**
     * Returns the unique ID of this blog entry.
     * @return The ID
     */
    String getId() {
		return null;
	}

    /**
     * Sets the unique ID of this blog entry. Once set, the ID is immutable.
     * @param id The ID
     */
    void setId(String id) {
	}

    /**
     * Returns the title of this blog entry.
     * @return The title
     */
    String getTitle() {
		return null;
	}

    /**
     * Sets the title of this blog entry.
     * @param title The title
     */
    void setTitle(String title) {
	}

    /**
     * Returns the content resource of this blog entry.
     * @return The content resource
     */
    Resource getContentResource() {
		return null;
	}

    /**
     * Returns the text of this blog entry.
     * @return The text
     */
    String getText() {
		return null;
	}

    /**
     * Sets the text of this blog entry.
     * @param text The text
     */
    void setText(String text) {
	}

    /**
     * Returns the author of this blog entry.
     * @return The author
     */
    String getAuthor() {
		return null;
	}

    /**
     * Sets the author of this blog entry.
     * @param author The author
     */
    void setAuthor(String author) {
	}

    /**
     * Returns the date this blog entry has been published on. If still unpublished, the date of the last
     * modification, or finally the creation date will be used.
     * @return The date
     */
    Date getDate() {
		return null;
	}

    /**
     * Sets the date this blog entry has been published on.
     * @param date The date
     */
    void setDate(Date date) {
	}

    /**
     * Returns the tags associated with this blog entry.
     * @return The tags
     */
    Tag[] getTags() {
		return null;
	}

    /**
     * Sets the tags associated with this blog entry. Refer to the {@link com.day.cq.tagging.TagManager} in order to
     * retrieve and/or create tags.
     * @param tags The tags
     */
    void setTags(Tag[] tags) {
	}

    /**
     * Returns a comma-separated string of names the specified tags.
     * @return The tags as string
     */
    String getTagsAsString() {
		return null;
	}

    /**
     * Returns a comma-separated string of HTML links for the specified tags.
     * @return The tags as HTML
     */
    String getTagsAsHTML() {
		return null;
	}

    /**
     * Returns a comma-separated string of HTML links for the specified tags.
     * @param resourceBundle <code>ResourceBundle</code> to use for translations
     * @return The tags as HTML
     */
    String getTagsAsHTML(ResourceBundle resourceBundle) {
		return null;
	}

    /**
     * Returns the paths to the attachments of this blog entry.
     * @return The attachments
     */
    List<String> getAttachments() {
		return null;
	}

    /**
     * Returns all comments (including comments to comments) to this blog entry in chronological order.
     * @return The comments
     */
    Iterator<Comment> getComments() {
		return null;
	}

    /**
     * Returns the number of comments (including comments to comments). Comments are excluded from the count if they
     * are marked as spam, or still unapproved in case comments are moderated.
     * @return The number of comments
     */
    int countComments() {
		return 0;
	}

    /**
     * Adds a comment to this blog entry sing the specified text.
     * @param text The text
     * @param author The author of the comment (optional)
     * @return The comment
     */
    Comment addComment(String text, String author) {
		return null;
	}

    /**
     * Adds a trackback to this blog entry.
     * @param title The title
     * @param excerpt The excerpt of the trackback
     * @param blogName The author's blogName
     * @param url The URL of the author's website
     * @param ip The ip of the author
     */
    void addTrackback(String title, String excerpt, String url, String blogName, String ip) {
	}

    /**
     * Returns the blog this blog entry belongs to or <code>null</code> if no blog can be found.
     * @return The blog
     */
    Blog getBlog() {
		return null;
	}

    /**
     * Returns the resource associated with this blog entry.
     * @return The resource
     */
    Resource getResource() {
		return null;
	}

    /**
     * Returns the page associated with this blog entry.
     * @return The page
     */
    Page getPage() {
		return null;
	}

    /**
     * Returns the server-local URL of this blog entry, omitting scheme, server name and port number.
     * @return The URL
     */
    String getUrl() {
		return null;
	}

    /**
     * Returns the full URL of this blog entry, including scheme, server name and port.
     * @return The full URL
     */
    String getFullUrl() {
		return null;
	}

    /**
     * Returns the URL of the feed for this blog entry as absolute URL. Same as {@link #getFeedUrl(boolean)
     * getFeedUrl(true)}.
     * @return The feed URL
     */
    String getFeedUrl() {
		return null;
	}

    /**
     * Returns the URL of the feed for this blog entry.
     * @param absolute true if the URL should be absolute (http://server.com/blog), false if it should be relative
     *            (/blog)
     * @return The feed URL
     */
    String getFeedUrl(boolean absolute) {
		return null;
	}

    /**
     * Returns the track back URL for this blog entry.
     * @return The track back URL
     */
    String getTrackBackUrl() {
		return null;
	}

    /**
     * Returns the URL this blog entry can be edited at, or <code>null</code> if not available.
     * @param request The request
     * @return The edit URL
     */
    String getEditUrl(SlingHttpServletRequest request) {
		return null;
	}

    /**
     * Returns <code>true</code> if this blog entry is a page that doesn't show up in the timeline of the blog, e.g.
     * an "About" page.
     * @return <code>true</code> if blog entry is a page, <code>false</code> otherwise
     */
    boolean isPage() {
		return false;
	}

    /**
     * Returns <code>true</code> if this blog entry is an unpublished draft.
     * @return <code>true</code> if blog entry is a draft, <code>false</code> otherwise
     */
    boolean isDraft() {
		return false;
	}

    /**
     * Returns <code>true</code> if this blog entry is filed under the tag at the specified path.
     * @param path the tag path
     * @return <code>true</code> if blog entry has this tag, <code>false</code> otherwise
     */
    boolean hasTag(String path) {
		return false;
	}

    /**
     * Returns <code>true</code> if this blog entry has one or more attachments.
     * @return <code>true</code> if blog entry has attachments, <code>false</code> otherwise
     */
    boolean hasAttachments() {
		return false;
	}

    /**
     * Returns <code>true</code> if this blog entry has one or more comments.
     * @return <code>true</code> if blog entry has comments, <code>false</code> otherwise
     */
    boolean hasComments() {
		return false;
	}

    /**
     * Returns whether this blog entry has been modified and needs to be saved.
     * @see #save()
     * @return <code>true</code> if blog entry is modified, <code>false</code> otherwise
     */
    boolean isModified() {
		return false;
	}

    /**
     * Saves all changes made to this blog entry. In case the blog entry doesn't exist yet, it will be created.
     * @throws BlogException if saving fails
     */
    void save() throws BlogException {
	}
}
