/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.blog;

import java.util.HashMap;

import org.osgi.service.event.Event;

import com.adobe.cq.social.commons.events.SocialEvent;

/**
 * Defines events that are triggered in blogs.
 */
public class BlogEvent extends SocialEvent<BlogEvent.BlogActions> {

    private static final long serialVersionUID = 1L;
    /**
     * The event topic suffix for blog events
     */
    public static final String BLOG_TOPIC = "blog";
    protected static final String BLOG_PATH = "BLOG_PATH";

    /**
     * @param entryPath - the jcr path to the blog entry page on which the event was triggered.
     * @param userId - the user id of the user who triggered the event.
     * @param blogPath - the jcr path to the blog in which the blog entry resides.
     * @param action - the blog action that triggered this event.
     */
    public BlogEvent(final String entryPath, final String userId, final String blogPath,
        final BlogEvent.BlogActions action) {
        super(BLOG_TOPIC, entryPath, userId, action, new HashMap<String, Object>(1) {

            private static final long serialVersionUID = 1L;

            {
                this.put(BLOG_PATH, blogPath);
            }
        });
    }

    /**
     * This constructor should be used to create a Blog Event given an OSGi Event.
     * @param event an OSGi event that is known to be a blog event.
     */
    private BlogEvent(final Event event) {
        super(event);
    }

    /**
     * List of available blog actions that can trigger a blog event.
     */
    public static enum BlogActions implements SocialEvent.SocialActions {
        CREATED, EDITED;
    }

    /**
     * @return the jcr path to the blog in which this event happened.
     */
    public String getBlogPath() {
		return null;
	}

    /**
     * Retrieves the blog event from the given generic {@link Event}.
     * @param event The event.
     * @return The blog event or <code>null</code> if the given event is not a blog event.
     */
    public static BlogEvent fromEvent(final Event event) {
		return null;
	}
}
