/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.blog;

import javax.servlet.http.HttpServletRequest;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.ResourceResolver;

/**
 * <code>BlogManager</code> is a central interface to manage blogs and blog entries. To retrieve a BlogManager, please
 * use {@link ResourceResolver#adaptTo(Class)}.
 */
public interface BlogManager {

    static final String ATTR_BLOG = "blog";

    /**
     * Returns the <code>Blog</code> at the specified path, or <code>null</code> if no blog can be found.
     * @param path The blog path
     * @return The blog
     */
    Blog getBlog(String path) {
		return null;
	}

    /**
     * Returns the <code>Blog</code> for the specified request. The corresponding resource will be retrieved from the
     * request, and the blog will be instantiated only once per request.
     * @param req The request
     * @return The blog
     */
    Blog getBlog(SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Returns the <code>Blog</code> at the specified path, or <code>null</code> if no blog can be found. The request
     * is needed for certain operations.
     * @param req The request
     * @param path The blog path
     * @return The blog
     */
    Blog getBlog(SlingHttpServletRequest req, String path) {
		return null;
	}

    /**
     * Creates a <code>Blog</code> at <code>path</code> using the specified title and description.
     * @param path The path of the blog
     * @param title The title of the blog
     * @return The blog
     */
    Blog createBlog(String path, String title) {
		return null;
	}

    /**
     * Returns the <code>BlogEntry</code> at the specified path or <code>null</code> if no blog entry can be found.
     * @param path The blog entry path
     * @return The blog entry
     */
    BlogEntry getBlogEntry(String path) {
		return null;
	}

    /**
     * Returns the <code>BlogEntry</code> for the specified request and resource. If the resource is <code>null</code>
     * , it will be retrieved from the request.
     * @param req The request
     * @param path The blog entry path (optional)
     * @return The blog entry
     */
    BlogEntry getBlogEntry(SlingHttpServletRequest req, String path) {
		return null;
	}

    /**
     * Returns the selected view for the blog. The value can be one of:
     * <ul>
     * <li>{@link Blog#VIEW_DEFAULT} (default)</li>
     * <li>{@link Blog#VIEW_DEFAULT}</li>
     * <li>{@link Blog#VIEW_NEW}</li>
     * <li>{@link Blog#VIEW_SEARCH}</li>
     * <li>{@link Blog#VIEW_ARCHIVE}</li>
     * </ul>
     * @param req The request
     * @return The view
     */
    String getView(final HttpServletRequest req) {
		return null;
	}

}
