/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.blog.antispam;

import java.util.Map;

public interface AkismetService {

    /**
     * Check to see if the API key has been verified.
     * @return <code>true</code> if the API key has been verified, <code>false</code> otherwise
     */
    boolean isVerifiedKey() {
		return false;
	}

    /**
     * Sets proxy configuration information. This method must be called before any calls to the API if you require
     * proxy configuration.
     * @param proxyHost Proxy host
     * @param proxyPort Proxy port
     */
    void setProxyConfiguration(String proxyHost, int proxyPort) {
	}

    /**
     * Sets proxy authentication information. This method must be called before any calls to the API if you require
     * proxy authentication.
     * @param proxyUsername Username to access proxy
     * @param proxyPassword Password to access proxy
     */
    void setProxyAuthenticationConfiguration(String proxyUsername, String proxyPassword) {
	}

    /**
     * Verify your API key.
     * @return <code>true</code> if the API key has been verified, <code>false</code> otherwise
     */
    boolean verifyAPIKey() {
		return false;
	}

    /**
     * From the API docs, This is basically the core of everything. This call takes a number of arguments and
     * characteristics about the submitted content and then returns a thumbs up or thumbs down. Almost everything is
     * optional, but performance can drop dramatically if you exclude certain elements. I would recommend erring on
     * the side of too much data, as everything is used as part of the Akismet signature."
     * @param ipAddress IP address of the comment submitter
     * @param userAgent User agent information
     * @param referrer The content of the HTTP_REFERER header should be sent here
     * @param permalink The permanent location of the entry the comment was submitted to
     * @param commentType May be blank, comment, trackback, pingback, or a made up value like "registration"
     * @param author Submitted name with the comment
     * @param authorEmail Submitted email address
     * @param authorURL Commenter URL
     * @param commentContent The content that was submitted
     * @param other In PHP there is an array of enviroment variables called $_SERVER which contains information about
     *            the web server itself as well as a key/value for every HTTP header sent with the request. This data
     *            is highly useful to Akismet as how the submited content interacts with the server can be very
     *            telling, so please include as much information as possible.
     * @return <code>true</code> if the comment is identified by Akismet as spam, <code>false</code> otherwise
     */
    @SuppressWarnings("unchecked")
    boolean commentCheck(String ipAddress, String userAgent, String referrer, String permalink, String commentType,
        String author, String authorEmail, String authorURL, String commentContent, Map other) {
		return false;
	}

    /**
     * From the API docs, This call is for submitting comments that weren't marked as spam but should have been. It
     * takes identical arguments as comment check."
     * @param ipAddress IP address of the comment submitter
     * @param userAgent User agent information
     * @param referrer The content of the HTTP_REFERER header should be sent here
     * @param permalink The permanent location of the entry the comment was submitted to
     * @param commentType May be blank, comment, trackback, pingback, or a made up value like "registration"
     * @param author Submitted name with the comment
     * @param authorEmail Submitted email address
     * @param authorURL Commenter URL
     * @param commentContent The content that was submitted
     * @param other In PHP there is an array of enviroment variables called $_SERVER which contains information about
     *            the web server itself as well as a key/value for every HTTP header sent with the request. This data
     *            is highly useful to Akismet as how the submited content interacts with the server can be very
     *            telling, so please include as much information as possible.
     */
    @SuppressWarnings("unchecked")
    void submitSpam(String ipAddress, String userAgent, String referrer, String permalink, String commentType,
        String author, String authorEmail, String authorURL, String commentContent, Map other) {
	}

    /**
     * From the API docs, This call is intended for the marking of false positives, things that were incorrectly
     * marked as spam. It takes identical arguments as comment check and submit spam."
     * @param ipAddress IP address of the comment submitter
     * @param userAgent User agent information
     * @param referrer The content of the HTTP_REFERER header should be sent here
     * @param permalink The permanent location of the entry the comment was submitted to
     * @param commentType May be blank, comment, trackback, pingback, or a made up value like "registration"
     * @param author Submitted name with the comment
     * @param authorEmail Submitted email address
     * @param authorURL Commenter URL
     * @param commentContent The content that was submitted
     * @param other In PHP there is an array of enviroment variables called $_SERVER which contains information about
     *            the web server itself as well as a key/value for every HTTP header sent with the request. This data
     *            is highly useful to Akismet as how the submited content interacts with the server can be very
     *            telling, so please include as much information as possible.
     */
    @SuppressWarnings("unchecked")
    void submitHam(String ipAddress, String userAgent, String referrer, String permalink, String commentType,
        String author, String authorEmail, String authorURL, String commentContent, Map other) {
	}
}
