/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.calendar;

import java.io.IOException;
import java.io.PrintWriter;

import org.apache.sling.api.resource.ResourceResolver;

/**
 * <code>CalendarManager</code> is a central interface to manage cq social calendars. To retrieve a CalendarManager,
 * please use {@link ResourceResolver#adaptTo(Class)}.
 * @since 5.6
 */
public interface CalendarManager {

    /**
     * Returns a calendar at the given path, if present, or <code>null</code> if there is no node or calendar at that
     * path.
     * @param path calendar path
     * @return the object for the calendar or <code>null</code>
     */
    CqCalendar getCalendar(String path) {
		return null;
	}

    /**
     * Tries to find a calendar at the given path or its ancestors, which is explicitly marked by node types. If none
     * is found, the result depends on the 'forceIfNotFound' flag: if <code>true</code>, the given path will be
     * assumed to be a {@link CqCalendar} and returned; if it is <code>false</code>, <code>null</code> will be
     * returned. Note that <code>null</code> can only be returned if 'forceIfNotFound' is <code>false</code> or if the
     * given path does not point to an existing resource.
     * @param path start path for searching
     * @param forceIfNotFound if <code>true</code>, force given path for returned {@link CqCalendar} if none can be
     *            found, otherwise will return <code>null</code> if no marked calendar was found
     * @return the object wrapping the calendar or <code>null</code>
     * @since 5.6
     */
    CqCalendar findCalendar(String path, boolean forceIfNotFound) {
		return null;
	}

    /**
     * Creates or gets a calendar at the given path. If there is an existing calendar at that path, it will be
     * returned; if there is a node at that path, it will be marked as calendar; otherwise the path and the calendar
     * node will be created.
     * @param path calendar path
     * @return the object for the calendar
     * @throws CalendarException if something went wrong during path creation
     */
    CqCalendar createCalendar(String path) throws CalendarException {
		return null;
	}

    /**
     * Implements iCalendar folding (breaking of long lines) as specified in <a
     * href="http://tools.ietf.org/html/rfc2445#section-4.1">RFC-2445, section 4.1</a>. According to the spec, folding
     * lines longer as 75 chars is strongly recommended, but to be consistent with Apple iCal, this method will fold
     * at 73 chars.
     * <p>
     * The passed string must already be in proper iCalendar format (eg. using {@link Event#toICalendarString()}). It
     * will be folded and written to the given {@link PrintWriter}.
     * @param icalString an iCalendar formatted string to be folded
     * @param writer where the folded string is written to
     * @throws IOException if something failed writing to the PrintWriter
     */
    void printICSFoldedString(String icalString, PrintWriter writer) throws IOException {
	}
}
