/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.calendar;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Iterator;

/**
 * {@linkplain CqCalendar} represents a single calendar, eg. a "work" or "home" calendar. Each calendar is made up of
 * a set of {@link Event events} and todos.
 * <p>
 * Note: this interface is named {@link CqCalendar} instead of just <code>Calendar</code> to avoid conflicts with
 * {@link Calendar java.util.Calendar} and iCal4j's <code>net.fortuna.ical4j.model.Calendar</code>.
 * @since 5.6
 */
public interface CqCalendar {

    /**
     * Returns the resource path of this calendar.
     */
    String getPath() {
		return null;
	}

    /**
     * Returns the title of this calendar.
     */
    String getName() {
		return null;
	}

    /**
     * Imports an iCalendar file (*.ics, RFC-2445) into this calendar.
     * @param icalendarStream a stream containing an iCalendar file
     * @throws IOException if reading/parsing the stream failed
     */
    void importICS(InputStream icalendarStream) throws IOException {
	}

    /**
     * Exports all events and todos in this calendar as an iCalendar file (*.ics, RFC-2445).
     * @param out the outputstream that will be written to
     * @throws IOException if writing to the outputstream failed
     * @throws CalendarException if no valid iCalendar could be built
     */
    void exportICS(OutputStream out) throws IOException, CalendarException {
	}

    /**
     * Returns the event at the given path. This is equivalent to
     * <code>resourceResolver.resolve(path).adaptTo(Event.class)</code> (if the resource exists).
     * @param path absolute resource path to an event
     * @return an event object or <code>null</code> if there is no resource or no event at the given path
     */
    Event getEvent(String path) {
		return null;
	}

    /**
     * Searches inside this calendar for an event with the given uid. In case of a recurring event, this will always
     * return the original event that defines the recurrence. If no matching event is found, <code>null</code> will be
     * returned.
     * <p>
     * If multiple matching events are found, which should not be the normal case, the first event found will be
     * returned.
     * @param uid an iCalendar UID, which must be globally unique, but can be any string
     * @return the event found or <code>null</code>
     */
    Event findEvent(String uid) {
		return null;
	}

    /**
     * Returns an iterator with all events found in this calendar. The order of events is not necessarily ensured.
     */
    Iterator<Event> getEvents() {
		return null;
	}

    /**
     * Custom iterator that recursively goes down the tree below this calendar's resource and looks for events. and
     * returns the events in sorted Order
     * @param path Path from where events are wanted
     * @return iterator Events Iterator
     */
    Iterator<Event> getSortedEvents(final String path) {
		return null;
	}

}
