/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.calendar;

import java.util.Calendar;
import java.util.Iterator;

import org.apache.sling.api.resource.ValueMap;

import com.adobe.cq.social.calendar.client.api.CalendarEvent;

/**
 * <code>Event</code> represents a single calendar event.
 * @since 5.6
 * @deprecated @since AEM 6.0. Use {@link CalendarEvent} insteand.
 */
@Deprecated
public interface Event {

    /**
     * Returns the resource path of this event.
     * @return the path
     */
    String getPath() {
		return null;
	}

    /**
     * Returns the icalendar UID, which is a globally unique identifier for events.
     * @return the uid
     */
    String getUid() {
		return null;
	}

    /**
     * Returns the start date of the event. If it is an all-day event, {@link #isDate()} will return true and the time
     * part can be ignored.
     * @return the start
     * @since 5.6
     */
    Calendar getStart() {
		return null;
	}

    /**
     * Returns the end date of the event. If it is an all-day event, {@link #isDate()} will return true and the time
     * part can be ignored.
     * @return the end
     * @since 5.6
     */
    Calendar getEnd() {
		return null;
	}

    /**
     * Returns if this event is an all-day event.
     * @return true, if is date
     * @since 5.6
     */
    boolean isDate() {
		return false;
	}

    /**
     * Returns a value map of the event content.
     * @return the properties
     */
    ValueMap getProperties() {
		return null;
	}

    /**
     * Returns the calendar that contains this event.
     * @return the calendar
     * @since 5.6
     */
    CqCalendar getCalendar() {
		return null;
	}

    /**
     * Returns if this event is part of a recurring series. This will return <code>true</code> if this is the original
     * event that defines a recurrence or if it is {@link #isRecurrence() a recurring event instance}.
     * @return <code>true</code> if this event is part of a recurring series
     * @see #isRecurrence()
     * @see #getOriginal()
     * @since 5.3
     */
    boolean isRecurringSeries() {
		return false;
	}

    /**
     * Returns whether this event is the recurring instance of an event with recurring rule(s). To get the original
     * event, use {@link #getOriginal()}. Note that for the original event, which is also part of the recurring event
     * series, this will return <code>false</code>. To find out if an event is part of a recurring series, use
     * @return <code>true</code> if this is a recurrence, <code>false</code> if this is the original event
     *         {@link #isRecurringSeries()}.
     *         <p>
     *         Recurring events are copied into the JCR for each instance to allow for simpler display and search,
     *         without the need to dynamically re-calculate the recurrence on each read access.
     * @see #isRecurringSeries()
     * @see #getOriginal()
     */
    boolean isRecurrence() {
		return false;
	}

    /**
     * Returns the original event of a recurring event. If this is no recurring event or if this event object already
     * denotes the original event, this event itself will be returned, ie. this method will never return
     * <code>null</code>. To find out if an event is part of a recurring series, use {@link #isRecurringSeries()}.
     * <p>
     * Recurring events are copied into the JCR for each instance to allow for simpler display and search, without the
     * need to dynamically re-calculate the recurrence on each read access.
     * @return original event or itself (never <code>null</code>)
     * @see #isRecurringSeries()
     * @see #isRecurrence()
     */
    Event getOriginal() {
		return null;
	}

    /**
     * Returns all recurring instances of this event found in the repository. This does not include this event itself
     * and is only applicable to an original event, ie. when <code>{@link #isRecurrence()} == false</code>.
     * @return iterator over the recurring event instances
     */
    Iterator<Event> getRecurrences() {
		return null;
	}

    /**
     * This will update the recurring instances of this event. Must be called whenever the properties of the
     * underlying event node have been changed.
     * <p>
     * This only works if this event is an original ( <code>{@link #isRecurrence()} == false</code>) event, if it
     * actually defines a recurrence rule and if the underlying JCR session allows write access.
     * <p>
     * In detail, this will remove the existing set of recurrences and create new ones. But nothing might happen if
     * the rule did not change.
     * @param save if true, will automatically save the underlying jcr session
     * @throws CalendarException if something goes wrong when saving
     * @since 5.6
     */
    void updateRecurrences(boolean save) throws CalendarException {
	}

    /**
     * This will take this single instance out of its recurring event series and create a separate event, if it is
     * part of a recurrence. This will give the event a new UID and also mark its date as an exception in the original
     * event.
     * @param save if true, will automatically save the underlying jcr session
     * @throws CalendarException if something goes wrong
     * @since 5.6
     */
    void takeOutOfRecurrence(boolean save) throws CalendarException {
	}

    /**
     * This will remove this event from its recurring event series (if present), but only this and not the whole
     * series.
     * @param save if true, will automatically save the underlying jcr session
     * @throws CalendarException if something goes wrong
     * @since 5.6
     */
    void deleteFromRecurrence(boolean save) throws CalendarException {
	}

    /**
     * Returns this event serialized as iCalendar object (RFC-2445).
     * <p>
     * Please note that in order to put this into a file, one has to ensure the correct <i>folding</i> of overlong
     * lines, using {@link CalendarManager#printICSFoldedString(String, java.io.PrintWriter)}.
     * <p>
     * Example:
     * 
     * <pre>
     * BEGIN:VEVENT
     * DTSTAMP:20090220T175158Z
     * UID:CCF41603-B770-11D7-8FBE-0050E4C50C66
     * DTSTART;VALUE=DATE:20031016
     * DTEND;VALUE=DATE:20031017
     * SUMMARY:Kill Bill Volume 1
     * END:VEVENT
     * </pre>
     * @return a icalendar-formatted string representation of this event
     */
    String toICalendarString() {
		return null;
	}
}
