/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.commons;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.ReferenceCardinality;
import org.apache.felix.scr.annotations.ReferencePolicy;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.request.RequestDispatcherOptions;
import org.apache.sling.api.request.RequestParameter;
import org.apache.sling.api.request.RequestParameterMap;
import org.apache.sling.api.resource.ModifiableValueMap;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.api.servlets.HttpConstants;
import org.apache.sling.api.wrappers.SlingHttpServletRequestWrapper;
import org.apache.sling.commons.osgi.OsgiUtil;
import org.apache.sling.jcr.resource.JcrResourceResolverFactory;
import org.apache.sling.settings.SlingSettingsService;
import org.apache.tika.metadata.HttpHeaders;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.comments.endpoints.AbstractCommentOperation;
import com.adobe.cq.social.commons.comments.endpoints.AbstractCommentOperationService;
import com.adobe.granite.security.user.UserProperties;
import com.day.cq.commons.Externalizer;
import com.day.cq.commons.jcr.JcrUtil;
import com.day.cq.replication.Agent;
import com.day.cq.replication.AgentFilter;
import com.day.cq.replication.ReplicationActionType;
import com.day.cq.security.NoSuchAuthorizableException;
import com.day.cq.security.UserManager;
import com.day.cq.security.UserManagerFactory;

/**
 * The <code>AbstractCreateCommentServlet</code> provides the basic facilities for creating comments via a POST
 * request. Implementations may vary on the properties persisted and request handling.
 * @since CQ 5.6.0
 * @deprecated @since AEM 6.0. Use {@link AbstractCommentOperationService} or {@link AbstractCommentOperation} instead.
 */
@Component(metatype = false, componentAbstract = true)
@Deprecated
public abstract class AbstractCreateCommentServlet extends AbstractSessionServlet {
    /**
     * Replication agents Filter that includes reverse replication ("distribute") agents.
     */
    public static final AgentFilter REV_AGENT_FILTER = new AgentFilter() {
        @Override
        public boolean isIncluded(final Agent agent) {
			return false;
		}
    };

    /**
     * The name of the property that contains defines whether the comment should be reverse-replicated.
     */
    public static final String PROPERTY_REVERSE_REPLICATE = "reverseReplicate";

    /**
     * The name of the property that holds a white list of form field names added as additional properties to the
     * comment.
     */
    public static final String PROPERTY_FIELD_WHITELIST = "fieldWhitelist";

    /**
     * The name of the property that holds a black list of attachment mime types that are unsafe.
     */
    public static final String PROPERTY_ATTACHMENT_TYPE_BLACKLIST = "attachmentTypeBlacklist";

    /**
     * The reverse replication state.
     */
    public static final boolean DEFAULT_REVERSE_REPLICATE = true;
    /**
     * List of properties that the comment will overwrite.
     */
    public static final String[] RESERVED_PROPERTY_NAMES = {Comment.PROP_MESSAGE, CollabUser.PROP_EMAIL,
        CollabUser.PROP_NAME, CollabUser.PROP_WEBSITE};

    /**
     * Serial ID.
     */
    private static final long serialVersionUID = 1L;

    /**
     * The regex used to match templates in the various components.
     */
    private static final String TEMPLATE_MASK = "-template\\..*";

    /**
     * The parameter name for a client requested a different template.
     */
    protected static final String TEMPLATE_FORM_ID = ":templatename";

    /**
     * Resource Resolver used for the request.
     */
    @Reference(cardinality = ReferenceCardinality.MANDATORY_UNARY, policy = ReferencePolicy.STATIC)
    protected JcrResourceResolverFactory jcrResourceResolverFactory;

    /**
     * Resource Resolver used for the request.
     */
    @Reference(cardinality = ReferenceCardinality.MANDATORY_UNARY, policy = ReferencePolicy.STATIC)
    protected Externalizer externalizer;

    /**
     * Sling repository for finding / creating nodes.
     * @Reference(cardinality = ReferenceCardinality.MANDATORY_UNARY, policy = ReferencePolicy.STATIC) protected
     *                        SlingRepository repository;
     */

    /**
     * To to allow / disallow reverseReplication for the comment creation.
     */
    protected boolean reverseReplicate;

    /**
     * White list field.
     */
    protected String[] fieldWhitelist;

    /**
     * Attachment type blacklist.
     */
    protected String[] attachmentTypeBlacklist;

    /**
     * User Manager Factory.
     */
    @Reference
    protected UserManagerFactory userManagerFactory;

    /**
     * Sling settings service.
     */
    @Reference
    protected SlingSettingsService settingsService;

    /**
     * Async reverse replicator.
     */
    @Reference
    protected AsyncReverseReplicator replicator;

    /**
     * The logger for this class.
     */
    private final Logger log = LoggerFactory.getLogger(this.getClass());

    @Override
    protected void doPost(final SlingHttpServletRequest request, final SlingHttpServletResponse response)
        throws ServletException, IOException {
		}

    /**
     * Checks if the request might have come from a bot.
     * @param request request to check for bot
     * @return if the request originated from a bot
     */
    protected boolean isBot(final SlingHttpServletRequest request) {
		return false;
	}

    /**
     * Retrieves the {@link ResourceResolver} based on the session.
     * @param session The {@link Session} for which to get a <code>ResourceResolver</code>.
     * @return The resource resolver.
     */
    protected ResourceResolver getResourceResolver(final Session session) {
		return null;
	}

    /**
     * Get the comment system for the given resource.
     * @param r The resource for which to retrieve the comment system.
     * @param session The {@link Session}.
     * @return The {@link com.day.cq.collab.commons.CommentSystem}.
     */
    protected CommentSystem getCommentSystem(final Resource r, final Session session) {
		return null;
	}

    /**
     * Retrieves a {@link Node} identified by the given <code>path</code>.
     * @param path The path.
     * @param session The {@link Session}.
     * @return The node.
     */
    protected Node getNode(final String path, final Session session) {
		return null;
	}

    /**
     * Redirects to the given <code>url</code>.
     * @param request The request. Used to retrieve the context path.
     * @param response The response upon which to redirect.
     * @param url The url to redirect to.
     */
    protected void redirect(final SlingHttpServletRequest request, final SlingHttpServletResponse response,
        final String url) {
		}

    /**
     * Triggers reverse replication for the node at the given path.
     * @param path path of node to replicate
     */
    protected void reverseReplicate(final String path) {
	}

    /**
     * Triggers reverse replication for the node at the given path.
     * @param actionType replication action type
     * @param path path of node to replicate
     */
    protected void reverseReplicate(final ReplicationActionType actionType, final String path) {
	}

    /**
     * Triggers reverse replication for the node at the given paths.
     * @param actionType replication action type
     * @param path paths of nodes to replicate
     */
    protected void reverseReplicate(final ReplicationActionType actionType, final List<String> paths) {
	}

    /**
     * Return all paths of comment's attachments.
     * @param session The {@link Session}.
     * @param path path of node
     * @return all paths of comment's attachments.
     */
    private List<String> getPaths(final Session session, final String path) {
		return null;
	}

    /**
     * This method sets the default properties on a comment being created. These properties are:
     * <ul>
     * <li>Date ({@link Comment#PROP_DATE}</li>
     * <li>Email ({@link CollabUser#PROP_EMAIL}</li>
     * <li>Website ({@link CollabUser#PROP_WEBSITE}</li>
     * <li>IP Address ({@link Comment#PROP_IP_ADDRESS}</li>
     * <li>User Agent ({@link Comment#PROP_USER_AGENT}</li>
     * <li>Referer ({@link Comment#PROP_REFERER}</li>
     * </ul>
     * @param request The request from which to retrieve the parameters for the properties to be set.
     * @param node The comment node upon which to set the properties.
     * @throws RepositoryException If an error occurs setting the properties.
     */
    protected void setCommentProperties(final SlingHttpServletRequest request, final Node node)
        throws RepositoryException {
		}

    protected void setCommentProperties(final SlingHttpServletRequest request, final Comment comment) {
	}

    /**
     * Set authorizable id.
     * @param request The sling request.
     * @param node The node where the authorizableId property is set
     * @throws RepositoryException if there is an error when saving to repository
     */
    private void setAuthorizableId(final SlingHttpServletRequest request, final Node node) throws RepositoryException {
	}

    /**
     * Should be joined up with the node based one, likely will need to deprecate the old one once the other layers
     * move to resource based calls.
     * @param request
     * @param properties
     */
    private void setAuthorizableId(final SlingHttpServletRequest request, final ValueMap properties) {
	}

    /**
     * check that we have the right authorizable id between the sessionProfile and the request, if not, throw an
     * exception.
     * @param request The request used to get parameters
     * @param adminSession The {@link Session}.
     * @throws RepositoryException if authorizable id between the sessionProfile and the request is incorrect
     */
    protected void checkAuthorizableId(final SlingHttpServletRequest request, final Session adminSession)
        throws RepositoryException {
		}

    /**
     * Check if the specified request parameter CollabUser.PROPNAME exists or not.
     * @param adminSession
     * @param sessionUserId the request user id
     * @param userId the userId value of the CollabUser.PROP_NAME
     * @return true if the specified userId exists, otherwise, return false.
     */
    private boolean validateAnonymousUserId(final Session adminSession, final String sessionUserId,
        final String userId) {
			return false;
		}

    /**
     * Return session userProperties.
     * @param request The sling request
     * @return session userProperties
     */
    private UserProperties getSessionUserProperties(final SlingHttpServletRequest request) {
		return null;
	}

    /**
     * Return request session.
     * @param request The sling request
     * @return request session.
     */
    private Session getRequestSession(final SlingHttpServletRequest request) {
		return null;
	}

    /**
     * Return if user exists.
     * @param userId The user id
     * @param session The {@link Session}.
     * @return if user exists
     */
    private boolean userExists(final String userId, final Session session) {
		return false;
	}

    /**
     * Return if user has permission.
     * @param userIdentifier The user id
     * @param requestSession The {@link Session}.
     * @param adminSession The administrator {@link Session}.
     * @return if user has permission.
     */
    private boolean hasPermissions(final String userIdentifier, final Session requestSession,
        final Session adminSession) {
			return false;
		}

    /**
     * Return if it is in author mode.
     * @return if it is in author mode.
     */
    private boolean isAuthorMode() {
		return false;
	}

    /**
     * Activate this component. Open the session and register event listeners.
     * @param context The component context
     */
    protected void activate(final ComponentContext context) {
	}

    /**
     * Writes a proper UGC response to the response. This handles both HTML and REST style responses.
     * @param request the current request
     * @param response the response to write to
     * @param resource the resource to serialize
     * @param location a different location to send to the client.
     * @return if it can write to UGC
     * @throws IOException IO Error
     * @throws ServletException Servlet Error
     */
    protected boolean writeUGCResponse(final SlingHttpServletRequest request,
        final SlingHttpServletResponse response, final Resource resource, final String location)
        throws ServletException, IOException {
			return false;
		}

    /**
     * Writes a proper UGC response to the response. This handles both HTML and REST style responses.
     * @param request the current request
     * @param response the response to write to
     * @param resource the resource to serialize
     * @return if it can write to UGC
     * @throws IOException IO Error
     * @throws ServletException Servlet Error
     */
    protected boolean writeUGCResponse(final SlingHttpServletRequest request,
        final SlingHttpServletResponse response, final Resource resource) throws ServletException, IOException {
			return false;
		}

    /**
     * Gets the SuperResourceType of a Resource to be created.
     * @param request request used to create the resource
     * @param resourceType Resource type being created
     * @return String that is the resource's supertype
     */
    protected String getResourceSuperType(final SlingHttpServletRequest request, final String resourceType) {
		return null;
	}

    /**
     * Searches up the resource super type hierarchy to create a list of available templates. This list can have
     * duplicates in it and should respect the app and libs folder.
     * @param resource the resource to start getting templates for
     * @return list of templates that are available for this resource
     */
    private List<String> collectTemplates(final Resource resource) {
		return null;
	}

    /**
     * Return if origins are the same.
     * @param request The sling request
     * @return if origins are the same.
     */
    protected Boolean isCORS(final SlingHttpServletRequest request) {
		return null;
	}

    /**
     * Allow origin access control.
     * @param request The sling request
     * @param response The sling response
     */
    protected void allowCORS(final SlingHttpServletRequest request, final SlingHttpServletResponse response) {
	}

    /**
     * Used to redirect a sling request to a get method so that templates don't need both get and POST JSPs.
     * @author mcweeney
     */
    private class TemplateHandlingRequest extends SlingHttpServletRequestWrapper {
        /**
         * Creates the wrapper Request.
         * @param wrappedRequest the request to wrap
         */
        public TemplateHandlingRequest(final SlingHttpServletRequest wrappedRequest) {
            super(wrappedRequest);
        }

        /**
         * Template includes always assume GET.
         * @return the HTTP method name.
         */
        @Override
        public String getMethod() {
			return null;
		}
    }
}
