/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.TreeSet;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.mail.Header;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.InternetHeaders;

import org.apache.commons.io.IOUtils;
import org.apache.commons.io.input.CountingInputStream;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.text.StrLookup;
import org.apache.commons.lang.text.StrSubstitutor;
import org.apache.commons.mail.Email;
import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.SimpleEmail;
import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.ReferenceCardinality;
import org.apache.felix.scr.annotations.ReferencePolicy;
import org.apache.felix.scr.annotations.Service;
import org.apache.sling.api.resource.LoginException;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceResolverFactory;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.api.wrappers.ValueMapDecorator;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.ugcbase.SocialResourceUtils;
import com.adobe.granite.security.user.UserProperties;
import com.adobe.granite.security.user.UserPropertiesManager;
import com.day.cq.commons.Externalizer;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.mailer.MailService;
import com.day.cq.security.UserManager;
import com.day.cq.security.UserManagerFactory;
import com.day.cq.workflow.WorkflowException;
import com.day.cq.workflow.WorkflowSession;
import com.day.cq.workflow.exec.WorkItem;
import com.day.cq.workflow.exec.Workflow;
import com.day.cq.workflow.exec.WorkflowData;
import com.day.cq.workflow.exec.WorkflowProcess;
import com.day.cq.workflow.metadata.MetaDataMap;

/**
 * Implements <code>WorkflowProcess</code> interface to be used for sending emails.
 */
@Component
@Service(value = {WorkflowProcess.class, EmailConfig.class})
@org.apache.felix.scr.annotations.Property(name = "process.label", value = "Send Email")
public class SendEmailProcess implements WorkflowProcess, EmailConfig {

    /** Default logger for this class. */
    private static final Logger LOG = LoggerFactory.getLogger(SendEmailProcess.class);

    /**
     * mail service.
     */
    @Reference(cardinality = ReferenceCardinality.OPTIONAL_UNARY, policy = ReferencePolicy.DYNAMIC)
    protected MailService mailService;

    /**
     * user manager factory.
     */
    @Reference
    protected UserManagerFactory userManagerFactory;

    /**
     * resource resolver factory.
     */
    @Reference
    private ResourceResolverFactory resourceResolverFactory;

    /**
     * Property for default character set.
     */
    private static final String DEFAULT_CHARSET = "utf-8";

    private Map<String, Map<String, String>> configValueMap;

    private Map<String, Map<String, String>> configPathMap;

    private Map<String, Long> lastConfigUpdateCheck;

    @org.apache.felix.scr.annotations.Property(value = "300000")
    private static final String CHECK_UPDATE_INTERVAL = "checkUpdate.interval";

    /**
     * The interval of time in miliseconds that the configuration should be checked again in case it has changed.
     */
    private long checkUpdateInterval = 300000;

    @Activate
    protected void activate(final ComponentContext context) {
	}

    /**
     * {@inheritDoc}
     * <p/>
     * Process arguments:
     * <p/>
     * args[0]: mail template.
     */
    @Override
    public void execute(final WorkItem item, final WorkflowSession wfSession, final MetaDataMap metaData)
        throws WorkflowException {
		}

    /**
     * Build arguments for SendEmailProcess to process.
     * @param metaData The metadata
     * @return the arguments for the process
     */
    public String[] buildArguments(final MetaDataMap metaData) {
		return null;
	}

    /**
     * Create email.
     * @param template The template
     * @param charSet The character set
     * @param substitutor The String substitutor
     * @return Email the email
     * @throws Exception when an error occurs
     */
    private Email createEmail(final String template, final String charSet, final StrSubstitutor substitutor)
        throws Exception {
			return null;
		}

    /**
     * Save remove header.
     * @param hdrs The header map
     * @param name The name of the header
     * @return String[] the headers
     */
    private static String[] saveRemoveHeader(final Map<String, String[]> hdrs, final String name) {
		return null;
	}

    /**
     * Get character set.
     * @param session The session
     * @param path The path of the node
     * @return String[] the headers
     */
    private String getCharSet(final Session session, final String path) {
		return null;
	}

    /**
     * Loads the mail templates from the repository.
     * @param path mail templates root path
     * @param session session
     * @param charSet The character set
     * @return a reader to the template or <code>null</code> if not valid.
     */
    public String loadTemplate(final Session session, final String path, final String charSet) {
		return null;
	}

    /**
     * Perform payload lookup.
     */
    private class PayloadLookup extends StrLookup {

        /**
         * Payload lookup properties.
         */
        private final ValueMap properties = new ValueMapDecorator(new HashMap<String, Object>());

        /**
         * Content node.
         */
        private Node content = null;
        private ValueMap contentAsVM = null;

        /**
         * PayloadLookup constructor to perform payload lookup.
         * @param workItem The work item
         * @param session session
         * @param userManager The user manager
         * @param resolver The resource resolver
         * @param configType The configuration type.
         * @throws RepositoryException if there is an issue with the repository.
         */
        private PayloadLookup(final WorkItem workItem, final WorkflowSession session, final UserManager userManager,
            final String configType, final ResourceResolver resolver) throws RepositoryException {

            // load work item properties
            put("item.id", workItem.getId());
            put("item.node.id", workItem.getNode().getId());
            put("item.node.title", workItem.getNode().getTitle());
            put("item.node.type", workItem.getNode().getType());
            putAll(workItem.getNode().getMetaDataMap(), "item.node.data.");

            final Workflow workflow = workItem.getWorkflow();

            // instance
            put("instance.id", workflow.getId());
            put("instance.state", workflow.getState());
            putAll(workflow.getMetaDataMap(), "instance.data.");

            // model
            put("model.title", workflow.getWorkflowModel().getTitle());
            put("model.description", workflow.getWorkflowModel().getDescription());
            put("model.id", workflow.getWorkflowModel().getId());
            put("model.version", workflow.getWorkflowModel().getVersion());
            putAll(workflow.getWorkflowModel().getMetaDataMap(), "model.data.");

            if (configValueMap.containsKey(configType)) {
                // Check if the config is up to date and then add the config values to the properties map.
                checkConfigUpdate(configType);
                properties.putAll(configValueMap.get(configType));
            }

            // initiator
            final String userPath =
                workItem.getWorkflow().getMetaDataMap().get(ToggleUtil.PN_WORKFLOW_METADATA_USER, "");

            final UserPropertiesManager upMgr = resolver.adaptTo(UserPropertiesManager.class);

            UserProperties initiator = null;
            if (StringUtils.isBlank(userPath)) {
                final String initiatorId = workflow.getInitiator();
                initiator = upMgr.getUserProperties(initiatorId, "profile");
            } else {
                final int lastBackSlashIndex = userPath.lastIndexOf('/');
                if (lastBackSlashIndex != -1) {
                    initiator = upMgr.getUserProperties(userPath.substring(lastBackSlashIndex + 1), "profile");
                }
            }

            if (initiator == null) {
                throw new NullPointerException("Null initiator with userPath=" + userPath + " and initiatorId="
                        + workflow.getInitiator());
            }
            putAll(initiator, "initiator.");

            // data
            final WorkflowData workflowData = workflow.getWorkflowData();
            putAll(workflowData.getMetaDataMap(), "data.");
            if ("JCR_PATH".equals(workflowData.getPayloadType())) {
                final String path = workflowData.getPayload().toString();
                put("payload.path", path);

                final Resource res = resolver.getResource(path);
                if (SocialResourceUtils.isSocialResource(res)) {
                    contentAsVM = res.adaptTo(ValueMap.class);
                } else {
                    try {
                        content = session.getSession().getNode(path);
                    } catch (final RepositoryException e) {
                        LOG.error("Payload content not accessible: {}", e.toString());
                    }
                }
            } else {
                put("payload.data", workflowData.getPayload());
                put("payload.type", workflowData.getPayloadType());
            }

            // add special props property for debugging
            final StringBuilder props = new StringBuilder();
            final TreeSet<String> names = new TreeSet<String>(properties.keySet());
            for (final String name : names) {
                final Object value = properties.get(name);
                LOG.debug("- {}:{}", name, value);
                props.append(name).append(":").append(value).append("\n");
            }
            put("properties", props.toString());
        }

        @Override
        public String lookup(final String key) {
			return null;
		}

        /**
         * Put method for payload lookup properties.
         * @param name The name of the property
         * @param value The value of the property
         */
        private void put(final String name, final Object value) {
		}

        /**
         * Add all date to the payload lookup properties.
         * @param metaData The metadata
         * @param prefix The prefix of the name
         */
        private void putAll(final MetaDataMap metaData, final String prefix) {
		}

        /**
         * Escape variables of the value.
         * @param value The value to escape the variables
         * @return the new value after escape the variables
         */
        private String escapeVariables(final String value) {
			return null;
		}

        /**
         * Add all date to the payload lookup properties.
         * @param auth The authoriable
         * @param prefix The prefix of the name
         */
        private void putAll(final UserProperties auth, final String prefix) throws RepositoryException {
		}

    }

    /**
     * Given a configuration node path and a property name, read the property and return it or return null if it does
     * not exist.
     * @param resolver the resource resolver.
     * @param configPath the configuration node path.
     * @param propertyName the name of the property.
     * @return read the property and return it or return null if it does not exist.
     * @throws RepositoryException thrown if there is an issue with the repository.
     * @throws IOException thrown if there is an issue reading the property file.
     */
    private String getProperty(final ResourceResolver resolver, final String configPath, final String propertyName)
        throws RepositoryException, IOException {
			return null;
		}

    /**
     * Read the config values.
     */
    private void readConfig(final String configType) {
	}

    /**
     * Check the last time the configuration was read and read it again if the next time interval has been reached.
     * @param configType the type of configuration we are checking.
     */
    private void checkConfigUpdate(final String configType) {
	}

    @Override
    public String getConfig(final String configType, final String name) {
		return null;
	}

    @Override
    public void setConfig(final String configType, final String name, final String configPath,
        final String defaultValue) {
		}
}