/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.client.api;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.RepositoryException;

import org.apache.commons.lang3.StringUtils;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.client.impl.UserImpl;
import com.adobe.cq.social.scoring.api.ScoringException;
import com.adobe.cq.social.scoring.api.ScoringService;
import com.adobe.cq.social.ugcbase.CollabUser;
import com.adobe.cq.social.ugcbase.SocialUtils;
import com.adobe.cq.social.ugcbase.SocialUtils.AVATAR_SIZE;
import com.adobe.granite.security.user.UserProperties;
import com.adobe.granite.security.user.UserPropertiesManager;
import com.day.cq.wcm.api.Page;
import com.fasterxml.jackson.annotation.JsonIgnore;

public abstract class AbstractUser extends BaseSocialComponent implements User {
    private static final String BADGE_IMG_PATH = "/jcr:content/image/file/jcr:content";

    private static final String RULE_SEPARATOR = "#";

    private static final String PN_MAPPINGS = "mappings";

    private static final String BADGE_PATH = "/jcr:content/badge";

    private static final String PN_NUMBER_OF_POSTS = "numberOfPosts";

    private static final String FORUM_PATH = "forum";

    private static final String HTML_EXT = ".html";

    private static final String PROFILE_EXT = ".form.html";

    private static final String PROFILE_PATH = "profile";

    private static final String SOCIAL_AUTHORS_PREFIX = "/social/authors/";

    private final UserProperties userProps;

    private static final Logger LOG = LoggerFactory.getLogger(UserImpl.class);

    /** Default avatar */
    static final String DEFAULT_AVATAR_PATH = SocialUtils.DEFAULT_AVATAR;

    private String name;
    private String avatarPath;
    private String userId;
    private final ClientUtilities clientUtils;
    private String largeAvatarPath;
    private final ScoringService scoring;

    public AbstractUser(final Resource resource, final ClientUtilities clientUtils, final UserPropertiesManager upm,
        final ScoringService scoring) {
        super(resource, clientUtils);
        this.clientUtils = clientUtils;
        this.scoring = scoring;
        try {
            userProps = getUserProperties(resource, upm);
        } catch (final RepositoryException e) {
            LOG.error("Unable to get userId from resource ", e);
            throw new RuntimeException("Unable to find userproperties for " + resource.getPath());
        }
        if (null == userProps) {
            LOG.error("Unable to get userId from resource ", resource.getPath());
            setUpDefaults();
        } else {
            final String authorId = userProps.getAuthorizableID();
            String avatarpath = DEFAULT_AVATAR_PATH;
            String largeAvatarpath = DEFAULT_AVATAR_PATH;
            String usrName = CollabUser.ANONYMOUS;
            this.userId = userProps.getAuthorizableID();
            if (authorId != null && authorId.length() > 0) {

                avatarpath = clientUtils.getSocialUtils().getAvatar(userProps, DEFAULT_AVATAR_PATH);
                largeAvatarpath =
                    clientUtils.getSocialUtils().getAvatar(userProps, DEFAULT_AVATAR_PATH, AVATAR_SIZE.FOURTY_EIGHT);
                try {
                    usrName = userProps.getDisplayName();
                } catch (final RepositoryException e) {
                    LOG.error("Unable to get username for ", userId);
                }
            }

            this.avatarPath = avatarpath;
            this.largeAvatarPath = largeAvatarpath;
            this.name = usrName;
        }
    }

    private void setUpDefaults() {
	}

    private UserProperties getUserProperties(final Resource resource, final UserPropertiesManager upm)
        throws RepositoryException {
			return null;
		}

    @Override
    public String getName() {
		return null;
	}

    @Override
    public String getAvatarUrl() {
		return null;
	}

    @Override
    public String getLargeAvatarUrl() {
		return null;
	}

    @Override
    @JsonIgnore
    public String getUserId() {
		return null;
	}

    @Override
    public String getProfileUrl() {
		return null;
	}

    @Override
    public Long getNumberOfPosts() {
		return null;
	}

    @Override
    public List<Badge> getBadges() {
		return null;
	}
    
    /**
     * @returns the {@link UserProperties} object associated with the the user that is represented by this instance.
     */
    protected UserProperties getUserProperties() {
		return null;
	}
}
