/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.client.api;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.request.RequestPathInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.client.endpoints.DefaultSocialGetServlet;

/**
 * Default implementation of the QueryRequestInfo.
 */
public class BaseQueryRequestInfo implements QueryRequestInfo {
    /** Logger for this class. */
    private static final Logger LOG = LoggerFactory.getLogger(BaseQueryRequestInfo.class);

    private final boolean isQueryRequest;
    private final Map<String, String[]> predicates;
    private CollectionPagination pagination;
    private CollectionSortedOrder order = CollectionSortedOrder.DEFAULT_ORDER;
    private List<String> sortBy;

    public static QueryRequestInfo DEFAULT_QUERY_REQUEST = new BaseQueryRequestInfo();
    private static String PN_QUERY_FILTER = "filter";

    /**
     * Private constructor, creating a default Query Request Info.
     */
    private BaseQueryRequestInfo() {
        this.pagination = CollectionPagination.DEFAULT_PAGINATION;
        this.isQueryRequest = false;
        this.predicates = Collections.emptyMap();
    }

    /**
     * Constructor which parses the info from the specified {@link SlingHttpServletRequest}.
     * @param request the http servlet request.
     */
    @SuppressWarnings("unchecked")
    public BaseQueryRequestInfo(final SlingHttpServletRequest request) {
        this.pagination = getPaginationFromRequest(request);
        this.isQueryRequest = isQueryRequest(request);
        this.predicates = (isQueryRequest) ? request.getParameterMap() : Collections.emptyMap();
    }

    /**
     * Constructor.
     * @param isQueryRequest
     * @param predicates
     * @param pagination
     */
    public BaseQueryRequestInfo(final boolean isQueryRequest, final Map<String, String[]> predicates,
        final CollectionPagination pagination) {
        this.isQueryRequest = isQueryRequest;
        this.predicates = predicates;
        this.pagination = pagination;
    }

    /**
     * Constructor.
     * @param isQueryRequest
     * @param predicates
     * @param pagination
     */
    public BaseQueryRequestInfo(final boolean isQueryRequest, final Map<String, String[]> predicates,
        final CollectionPagination pagination, final CollectionSortedOrder order) {
        this.isQueryRequest = isQueryRequest;
        this.predicates = predicates;
        this.pagination = pagination;
        this.order = order;
    }

    /**
     * Constructor.
     * @param isQueryRequest
     * @param predicates
     * @param pagination
     */
    public BaseQueryRequestInfo(final boolean isQueryRequest, final Map<String, String[]> predicates,
        final CollectionPagination pagination, final CollectionSortedOrder order, final List<String> sortBy) {
        this.isQueryRequest = isQueryRequest;
        this.predicates = predicates;
        this.pagination = pagination;
        this.order = order;
        this.sortBy = sortBy;
    }

    /**
     * Parse the request and produce the <code>CollectionPagination</code>. This method assumes that the offset and
     * length selector is the last 2 selectors.
     * @param request
     * @return the specified pagination in the request; if the specified request is null, or there is no pagination
     *         selector in the request, the method returns the default
     *         <code>CollectionPagination.DEFAULT_PAGINATION</code>
     */
    protected CollectionPagination getPaginationFromRequest(final SlingHttpServletRequest request) {
		return null;
	}

    /**
     * Check whether the specified request contains queryable parameters or not.
     * @param request the specified request.
     * @return
     */
    private boolean isQueryRequest(final SlingHttpServletRequest request) {
		return false;
	}

    @Override
    public boolean isQuery() {
		return false;
	}

    @Override
    public Map<String, String[]> getPredicates() {
		return null;
	}

    @Override
    public CollectionPagination getPagination() {
		return null;
	}

    @Override
    public void setPagination(final CollectionPagination pagination) {
	}

    @Override
    public void setSortBy(final List<String> sortBy) {
	}

    @Override
    public List<String> getSortBy() {
		return null;
	}

    @Override
    public void setSortOrder(final CollectionSortedOrder order) {
	}

    @Override
    public CollectionSortedOrder getSortOrder() {
		return null;
	}

}
