/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.client.api;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ValueMap;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

/**
 * Default SocialComponent implementation for all Resource Types. This class should be extended to implement
 * SocialComponent for other Resource Types.
 */
@JsonAutoDetect(fieldVisibility = JsonAutoDetect.Visibility.PUBLIC_ONLY, getterVisibility = Visibility.PUBLIC_ONLY,
        isGetterVisibility = Visibility.PUBLIC_ONLY, setterVisibility = Visibility.NONE)
@JsonInclude(Include.NON_EMPTY)
public class BaseSocialComponent implements SocialComponent {

    private static ObjectMapper objectMapper;

    protected final Resource resource;
    protected final ResourceID id;
    protected final Properties properties;

    static {
        objectMapper = new ObjectMapper();
        objectMapper.configure(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS, true);
    }

    /**
     * Instance of ClientUtilities for accessing helper methods for XSS support and generating URLs.
     */
    protected ClientUtilities clientUtils;

    /**
     * A list of properties that will be omitted when you getProperties for the resource. The default will omit
     * 'cq.*', 'ip', 'email', 'userAgent'. This list can contain regex.
     */
    protected final List<String> ignoredProperties = new ArrayList<String>(Arrays.asList("cq:.*", "ip", "email",
        "userAgent", "sling:.*"));

    private final List<String> whiteList = Arrays.asList("cq:tags");

    /**
     * @param resource the resource for which this BaseSocialComponent will be created for.
     * @param clientUtils an instance of ClientUtilities for accessing helper methods for XSS support and generating
     *            URLs.
     */
    public BaseSocialComponent(final Resource resource, final ClientUtilities clientUtils) {
        this.resource = resource;
        this.id = new DefaultResourceID(resource);
        this.properties = this.getPropertiesFromValueMap();
        this.clientUtils = clientUtils;
    }

    @Override
    public ResourceID getId() {
		return null;
	}

    @Override
    public String getResourceType() {
		return null;
	}

    @Override
    public String getUrl() {
		return null;
	}

    @Override
    public Properties getProperties() {
		return null;
	}

    private Properties getPropertiesFromValueMap() {
		return null;
	}

    /**
     * This method determines whether a property should be omitted or not by checking against the ignore list.
     * @param key property name
     * @return true if this property is to be omitted, false otherwise.
     */
    protected boolean isKeyIgnored(final String key) {
		return false;
	}

    /**
     * This method returns the list of property names/regex that need to be omitted. Override this method to return a
     * more comprehensive list specific to your component.
     * @return a list of property names/regex that need to be omitted.
     */
    protected List<String> getIgnoredProperties() {
		return null;
	}

    @Override
    @JsonIgnore
    public String toJSONString(final boolean tidy) throws JsonException {
		return null;
	}

    @Override
    @JsonIgnore
    public Resource getResource() {
		return null;
	}

    /**
     * @param path absolute path to the resource
     * @return an externalized URL to the resource
     */
    protected String externalizeURL(final String path) {
		return null;
	}

    @Override
    @SuppressWarnings("unchecked")
    @JsonIgnore
    public Map<String, Object> getAsMap() {
		return null;
	}

}
