/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.client.api;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;

import com.adobe.cq.social.ugcbase.SocialUtils;
import com.adobe.granite.translation.core.MachineTranslationCloudConfig;
import com.day.cq.wcm.api.Page;
import com.day.cq.wcm.webservicesupport.Configuration;

/**
 * Provides helper methods that aid in performing some request specific functions like XSS, URL generations, etc.
 */
public interface ClientUtilities {

    /** JSON extension. */
    String JSON_EXTENSION = ".json";
    /** GET Selector. */
    String SOCO_SELECTOR = ".social";

    /** The query selector. */
    String QUERY_SELECTOR = ".query";
    
    /**
     * Computes the externalized URL to the resource at the given path.
     * @param path path to the resource
     * @return an externalized url to the resource
     */
    String externalLink(final String path) {
		return null;
	}

    /**
     * Computes the externalized URL to the resource at the given path.
     * @param path path to resource
     * @param needExtension boolean to indicate whether the .social.json extension needs to be appended to the URL
     * @return an externalized url to the resource with or without .social.json suffix.
     */
    String externalLink(final String path, final Boolean needExtension) {
		return null;
	}

    /**
     * Get the current extension of the request. Useful when building URLS based on the requested content type.
     * If the constructor that takes in a ResourceResolver was used then this information is not available and
     * null is returned.
     * @return the extension without a leading dot
     */
    String getRequestExtension() {
		return null;
	}

    /**
     * Calls {@Link XSSAPI} filterHTML on the provided String.  If no reference to XSSAPI is available
     * the String is returned as is.
     * @param valueToBeFiltered The String to be filtered.
     * @return The filtered String.
     */
    String filterHTML(final String valueToBeFiltered) {
		return null;
	}

    /**
     * Get the {@link SocialComponentFactoryManager} service.
     * @return the SocialComponentFactoryManager service.
     */
    SocialComponentFactoryManager getSocialComponentFactoryManager() {
		return null;
	}

    /**
     * Gets the {@link SocialUtils} service.
     * @return a SocialUtils implementation
     */
    SocialUtils getSocialUtils() {
		return null;
	}

    /**
     * Gets the authorized user ID.  If the constructor that takes a ResourceResolver was used then this
     * information is not available and null will be returned.
     * @return The authorized user ID, or null.
     */
    String getAuthorizedUserId() {
		return null;
	}

    /**
     * Helper method to check whether the user is anonymous (not logged in) or not.  If the constructor that
     * takes a ResourceResolver was used then this will always return true.
     * @return true if the authorized userID is null or anonymous
     */
    boolean userIsAnonymous() {
		return false;
	}

    /**
     * Get the original request
     * @return the {@link SlingHttpServletRequest} request
     */
    SlingHttpServletRequest getRequest() {
		return null;
	}

    /**
     * Helper method to get the configured social profile path for the page on which the resource is contained.
     * If the constructor that takes a ResourceResolver is used this information is not available and null will
     * be returned.
     * @return The configured social profile (public) path if one exists, or null.
     */
    String getSocialProfilePath() {
		return null;
	}

    /**
     * Returns the page that contains the resource at the given path if path is absolute. If path is relative, the
     * request resources path is pre-pended to the path in order to construct an absolute path.  If the constructor
     * that takes a ResourceResolver was used then this will always return null.
     * request path to construct an absolute path.
     * @param path the path to the resource for which the containing page is to be found
     * @return the page that contains the resource if one is found
     */
    Page getContainingPage(String path) {
		return null;
	}

    /**
     * Returns the {@Link Page} from the specified path.
     * @param pathToPage path to a page
     * @return {@link Page} object from the page path, or null.
     */
    Page getPage(String pathToPage) {
		return null;
	}

    /**
     * Get the {@Link User} for the specified user ID.
     * @param userId The user ID of the user.
     * @return A {@link User} representing the user id.
     */
    User getUser(String userId, ResourceResolver resolver) {
		return null;
	}
    
    /**
     * Gets an active storage cloud service configuration based on the requested resource.
     * @param resource The resource to get the active storage cloud service for.
     * @return a Cloud {@Link Configuration} for the specified resource, or null if none exists.
     */
    Configuration getStorageCloudConfig(Resource resource) {
		return null;
	}

    /**
     * Gets an Machine Translation config for the requested resource.
     * @param resource The resource to get the Machine Translation config for.
     * @return A {@Link MachineTranslationCloudConfig} for the requested resource, or null if none exists.
     */
    MachineTranslationCloudConfig getMTCloudConfig(Resource resource) {
		return null;
	}

}
