/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.commons.comments.api;

import java.util.Calendar;
import java.util.List;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.CommentSystem;
import com.adobe.cq.social.commons.client.api.BaseSocialComponent;
import com.adobe.cq.social.commons.client.api.ClientUtilities;
import com.adobe.cq.social.commons.client.api.CollectionPagination;
import com.adobe.cq.social.commons.client.api.CollectionSortedOrder;
import com.adobe.cq.social.commons.client.api.QueryRequestInfo;
import com.adobe.cq.social.commons.client.api.ResourceID;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentList;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentListProvider;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentListProviderManager;
import com.adobe.cq.social.ugcbase.SocialException;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;

/**
 * Base SocialComponentCollection class which maintains a list of {@link Comment}.
 * @param <C> any SocialComponent type that extends the {@link Comment} data type.
 */
public abstract class AbstractCommentCollection<C extends Comment, T extends CommentCollectionConfiguration> extends
    BaseSocialComponent implements CommentCollection<C, T> {
    private static final Logger LOG = LoggerFactory.getLogger(AbstractCommentCollection.class);
    private CommentSocialComponentListProviderManager commentListProviderManager;
    private Calendar lastModified;
    private Calendar created;
    private CommentSocialComponentList comments;
    private QueryRequestInfo queryInfo;
    private PageInfo pageInfo;
    private CommentSystem cs;

    private T configuration;

    /**
     * Constructor using the specified {@link ResourceID} which should be the root of the collection without paging
     * specification. Paging information is not created until the pagination is specified.
     * @param id the resource id of the <code>CommentCollection</code>
     * @param resolver the resource resolver used to obtain the resource in the JCR.
     * @param clientUtils the clientUtils
     * @see AbstractCommentCollection#setPagination(CollectionPagination)
     */
    public AbstractCommentCollection(final ResourceID id, final ResourceResolver resolver,
        final ClientUtilities clientUtils, final CommentSocialComponentListProviderManager commentListProviderManager) {
        this(resolver.resolve(id.getResourceIdentifier()), clientUtils, commentListProviderManager);
    }

    /**
     * Constructor using the specified {@link Resource} which should be the root of the collection without paging
     * specification. Paging information is not created until the pagination is specified.
     * @param resource the resource.
     * @param clientUtils the clientUtils
     * @see AbstractCommentCollection#setPagination(CollectionPagination)
     */
    public AbstractCommentCollection(final Resource resource, final ClientUtilities clientUtils,
        final CommentSocialComponentListProviderManager commentListProviderManager) {
        super(resource, clientUtils);
        /** Logger for this class. */
        init(resource, clientUtils, QueryRequestInfo.DEFAULT_QUERY_INFO_FACTORY.create(), false,
            commentListProviderManager);

    }

    /**
     * Construct a {@link CommentCollection} using the specified {@link Resource} which should be the root of the
     * collection.
     * @param resource the resource where the <code>CommentCollection</code> is located
     * @param clientUtils the clientUtilities instance
     */
    public AbstractCommentCollection(final Resource resource, final ClientUtilities clientUtils,
        final QueryRequestInfo queryInfo, final CommentSocialComponentListProviderManager commentListProviderManager) {
        super(resource, clientUtils);
        init(resource, clientUtils, queryInfo, true, commentListProviderManager);
    }

    /**
     * Constructor initialization helper method.
     * @param resource
     * @param clientUtils
     * @param queryInfo
     * @param buildPages
     */
    private void init(final Resource resource, final ClientUtilities clientUtils, final QueryRequestInfo queryInfo,
        final boolean buildPages, final CommentSocialComponentListProviderManager commentListProviderManager)
        throws SocialException {
		}

    /**
     * Set the collection list range. If this is not set, then the return list will start at offset 0 and the default
     * size.
     * @param pagination pagination configuration
     */
    @Override
    public void setPagination(final CollectionPagination pagination) {
	}

    /**
     * Get the pagination settings.
     * @return the pagination settings.
     */
    protected CollectionPagination getPagination() {
		return null;
	}

    /**
     * Get the query info
     */
    protected QueryRequestInfo getQueryRequestInfo() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Calendar getCreated() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Calendar getLastModified() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public int getTotalSize() {
		return 0;
	}

    /**
     * Get the comments of this {@link CommentCollection} base on the pagination configuration.
     * @return List<Object> a list of Comment
     */
    @Override
    public List<Object> getItems() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void setSortedOrder(final CollectionSortedOrder sortedOrder) {
	}

    @Override
    public PageInfo getPageInfo() {
		return null;
	}

    @Override
    @JsonIgnoreProperties
    public boolean isTaggingAllowed() {
		return false;
	}

    protected void setLastModified(final Calendar lastModified) {
	}

    protected void setCreated(final Calendar created) {
	}

    protected CommentSocialComponentListProviderManager getCommentListProviderManager() {
		return null;
	}

    protected void setCommentListProviderManager(
        final CommentSocialComponentListProviderManager commentListProviderManager) {
		}

    @Override
    public T getConfiguration() {
		return null;
	}

    @Override
    public T createConfiguration(final Resource resource, final CommentSystem cs) {
		return null;
	}
    
    /**
     * @return a reference to the {@link CommentSystem} object that is represented by this instance.
     */
    protected CommentSystem getCommentSystem() {
		return null;
	}
}