/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.comments.api;

import org.apache.sling.api.resource.Resource;

import com.adobe.cq.social.commons.CommentSystem;
import com.adobe.cq.social.commons.CommentUtil;
import com.adobe.cq.social.commons.client.api.CollectionPagination;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Base implementation of the {@link CommentCollectionConfiguration}.
 */
public class AbstractCommentCollectionConfiguration implements CommentCollectionConfiguration {

    private final boolean allowsVoting;
    private final boolean isRTEEnabled;
    private final boolean allowsAttachment;
    private boolean allowsCustomFlagReason = true;
    private int pageSize = 10;

    /**
     * Constructor
     * @param resource the target resource of the collection or comment
     * @param cs the {@link CommentSystem} instance of the CommentCollection
     */
    public AbstractCommentCollectionConfiguration(final Resource resource, final CommentSystem cs) {
        allowsCustomFlagReason = cs.allowCustomFlagReason();
        final Resource component = cs.getResource();
        allowsVoting = CommentUtil.isAllowVoting(component);

        // Need to use CommentUtil for consistency
        isRTEEnabled = cs.isRteEnabled();
        allowsAttachment = cs.allowsAttachment();
        pageSize = cs.getProperty("maxPerPage", CollectionPagination.DEFAULT_PAGE_SIZE);
    }

    @Override
    @JsonProperty("isVotingAllowed")
    public boolean isVotingAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isAttachmentAllowed")
    public boolean isAttachmentAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isRTEEnabled")
    public boolean isRTEEnabled() {
		return false;
	}

    @Override
    @JsonProperty("isCustomFlagReasonAllowed")
    public boolean isCustomFlagReasonAllowed() {
		return false;
	}

    @Override
    public int getPageSize() {
		return 0;
	}

}
