/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.digest;

import java.util.Calendar;

/**
 * Type of digest.
 */
public enum DigestEventType {
    /**
     * The event fired when a daily digest is created. Occurs after every 1 day.
     */
    Daily(Calendar.DATE, -1),
    /**
     * The event fired when a weekly digest is created. Occurs after every 7 days.
     */
    Weekly(Calendar.WEEK_OF_YEAR, -1),
    /**
     * The event fired when a fortnightly digest is created. Occurs after every 14 days.
     */
    Fortnightly(Calendar.WEEK_OF_YEAR, -2),

    /**
     * The event fired when a monthly digest is created. Occurs after 1 month.
     */
    Monthly(Calendar.MONTH, -1);

    /**
     * Calendar field type.
     */
    private int fieldType;

    /**
     * Number of units of time to be added or subtracted.
     */
    private int interval;

    /**
     * constructor.
     * @param fieldType type of calendar field.
     * @param interval number of units of time to be added or subtracted.
     */
    DigestEventType(final int fieldType, final int interval) {
        this.fieldType = fieldType;
        this.interval = interval;
    }

    /**
     * @return type of calendar field
     */
    public int getFieldType() {
        return fieldType;
    }

    /**
     * @return number of units of time to be added or subtracted
     */
    public int getInterval() {
        return interval;
    }
}
