/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.commons.emailreply;

import java.io.InputStream;
import java.util.Properties;

import javax.jcr.Session;

import org.apache.commons.mail.Email;
import org.apache.sling.api.resource.Resource;

/**
 * Provides ability to reply to notification emails and to parse the reply emails.
 */
public interface EmailReplyService {

    /**
     * Property for message contents parsed from email body.
     */
    String EMAIL_MESSAGE_PROPERTY = "email.message";

    /**
     * Property for author of email, i.e. "From" address.
     */
    String EMAIL_AUTHOR_PROPERTY = "email.author";

    /**
     * Property for subject of email.
     */
    String EMAIL_SUBJECT_PROPERTY = "email.subject";

    /**
     * Property for email client that parsed the reply email.
     */
    String EMAIL_CLIENT_PROVIDER_PROPERTY = "email.parsingEmailClient";

    /**
     * Property for path of topic resource to which comment parsed from email is to be added.
     */
    String TOPIC_RESOURCE_PATH_PROPERTY = "topic.resourcePath";

    /**
     * Property unique Id of topic resource to which comment parsed from email is to be added.
     */
    String TOPIC_UNIQUE_ID_PROPERTY = "topic.uniqueId";

    /**
     * Adds unique tracker id information in subject/replyto before sending notification email creation. The tracker
     * id or topic unique id is hash generated from topicPath itself.
     * @param adminSession session of admin user
     * @param email original notification email
     * @param topicPath the path of the node where tracker id property is to be added.
     * @return email with unique tracker id information
     */
    Email addTrackerId(Session adminSession, Email email, String topicPath) {
		return null;
	}

    /**
     * @deprecated Parses reply email data and returns information needed to create a comment. Mandatory parameters to
     *             be parsed from mail are email message, topic resource path and author id. Currently author email id
     *             is used as author id for author profile identification. Topic path returned as a property assumes
     *             topic is of node type <code>Comment.NODE_TYPE</code> and all topics are stored under
     *             <code>CommentSystem.PATH_UGC</code>
     * @param adminSession session of admin user
     * @param mailStream input stream representing mail
     * @param postOriginalOnFailure true if original email contents with quoted text to be returned on failure to
     *            parse
     * @param topicResourceType type of topic resource to which comment parsed from email needs to be added
     * @return properties of mail containing information about email content, topic resource path, author id and
     *         parsing status message
     */
    @Deprecated
    Properties getMailProperties(final Session adminSession, final InputStream mailStream,
        final boolean postOriginalOnFailure, final String topicResourceType) {
		return null;
	}

    /**
     * @deprecated Parses reply email data and returns information needed to create a comment. Mandatory parameters to
     *             be parsed from mail are email message, topic unique id and author id. Currently author email id is
     *             used as author id for author profile identification.
     * @param adminSession session of admin user
     * @param mailStream input stream representing mail
     * @param postOriginalOnFailure true if original email contents with quoted text to be returned on failure to
     *            parse
     * @param topicResourceType type of topic resource to which comment parsed from email needs to be added
     * @param topicNodeType type of topic node
     * @param topicRootPath path under which all topics nodes are stored
     * @return properties of mail containing information about email content, topic resource path, author id and
     *         parsing status message
     */
    @Deprecated
    Properties getMailProperties(final Session adminSession, final InputStream mailStream,
        final boolean postOriginalOnFailure, final String topicResourceType, final String topicNodeType,
        final String topicRootPath) {
		return null;
	}

    /**
     * @deprecated Returns the path of topic resource with given topicUniqueId property.
     * @param topicUniqueId tracker id for a topic
     * @param adminSession session of an admin user
     * @param topicResourceType type of topic resource to which comment parsed from email needs to be added
     * @param topicNodeType type of topic node
     * @param topicRootPath path under which all topics nodes are stored
     * @return the path of topic resource to which comment parsed from email is to be added
     */
    @Deprecated
    String getTopicPath(final String topicUniqueId, final Session adminSession, final String topicResourceType,
        final String topicNodeType, final String topicRootPath) {
		return null;
	}

    /**
     * Parses reply email data and returns information needed to create a comment including list of attachments.
     * Mandatory parameters to be parsed from mail are email message, topic unique id and author id. Currently author
     * email id is used as author id for author profile identification.
     * @param resource the e-mail resource that has been created.
     * @return mailContent object that contains properties of mail containing information about email content, topic
     *         resource path, author id and parsing status message and the list of attachments in the e-mail.
     */
    EmailContent getMailContent(final Resource resource) {
		return null;
	}

    /**
     * Sends failure notification mail to the sender of the email if the email fails to be processed.
     * @param emailContent the parsed content of the mail which failed to perform the desired task.
     */
    void sendFailureEmailReply(final EmailContent emailContent) {
	}
}
