/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.notification;

import org.apache.sling.api.resource.ResourceResolver;

import com.day.cq.security.Authorizable;

/**
 * Defines methods for subscribing or un-subscribing a user.
 */
public interface Subscribable {
    /**
     * Subscribes the given <code>authorizable</code> to receive notifications.
     * @param authorizable The {@link com.day.cq.security.Authorizable} to subscribe.
     * @param workflowModel the path to the subscribe workflow model.
     * @param resourceResolver the {@link ResourceResolver} to be used to start the subscription workflow.
     */
    void subscribe(Authorizable authorizable, String workflowModel, ResourceResolver resourceResolver) {
	}

    /**
     * Unsubscribes the given <code>authorizable</code>
     * @param authorizable The {@link com.day.cq.security.Authorizable} to unsubscribe.
     * @param workflowModel the path to the unsubscribe workflow model.
     * @param resourceResolver the {@link ResourceResolver} to be used to start the unsubscription workflow.
     */
    void unsubscribe(Authorizable authorizable, String workflowModel, ResourceResolver resourceResolver) {
	}

    /**
     * Indicates whether the given <code>authorizable</code> (user) is currently subscribed.
     * @param authorizable The user to check.
     * @return <code>true</code> if the given user is subscribed to it, <code>false</code> otherwise.
     */
    boolean isSubscribed(Authorizable authorizable) {
		return false;
	}

    /**
     * @return path referencing the subscribed entry.
     */
    String getSubscriptionPath() {
		return null;
	}
}
