/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.security;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.request.RequestDispatcherOptions;
import org.apache.sling.api.request.RequestPathInfo;
import org.apache.sling.api.servlets.OptingServlet;
import org.apache.sling.api.servlets.SlingAllMethodsServlet;

import com.adobe.cq.social.commons.SaferSlingPostValidator;

/**
 * POST servlet which wraps the sling default post servlet to disable some of the options which excessively expand the
 * security attack surface area. This servlet is fired if the url contains the selector .social. *or* the query
 * parameter :socialPostFilter. In the case of the query parameter the value indicates the depth to which :applyTo may
 * refer to a descendant of the resource selected by the URL. A value of 0 indicates only the resource identified by
 * the URL may be referenced, a value of 1 allows reference to the direct descendants of the resource identified. To
 * achieve security POST requests which would otherwise go to the default post servlet directly *must* be blocked
 * unless they use either the .social. selector. Dispatcher 4.1.5 or above is
 * required. First all POSTs should be blocked, e.g.: /0000 { /type "deny" /method "POST" }/
 *  To allow POSTs to SCF endpoints which use the .json extension: (single line comment used so as not to terminate
 *   this comment block)
 */
// /0001 { /type "allow" /method "POST" /url "/content/*/*.social.json" }
/*
 * Note that * must not be used *after* .social. because * will match / and so it wouldn't necessarily be seen as a
 * selector.
 */
@Component(immediate = true, specVersion = "1.1", metatype = false)
@Service(value = {Servlet.class, Filter.class})
@org.apache.felix.scr.annotations.Properties({
    @Property(name = "service.description", value = "AEM Social Communities Safer Sling Post Servlet",
            propertyPrivate = true),
    @Property(name = "service.vendor", value = "Adobe Systems Incorporated", propertyPrivate = true),
    /*
     * Service ranking of MAX_VALUE - 1000 puts this filter after everything but the request logger, but leaves room
     * if other filters need to go after this one. Being late is good because during the include only INCLUDE scope
     * filters will be invoked, so the first time through the the only chance that all other filter have to apply to
     * the request.
     */
    @Property(name = "service.ranking", intValue = Integer.MAX_VALUE - 1000, propertyPrivate = true),
    @Property(name = "sling.filter.scope", value = "REQUEST", propertyPrivate = true),
    @Property(name = "sling.servlet.resourceTypes", value = SaferSlingPostServlet.POST_SERVLET_RESOURCE_TYPE,
            propertyPrivate = true),
    @Property(name = "sling.servlet.selectors", value = "social", propertyPrivate = true),
    @Property(name = "sling.servlet.methods", value = "POST", propertyPrivate = true)})
public class SaferSlingPostServlet extends SlingAllMethodsServlet implements Filter, OptingServlet {

    /**
     * Resource type for the default sling post servlet.
     */
    static final String POST_SERVLET_RESOURCE_TYPE = "sling/servlet/default";

    private static final long serialVersionUID = 1L;

    private static final String DROP_SELECTORS_ATTRIBUTE = SaferSlingPostServlet.class.getName() + ".dropSelectors";
    private static final String ACCEPTED_ATTRIBUTE = SaferSlingPostServlet.class.getName() + ".accepted";

    @Reference
    private SaferSlingPostValidator validator;

    @Override
    protected void doPost(final SlingHttpServletRequest request, final SlingHttpServletResponse response)
        throws ServletException, IOException {
		}

    @Override
    public void doFilter(final ServletRequest request, final ServletResponse response, final FilterChain chain)
        throws IOException, ServletException {
		}

    @Override
    public void init(final FilterConfig filterConfig) throws ServletException {
	}

    private String checkSelector(final ServletRequest request, final String selector) {
		return null;
	}

    private String checkSelectors(final SlingHttpServletRequest request, final RequestPathInfo rpi) {
		return null;
	}

    private String checkSuffix(final SlingHttpServletRequest request, final RequestPathInfo rpi) {
		return null;
	}

    private void include(final SlingHttpServletRequest request, final ServletResponse response,
        final RequestPathInfo rpi, final String selector) throws ServletException, IOException {
		}

    @Override
    public boolean accepts(final SlingHttpServletRequest request) {
		return false;
	}
}