/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.forum.api;

import java.util.Iterator;
import java.util.List;

import javax.activation.DataSource;
import javax.jcr.RepositoryException;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.ResourceResolver;

import com.adobe.cq.social.commons.CommentSystem;
import com.adobe.cq.social.commons.notification.Subscribable;

/**
 * The <code>Forum</code> interface specifies the contract for a forum.
 * @since CQ 5.6.0
 * @deprecated @since AEM 6.0. Use {@link com.adobe.cq.social.forum.client.api.Forum} instead.
 */
@Deprecated
public interface Forum extends Subscribable {

    /**
     * Property for forum attribute name.
     */
    static final String ATTRIBUTE_NAME = Forum.class.getName();

    /**
     * Property for forum resource type.
     */
    static final String RESOURCE_TYPE = "social/forum/components/forum";

    /**
     * Property for default max per page.
     */
    static final int DEFAULT_MAXPERPAGE = 25;

    /**
     * Property for closed.
     */
    static final String PN_CLOSED = "closed";

    /**
     * Property for empty text.
     */
    static final String PN_EMPTYTEXT = "emptyText";

    /**
     * Property for feed enabled.
     */
    static final String PN_FEEDENABLED = "feedEnabled";

    /**
     * Property for rte enabled.
     */
    static final String PN_RTEENABLED = "rteEnabled";

    /**
     * Property for forum closed text.
     */
    static final String PN_FORUMCLOSEDTEXT = "forumClosedText";

    /**
     * Property for max per page.
     */
    static final String PN_MAXPERPAGE = "maxPerPage";

    /**
     * Property for moderate comments.
     */
    static final String PN_MODERATED = "moderateComments";

    /**
     * Property for topics sorting on date ascending.
     */
    static final String PN_TOPICS_SORT_DATE_ASCENDING = "topicsSortDateAscending";

    /**
     * Property for posts sorting on date descending.
     */
    static final String PN_POSTS_SORT_DATE_DESCENDING = "postsSortDateDescending";

    /**
     * Property for nested comment system.
     */
    static final String PN_NESTEDCOMMENTSYSTEM = "nestedCommentSystem";

    /**
     * Property for forum no permission text.
     */
    static final String PN_NOPERMISSIONTEXT = "forumNoPermissionText";

    /**
     * Property for no topics text.
     */
    static final String PN_NOTOPCISTEXT = "noTopicsText";

    /**
     * Property for topics not found text.
     */
    static final String PN_TOPICNOTFOUNDTEXT = "topicNotFoundText";

    /**
     * Property for file upload.
     */
    static final String PN_FILEUPLOADS = CommentSystem.PROP_FILE_UPLOAD;

    /**
     * Property for file upload size.
     */
    static final String PN_FILEUPLOAD_SIZE = CommentSystem.PROP_FILE_UPLOAD_SIZE;

    /**
     * Property for image upload size.
     */
    static final String PN_IMAGEUPLOAD_SIZE = CommentSystem.PROP_IMAGE_UPLOAD_SIZE;

    /**
     * The property name for the character limit of message.
     */
    static final String PN_MESSAGE_LENGTH = CommentSystem.PROP_MESSAGE_LENGTH;

    /**
     * The property name for the character limit of subject.
     */
    static final String PN_SUBJECT_LENGTH = "maxSubjectLength";

    /**
     * The default character limit of subject.
     */
    static final long PN_DEFAULT_MAX_SUBJECT_LENGTH = 104857600;

    /**
     * Property for file upload type.
     */
    static final String PN_FILEUPLOAD_TYPES = CommentSystem.PROP_FILE_UPLOAD_TYPES;

    /**
     * The JCR property name that signifies if a Forum allows Replies to also have replies.
     */
    String PN_ALLOW_THREADEDREPLY = "allowReplyThread";

    /**
     * Property for Forum.topic attribute name.
     */
    static final String ATTRIBUTE_NAME_TOPIC = Forum.class.getName() + ".topic";

    /**
     * Property for topic resource type.
     */
    static final String RESOURCE_TYPE_TOPIC = "social/forum/components/topic";

    /**
     * Property for post resource type.
     */
    static final String RESOURCE_TYPE_POST = "social/forum/components/post";

    /**
     * Property for post attribute name.
     */
    static final String ATTRIBUTE_NAME_POST = Post.class.getName();

    /**
     * Property for Post.filtered attribute name.
     */
    static final String ATTRIBUTE_NAME_FILTERED_POSTS = Post.class.getName() + ".filtered";

    /**
     * Property for Post.request attribute name.
     */
    static final String ATTRIBUTE_NAME_REQUEST_POST = Post.class.getName() + ".request";

    /**
     * Property for not replicated attribute name.
     */
    static final String ATTRIBUTE_NAME_NOT_REPLICATED = "NOT_REPLICATED";

    /**
     * Property for topic's postfix.
     */
    static final String TOPIC_POSTFIX = "_ciot";

    /**
     * Adds a new topic (top level post) to this forum.
     * @param resourceResolver The {@link org.apache.sling.api.resource.ResourceResolver} to use for persisting
     *            values.
     * @param post The {@link com.day.cq.collab.forum.api.Post} to add.
     * @return The newly created topic (top level post).
     * @throws ForumException If an error occurs while adding the topic.
     */
    Post addTopic(ResourceResolver resourceResolver, Post post) throws ForumException {
		return null;
	}

    /**
     * Adds a new topic (top level post) to this forum.
     * @param resourceResolver The {@link org.apache.sling.api.resource.ResourceResolver} to use for persisting
     *            values.
     * @param fromRequest The {@link com.day.cq.collab.forum.api.Post} to add.
     * @param attachmentDataSources The datasource of the attachment to add.
     * @return The newly created topic (top level post).
     * @throws ForumException If an error occurs while adding the topic.
     */
    Post addTopic(ResourceResolver resourceResolver, Post fromRequest, List<DataSource> attachmentDataSources)
        throws ForumException {
		return null;
	}

    /**
     * Get the text to be displayed when nothing is yet entered in the input fields of the create post form.
     * @return The text or null if not defined.
     */
    String getEmptyText() {
		return null;
	}

    /**
     * Return the maximum number of posts/topics to display per page.
     * @return The limit.
     */
    int getLimit() {
		return 0;
	}

    /**
     * Return the number of topics.
     * @return the topic count.
     * @throws RepositoryException If an error occurs getting the topic count.
     */
    int getTopicCount() throws RepositoryException {
		return 0;
	}

    /**
     * Returns the text configured for this forum for when a user has no permission to post or is not logged in.
     * @return The no permission text.
     */
    String getNoPermissionText() {
		return null;
	}

    /**
     * Returns the text to be displayed if this forum has no topics.
     * @return The text or null if not defined.
     */
    String getNoTopicsText() {
		return null;
	}

    /**
     * Return the topics (top level posts) of this forum, ordered by pinned status and then in reverse chronological
     * order (modification date, descending).
     * @param resourceResolver The {@link org.apache.sling.api.resource.ResourceResolver} to use for retrieving
     *            values.
     * @return The list of topics.
     */
    Iterator<Post> getTopics() {
		return null;
	}

    /**
     * Return a subset of the topics (top level posts) of this forum as defined by the given offset and limit. The
     * posts are ordered by pinned status and then in reverse chronological order.
     * @param offset The offset.
     * @param limit The limit.
     * @return The list of topics.
     */
    List<Post> getTopics(final int offset, final int limit) {
		return null;
	}

    /**
     * Return a subset of the topics (top level posts) of this forum as defined by the given offset and limit. The
     * posts are ordered by pinned status and then in reverse chronological order.
     * @param offset The offset.
     * @param limit The limit.
     * @param sortDateAscending true if topics are sorted by modified date in ascending order and false if descending.
     * @return The list of topics.
     */
    List<Post> getTopics(final int offset, final int limit, boolean sortDateAscending) {
		return null;
	}

    /**
     * Returns the requested post as identified by the request suffix.
     * @param request The request to check.
     * @return The {@link Post} as identified by the request suffix, or <code>null</code> if no request suffix is
     *         present.
     * @throws PostNotFoundException If a request suffix is present but doesn't map to an existing post.
     */
    Post getPostFromRequest(final SlingHttpServletRequest request) throws PostNotFoundException {
		return null;
	}

    /**
     * Returns the text to be displayed if a topic was requested that doesn't exist.
     * @return The text.
     */
    String getTopicNotFoundText() {
		return null;
	}

    /**
     * Retrieves a post (from any hierarchy level) within this forum as identified by the given <code>id</code>.
     * @param id The id of the post.
     * @return the {@link Post}.
     * @throws PostNotFoundException If no post is found matching the given <code>id</code>.
     */
    Post getPost(String id) throws PostNotFoundException {
		return null;
	}

    /**
     * Indicates whether this forum requires moderation for new topcis/posts.
     * @return <code>true</code> if this forum is moderated.
     */
    boolean isModerated() {
		return false;
	}

    /**
     * Indicates whether this forum has topics sorted by date in descending order.
     * @return <code>true</code> if topics are sorted by date in descending order.
     */
    boolean isTopicsSortDateAscending() {
		return false;
	}

    /**
     * Indicates whether this forum has posts sorted by date in ascending order.
     * @return <code>true</code> if posts are sorted by date in ascending order.
     */
    boolean isPostsSortDateDescending() {
		return false;
	}

    /**
     * Indicates whether this forum is closed for further addition of topics/posts.
     * @return <code>true</code> if this forum is closed.
     */
    boolean isClosed() {
		return false;
	}

    /**
     * Returns the text to be displayed if this forum has been marked closed for further addition of posts/topics.
     * @return A <code>String</code> representing the forum closed message or <code>null</code> if not defined.
     */
    String getForumClosedText() {
		return null;
	}

    /**
     * Returns the title of this forum.
     * @return The title.
     */
    String getTitle() {
		return null;
	}

    /**
     * Returns the path of this forum.
     * @return The path or <code>null</code> if undefined.
     */
    String getPath() {
		return null;
	}

    /**
     * Returns the URL of this forum (i.e. its page path with .html extension).
     * @return The URL or <code>null</code> if the forum's page cannot be found.
     */
    String getUrl() {
		return null;
	}

    /**
     * Indicates whether RSS feeds for this forum are considered enabled or not. Default: true.
     * @return <code>true</code> if forum feeds are enabled.
     */
    boolean isFeedEnabled() {
		return false;
	}

    /**
     * Indicates whether the Rich Text Editor for the Forum is enabled or not. Default: false.
     * @return <code>false</code> if Rich Text editor is not enable
     */
    boolean isRTEEnabled() {
		return false;
	}

    /**
     * Indicates whether the File Attachments for the Forum is enabled or not. Default: false.
     * @return <code>false</code> if file upload capability is not enabled
     */
    boolean allowFileUploads() {
		return false;
	}

    /**
     * Returns the Maximum number of bytes a file may have if the comment system has a property to allow attachments. @See
     * Forum#PN_FILEUPLOAD_SIZE
     * @return file attachment size if the comment system allows attachments
     */
    long getAttachmentSizeLimit() {
		return 0L;
	}

    long getImageAttachmentSizeLimit() {
		return 0L;
	}

    /**
     * Returns the Maximum number of characters a forum message text may have. @See FORUM#PN_MESSAGE_LENGTH
     * @return the maximum number of characters in forum message text
     */
    long getMessageCharacterLimit() {
		return 0L;
	}

    /**
     * Returns the Maximum number of characters a forum subject text may have. @See FORUM#PN_SUBJECT_LENGTH
     * @return the maximum number of characters in forum subject text
     */
    long getSubjectCharacterLimit() {
		return 0L;
	}

    /**
     * Returns the string that is a list of Comma separated file extensions with dots. @See Forum#PN_FILEUPLOAD_TYPES
     * @return List of allowed file extensions (white list) with dots, example [".jpg", ".jpeg", ".png", ".doc",
     *         ".docx", ".ppt", ".zip", ".rar"]
     */
    List<String> getAllowedFileTypes() {
		return null;
	}

    /**
     * Return the number of pages in this forum.
     * @return the number of pages.
     * @throws ForumException thrown when an error occurs within Forum
     */
    int getNumberOfPages() throws ForumException {
		return 0;
	}

    /**
     * Return the list of topics for the given page of a given size.
     * @param pageNumber the number of page to return.
     * @return the list of topics for that page.
     */
    List<Post> getTopics(final int pageNumber) {
		return null;
	}

    /**
     * Get the page URL for the specified page index.
     * @param pageIndex The page index
     * @return page URL
     */
    String getPageUrl(final int pageIndex) {
		return null;
	}

    /**
     * Get the request page number for the specified request.
     * @param request The sling request
     * @return request page number
     */
    int getRequestPageNumber(final SlingHttpServletRequest request) {
		return 0;
	}

    /**
     * Checks if the forum allows replies to replies.
     * @return true if the forum allows it otherwise false
     */
    Boolean allowThreadedReply() {
		return null;
	}

    /**
     * enum for mode. It can be TOPIC_LIST, TOPIC, POST, SUBSCRIBE, SETTINGS, PROFILE.
     */
    static enum MODE {
        TOPIC_LIST, TOPIC, POST, SUBSCRIBE, SETTINGS, PROFILE
    }

}
