/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.forum.api;

import java.util.Dictionary;

import org.osgi.service.event.Event;

import com.adobe.cq.social.commons.digest.DigestEvent;

/**
 * Event fired when forum digest is created.
 */
public final class ForumDigestEvent extends DigestEvent {

    public enum EventType {
        TOPIC(EVENT_TOPIC), FORUM(FORUM_EVENT_TOPIC);
        private final String topic;

        private EventType(final String eventTopic) {
            topic = eventTopic;
        }

        public String getTopic() {
            return topic;
        }
    }

    /**
     * The topic name for topic digest events.
     */
    public static final String EVENT_TOPIC = "com/adobe/cq/social/forum/digest";

    /**
     * The topic name for forum digest events.
     */
    public static final String FORUM_EVENT_TOPIC = "com/adobe/cq/social/forumlevel/digest";

    /** A serialVersionUID for this class. */
    private static final long serialVersionUID = 1L;

    /**
     * @deprecated Construct ForumDigestEvent using a supplied properties dictionary.
     * @param properties a dictionary of properties for this event.
     */
    @Deprecated
    public ForumDigestEvent(final Dictionary<String, Object> properties) {
        super(EVENT_TOPIC, properties);
    }

    /**
     * Construct ForumDigestEvent using a supplied properties dictionary.
     * @param eventType whether the event is forum-level or topic-level
     * @param properties a dictionary of properties for this event.
     */
    public ForumDigestEvent(final Dictionary<String, Object> properties, final EventType eventType) {
        super(eventType.getTopic(), properties);
    }

    private ForumDigestEvent(final Event event) {
        super(event);
    }

    /**
     * Retrieves the forum digest event from the given generic {@link org.osgi.service.event.Event}.
     * @param event The event.
     * @return The forum digest event or <code>null</code> if the given event is not a forum digest event.
     */
    public static ForumDigestEvent fromEvent(final Event event) {
		return null;
	}
}
