/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.forum.api;

import java.io.Serializable;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import org.osgi.service.event.Event;

import com.adobe.cq.social.commons.events.SocialEvent;
import com.adobe.cq.social.forum.api.ForumEvent.Type;

/**
 * Event fired for changes occurring within a forum.
 * @since CQ 5.6.0
 */
public final class ForumEvent extends SocialEvent<Type> implements Serializable {

    /** A serialVersionUID for this class. */
    private static final long serialVersionUID = 2L;

    /**
     * Property for event type.
     */
    private static final String PROP_EVENT_TYPE = "EventType";

    /**
     * Property for event path.
     */
    private static final String PROP_EVENT_PATH = "EventPath";

    /**
     * Property for forum path.
     */
    private static final String PROP_FORUM_PATH = "ForumPath";

    private static final String FORUM_TOPIC = "forum";

    /**
     * The topic name for event topic.
     */
    public static final String EVENT_TOPIC = SocialEvent.SOCIAL_EVENT_TOPIC_PREFIX + FORUM_TOPIC;

    /**
     * We must handle collab events to support notifications in collab forums. The old code that collabs depended on
     * for notifications in 5.5 no longer exists. ForumEventBuilder has been moved to social-forum so we need to
     * handle collab notifications in social.
     * @deprecated - this has been deprecated, we no longer need to be triggering collab events from social bundles.
     */
    @Deprecated
    public static final String COLLAB_EVENT_TOPIC = "com/day/cq/collab/forum";

    protected static final String PROP_TOPIC_PATH = "TopicPath";

    /**
     * The event types supported for the forum and its topics/posts.
     */
    public static enum Type implements com.adobe.cq.social.commons.events.SocialEvent.SocialActions {

        /**
         * The event fired upon a topic being added.
         */
        TopicAdded,

        /**
         * The event fired upon a post being added.
         */
        PostAdded,

        /**
         * The event fired upon approval of a moderated post.
         */
        PostApproved,

        TopicEdited,

        PostEdited,

        TopicDeleted,

        PostDeleted;

    }

    /**
     * Constructor for event fired for changes occurring within a forum.
     * @param properties The properties of the event
     * @deprecated use {@link #ForumEvent(Type, Post)} instead.
     */
    @Deprecated
    public ForumEvent(final Dictionary<String, Object> properties) {
        super(FORUM_TOPIC, (String) properties.get(PROP_EVENT_PATH), (String) properties
            .get(com.adobe.cq.social.ugcbase.events.SocialEvent.USER_ID), (Type) properties.get(PROP_EVENT_TYPE),
            getPropertiesMap(properties));
    }

    private static Map<String, Object> getPropertiesMap(final Dictionary<String, Object> properties) {
		return null;
	}

    /**
     * Constructor for event fired for changes occurring within a forum.
     * @param type The type of the properties of the event
     * @param post The post.
     */
    public ForumEvent(final Type type, final Post post) {
        super(FORUM_TOPIC, post.getPath(), post.getCreatedBy().getUserID(), type, post.isTopic()
            ? buildTopicProperties(post) : buildPostProperties(post));
    }

    private static Map<String, Object> buildPostProperties(final Post post) {
		return null;
	}

    private static Map<String, Object> buildTopicProperties(final Post post) {
		return null;
	}

    private ForumEvent(final Event event) {
        super(event);
    }

    public ForumEvent(final String postPath, final String forumPath, final String topicPath, final String id,
        final Type type) {
        super(FORUM_TOPIC, postPath, id, type, buildPathMap(forumPath, topicPath));
    }

    private static Map<String, Object> buildPathMap(final String forumPath, final String topicPath) {
		return null;
	}

    /**
     * Returns the path of the forum associated with the topic/post affected by the event.
     * @return A <code>String</code> representing the path of the forum.
     */
    public String getForumPath() {
		return null;
	}

    public String getTopicPath() {
		return null;
	}

    /**
     * Returns the type of the event.
     * @return The {@link Type} of the event or <code>null</code> if the event carries an unknown type.
     * @deprecated use getAction()
     */
    @Deprecated
    public Type getType() {
		return null;
	}

    /**
     * Creates a distributable event from this forum event.
     * @return this {@link Event}.
     * @deprecated Do not use this method. It is provided for API backward compatibility, but it does not return a
     *             distributable event form of this Event.
     */
    @Deprecated
    public Event toEvent() {
		return null;
	}

    /**
     * Retrieves the forum event from the given generic {@link Event}.
     * @param event The event.
     * @return The forum event or <code>null</code> if the given event is not a forum event.
     */
    public static ForumEvent fromEvent(final Event event) {
		return null;
	}

    @Override
    public String toString() {
		return null;
	}
}
