/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.forum.api;

import java.util.List;

import com.adobe.cq.social.commons.comments.search.SearchComponent;
import com.adobe.cq.social.commons.comments.search.SearchResultComponent;
import com.day.cq.search.Predicate;
import com.day.cq.search.result.ResultPage;
import com.day.cq.search.result.SearchResult;

/**
 * Provides forum search capability. The search searches all forums within <i>/content</i>, unless the user restricted
 * the search to one or more forum paths.
 * <p/>
 * Instantiating a search will make the search instance available as a request attribute of name
 * {@link #ATTRIBUTE_NAME_SEARCH}. Currently the search can only search one forum at a time, as specified by the
 * respective content property.
 * @since CQ 5.6.0
 * @deprecated @since AEM 6.0. Use {@link SearchResultComponent} instead.
 */
@Deprecated
public interface ForumSearch {

    /**
     * Property for forum search result pages.
     */
    static final String ATTRIBUTE_NAME_RESULTPAGES = ForumSearch.class.getName() + ".resultpages";

    /**
     * Property for forum search.
     */
    static final String ATTRIBUTE_NAME_SEARCH = ForumSearch.class.getName() + ".search";

    /**
     * Property for search paths.
     */
    static final String PN_SEARCHPATHS = "searchPaths";

    /**
     * Property for text no results.
     */
    static final String PN_TEXTNORESULTS = "textNoResults";

    /**
     * Property for text one match.
     */
    static final String PN_TEXTONEMATCH = "textOneMatch";

    /**
     * Property for text results.
     */
    static final String PN_TEXTRESULTS = "textResults";

    /**
     * Property for text search button.
     */
    static final String PN_TEXTSEARCHBUTTON = "textSearchButton";

    /**
     * Property for request parameter query.
     */
    static final String REQUEST_PARAM_QUERY = "q";

    /**
     * Property for request parameter query scope.
     */
    static final String REQUEST_PARAM_QUERY_SCOPE = "s";

    /**
     * Property for request parameter start.
     */
    static final String REQUEST_PARAM_START = "start";

    /**
     * Returns the search query as entered by the user and retrieved from the request.
     * @return A <code>String</code> representing the search query or <code>null</code> if none was specified.
     */
    String getQuery() {
		return null;
	}

    /**
     * Returns the scope properties as selected by the user in the search form, i.e. which properties are to be
     * searched.
     * @return A <code>String[]</code> representing the selected scope properties or an empty array if none was
     *         defined.
     */
    String[] getScopeProperties() {
		return null;
	}

    /**
     * Executes the query and returns the results. Consecutive calls to this method will always return the same result
     * (cached).
     * @return A {@link SearchResult}.
     * @throws ForumException If an error occurred while searching.
     */
    SearchResult getResult() throws ForumException {
		return null;
	}

    /**
     * Executes the search (see {@link #getResult()} and returns the results split into the result pages. The method
     * also sets a request attribute of name {@link #ATTRIBUTE_NAME_RESULTPAGES} containing the list of pages.
     * Consecutive calls to this method will always return the same page list (cached).
     * @return An unmodifiable list of {@link com.day.cq.collab.forum.api.ForumSearch.Page}s.
     * @throws ForumException If an error occurred while searching.
     */
    List<Page> getResultPages() throws ForumException {
		return null;
	}

    /**
     * Allows the addition of custom query builder {@link Predicate}s to refine the search.
     * @param predicate The predicate to be added.
     */
    void addPredicate(Predicate predicate) {
	}

    /**
     * Returns the text configured for this search for when no search results were found.
     * @return A <code>String</code> representing the text.
     */
    String getTextNoResults() {
		return null;
	}

    /**
     * Returns the text configured for this search for when exactly one match was found.
     * @return A <code>String</code> representing the text.
     */
    String getTextOneMatch() {
		return null;
	}

    /**
     * Returns the text configured for this search for information about the results found.
     * @return A <code>String</code> representing the text.
     */
    String getTextResults() {
		return null;
	}

    /**
     * Returns the text configured for this search for the label of the search button.
     * @return A <code>String</code> representing the text.
     */
    String getTextSearchButton() {
		return null;
	}

    /**
     * <code>Page</code> extends from <code>ResultPage</code> which represents one page of the search result.
     */
    interface Page extends ResultPage {

        /**
         * Return URL of the page.
         * @return URL of the page.
         */
        String getUrl() {
			return null;
		}
    }
}
