/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.forum.api;

import java.io.InputStream;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.activation.DataSource;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ValueMap;

import com.adobe.cq.social.commons.Comment;
import com.adobe.cq.social.commons.notification.Subscribable;
import com.adobe.granite.security.user.UserProperties;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.security.Authorizable;
import com.day.cq.security.profile.Profile;
import com.day.cq.tagging.Tag;
import com.day.cq.wcm.api.NameConstants;

/**
 * The <code>Post</code> interface specifies the contract for a forum post.
 * @deprecated @since AEM 6.0. Use {@link com.adobe.cq.social.forum.client.api.Post} instead.
 */
@Deprecated
public interface Post extends Subscribable {

    /**
     * The name of the reverse replication property.
     * @deprecated use {@link com.day.cq.replication.Replicator} directly to trigger reverse replication.
     */
    @Deprecated
    public static final String PN_DISTRIBUTE = "cq:distribute";

    /**
     * Property for added.
     */
    static final String PN_DATE = Comment.PROP_DATE;

    /**
     * Property for jcr:description.
     */
    static final String PN_MESSAGE = Comment.PROP_MESSAGE;

    /**
     * Property for notify.
     */
    static final String PN_NOTIFY = "notify";

    /**
     * Property for pinned.
     */
    static final String PN_PINNED = "pinned";

    /**
     * Property for jcr:title.
     */
    static final String PN_SUBJECT = JcrConstants.JCR_TITLE;

    // metadata properties
    /**
     * Property for ip.
     */
    static final String PN_IP_ADDRESS = Comment.PROP_IP_ADDRESS;

    /**
     * Property for referer.
     */
    static final String PN_REFERER = Comment.PROP_REFERER;

    /**
     * Property for userAgent.
     */
    static final String PN_USER_AGENT = Comment.PROP_USER_AGENT;

    /**
     * Property for cq:tags.
     */
    static final String PN_TAGS = NameConstants.PN_TAGS;

    /**
     * Property for post's postfix.
     */
    static final String POST_POSTFIX = "_tosp";

    /**
     * Property for url path splitter.
     */
    static final String ID_URL_PATH_SPLIT = "_s_";

    /**
     * Property for forum topic resource type.
     */
    String RESOURCE_TYPE_TOPIC = Forum.RESOURCE_TYPE_TOPIC;

    /**
     * Property for forum post resource type.
     */
    String RESOURCE_TYPE_POST = Forum.RESOURCE_TYPE_POST;

    /**
     * Adds a new post (reply) to this post.
     * @param resourceResolver The {@link ResourceResolver} to use for persisting values.
     * @param post The {@link Post} to add.
     * @return The newly created post.
     * @throws ForumException If an error occurs while adding the post.
     */
    Post addPost(ResourceResolver resourceResolver, Post post) throws ForumException {
		return null;
	}

    /**
     * Adds a new post (reply) to this post with attachments.
     * @param resourceResolver The {@link ResourceResolver} to use for persisting values.
     * @param fromRequest The {@link Post} to add.
     * @param attachmentDataSource The datasource of the attachment to add.
     * @return The newly created post.
     * @throws ForumException If an error occurs while adding the post.
     * @deprecated @see {@link #addAttachment(String, InputStream, String)}
     */
    @Deprecated
    Post addPost(ResourceResolver resourceResolver, Post fromRequest, List<DataSource> attachmentDataSource)
        throws ForumException {
		return null;
	}

    /**
     * This method returns the forum post's message as entered by the user. A message must always be present in a
     * post.
     * @return A <code>String</code> representing the post's message.
     */
    String getMessage() {
		return null;
	}

    /**
     * This method returns the forum post's subject as entered by the user. In general only top-level posts (topics)
     * have a subject. If the post doesn't have a subject, the subject of top level parent (topic) is returned.
     * @return A <code>String</code> representing the post's subject or <code>null</code> if not defined.
     */
    String getSubject() {
		return null;
	}

    /**
     * This method returns the date and time this post was created.
     * @return A {@link Date} representing the creation date and time.
     */
    Date getCreated() {
		return null;
	}

    /**
     * Returns the id of the user that added this post.
     * @return The user name.
     */
    ForumUser getCreatedBy() {
		return null;
	}

    /**
     * Returns the date this post was last modified, either directly or via addition of a reply (child post).
     * @return The date of last modification or the value of {@link #getCreated()} if never modified.
     */
    Date getModified() {
		return null;
	}

    /**
     * Returns the id of the user that effected the last modification of this post.
     * @return The user name or the value of {@link #getCreatedBy()} if never modified.
     */
    ForumUser getModifiedBy() {
		return null;
	}

    /**
     * Returns the date this topic was last modified directly.
     * @return The date of last modification or the value of {@link #getCreated()} if never modified.
     */
    Date getSelfModified() {
		return null;
	}

    /**
     * Returns the id of the user that effected the last modification of this topic.
     * @return The user name or the value of {@link #getCreatedBy()} if never modified.
     */
    ForumUser getSelfModifiedBy() {
		return null;
	}

    /**
     * Returns the ID of this post, specifically the node name.
     * @return The id.
     */
    String getId() {
		return null;
	}

    /**
     * The maximum number of posts per page.
     * @return The limit.
     */
    int getLimit() {
		return 0;
	}

    /**
     * Returns the path of this post.
     * @return A <code>String</code> representing the path of this post.
     */
    String getPath() {
		return null;
	}

    /**
     * Returns the top level post this post is a descendant of.
     * @return Returns the topic or itself if this post is a topic itself.
     * @throws ForumException if the topic couldn't be retrieved.
     */
    Post getTopic() throws ForumException {
		return null;
	}

    /**
     * Returns the post that this post is a child of.
     * @return Returns the post this post is a child of or itself if this is a top level post.
     * @throws ForumException if the parent couldn't be retrieved.
     */
    Post getParent() throws ForumException {
		return null;
	}

    /**
     * Indicates whether this post is pinned. A post is pinned if it is a topic (top level post) or its parent topic
     * is pinned.
     * @return <code>true</code> if this post is pinned.
     */
    boolean isPinned() {
		return false;
	}

    /**
     * Indicates whether this topic has been classified as spam.
     * @return <code>true</code> if this topic is spam.
     */
    boolean isSpam() {
		return false;
	}

    /**
     * Indicates whether this topic has been approved by a moderator. Only returns true if this topic is in fact
     * moderated and approved.
     * @return <code>true</code> if this topic is moderated and approved.
     * @see #isModerated()
     */
    boolean isApproved() {
		return false;
	}

    /**
     * Indicates whether this forum requires moderation for new topcis/posts.
     * @return <code>true</code> if this forum is moderated.
     */
    boolean isModerated() {
		return false;
	}

    /**
     * Indicates whether this forum is closed for further addition of topics/posts.
     * @return <code>true</code> if this forum is closed.
     */
    boolean isClosed() {
		return false;
	}

    /**
     * Indicates whether this post is in fact a topic (top level post).
     * @return <code>true</code> if this post represents a topic.
     */
    boolean isTopic() {
		return false;
	}

    /**
     * Returns the first level of posts belonging to this topic in chronological order (addition date, ascending).
     * @return An unmodifiable {@link Iterator} containing the posts.
     */
    Iterator<Post> getPosts() {
		return null;
	}

    /**
     * Returns the {@link ValueMap} containing the properties of this post.
     * @return The value map.
     */
    ValueMap getProperties() {
		return null;
	}

    /**
     * Returns the number of replies (child posts) this post has.
     * @return The number of replies.
     */
    int getRepliesCount() {
		return 0;
	}

    /**
     * Returns the number of replies (child posts) this post has.
     * @param request the SlingHttpServletRequest. This is checked to see if a new reply was just created in the
     *            previous request. If there was a new reply created and it hasn't been index yet on this server,
     *            increment the replies count by 1 to still display the new reply in the UI despite the lack of an
     *            index.
     * @return The number of replies.
     */
    int getRepliesCount(SlingHttpServletRequest request) {
		return 0;
	}

    /**
     * Returns the number of reply pages.
     * @return the number of reply pages.
     */
    public int getNumberOfPages() {
		return 0;
	}

    /**
     * The URL of this post.
     * @return A <code>String</code> representing the URL of this post.
     */
    String getUrl() {
		return null;
	}

    /**
     * The URL of this post. Optionally without forum page index, which saves processing time.
     * @param withPageIndex Whether to have the forum page index in the url. This has a performance implication.
     * @return A <code>String</code> representing the URL of this post.
     */
    String getUrl(boolean withPageIndex) {
		return null;
	}

    /**
     * The URL of this post with the page index and optionally the post id.
     * @param pageIndex the page index.
     * @param includePostId true if the selector should include post id.
     * @param includeTopicBucketId true if the selector should include the topic bucket id.
     * @return URL of this post with the page index and optionally the post id.
     */
    String getUrl(final int pageIndex, final boolean includePostId, boolean includeTopicBucketId) {
		return null;
	}

    /**
     * Returns the forum this post belongs to.
     * @return Return the {@link Forum} or <code>null</code> if not found.
     */
    Forum getForum() {
		return null;
	}

    /**
     * Return a subset of the child posts of this post as defined by the given page. The posts are returned in reverse
     * chronological order for topics and in chronological order for replies.
     * @param page the page of posts, start at 1.
     * @return The list of posts at the specified page.
     * @throws RepositoryException if error occurs when accessing repository.
     */
    List<Post> getPosts(int page) throws RepositoryException {
		return null;
	}

    /**
     * Return a subset of the child posts of this post as defined by the given offset and size. The posts are returned
     * in reverse chronological order for topics and in chronological order for replies.
     * @param offset the offset starting from 0.
     * @param size the max number of elements to return.
     * @param sortDateDescending true if posts are sorted by modified date in descending order and false if ascending.
     * @return The list of posts with the specified offset and size.
     * @throws RepositoryException if error occurs when accessing repository.
     */
    List<Post> getPosts(int offset, int size, boolean sortDateDescending) throws RepositoryException {
		return null;
	}

    /**
     * Return a subset of the child posts of this post as defined by the given offset and size. The posts are returned
     * in reverse chronological order for topics and in chronological order for replies.
     * @param offset the offset starting from 0.
     * @param size the max number of elements to return.
     * @return The list of posts with the specified offset and size.
     * @throws RepositoryException if error occurs when accessing repository.
     */
    List<Post> getPosts(int offset, int size) throws RepositoryException {
		return null;
	}

    /**
     * Return a subset of the child posts of this post as defined by the given page. The posts are returned in reverse
     * chronological order.
     * @param page the page of topics, start at 1.
     * @param request the SlingHttpServletRequest. This is checked to see if a new reply was just created in the
     *            previous request. If there was a new reply created and it hasn't been index yet on this server, add
     *            that reply to the end of the posts list so that it can still be displayed.
     * @return The list of topics at the specified page.
     * @throws RepositoryException if error occurred when accessing repository.
     * @TODO: Can we remove this API. This is a new API we added
     */
    List<Post> getPosts(int page, SlingHttpServletRequest request) throws RepositoryException {
		return null;
	}

    /**
     * Returns the latest post, i.e. the post with the most recent modification date. If this post is not a topic or
     * doesn't have any posts, <code>null</code> is returned.
     * @return The latest post of this topic or <code>null</code> if not found.
     */
    Post getLatestPost() {
		return null;
	}

    /**
     * Subscribes the given <code>authorizable</code> to receive change notifications for this topic.
     * @param authorizable The {@link com.day.cq.security.Authorizable} to subscribe.
     * @throws ForumException - Thrown, if an error during subscription occurs or an attempt is made to subscribe to a
     *             post that is not a topic ({@link #isTopic()} == false).
     */
    void subscribe(Authorizable authorizable) throws ForumException {
	}

    /**
     * Unsubscribes the given <code>authorizable</code> from this post.
     * @param authorizable The {@link com.day.cq.security.Authorizable} to unsubscribe.
     * @throws ForumException - Thrown, if an error during unsubscription occurs.
     */
    void unsubscribe(Authorizable authorizable) throws ForumException {
	}

    /**
     * Subscribes the given <code>authorizable</code> to receive change notifications for this topic.
     * @param authorizable The {@link com.day.cq.security.Authorizable} to subscribe.
     * @param workflowModel the path to the subscribe workflow model.
     * @throws ForumException - Thrown, if an error during subscription occurs or an attempt is made to subscribe to a
     *             post that is not a topic ({@link #isTopic()} == false).
     */
    void subscribe(Authorizable authorizable, String workflowModel) throws ForumException {
	}

    /**
     * Unsubscribes the given <code>authorizable</code> from this post.
     * @param authorizable The {@link com.day.cq.security.Authorizable} to unsubscribe.
     * @param workflowModel the path to the unsubscribe workflow model.
     * @throws ForumException - Thrown, if an error during unsubscription occurs.
     */
    void unsubscribe(Authorizable authorizable, String workflowModel) throws ForumException {
	}

    /**
     * Subscribes the given <code>authorizable</code> to receive change notifications for this topic.
     * @param authorizable The {@link com.day.cq.security.Authorizable} to subscribe.
     * @param workflowModel the path to the subscribe workflow model.
     * @param session the session to the subscribe workflow model.
     * @throws ForumException - Thrown, if an error during subscription occurs or an attempt is made to subscribe to a
     *             post that is not a topic ({@link #isTopic()} == false).
     * @deprecated Use
     *             {@link com.adobe.cq.social.commons.notification.Subscribable#subscribe(com.day.cq.security.Authorizable, String, org.apache.sling.api.resource.ResourceResolver)}
     *             instead.
     */
    @Deprecated
    void subscribe(Authorizable authorizable, String workflowModel, Session session) throws ForumException {
	}

    /**
     * Unsubscribes the given <code>authorizable</code> from this post.
     * @param authorizable The {@link com.day.cq.security.Authorizable} to unsubscribe.
     * @param workflowModel the path to the unsubscribe workflow model.
     * @param session the session to the unsubscribe workflow model.
     * @throws ForumException - Thrown, if an error during unsubscription occurs.
     * @deprecated Use
     *             {@link com.adobe.cq.social.commons.notification.Subscribable#unsubscribe(com.day.cq.security.Authorizable, String, org.apache.sling.api.resource.ResourceResolver)}
     *             instead.
     */
    @Deprecated
    void unsubscribe(Authorizable authorizable, String workflowModel, Session session) throws ForumException {
	}

    /**
     * Indicates whether the given <code>authorizable</code> (user) is currently subscribed to this topic.
     * @param authorizable The user to check.
     * @return <code>true</code> if this post is a topic and the given user is subscribed to it.
     */
    @Override
    boolean isSubscribed(Authorizable authorizable) {
		return false;
	}

    /**
     * Returns the {@link Tag}s this post is tagged with.
     * @return An array containing the {@link Tag}s of this post.
     */
    Tag[] getTags() {
		return null;
	}

    /**
     * Returns a comma-separated string of names the specified tags.
     * @return The tags as string
     */
    String getTagsAsString() {
		return null;
	}

    /**
     * Returns the number of unique participants of this topic, i.e. the size of the list of unique usernames that
     * replied in this topic.
     * @return the number of participants.
     * @deprecated This method has serious performance implications and its use is not advised
     */
    @Deprecated
    int getNumberOfParticipants() {
		return 0;
	}

    /**
     * Returns the <code>profile</code> of the user.
     * @return The profile or <code>null</code> if not set
     * @deprecated Use {@link #getUserProperties()} instead.
     */
    @Deprecated
    Profile getProfile() {
		return null;
	}

    /**
     * Returns the <code>userProperties</code> of the user.
     * @return The userProperties or <code>null</code> if not set
     */
    UserProperties getUserProperties() {
		return null;
	}

    /**
     * Return the index of the page this post is on (pagination).
     * @return The index.
     */
    int getPageIndex() {
		return 0;
	}

    /**
     * Return the index of this post.
     * @return the index of this post
     */
    int getIndex() {
		return 0;
	}

    /**
     * Get the url of this post with the given <code>index</code> this post is on. contained in the URL.
     * @param index The page index.
     * @return The url.
     * @see #getPageIndex()
     */
    String getUrl(int index) {
		return null;
	}

    /**
     * If this post is a topic, retrieve the child post with the given <code>id</code>.
     * @param id The id.
     * @return The post or <code>null</code> if not found.
     */
    Post getPost(String id) {
		return null;
	}

    /**
     * Return the path of the parent post, relative to the root content folder for the forum. If this post is a topic,
     * the relative path of itself is returned.
     * @return the path of the parent post, relative to the root content folder for the forum.
     */
    String getParentRelativePath() {
		return null;
	}

    /**
     * Return the default topic resource type.
     * @return the default topic resource type.
     */
    String getDefaultTopicResourceType() {
		return null;
	}

    /**
     * Return the default post resource type.
     * @return the default post resource type.
     */
    String getDefaultPostResourceType() {
		return null;
	}

    /**
     * The resource object that this object was adapted from.
     * @return The {@link Resource} this object wraps.
     */
    Resource getResource() {
		return null;
	}

    /**
     * Attach a file to this comment.
     * @param name The name of the file to attach.
     * @param inputStream The {@link InputStream} representing the contents of the file.
     * @param mimeType The mime-type of this file.
     * @return The {@link Resource} representing the attachment just added.
     * @throws CommentException Upon encountering an error during persistence of the attachment.
     * @throws IllegalArgumentException if name or inputStream or mimeType is null or empty.
     */
    Resource addAttachment(String name, InputStream inputStream, String mimeType) {
		return null;
	}

    /**
     * Retrieve an attachment of this comment by its file name.
     * @param name The file name of the attachment.
     * @return A {@link Resource} representing the attachment, or <code>null</code> if no attachment with the given
     *         name exists.
     */
    Resource getAttachment(String name) {
		return null;
	}

    /**
     * Retrieves a map, an entry of which represents a file-name/attachment pair. The map contains all the attachments
     * of the comment.
     * @return A {@link Map} representing the available attachments, or an empty map if no attachments are present.
     */
    Map<String, Resource> getAttachmentMap() {
		return null;
	}

    /**
     * Removes an attachment from this comment, as identified by its name.
     * @param name The name of the attachment to remove.
     * @throws CommentException If an error occurs removing the attachment.
     */
    void removeAttachment(String name) {
	}
}
