/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.forum.client.api;

import javax.jcr.RepositoryException;

import org.apache.commons.lang.StringUtils;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.CommentSystem;
import com.adobe.cq.social.commons.client.api.BaseQueryRequestInfo;
import com.adobe.cq.social.commons.client.api.ClientUtilities;
import com.adobe.cq.social.commons.client.api.CollectionPagination;
import com.adobe.cq.social.commons.client.api.DefaultResourceID;
import com.adobe.cq.social.commons.client.api.QueryRequestInfo;
import com.adobe.cq.social.commons.client.api.ResourceID;
import com.adobe.cq.social.commons.client.api.User;
import com.adobe.cq.social.commons.comments.api.AbstractComment;
import com.adobe.cq.social.commons.comments.listing.CommentIndexElementFactory;
import com.adobe.cq.social.commons.comments.listing.CommentSocialComponentListProviderManager;
import com.adobe.cq.social.commons.listing.impl.CommentIndexElementFactory.CommentResourceElement;
import com.adobe.cq.social.forum.api.Forum;
import com.adobe.cq.social.forum.client.impl.ForumUtils;
import com.adobe.cq.social.forum.client.impl.PostImpl;
import com.adobe.cq.social.forum.client.listing.impl.PostIndexElementFactory;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

public abstract class AbstractPost<T extends ForumConfiguration> extends AbstractComment<T> implements Post<T> {

    private static final String TOPIC_HTML_SUFFIX = ".topic.html";
    private Resource latestPost;
    private final Resource parent;
    private int numReplies;
    private ValueMap properties;
    private boolean isTopic;
    private final int numParticipants = -1;
    private User lastModifiedUser;
    private CommentSocialComponentListProviderManager listProviderManager;
    private static final Logger LOG = LoggerFactory.getLogger(PostImpl.class);

    public AbstractPost(final Resource resource, final ClientUtilities clientUtils,
        final CommentSocialComponentListProviderManager listProviderManager) throws RepositoryException {
        this(resource, clientUtils, QueryRequestInfo.DEFAULT_QUERY_INFO_FACTORY.create(), listProviderManager);
    }

    /**
     * Constructor of a comment.
     * @param comment the specified {@link CommentResourceElement}
     * @param clientUtils the client utilities instance
     * @param pagination the list pagination configuration of this comment
     * @param commentFactory comment factory used to create a legacy comment instance to a Post
     * @throws RepositoryException if an error occurs
     */
    public AbstractPost(final Resource resource, final ClientUtilities clientUtils, final QueryRequestInfo queryInfo,
        final CommentSocialComponentListProviderManager listProviderManager) throws RepositoryException {
        super(resource, clientUtils, queryInfo, listProviderManager);
        properties = ResourceUtil.getValueMap(resource);
        isTopic = isTopLevel();
        parent = ForumUtils.getParent(resource, isTopic, false);
        this.listProviderManager = listProviderManager;
    }

    public AbstractPost(final Resource resource, final ClientUtilities clientUtils, final QueryRequestInfo queryInfo,
        final Resource latestPost, final int numReplies,
        final CommentSocialComponentListProviderManager listProviderManager) throws RepositoryException {
        this(resource, clientUtils, queryInfo, listProviderManager);
        this.latestPost = latestPost;
        this.numReplies = numReplies;
        this.listProviderManager = listProviderManager;
    }

    @Override
    public String getSubject() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    @JsonInclude(value = Include.NON_NULL)
    public User getLastModifiedUser() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPinned() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isClosed() {
		return false;
	}

    /**
     * @return true if the topic is closed (this is based off AbstractComment).
     */
    @Override
    public boolean isTopicClosed() {
		return false;
	}

    /**
     * @return true if the entire forum is closed.
     */
    @Override
    public boolean isForumClosed() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isTopic() {
		return false;
	}

    /**
     * @inheritDoc
     */
    @Override
    public String getForumId() {
		return null;
	}

    /**
     * @inheritDoc
     */
    @Override
    @JsonInclude(value = Include.NON_NULL)
    public String getLatestPost() {
		return null;
	}

// /**
// * Get total size of the collection.
// * @return the size of the collection.
// */
// public int getTotalSize() {
// return numReplies;
// }

    @Override
    protected CommentIndexElementFactory getIndexElementFactory() {
		return null;
	}

    @Override
    public void setLatestPost(final Resource post) {
	}

    @Override
    public void setNumReplies(final int replies) {
	}

    @Override
    public String getFriendlyUrl() {
		return null;
	}

    @Override
    public void setPagination(final CollectionPagination pagination) {
	}

    @Override
    public T createConfiguration(final Resource resource, final CommentSystem cs) {
		return null;
	}
}
