/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.handlebars;

import java.util.Collections;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ValueMap;

import com.github.jknack.handlebars.ValueResolver;

/**
 * The Enum SlingResourceResolver.
 */
public enum SlingResourceResolver implements ValueResolver {

    /** The instance. */
    INSTANCE;

    /**
     * A hook interface for resolving values from the {@link Context context stack}.
     * @param context Object Context
     * @param name The name to be resolved
     * @return value The object value
     * @see com.github.jknack.handlebars.ValueResolver#resolve(java.lang.Object, java.lang.String)
     */
    @Override
    public Object resolve(final Object context, final String name) {
        Object value = null;
        if (context instanceof Resource) {
            final ValueMap vm = ((Resource) context).adaptTo(ValueMap.class);
            value = vm.get(name);
        }
        return value == null ? UNRESOLVED : value;
    }

    /**
     * List all the properties and their values for the given object.
     * @param context The context object. Not null.
     * @return All the properties and their values for the given object.
     */
    @Override
    public Set<Entry<String, Object>> propertySet(final Object context) {
        if (context instanceof Resource) {
            final ValueMap vm = ((Resource) context).adaptTo(ValueMap.class);
            return vm.entrySet();
        }
        return Collections.emptySet();
    }
}
