/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.journal;

import java.text.DateFormat;
import java.util.Date;
import java.util.List;
import java.util.ResourceBundle;

import javax.activation.DataSource;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;

import com.day.cq.tagging.Tag;
import com.day.cq.wcm.api.Page;

/**
 * {@linkplain Journal} represents a journal. It consisting of multiple instances of {@link JournalEntry}.
 */
public interface Journal {

    public static final String ANONYMOUS = "Anonymous";

    public static final String NODE_ENTRIES = "journalentries";

    public static final String URL_SUFFIX = ".html";
    public static final String FEED_SUFFIX = "/jcr:content.feed";
    public static final String TRACKBACK_SUFFIX = ".trackback.html";

    public static final String GENERATOR_NAME = "CQ5 Social Collaboration";
    public static final String GENERATOR_URL = "http://www.day.com/communique";

    public static final String PARAM_VIEW = "journal";
    public static final String PARAM_PERIOD = "period";
    public static final String PARAM_QUERY = "query";
    public static final String PARAM_TAG = "tag";
    public static final String PARAM_AUTHOR = "author";

    public static final String VIEW_DEFAULT = "default";
    public static final String VIEW_NEW = "new";
    public static final String VIEW_EDIT = "edit";
    public static final String VIEW_SEARCH = "search";
    public static final String VIEW_ARCHIVE = "archive";
    public static final String VIEW_AUTHOR = "author";
    public static final String VIEW_TAG = "tag";

    public static final String PROP_TITLE = "jcr:title";
    public static final String PROP_DESCRIPTION = "jcr:description";
    public static final String PROP_TEMPLATE = "cq:template";
    public static final String PROP_RESOURCE_TYPE = "sling:resourceType";
    public static final String PROP_JOURNAL_ARCHIVE = "journalarchive";

    public static final String DEFAULT_TEMPLATE = "/libs/social/journal/templates/page";
    public static final String DEFAULT_RESOURCE_TYPE = "social/journal/components/page";
    public static final String DEFAULT_TITLE = "Untitled";

    String AUTHOR_MAPPING_PREFIX = "/collabauthor";

    /**
     * Returns the entries in this journal.
     * @return The journal entries
     */
    List<JournalEntry> getEntries() {
		return null;
	}

    /**
     * Returns the entries in this journal using the specified filter.
     * @param filter The journal entry filter
     * @return The journal entries
     */
    List<JournalEntry> getEntries(JournalEntryFilter filter) {
		return null;
	}

    /**
     * Returns the entries in this journal using the specified filters.
     * @param filters The journal entry filters
     * @return The journal entries
     */
    List<JournalEntry> getEntries(List<JournalEntryFilter> filters) {
		return null;
	}

    /**
     * Returns the pages in this journal. Pages are special journal entries that don't show up in the timeline, e.g.
     * an "About" page.
     * @return The pages
     */
    List<JournalEntry> getPages() {
		return null;
	}

    /**
     * Returns the pages in this journal as a list of HTML links.
     * @return The pages as HTML
     * @see #getPages()
     */
    String getPagesAsHTML() {
		return null;
	}

    /**
     * Adds a new entry to this journal.
     * @param title The title of the journal entry
     * @param text The text of the journal entry
     * @param date The date of the journal entry (optional)
     * @param author the Author of the journal entry
     * @return The journal entry
     */
    JournalEntry addEntry(String title, String text, Date date, String author) {
		return null;
	}

    /**
     * Adds a new entry to this journal.
     * @param title The title of the journal entry
     * @param text The text of the journal entry
     * @param date The date of the journal entry (optional)
     * @param author the Author of the journal entry
     * @param attachmentDataSources a list of file attachments to include in the post
     * @return The journal entry
     */
    JournalEntry addEntry(String title, String text, Date date, String author, List<DataSource> attachmentDataSources) {
		return null;
	}

    /**
     * Adds a new page to this journal. Pages are special journal entries that don't show up in the timeline, e.g. an
     * "About" page.
     * @param title The title of the journal entry
     * @param text The text of the journal entry
     * @param author the author of the journal entry
     * @return The journal entry
     */
    JournalEntry addPage(String title, String text, String author) {
		return null;
	}

    /**
     * Adds a new page to this journal. Pages are special journal entries that don't show up in the timeline, e.g. an
     * "About" page.
     * @param title The title of the journal entry
     * @param text The text of the journal entry
     * @param author the author of the journal entry
     * @param attchmentDataSources a list of file attachments to include in the post
     * @return The journal entry
     */
    JournalEntry addPage(String title, String text, String author, List<DataSource> attachmentDataSources) {
		return null;
	}

    /**
     * Returns the entry before the specified entry or <code>null</code> if the specified entry is the first one.
     * @param refEntry The reference entry
     * @return The previous entry
     */
    JournalEntry getPreviousEntry(JournalEntry refEntry) {
		return null;
	}

    /**
     * Returns the entry after the specified entry or <code>null</code> if the specified entry is the last one.
     * @param refEntry The reference entry
     * @return The next entry
     */
    JournalEntry getNextEntry(JournalEntry refEntry) {
		return null;
	}

    /**
     * Returns all tags entries in this journal are filed under.
     * @return The tags
     */
    Tag[] getTags() {
		return null;
	}

    /**
     * Returns the specified tag as an HTML link.
     * @param tag The tag
     * @return The HTML link
     */
    String getTagAsHTML(Tag tag) {
		return null;
	}

    /**
     * Returns the specified tag as an HTML link.
     * @param tag The tag
     * @param resourceBundle <code>ResourceBundle</code> to use for translations
     * @return The HTML link
     */
    String getTagAsHTML(Tag tag, ResourceBundle resourceBundle) {
		return null;
	}

    /**
     * Returns all tags entries in this journal are filed under as a list of HTML links.
     * @return The tags as HTML
     */
    String getTagsAsHTML() {
		return null;
	}

    /**
     * Returns all tags entries in this journal are filed under as a list of HTML links.
     * @param resourceBundle <code>ResourceBundle</code> to use for translations
     * @return The tags as HTML
     */
    String getTagsAsHTML(ResourceBundle resourceBundle) {
		return null;
	}

    /**
     * Returns the number of journal entries filed under the specified tag.
     * @param tag The tag
     * @return The number of journal entries
     */
    int getTagCount(Tag tag) {
		return 0;
	}

    /**
     * Returns all authors of this journal.
     * @return The authors
     */
    String[] getAuthors() {
		return null;
	}

    /**
     * Returns the specified author as an HTML link.
     * @param author The author name
     * @return The HTML link
     */
    String getAuthorAsHTML(String author) {
		return null;
	}

    /**
     * Returns the specified author as an HTML link.
     * @param author The author name
     * @param resourceBundle <code>ResourceBundle</code> to use for translations
     * @return The HTML link
     */
    String getAuthorAsHTML(String author, ResourceBundle resourceBundle) {
		return null;
	}

    /**
     * Returns all authors of this journal as a list of HTML links.
     * @return The tags as HTML
     */
    String getAuthorsAsHTML() {
		return null;
	}

    /**
     * Returns all authors of this journal as a list of HTML links.
     * @param resourceBundle <code>ResourceBundle</code> to use for translations
     * @return The tags as HTML
     */
    String getAuthorsAsHTML(ResourceBundle resourceBundle) {
		return null;
	}

    /**
     * Returns the page associated with this journal.
     * @return The page
     */
    Page getPage() {
		return null;
	}

    /**
     * Returns the resource associated with this journal entry.
     * @return The resource
     */
    Resource getResource() {
		return null;
	}

    /**
     * Returns the journal entry if the underlying page is a journal entry page, or <code>null</code> if it is a main
     * journal page.
     * @return The journal entry
     */
    JournalEntry getEntry() {
		return null;
	}

    /**
     * Returns whether the underlying resource is a journal entry.
     * @return <code>true</code> if the underlying resource is a journal entry, <code>false</code> otherwise
     */
    boolean isEntry() {
		return false;
	}

    /**
     * Returns whether the underlying resource is a journal archive, i.e. a page between the main journal page and the
     * journal entries.
     * @return <code>true</code> if the underlying resource is a journal archive, <code>false</code> otherwise
     */
    boolean isArchive() {
		return false;
	}

    /**
     * Returns the title of this journal.
     * @return The title
     */
    String getTitle() {
		return null;
	}

    /**
     * Sets the title of this journal.
     * @param title The title
     */
    void setTitle(String title) {
	}

    /**
     * Returns the description of this journal.
     * @return The description
     */
    String getDescription() {
		return null;
	}

    /**
     * Sets the description of this journal.
     * @param desc The description
     */
    void setDescription(String desc) {
	}

    /**
     * Returns the server-local URL of this journal, omitting scheme, server name and port number.
     * @return The URL
     */
    String getUrl() {
		return null;
	}

    /**
     * Returns the full URL of this journal, including scheme, server name and port.
     * @return The full URL
     */
    String getFullUrl() {
		return null;
	}

    /**
     * Returns the URL of the feed for this journal as absolute URL.
     * @return The feed URL
     */
    String getFeedUrl() {
		return null;
	}

    /**
     * Returns the URL of the feed for this journal.
     * @param absolute true if the URL should be absolute (http://server.com/journal), false if it should be relative
     *            (/journal)
     * @return The feed URL
     */
    String getFeedUrl(boolean absolute) {
		return null;
	}

    /**
     * Returns the URL for view of this journal. The value can be one of:
     * <ul>
     * <li>{@link Journal#VIEW_DEFAULT} (default)</li>
     * <li>{@link Journal#VIEW_EDIT}</li>
     * <li>{@link Journal#VIEW_SEARCH}</li>
     * <li>{@link Journal#VIEW_ARCHIVE}</li>
     * </ul>
     * @param view The view
     * @return The view URL
     */
    String getViewUrl(String view) {
		return null;
	}

    /**
     * Returns the URL where a new journal entry can be added, or <code>null</code> if not available.
     * @param request The request
     * @return The URL to add a new journal entry
     */
    String getAddEntryUrl(SlingHttpServletRequest request) {
		return null;
	}

    /**
     * Returns whether the journal contains any journal entries.
     * @return <code>true</code> if journal is empty, <code>false</code> otherwise
     */
    boolean isEmpty() {
		return false;
	}

    /**
     * Returns the archive for this journal. If no date format is passed, a default date format will be used.
     * @param max The maximum number of items (optional)
     * @param dateFormat The date format (optional)
     * @return The archive
     */
    JournalArchive getArchive(int max, DateFormat dateFormat) {
		return null;
	}

    /**
     * Returns an HTML list created from the archive of this journal. If no date format is passed, a default date
     * format will be used.
     * @param max The maximum number of items (optional)
     * @param dateFormat The date format (optional)
     * @return The archive as HTML
     */
    String getArchiveAsHTML(int max, DateFormat dateFormat) {
		return null;
	}

    /**
     * Returns an HTML list created from the archive of this journal. If no date format is passed, a default date
     * format will be used.
     * @param max The maximum number of items (optional)
     * @param dateFormat The date format (optional)
     * @param resourceBundle <code>ResourceBundle</code> to use for translations
     * @return The archive as HTML
     */
    String getArchiveAsHTML(int max, DateFormat dateFormat, ResourceBundle resourceBundle) {
		return null;
	}

    /**
     * Returns whether this journal has been modified and needs to be saved.
     * @see #save()
     * @return <code>true</code> if journal is modified, <code>false</code> otherwise
     */
    boolean isModified() {
		return false;
	}

    /**
     * Saves all changes made to this journal. In case the journal doesn't exist yet, it will be created.
     * @throws JournalException if saving fails
     */
    void save() throws JournalException {
	}

    /**
     * Deletes the journal entry
     */
    void deleteEntry(JournalEntry entry) {
	}

}
