/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.journal;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import com.day.cq.wcm.api.Page;

/**
 * {@linkplain JournalArchive} represents a journal archive. It consists of multiple {@link JournalArchive.Period}s.
 */
public class JournalArchive {

    private final List<Period> periods;
    private static final DateFormat DEFAULT_TITLE_FORMAT = new SimpleDateFormat("MMMMM yyyy");
    private static final DateFormat DEFAULT_YEAR_FORMAT = new SimpleDateFormat("yyyy");
    private final DateFormat titleFormat;
    private final Journal journal;
    private final int max;

    /**
     * Creates a new <code>JournalArchive</code> for the specified journal. The optional date format is used to render
     * the titles of the periods. The default is <tt>MMMMM yyyyy</tt>.
     * @param j The journal
     * @param m The maximum number of periods (optional)
     * @param df The date format (optional)
     */
    public JournalArchive(final Journal j, final int m, final DateFormat df) {
        journal = j;
        max = m;
        titleFormat = df != null ? df : DEFAULT_TITLE_FORMAT;
        periods = new ArrayList<Period>();
    }

    /**
     * Returns the periods in this journal archive in descending order.
     * @return The periods
     */
    public List<Period> getPeriods() {
		return null;
	}

    /**
     * Returns a {@link JournalArchive.Period} for the specified period. An existing one will be returned, otherwise a
     * new period will be created.
     * @param period The period
     * @return The period
     */
    private Period getPeriod(final String period) {
		return null;
	}

    /**
     * Returns the title for a period.
     * @param period The period
     * @param monthDf The date format (optional)
     * @return The period title
     */
    public static String getPeriodTitle(final String period, final DateFormat monthDf) {
		return null;
	}

    /**
     * Returns the title for a period.
     * @param period The period
     * @param monthDf The date format for a date containing a month (optional)
     * @param yearDf The date format for a date only containing a year (optional)
     * @return The period title
     */
    public static String getPeriodTitle(final String period, DateFormat monthDf, DateFormat yearDf) {
		return null;
	}

    /**
     * {@linkplain JournalArchive.Period} represents a period in the journal archive.
     */
    public class Period {

        private final String id;
        private final String url;
        private final String title;

        /**
         * Creates a new instance of <code>Period</code> for the specified period.
         * @param period The period
         * @param t The title
         */
        public Period(final String period, final String t) {
            id = period;
            title = t;
            url = getUrl(period);
        }

        /**
         * Returns the URL of this period.
         * @return The period URL
         */
        public String getUrl() {
			return null;
		}

        /**
         * Returns the title of this period.
         * @return The period title
         */
        public String getTitle() {
			return null;
		}

        /**
         * Returns the ID of this period.
         * @return The ID
         */
        public String getId() {
			return null;
		}

        /**
         * Returns the URL for a period.
         * @param period The period
         * @return The period URL
         */
        private String getUrl(final String period) {
			return null;
		}

    }

}
