/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.journal;

import java.util.HashMap;

import org.osgi.service.event.Event;

import com.adobe.cq.social.commons.events.SocialEvent;
import com.adobe.cq.social.commons.events.SocialEvent.SocialActions;

/**
 * Defines events that are triggered in journals.
 */
public class JournalEvent extends SocialEvent<JournalEvent.JournalActions> {

    private static final long serialVersionUID = 1L;
    /**
     * The event topic suffix for journal events.
     */
    public static final String JOURNAL_TOPIC = "journal";
    protected static final String JOURNAL_PATH = "JOURNAL_PATH";

    /**
     * @param entryPath - the jcr path to the journal entry page on which the event was triggered.
     * @param userId - the user id of the user who triggered the event.
     * @param journalPath - the jcr path to the jounral in which the journal entry resides.
     * @param action - the journal action that triggered this event.
     */
    public JournalEvent(final String entryPath, final String userId, final String journalPath,
        final JournalEvent.JournalActions action) {
        super(JOURNAL_TOPIC, entryPath, userId, action, new HashMap<String, Object>(1) {

            private static final long serialVersionUID = 1L;

            {
                this.put(JOURNAL_PATH, journalPath);
            }
        });
    }

    /**
     * This constructor should be used to create a Journal Event given an OSGi Event.
     * @param event an OSGi event that is known to be a journal event.
     */
    private JournalEvent(final Event event) {
        super(event);
    }

    /**
     * List of available journal actions that can trigger a journal event.
     */
    public static enum JournalActions implements SocialActions {
        CREATED, EDITED, DELETED;
    }

    /**
     * @return the jcr path to the journal in which this event happened.
     */
    public String getJournalPath() {
		return null;
	}

    /**
     * Retrieves the journal event from the given generic {@link Event}.
     * @param event The event.
     * @return The journal event or <code>null</code> if the given event is not a journal event.
     */
    public static JournalEvent fromEvent(final Event event) {
		return null;
	}
}
