/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.messaging.api;

import java.util.Calendar;
import java.util.List;

import org.apache.sling.api.adapter.Adaptable;

/**
 * Message interface.
 */
public interface Message extends Adaptable {
    /**
     * property to specify message id.
     */
    String PN_MESSAGE_ID = "messageId";
    /**
     * property to specify recipients.
     */
    String PN_MESSAGE_RECIPIENTS = "Recipients";
    /**
     * property to specify sender name.
     */
    String PN_MESSAGE_SENDER_NAME = "senderName";
    /**
     * property to specify recipients' names.
     */
    String PN_MESSAGE_RECIPIENT_NAMES = "recipientNames";
    /**
     * property to specify message subject.
     */
    String PN_MESSAGE_SUBJECT = "subject";
    /**
     * property to specify message content.
     */
    String PN_MESSAGE_CONTENT = "content";
    /**
     * property to specify sender id.
     */
    String PN_MESSAGE_SENDER = "sender";
    /**
     * property to specify to specify read or unread.
     */
    String PN_MESSAGE_READ = "read";
    /**
     * property to specify trashed or not.
     */
    String PN_MESSAGE_DELETED = "deleted";
    /**
     * property to specify time stamp of message.
     */
    String PN_MESSAGE_TIMESTAMP = "timestamp";
    /**
     * node type for message Nodes.
     */
    String NT_MESSAGE_NODE_TYPE = "nt:unstructured";

    /**
     * Name of child node under which attachments are stored.
     */
    String NN_MESSAGE_ATTACHMENTS = "attachments";

    /**
     * get message id.
     * @return id of message.
     */
    String getId() {
		return null;
	}

    /**
     * get id of sender.
     * @return sender id.
     */
    String getSenderId() {
		return null;
	}

    /**
     * get name of sender.
     * @return sender name
     */
    String getSenderName() {
		return null;
	}

    /**
     * get subject of message.
     * @return subject of message.
     */
    String getSubject() {
		return null;
	}

    /**
     * get content of message.
     * @return content of message.
     */
    String getContent() {
		return null;
	}

    /**
     * get list of ids of recipients.
     * @return list of ids of recipients
     */
    List<String> getRecipientIdList() {
		return null;
	}

    /**
     * get list of recipient names.
     * @return list of recipient names.
     */
    List<String> getRecipientNameList() {
		return null;
	}

    /**
     * get a custom property stored on message.
     * @param propertyName name of property to be searched for
     * @return value of a custom property
     */
    Object getCustomProperty(String propertyName) {
		return null;
	}

    /**
     * tells if message has been read.
     * @return true for read message.
     */
    boolean isRead() {
		return false;
	}

    /**
     * tells if the message has been trashed.
     * @return true for trashed message.
     */
    boolean isDeleted() {
		return false;
	}

    /**
     * time at which message had been sent.
     * @return message sending time.
     */
    Calendar getTimestamp() {
		return null;
	}

    /**
     * mark the message as trashed or un-trashed.
     * @param value true if the message is to be trashed, false otherwise.
     * @return true if existing value on message was different from passed value, false otherwise.
     */
    boolean setDeleted(boolean value) {
		return false;
	}

    /**
     * mark the message as read or unread.
     * @param value true if the message is to be marked read, false otherwise.
     * @return true if existing value on message was different from passed value, false otherwise.
     */
    boolean setRead(boolean value) {
		return false;
	}

    /**
     * change the value of an existing custom property on message.
     * @param name name of property
     * @param value value to be set
     * @return old value of the property if successful. <code>null</code> if the property is not an existing custom
     *         property of the message and hence could not be set.
     */
    Object setCustomProperty(String name, Object value) {
		return null;
	}
}
