/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.messaging.api;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.service.event.Event;

import com.adobe.cq.social.commons.events.SocialEvent;

/**
 * class to represent private messaging events.
 */

public final class MessagingEvent extends SocialEvent<MessagingEvent.MessagingActions> {

    private static final long serialVersionUID = 1L;
    private static final String MESSAGE_TOPIC = "message";
    /**
     * The topic name for event topic.
     */
    public static final String EVENT_TOPIC = SocialEvent.SOCIAL_EVENT_TOPIC_PREFIX + MESSAGE_TOPIC;
    private static final String PN_PROPERTIES_MODIFIED = "propertiesUpdated";
    private static final String PN_RECIPIENT_MESSAGE_PATHS = "pathsToSentMessages";

    /**
     * The event types supported under messaging.
     */
    public static enum MessagingActions implements SocialEvent.SocialActions {

        /**
         * The event fired upon a message sending.
         */
        MessageSent,

        /**
         * The event fired upon a message update.
         */
        MessageUpdated,

        /**
         * The event fired upon permanently deleting a message.
         */
        MessageDeleted,

    }

    /**
     * constructor to be called for message event creation.
     * @param messagePath path to message which has been created, updated or deleted.
     * @param createdMessagePaths paths to messages which have been created under inbox nodes of recipients
     * @param userId id of user who initiated the event
     * @param messagingAction {@link com.adobe.cq.social.messaging.api.MessagingEvent.MessagingActions}
     * @param modifiedProperties list of names of modified properties, should be null in case of message sending event
     */
    public MessagingEvent(final String messagePath, final List<String> createdMessagePaths, final String userId,
        final MessagingActions messagingAction, final List<String> modifiedProperties) {
        super(MESSAGE_TOPIC, messagePath, userId, messagingAction, buildProperties(modifiedProperties,
            createdMessagePaths));
    }

    /**
     * Retrieves the messaging event from the given generic {@link Event}.
     * @param event The event.
     * @return The messaging event or <code>null</code> if the given event is not a {@link MessagingEvent}.
     */
    public static MessagingEvent fromEvent(final Event event) {
		return null;
	}

    /**
     * get list of names of updated properties of message.
     * @return list of names of updated properties.
     */
    public List<String> getUpdatedProperties() {
		return null;
	}

    /**
     * get paths to the messages created under inbox nodes of recipients.
     * @return paths to created messages.
     */
    public List<String> getPathsToReceivedMessages() {
		return null;
	}

    private static Map<String, Object> buildProperties(final List<String> modifiedProperties,
        final List<String> createdMessagePaths) {
			return null;
		}
}
