/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.messaging.api;

import java.util.List;

import javax.jcr.RepositoryException;

import org.apache.sling.api.resource.PersistenceException;
import org.apache.sling.api.resource.ResourceResolver;

/**
 * Messaging service exposed for sending messages. Also extends {@link MessageSearch} for retrieving messages.
 */
public interface MessagingService extends MessageSearch {

    /** Custom properties Names whitelist service property name. */
    String MESSAGE_CUSTOM_PROPERTIES_WHITE_LIST = "message.properties";

    /**
     * return a new {@link MessageBuilder}.
     * @param resourceResolver {@link ResourceResolver}
     * @return a new message sender.
     * @deprecated use {@link #newMessageSender(String, boolean)} instead
     */
    @Deprecated
    MessageBuilder newMessageSender(ResourceResolver resourceResolver) {
		return null;
	}

    /**
     * return a new {@link MessageBuilder}.
     * @param senderId user id of the sender.
     * @param isFiltered if true, the white list filtering is applied
     * @return a new message sender.
     */
    MessageBuilder newMessageSender(String senderId, final boolean isFiltered) {
		return null;
	}

    /**
     * mark all the messages in message list as read or unread.
     * @param messageList list of {@link Message}s
     * @param value true if the message is to be marked read, false otherwise.
     * @param resourceResolver {@link ResourceResolver}
     * @throws PersistenceException in case of failure on session save.
     */
    void setRead(List<Message> messageList, boolean value, ResourceResolver resourceResolver)
        throws PersistenceException {
	}

    /**
     * mark all the messages in message list as trashed or un-trashed.
     * @param messageList list of {@link Message}s
     * @param value true if the message is to be marked trashed, false otherwise.
     * @param resourceResolver {@link ResourceResolver}
     * @throws PersistenceException in case of failure on session save.
     */
    void setDeleted(List<Message> messageList, boolean value, ResourceResolver resourceResolver)
        throws PersistenceException {
	}

    /**
     * change the value of an existing custom property on all messages in message list.
     * @param messageList list of {@link Message}s
     * @param name name of property to be set
     * @param value value to be set
     * @param resourceResolver {@link ResourceResolver}
     * @throws PersistenceException in case of failure on session save.
     */
    void setCustomProperty(List<Message> messageList, String name, Object value, ResourceResolver resourceResolver)
        throws PersistenceException {
	}

    /**
     * permanently delete all the messages in message list.
     * @param messageList list of {@link Message}s
     * @param resourceResolver {@link ResourceResolver}
     * @throws PersistenceException in case of failure on session save.
     */
    void delete(List<Message> messageList, ResourceResolver resourceResolver) throws PersistenceException {
	}

    /**
     * get the usage statistics of the user corresponding to the passed resolver.
     * @param resolver {@link ResourceResolver} of the user whose statistics is to be known.
     * @return {@link Usage}
     * @throws RepositoryException {@link RepositoryException} if some exception occurs in calculating the statistics.
     */
    Usage getUserStatistics(ResourceResolver resolver) throws RepositoryException {
		return null;
	}
}
