/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.messaging.client.endpoints;

import java.util.List;
import java.util.Map;

import javax.activation.DataSource;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;

import com.adobe.cq.social.commons.client.endpoints.OperationException;

/**
 * CRUD Operation of a {@link com.adobe.cq.social.messaging.api.Message}.
 */
public interface MessagingOperationsService {
    /**
     * Creates a new message from/based on {@link SlingHttpServletRequest}.
     * @param request the client request
     * @return Http response status code.
     * @throws com.adobe.cq.social.commons.client.endpoints.OperationException if there is an error occurs
     */
    short create(final SlingHttpServletRequest request) throws OperationException {
		return 0;
	}

    /**
     * Create a a new message with the specified {@link java.util.Map} of properties and {@link java.util.List} of
     * attachments.
     * @param resolver the root resource of the message to be created.
     * @param senderAuthId Authorizable Id of the message sender.
     * @param props message properties passed as a java map.
     * @return the new {@link com.adobe.cq.social.messaging.api.Message}
     * @throws com.adobe.cq.social.commons.client.endpoints.OperationException if there is an error occurs
     */
    short create(final ResourceResolver resolver, final String senderAuthId, final Map<String, Object> props,
        final List<DataSource> attachments) throws OperationException {
		return 0;
	}

    /**
     * Update a {@link com.adobe.cq.social.messaging.api.Message} base on the specified
     * {@link org.apache.sling.api.SlingHttpServletRequest} using the specified. {@link javax.jcr.Session}.
     * @param request
     * @return the update message.
     * @throws com.adobe.cq.social.commons.client.endpoints.OperationException if there is an error occurs
     */
    Resource update(final SlingHttpServletRequest request) throws OperationException {
		return null;
	}

    /**
     * Update the specified message with the specified {@link java.util.Map} of properties and {@link java.util.List}
     * of attachments.
     * @param commentResource the target message resource.
     * @param props a map of the properties to be updated
     * @param attachments a list of attachment to be added to the specified message.
     * @return the new updated message resource.
     * @throws com.adobe.cq.social.commons.client.endpoints.OperationException if there is an error occurs
     */
    Resource update(final Resource commentResource, final Map<String, Object> props,
        final List<DataSource> attachments) throws OperationException {
		return null;
	}

    /**
     * Delete a {@link com.adobe.cq.social.messaging.api.Message} specified by the
     * {@link org.apache.sling.api.SlingHttpServletRequest} using the specified {@link javax.jcr.Session}.
     * @param request the http request which specifies the target message.
     * @throws com.adobe.cq.social.commons.client.endpoints.OperationException if there is an error occurs
     */
    void delete(final SlingHttpServletRequest request) throws OperationException {
	}

    /**
     * Delete a {@link com.adobe.cq.social.messaging.api.Message} using the specified {@link javax.jcr.Session}.
     * @throws com.adobe.cq.social.commons.client.endpoints.OperationException if there is an error occurs
     */
    void delete(final Resource messageResource) throws OperationException {
	}

}
