/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.qna.api;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.event.Event;

import com.adobe.cq.social.commons.events.SocialEvent;
import com.adobe.cq.social.qna.api.QnaEvent.QnaActions;

/**
 * Defines events that are triggered in a QnA Forum .
 */
public class QnaEvent extends SocialEvent<QnaEvent.QnaActions> {

    private static final String QUESTION_PATH = "QUESTION_PATH";

    /**
     * The event topic suffix for QnA events.
     */
    public static final String QNA_TOPIC = "qna";

    private static final String ANSWER_PATH = "ANSWER_PATH";

    private static final String QNA_FORUM_PATH = "QNA_FORUM_PATH";

    private static final long serialVersionUID = 1L;

    /**
     * List of available QnA actions that can trigger a QnA event.
     */
    public static enum QnaActions implements SocialEvent.SocialActions {
        QUESTION_CREATED, QUESTION_EDITED, QUESTION_DELETED, ANSWER_SELECTED, ANSWER_UNSELECTED, ANSWER_CREATED, ANSWER_EDITED, ANSWER_DELETED;
    }

    /**
     * @param path - the jcr path to either the question node or the answer node on which the event was triggered.
     * @param userId - the user id of the user who triggered the event.
     * @param action - the action that was taken to trigger the event.
     * @param additionalData - any additional data that is part of the event payload.
     */
    private QnaEvent(final String path, final String userId, final QnaActions action,
        final Map<String, Object> additionalData) {
        super(QNA_TOPIC, path, userId, action, additionalData);
    }

    /**
     * This constructor should be used to create a QnA Event given an OSGi Event.
     * @param event an OSGi event that is known to be a QnA event.
     */
    private QnaEvent(final Event event) {
        super(event);
    }

    private static QnaEvent buildQuestionEvent(final String questionPath, final String userId,
        final String qnaForumPath, final QnaActions action) {
			return null;
		}

    /**
     * Creates a QnAEvent that represents an action of {@link QnaActions.ANSWER_SELECTED}
     * @param questionPath - the jcr node path of the question for which an answer was selected.
     * @param answerPath - the jcr node path of the answer that was chosen as the correct answer.
     * @param qnaForumPath - the jcr node path to the QnA forum to which the question belongs.
     * @param userId - the user id of the user who selected the answer.
     */
    public static QnaEvent answerSelected(final String questionPath, final String answerPath,
        final String qnaForumPath, final String userId) {
			return null;
		}

    /**
     * Creates a QnAEvent that represents an action of {@link QnaActions.ANSWER_UNSELECTED}
     * @param questionPath - the jcr node path of the question for which an answer was unselected.
     * @param answerPath - the jcr node path of the answer that was removed as the correct answer.
     * @param qnaForumPath - the jcr node path to the QnA forum to which the question belongs.
     * @param userId - the user id of the user who unselected the answer.
     */
    public static QnaEvent answerUnselected(final String questionPath, final String answerPath,
        final String qnaForumPath, final String userId) {
			return null;
		}

    /**
     * @return the jcr node path to the QnA Forum in which this event happened.
     */
    public String getQnaForumPath() {
		return null;
	}

    /**
     * @return the jcr path to the answer node that was part of this event. For QnAEvents that represent a question
     *         being created, deleted or edited, this will be null.
     */
    public String getAnswerPath() {
		return null;
	}

    /**
     * @return the jcr path to the question node that was part of this event.
     */
    public String getQuestionPath() {
		return null;
	}

    /**
     * Creates a QnAEvent that represents an action of {@link QnaActions.QUESTION_CREATED}
     * @param questionPath - the jcr node path to the question that was created.
     * @param qnaForumPath - the jcr node path to the QnA Forum in which the question was created.
     * @param userId - the user id of the user who created the question.
     */
    public static QnaEvent questionCreated(final String questionPath, final String qnaForumPath, final String userId) {
		return null;
	}

    /**
     * Creates a QnAEvent that represents an action of {@link QnaActions.QUESTION_EDITED}
     * @param questionPath - the jcr node path to the question that was edited.
     * @param qnaForumPath - the jcr node path to the QnA Forum in which the question was edited.
     * @param userId - the user id of the user who edited the question.
     */
    public static QnaEvent questionEdited(final String questionPath, final String qnaForumPath, final String userId) {
		return null;
	}

    /**
     * Creates a QnAEvent that represents an action of {@link QnaActions.QUESTION_DELETED}
     * @param questionPath - the jcr node path to the question that was deleted.
     * @param qnaForumPath - the jcr node path to the QnA Forum in which the question was deleted.
     * @param userId - the user id of the user who deleted the question.
     */
    public static QnaEvent questionDeleted(final String questionPath, final String qnaForumPath, final String userId) {
		return null;
	}

    /**
     * Creates a QnAEvent that represents an action of {@link QnaActions.ANSWER_CREATED}
     * @param answerPath - the jcr node path to the answer that was created.
     * @param questionPath - the jcr node path to the question for which the answer was created.
     * @param qnaForumPath - the jcr node path to the QnA Forum in which the answer was created.
     * @param userId - the user id for the user who created the answer.
     */
    public static QnaEvent answerCreated(final String answerPath, final String questionPath,
        final String qnaForumPath, final String userId) {
			return null;
		}

    /**
     * Creates a QnAEvent that represents an action of {@link QnaActions.ANSWER_DELETED}
     * @param answerPath - the jcr node path to the answer that was deleted.
     * @param questionPath - the jcr node path to the question for which the answer was deleted.
     * @param qnaForumPath - the jcr node path to the QnA Forum in which the answer was deleted.
     * @param userId - the user id for the user who deleted the answer.
     */
    public static QnaEvent answerDeleted(final String answerPath, final String questionPath,
        final String qnaForumPath, final String userId) {
			return null;
		}

    /**
     * Creates a QnAEvent that represents an action of {@link QnaActions.ANSWER_EDITED}
     * @param answerPath - the jcr node path to the answer that was edited.
     * @param questionPath - the jcr node path to the question for which the answer was edited.
     * @param qnaForumPath - the jcr node path to the QnA Forum in which the answer was edited.
     * @param userId - the user id for the user who edited the answer.
     */
    public static QnaEvent answerEdited(final String answerPath, final String questionPath,
        final String qnaForumPath, final String userId) {
			return null;
		}

    private static QnaEvent buildAnswerEvent(final String path, final String userId, final String qnaForumPath,
        final String questionPath, final QnaActions action) {
			return null;
		}

    /**
     * Retrieves the QnA event from the given generic {@link Event}.
     * @param event The event.
     * @return The QnA event or <code>null</code> if the given event is not a QnA event.
     */
    public static QnaEvent fromEvent(final Event event) {
		return null;
	}

}
