/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *
 *************************************************************************/

package com.adobe.cq.social.qna.api;

import javax.jcr.RepositoryException;
import javax.jcr.Session;

import com.adobe.cq.social.forum.api.Post;
import com.day.cq.replication.ReplicationException;
import com.day.cq.replication.Replicator;

/**
 * The <code>Post</code> interface specifies the contract for a forum post.
 * @since CQ 5.6.0
 */
public interface QnaPost extends Post {
    /**
     * Property for qna topic resource type.
     */
    String RESOURCE_TYPE_TOPIC = "social/qna/components/topic";

    /**
     * Property for qna post resource type.
     */
    String RESOURCE_TYPE_POST = "social/qna/components/post";

    /**
     * Indicates whether this question is answered.
     * @return <code>true</code> if this question is answered.
     */
    public boolean isAnswered() {
		return false;
	}

    /**
     * Indicates whether this is chosen answer.
     * @return <code>true</code> if this is chosen answer.
     */
    public boolean isChosenAnswer() {
		return false;
	}

    /**
     * Unchoose this answer if it is the chosen answer. Replicate the change if replicator is not null. Delete the
     * answer if delete flag is true. If this is not an answer, do nothing.
     * @param replicator the replicator.
     * @param session the user session.
     * @param delete if true, delete the answer.
     * @param checkClosedModerated if true, check if the moderation information and throw a ForumException if the
     *            question is closed or if the question or answer are marked spam.
     * @throws RepositoryException thrown if there is an issue with the repository.
     * @throws ReplicationException thrown if there is a replication issue.
     */
    public void removeAnswer(final Replicator replicator, final Session session, final boolean delete,
        final boolean checkClosedModerated) throws RepositoryException, ReplicationException {
	}
}
