/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.review;

import java.io.Serializable;
import java.util.HashMap;

import org.apache.commons.lang3.StringUtils;
import org.osgi.service.event.Event;

import com.adobe.cq.social.commons.events.SocialEvent;
import com.adobe.cq.social.review.ReviewEvent.Type;

/**
 * Event fired for changes occurring within a review system.
 */
public class ReviewEvent extends SocialEvent<Type> implements Serializable {

    /** A serialVersionUID for this class. */
    private static final long serialVersionUID = 2L;

    /**
     * The event topic suffix for review events
     */
    public static final String REVIEW_TOPIC = SocialEvent.SOCIAL_EVENT_TOPIC_PREFIX + "review";
    protected static final String REVIEW_SYSTEM_PATH = "REVIEW_SYSTEM_PATH";

    /**
     * The event types supported for the forum and its topics/posts.
     */
    public static enum Type implements com.adobe.cq.social.commons.events.SocialEvent.SocialActions {

        /**
         * The event fired upon a review being added.
         */
        ReviewAdded,

        /**
         * The event fired upon approval of a moderated review.
         */
        ReviewApproved,

        ReviewEdited,

        ReviewDeleted,

    }

    public ReviewEvent(final String path, final String parentPath, final String userId, final Type type) {
        super(REVIEW_TOPIC, path, userId, type, new HashMap<String, Object>(1) {

            private static final long serialVersionUID = 1L;

            {
                this.put(REVIEW_SYSTEM_PATH, parentPath);
            }
        });
    }

    private ReviewEvent(final Event event) {
        super(event);
    }

    /**
     * Retrieves the review event from the given generic {@link Event}.
     * @param event The event.
     * @return The forum event or <code>null</code> if the given event is not a forum event.
     */
    public static ReviewEvent fromEvent(final Event event) {
		return null;
	}

    /**
     * @return the path to the review system of the review on which this event was triggered.
     */
    public String getReviewSystemPath() {
		return null;
	}
}
