/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.storage.buckets;

import java.util.Iterator;

import javax.jcr.Node;

import org.apache.sling.api.resource.Resource;

/**
 * The NestedBucketStorageSystem is a facility which manages JCR node creation under a single root node. Nodes are
 * organized in buckets of 1000 nodes. The client can specify the bucket postfix string and the bucket resource type.
 * The default bucket postfix string is "Bucket", and "sling:Folder" is the bucket resource type. An instance of the
 * NestedBucketStorageSystem should be obtain using the sling adatpTo mechanism: <code>
 *     NestedBucketStorageSystem nestedBucketStorageSystem = root.adaptTo(NestedBucketStorageSystem.class);
 * </code>
 */
public interface NestedBucketStorageSystem {
    /** The maximum number of nodes per bucket. */
    int MAX_NODES_PER_BUCKET = 1000;

    /**
     * The default bucket type.
     */
    String BUCKET_TYPE = "sling:Folder";

    /**
     * The default bucket postfix.
     */
    String DEFAULT_BUCKET_POSTFIX = "Bucket";

    /**
     * Returns the root node of the storage system, for example,
     * /content/usergenerated/content/geometrixx/en/community/jcr:content/forum node.
     * @return The root node
     */
    Resource getRoot() {
		return null;
	}

    /**
     * Set the bucket postfix.
     */
    void setBucketPostfix(final String postfix) {
	}

    /**
     * Get the bucket postfix.
     */
    String getBucketPostfix() {
		return null;
	}

    /**
     * Set the bucket type.
     */
    void setBucketResourceType(String type) {
	}

    /**
     * Get the bucket type.
     */
    String getBucketResourceType() {
		return null;
	}

    /**
     * Returns the number of resource located under the root node.
     */
    long getNumberChildren() throws NestedBucketStorageException {
		return 0L;
	}

    /**
     * Returns an iterator of the children of this bucket storage system.
     * @return The comments
     */
    Iterator<Resource> listChildren() throws NestedBucketStorageException {
		return null;
	}

    /**
     * Add a new resource with the given name and primary node type name.
     * @param name - name of the resource to add
     * @param primaryNodeTypeName - primary node type of the node to add
     * @return <code>Resource</code> The new resource
     */
    Resource addResource(final String name, final String primaryNodeTypeName) throws NestedBucketStorageException {
		return null;
	}

    /**
     * Add a new node with the given name and primary node type name.
     * @param name - name of the resource to add
     * @param primaryNodeTypeName - primary node type of the node to add
     * @return Node the new node
     */
    Node addNode(final String name, final String primaryNodeTypeName) throws NestedBucketStorageException {
		return null;
	}

    /**
     * List the buckets.
     * @return <code>Iterator<Resource>
     * @throws NestedBucketStorageException
     */
    Iterator<Resource> listBuckets() throws NestedBucketStorageException {
		return null;
	}

}
