/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.storage.index;

import java.util.Set;

import org.apache.sling.api.SlingConstants;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Base class for IndexHandlers that has common code for filtering events.
 */
public abstract class AbstractBaseIndexHandler implements IndexHandler {
    private static final Logger LOG = LoggerFactory.getLogger(AbstractBaseIndexHandler.class);
    private Set<String> targetModifiedAttributes;
    private Set<String> targetAddedRemovedAttributes;
    private Set<String> addedResourceTypes;
    private Set<String> changedResourceTypes;
    private boolean indexResourceSubTypes;

    public AbstractBaseIndexHandler(final Set<String> targetModifiedAttributes,
        final Set<String> targetAddedRemovedAttributes, final Set<String> addedResourceTypes,
        final Set<String> changedResourceTypes) {
        init(targetModifiedAttributes, targetAddedRemovedAttributes, addedResourceTypes, changedResourceTypes, false);
    }

    public AbstractBaseIndexHandler(final Set<String> targetModifiedAttributes,
        final Set<String> targetAddedRemovedAttributes, final Set<String> addedResourceTypes,
        final Set<String> changedResourceTypes, final boolean indexResourceSubTypes) {
        init(targetModifiedAttributes, targetAddedRemovedAttributes, addedResourceTypes, changedResourceTypes,
            indexResourceSubTypes);
    }

    private void init(final Set<String> targetModifiedAttributes, final Set<String> targetAddedRemovedAttributes,
        final Set<String> addedResourceTypes, final Set<String> changedResourceTypes,
        final boolean indexResourceSubTypes) {
		}

    /**
     * {@inheritDoc}
     */
    @Override
    public void registerAddResourceTypes(final String... resourceTypes) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void registerChangeResourceTypes(final String... resourceTypes) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void unregisterAddResourceTypes(final String... resourceTypes) {
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public void unregisterChangeResourceTypes(final String... resourceTypes) {
	}

    /**
     * Given the node path, resource type and supported resource types, return true if this node should be indexed.
     * @param resolver the ResourceResolver.
     * @param nodePath the path to the node.
     * @param resourceType the resource type of the node.
     * @param supportedResourceTypes the set of resource types that we should index.
     * @return true if the given node should be indexed.
     */
    private boolean checkResourceType(final ResourceResolver resolver, final String nodePath,
        final String resourceType, final Set<String> supportedResourceTypes) {
			return false;
		}

    /**
     * Check the given array of attributes and return true if any string in the array appears in the target set of
     * strings.
     * @param attributes the array of strings to check.
     * @param targets the set of strings we are looking for.
     * @return true if any string in the array appears in the target set of strings.
     */
    private boolean checkAttributes(final String[] attributes, final Set<String> targets) {
		return false;
	}

    protected abstract void indexExec(ResourceResolver resolver, String nodePath,
        Enum<EventTopicsType> eventTopicsType) throws Exception {
	}

    /**
     * {@inheritDoc}
     */
    public boolean handleEvent(final ResourceResolver resolver, final String resourceType, final String eventTopics,
        final String nodePath, final String[] resourceChangedAttributes, final String[] resourceAddedAttributes,
        final String[] resourceRemovedAttributes, final Enum<EventTopicsType> eventTopicsType) throws Exception {
			return false;
		}

}
