/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.storage.index;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.storage.buckets.NestedBucketStorageSystem;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.commons.jcr.JcrUtil;

/**
 * Base class for Indexer implementation.
 */
public abstract class AbstractBaseIndexer<K, E> implements Indexer<K, E> {
    private static final Logger log = LoggerFactory.getLogger(AbstractBaseIndexer.class);

    /**
     * Root path to index.
     */
    protected String rootIndexResourcePath;

    /**
     * Maximum number of indices that each Index node can store.
     */
    private static final int MAX_BUCKET_INDEX_SIZE = NestedBucketStorageSystem.MAX_NODES_PER_BUCKET;

    private static final String BUCKET_RESOURCE_TYPE = "sling:OrderedFolder";

    /**
     * The location in the comma seperated index string where they key is stored.
     */
    public static final int KEY_INDEX = 0;

    /**
     * The comparator used to sort the indices.
     */
    private final Comparator<K> comparator;

    /**
     * Used to generate random numbers for new name when bucket splitting.
     */
    private static final Random random = new Random();

    private final boolean writeMode;
    private final String indexPath;
    private final String ugcBasePath;

    /**
     * A comparator used for comparing index keys of type Long. This comparator uses Object so that it works with a
     * Long and a String.
     */
    public static class LongComparator implements Comparator<Long> {

        @Override
        public int compare(final Long o1, final Long o2) {
			return 0;
		}
    }

    /**
     * A response object for an index search operation.
     */
    private static class FindIndexWrapper {
        private final int indexLoc;
        private final Resource indexResource;
        private final String index;

        private FindIndexWrapper(final int indexLoc, final Resource indexResource, final String index) {
            this.indexLoc = indexLoc;
            this.indexResource = indexResource;
            this.index = index;
        }
    }

    /**
     * A response object for finding the bucket and location within bucket of a given key.
     */
    private static class FindBucketAndLocWrapper {
        private final int bucketLoc;
        private final int indexLoc;

        private FindBucketAndLocWrapper(final int bucketLoc, final int indexLoc) {
            this.bucketLoc = bucketLoc;
            this.indexLoc = indexLoc;
        }
    }

    /**
     * Construct a BaseIndexer.
     * @param basePath base path of component being indexed. The path can be from the content or usergenerated
     *            directory.
     * @param indexPath the relative path from the root index path (derived from basePath) where this index should be
     *            stored.
     * @param comparator Comparator used to sort index.
     * @param writeMode if true, prepare index directory for writing. If false, assume API user will only do index
     *            reading.
     */
    public AbstractBaseIndexer(final ResourceResolver resolver, String basePath, final String indexPath,
        final Comparator<K> comparator, final boolean writeMode) throws RepositoryException {

        this.comparator = comparator;
        // TODO: Need to refactor this so that we can support non-ugc use case
        // Handle UGC case.
        final int ugcIndex = basePath.indexOf(PATH_UGC);
        if (basePath.indexOf(PATH_UGC) != -1) {
            basePath = basePath.substring(ugcIndex + PATH_UGC.length());
        }

        ugcBasePath = writeMode ? IndexUtil.prepareUserGeneratedContent(resolver, basePath) : PATH_UGC + basePath;

        final StringBuilder path = new StringBuilder(ugcBasePath);
        if (!path.toString().endsWith("/")) {
            path.append("/");
        }
        path.append(indexPath);
        rootIndexResourcePath = path.toString();

        this.writeMode = writeMode;
        this.indexPath = indexPath;

    }

    private Resource getRootIndexResource(final ResourceResolver resolver) {
		return null;
	}

    /**
     * Return the IndexCounter for this index.
     * @return the IndexCounter for this index.
     */
    protected IndexCounter getCounter(final ResourceResolver resolver) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public long getSize(final ResourceResolver resolver) {
		return 0L;
	}

    /**
     * Get the storage system
     * @return a new storage system instance
     */
    private NestedBucketStorageSystem getStorageSystem(final ResourceResolver resolver) {
		return null;
	}

    private NestedBucketStorageSystem getStorageSystem(final Resource root) {
		return null;
	}

    /**
     * Return the list of Resource objects representing index buckets for this index.
     * @param indexResource the Resource of the root index folder.
     * @return list of Resource objects representing index buckets for this index.
     */
    private List<Resource> getIndexBuckets(final ResourceResolver resolver, final Resource indexResource) {
		return null;
	}

    /**
     * Create a new Index bucket.
     * @param newNodeName the name of the new bucket.
     * @param numBuckets the number of buckets we currently have.
     * @param counters the current bucket counters.
     * @param index the first index to add to this new bucket.
     * @param rootIndexResource the Resource of the root index folder.
     * @return the new counters array that includes the new index bucket.
     */
    private int[] createIndexBucket(final String newNodeName, final int numBuckets, final int[] counters,
        final String index, final Resource rootIndexResource, final Session session) throws RepositoryException {
			return null;
		}

    /**
     * Add the index to an existing index bucket.
     * @param indexLoc the index of the bucket to add to.
     * @param counters the current bucket counters.
     * @param targetKey the key of the new index.
     * @param index the index to add.
     * @param indexBucketResource the resource of the index we are adding to.
     * @param targetIndex the Index representing the bucket we are adding to.
     */
    private void addIndex(final int indexLoc, final int[] counters, final K targetKey, final String index,
        final Session session, final Resource indexBucketResource, final Index targetIndex)
        throws RepositoryException {
		}

    /**
     * Given the previous and target index bucket name, generate a new name that can be used for the new index bucket
     * that is created after spliting the target bucket.
     * @param prevIndexName the name of the previous index bucket.
     * @param targetIndexName the name of the target index bucket we want to split.
     * @return a new name that can be used for the new index bucket that is created after spliting the target bucket.
     */
    private String getNewSplitIndexName(final String prevIndexName, final String targetIndexName) {
		return null;
	}

    /**
     * Split a given index and return the new bucket counters.
     * @param prevIndexName the name of the previous index bucket.
     * @param targetIndexLoc the location of the bucket we want to split.
     * @param counters the current bucket counters.
     * @param targetKey the key of the new index we want to add.
     * @param index the content of the new index we want to add.
     * @param rootIndexResource the Resource of the root index folder.
     * @param targetResource the Resource of the index bucket we want to split.
     * @param targetIndex the Index object representing the index bucket we want to split.
     * @return the modified counters array that includes the new split index bucket.
     */
    private int[] splitIndex(final String prevIndexName, final int targetIndexLoc, final int[] counters,
        final K targetKey, final String index, final Resource rootIndexResource, final Session session,
        final Resource targetResource, final Index targetIndex) throws RepositoryException {
			return null;
		}

    /**
     * Given list of index bucket resources, return a name that can be used for the next bucket to be added to the
     * list.
     * @param indexNodes The index bucket resources.
     * @return a name that can be used for the next bucket to be added to the list.
     */
    private String getNextNodeName(final List<Resource> indexNodes) {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String index(final ResourceResolver resolver, final E target, final K indexKey, final String indexContent)
        throws RepositoryException {
			return null;
		}

    /**
     * {@inheritDoc}
     */
    @Override
    public String getIndex(final ResourceResolver resolver, final K key, final String endsWith) {
		return null;
	}

    private FindBucketAndLocWrapper findBucketAndLoc(final ResourceResolver resolver, final K key,
        final String endsWith) {
			return null;
		}

    public int getIndexLocForwards(final ResourceResolver resolver, final K key, final String endsWith) {
		return 0;
	}

    public int getIndexLocBackwards(final ResourceResolver resolver, final K key, final String endsWith) {
		return 0;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public String reindex(final ResourceResolver resolver, final E target, final K oldIndexKey, final K indexKey,
        final String indexContent, final boolean force, final String endsWith) throws RepositoryException {
			return null;
		}

    /**
     * Find the best bucket that this key could be added to or already exist in. Return -1 if a new bucket should be
     * created at the end of the list.
     * @param indexBuckets the index bucket resources.
     * @param targetKey the key of the index we are trying to find within or add to a bucket.
     * @return the best bucket that this key could be added to or already exist in. Return -1 if a new bucket should
     *         be created at the end of the list.
     */
    private int findIndexBucket(final List<Resource> indexBuckets, final K targetKey) {
		return 0;
	}

    /**
     * {@inheritDoc}
     */
    public FindIndexWrapper findIndex(final List<Resource> indexNodes, final int indexNodeLoc, final K targetKey,
        final String endsWith) {
			return null;
		}

    /**
     * {@inheritDoc}
     */
    @Override
    public void unindex(final ResourceResolver resolver, final K indexKey, final String endsWith)
        throws RepositoryException {
		}

    /**
     * Given a starting point and max size, traverse the index backwards and return a list of elements.
     * @param start the index of the first element in the list to be returned.
     * @param size the maximum number of elements to return.
     * @param elementRootPath the root path of the elements in this index.
     * @param elementRetriever a helper that retrieves an element of a specific type, such as Comment, Topics, etc.
     * @return the requested list of elements.
     */
    protected List<E> getElementsBackwards(final ResourceResolver resolver, final int start, final int size,
        final String elementRootPath) {
			return null;
		}

    /**
     * Given a starting point and max size, traverse the index backwards and return a list of elements.
     * @param start the index of the first element in the list to be returned.
     * @param size the maximum number of elements to return.
     * @param elementRootPath the root path of the elements in this index.
     * @param resourceElementFactory a resource element factory used to adapt an index element to a specific component
     *            element, such as Comment, Topics, etc.
     * @return the requested list of elements.
     */
    protected <C> List<C> getComponentElementsBackwards(final ResourceResolver resolver, final int start,
        final int size, final String elementRootPath, final IndexElementFactory<C> resourceElementFactory) {
			return null;
		}

    /**
     * Given a starting point and max size, traverse the index forwards and return a list of elements.
     * @param <C>
     * @param start the index of the first element in the list to be returned.
     * @param size the maximum number of elements to return.
     * @param elementRootPath the root path of the elements in this index.
     * @return the requested list of elements.
     */
    protected List<E> getElementsForwards(final ResourceResolver resolver, int start, final int size,
        final String elementRootPath) {
			return null;
		}

    protected <C> List<C> getComponentElementsForwards(final ResourceResolver resolver, int start, final int size,
        final String elementRootPath, final IndexElementFactory<C> resourceElementFactory) {
			return null;
		}

    /**
     * Given an index and elementRootPath, retrieve the element and return it.
     * @param resolver the resource resolver
     * @param elementRootPath the root path of the element.
     * @param index the index content.
     * @return the target element.
     */
    protected abstract E getElementFromIndex(ResourceResolver resolver, String elementRootPath, String index) {
		return null;
	}

    /**
     * Given an index and elementRootPath, retrieve the component element and return it.
     * @param resolver the resource resolver
     * @param elementRootPath the root path of the element.
     * @param index the index content.
     * @param indexElementFactory the index element factory
     * @return the target element.
     */
    protected abstract <C> C getComponentElementFromIndex(ResourceResolver resolver, String elementRootPath,
        String index, IndexElementFactory<C> indexElementFactory) {
		return null;
	}

}
