/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.storage.index;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.sling.api.resource.LoginException;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceResolverFactory;
import org.apache.sling.jcr.api.SlingRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.storage.buckets.NestedBucketStorageSystem;
import com.day.cq.commons.jcr.JcrConstants;

/**
 * Base class for reindexing content.
 */
public abstract class AbstractBaseReindexer {
    private static final Logger log = LoggerFactory.getLogger(AbstractBaseReindexer.class);

    // The number of items we reindex before we log progress.
    private static final int PROGRESS_LOG_COUNT = 100;

    private Thread wThread = null;

    class WorkerThread implements Runnable {
        @Override
        public void run() {
		}
    }

    /**
     * Start a worker thread to perform reindexing so that it happens asynchronously.
     */
    protected void startReindexing() {
	}

    /**
     * Perform all the reindexing work here.
     */
    protected abstract void performReindexing() {
	}

    public abstract boolean checkResource(Resource resource) {
		return false;
	}

    public abstract String getUGCPath(Resource resource) {
		return null;
	}

    public abstract String getTopLevelPrefix() {
		return null;
	}

    public abstract String getLowerLevelPrefix() {
		return null;
	}

    /**
     * Index the node at the given path if it has not been indexed already.
     * @param resolver the ResourceResolver.
     * @param nodePath the path of the node to be indexed.
     * @throws Exception thrown if there are any indexing issues encountered.
     */
    public abstract void indexExec(ResourceResolver resolver, String nodePath) throws Exception {
	}

    protected List<String> findReindexPaths(final ResourceResolverFactory resolverFactory, final String rootPath,
        final String componentPathProp) {
			return null;
		}

    /**
     * Return the size of the index at the given content root path.
     * @param resolver the ResourceResolver.
     * @param contentRootPath the content root path.
     * @return the size of the index at the given content root path.
     * @throws RepositoryException thrown if there are issues with the repository.
     */
    protected abstract long getIndexSize(ResourceResolver resolver, String contentRootPath)
        throws RepositoryException {
		return 0L;
	}

    /**
     * Return true if the size of the index does not match the number of content nodes under the root.
     * @param resolver the ResourceResolver.
     * @param contentRoot the content root to check.
     * @return true if the size of the index does not match the number of content nodes under the root.
     * @throws RepositoryException thrown if repository issue occurs.
     */
    private boolean indexIncomplete(final ResourceResolver resolver, final Resource contentRoot)
        throws RepositoryException {
			return false;
		}

    private void findReindexPaths(final ResourceResolver resolver, final Node folder, final List<String> results,
        final String componentPathProp) throws RepositoryException {
		}

    protected void reindex(final String path, final int throttle, final ResourceResolverFactory resolverFactory,
        final SlingRepository repository) throws Exception {
		}

    private void reindex(final int throttle, int count, final Resource root, final String postfix) throws Exception {
	}
}
