/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.storage.index;

import java.util.List;

import javax.jcr.RepositoryException;

import org.apache.sling.api.resource.ResourceResolver;

/**
 * Interface for class that manages list of Indexers.
 */
public interface IndexManager<K, E> {

    /**
     * Register a new Indexer.
     * @param indexer the Indexer to register.
     */
    void registerIndexer(Indexer<K, E> indexer) {
	}

    /**
     * Return true if an Indexer with the given name is registered.
     * @param name the name of the Indexer to check.
     * @return true if an Indexer with the given name is registered.
     */
    boolean isRegistered(String name) {
		return false;
	}

    /**
     * Given an index name, return the size of that index or -1 if the Indexer is not registered.
     * @param indexName the name of the index.
     * @return the size of that index or -1 if the Indexer is not registered.
     */
    long getSize(String indexName, final ResourceResolver resolver) throws RepositoryException {
		return 0L;
	}

    /**
     * Given an index name, return the number of pages that index contains.
     * @param indexName the name of the index.
     * @param pageSize the size of the page.
     * @return the number of pages this index contains.
     */
    long getNumberOfPages(String indexName, int pageSize, final ResourceResolver resolver) throws RepositoryException {
		return 0L;
	}

    /**
     * Return the list of elements in the specified page number.
     * @param indexName the name of the index.
     * @param pageSize the size of the page.
     * @param pageNumber the page number.
     * @param rootPath the root path of the elements.
     * @return list of elements in the specified page.
     */
    List<E> getPageByNumber(String indexName, int pageSize, final ResourceResolver resolver, int pageNumber,
        String rootPath) {
		return null;
	}

    /**
     * Return the list of elements from the specified offset and size.
     * @param indexName the name of the index.
     * @param pageSize the size of the page.
     * @param offset the offset.
     * @param rootPath the root path of the elements.
     * @return list of elements from the specified offset and size.
     */
    List<E> getPageByOffset(String indexName, int pageSize, final ResourceResolver resolver, int offset,
        String rootPath) {
		return null;
	}

    /**
     * Return the list of elements from the specified offset and size.
     * @param indexName the name of the index.
     * @param pageSize the size of the page.
     * @param resolver the resource resolver.
     * @param offset the offset.
     * @param rootPath the root path of the elements.
     * @param elementConstructor the element construction helper of class type C
     * @return list of elements from the specified offset and size.
     */
    <C> List<C> getIndexElementPageByOffset(String indexName, int pageSize, final ResourceResolver resolver,
        int offset, String rootPath, IndexElementFactory<C> resourceElementFactory) {
		return null;
	}

    /**
     * Index the target element.
     * @param indexName the name of the index.
     * @param keyIndexName the name of the key index associated with this index.
     * @param targetId the id of the object being indexed.
     * @param target the object to index.
     * @param key the index key.
     * @param indexContent the index content.
     */
    void index(String indexName, final ResourceResolver resolver, String keyIndexName, String targetId, E target,
        K key, String indexContent) throws RepositoryException {
	}

    /**
     * Reindex the given target object.
     * @param indexName the name of the index.
     * @param keyIndexName the name of the key index associated with this index.
     * @param targetId the id of the object being indexed.
     * @param target the object to index.
     * @param prevIndexKey the previous index key.
     * @param indexKey the new index key.
     * @param indexContent the new index content.
     * @param force if false, don't reindex the target if the index key has not changed.
     * @param endsWith the string that the index is expected to end with or null if unspecified. This is used to
     *            handles cases where there are multiple indices with the same key value.
     */
    void reindex(String indexName, final ResourceResolver resolver, String keyIndexName, String targetId, E target,
        K prevIndexKey, K indexKey, String indexContent, boolean force, String endsWith) throws RepositoryException {
	}

    /**
     * Unindex the entry with the given key.
     * @param indexName the name of the index.
     * @param keyIndexName the name of the key index associated with this index.
     * @param targetId the id of the object being unindexed.
     * @param key the key of the target index.
     * @param endsWith the string that the index is expected to end with or null if unspecified. This is used to
     *            handles cases where there are multiple indices with the same key value.
     */
    void unindex(String indexName, final ResourceResolver resolver, String keyIndexName, String targetId, K key,
        String endsWith) throws RepositoryException {
	}

    /**
     * Return an index with the given key or null if it does not exist.
     * @param indexName the name of the index.
     * @param key the key of the target index.
     * @param endsWith the string that the index is expected to end with or null if unspecified. This is used to
     *            handles cases where there are multiple indices with the same key value.
     * @return index with the given key or null if it does not exist.
     */
    String getIndex(String indexName, ResourceResolver resolver, K key, String endsWith) {
		return null;
	}

    /**
     * Return the index location for the given key starting at 0. For example, if the key belongs to the 5th object in
     * the index, 4 will be returned. If the key is unindexed, -1 is returned.
     * @param indexName the name of the index.
     * @param key the index key.
     * @param endsWith the string that the index is expected to end with or null if unspecified. This is used to
     *            handles cases where there are multiple indices with the same key value.
     * @return the index location for the given key starting at 0.
     */
    int getIndexLoc(String indexName, ResourceResolver resolver, K key, String endsWith) {
		return 0;
	}

    /**
     * Return the Indexer with the specified name or null if this Indexer is not registered.
     * @param indexName the name of the index.
     * @return the Indexer with the specified name or null if this Indexer is not registered.
     */
    Indexer<K, E> getIndexer(String indexName) {
		return null;
	}

    /**
     * Return the KeyIndexer with the given name.
     * @param indexName the name of the KeyIndex.
     * @return the target KeyIndex or null of it does not exist.
     */
    KeyIndexer<E> getKeyIndexer(String indexName) {
		return null;
	}
}
