/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.storage.index;

import java.util.List;

import javax.jcr.RepositoryException;

import org.apache.sling.api.resource.ResourceResolver;

/**
 * Interface for reading and writing to an index.
 */
public interface Indexer<K, E> {

    /**
     * The path to the user-generated content.
     */
    public static final String PATH_UGC = "/content/usergenerated";

    /**
     * Return the unique name of this index. This would be used as an id when storing an Indexer in an IndexManager.
     * @return the name of this index.
     */
    String getName() {
		return null;
	}

    /**
     * Return an index with the given key or null if it does not exist.
     * @param key the key of the target index.
     * @param endsWith the string that the index is expected to end with or null if unspecified. This is used to
     *            handles cases where there are multiple indices with the same key value.
     * @return index with the given key or null if it does not exist.
     */
    String getIndex(ResourceResolver resolver, K key, String endsWith) {
		return null;
	}

    /**
     * Return the index location for the given key starting at 0. For example, if the key belongs to the 5th object in
     * the index, 4 will be returned. If the key is unindexed, -1 is returned.
     * @param key the index key.
     * @param endsWith the string that the index is expected to end with or null if unspecified. This is used to
     *            handles cases where there are multiple indices with the same key value.
     * @return the index location for the given key starting at 0.
     */
    int getIndexLoc(ResourceResolver resolver, K key, String endsWith) {
		return 0;
	}

    /**
     * Using the indexed relative paths, return a list of elements.
     * @param start The location in the index where reading should start.
     * @param size The maximum number of elements to return.
     * @param rootPath The root path where the elements are located. This is prepended to each indexed relative path
     *            to retrieve the element.
     * @return the list of elements.
     */
    List<E> getElements(final ResourceResolver resolver, int start, int size, String rootPath) {
		return null;
	}

    /**
     * Retrieve the indexed relative paths, return a list of element of the type supported by the specified
     * {@link IndexElementFactory}.
     * @param start The location in the index where reading should start.
     * @param size The maximum number of elements to return.
     * @param rootPath The root path where the elements are located. This is prepended to each indexed relative path
     *            to retrieve the element.
     * @param indexElementFactory The index element factory for a specific type of element.
     * @return the list of elements.
     */
    <C> List<C> getIndexElements(final ResourceResolver resolver, int start, int size, String rootPath,
        IndexElementFactory<C> indexElementFactory) {
		return null;
	}

    /**
     * Unindex the entry with the given key.
     * @param key the key of the target index.
     * @param endsWith the string that the index is expected to end with or null if unspecified. This is used to
     *            handles cases where there are multiple indices with the same key value.
     */
    void unindex(final ResourceResolver resolver, K key, String endsWith) throws RepositoryException {
	}

    /**
     * Index the given target object.
     * @param target the object to index.
     * @param key the index key.
     * @param indexContent the index content.
     * @return index info about where the target object was indexed. This can be used as the indexContent for a key
     *         index and later used to unindex this target.
     */
    String index(final ResourceResolver resolver, E target, K key, String indexContent) throws RepositoryException {
		return null;
	}

    /**
     * Reindex the given target object.
     * @param target the object to index.
     * @param oldIndexKey the previous index key.
     * @param indexKey the new index key.
     * @param indexContent the new index content.
     * @param force if false, don't reindex the target if the index key has not changed.
     * @param endsWith the string that the index is expected to end with or null if unspecified. This is used to
     *            handles cases where there are multiple indices with the same key value.
     * @return index info about where the target object was indexed. This can be used as the indexContent for a key
     *         index and later used to unindex this target.
     */
    String reindex(final ResourceResolver resolver, E target, K oldIndexKey, K indexKey, String indexContent,
        boolean force, String endsWith) throws RepositoryException {
		return null;
	}

    /**
     * Return the number of elements in this index.
     * @return the number of elements in this index.
     */
    long getSize(final ResourceResolver resolver) {
		return 0L;
	}

    /**
     * Given a string representation of a key, convert it to type K and return it.
     * @return given a string representation of a key, convert it to type K and return it.
     */
    K stringToK(String key) {
		return null;
	}
}
