/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.cq.social.ugcbase.events;

import java.io.Serializable;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Map;
import java.util.Map.Entry;

import org.osgi.service.event.Event;

import com.adobe.cq.social.ugcbase.events.SocialEvent.SocialActions;

/**
 * @author palanisw A generic class to define a Social Event and provide the basic framework and data exposed by
 *         events triggered by social actions. This class can be subtyped to define specific social events.
 * @param <T> type representing the actions supported by the SocialEvent
 */
public abstract class SocialEvent<T extends SocialActions> extends Event implements Serializable {

    private static final long serialVersionUID = 1L;

    protected static final String USER_ID = "userId";
    protected static final String ACTION = "action";
    protected static final String PATH = "path";

    /**
     * All {@link Event} instances created using SocialEvents will have an event topic starting with this string.
     */
    public static final String SOCIAL_EVENT_TOPIC_PREFIX = "com/adobe/cq/social/";

    /**
     * @author palanisw an empty interface to make sure that all Social Actions are strongly typed.
     */
    public interface SocialActions {

    }

    /**
     * This constructor should be used to create a Social Event given an OSGi Event.
     * @param event an OSGi event that is known to be a social event
     */
    public SocialEvent(final Event event) {
        super(event.getTopic(), buildPropertiesFromEvent(event));
    }

    private static Dictionary<String, Object> buildPropertiesFromEvent(final Event event) {
		return null;
	}

    /**
     * @param topic - the topic string for the event.
     * @param path - the jcr node path where the event occurred.
     * @param userId - the user id of the user who triggered the event.
     * @param action - the action that was taken to trigger the event.
     * @param additionalData - a key value pair of any other data that should be part of the event payload.
     */
    public SocialEvent(final String topic, final String path, final String userId, final T action,
        final Map<String, Object> additionalData) {
        super(SOCIAL_EVENT_TOPIC_PREFIX + topic, buildProps(path, userId, action, additionalData));
    }

    private static Dictionary<String, Object> buildProps(final String path, final String userId,
        final SocialActions action, final Map<String, Object> additionalData) {
			return null;
		}

    /**
     * @return the user id of the user who triggered the event
     */
    public String getUserId() {
		return null;
	}

    /**
     * @return the action that triggered this event.
     */
    @SuppressWarnings("unchecked")
    public T getAction() {
		return null;
	}

    /**
     * @return the jcr node path where this event occurred.
     */
    public String getPath() {
		return null;
	}

}
