/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.auth.saml.util;

import java.security.Key;
import java.security.cert.Certificate;

/**
 * KeyProvider is an interface which abstracts away the details of retrieving a private and public 
 * key pair. The private key must be capable of decrypting messages encrypted by using the public key.
 * 
 * @author jasonhightower
 */
public interface KeyProvider {
	
	/**
	 * Retrieves the private key which can be used to decrypt messages encrypted by the public key.
	 * @return Key which can be used to decrypt messages encrypted by the public key.
	 */
	Key getDecryptionKey() {
		return null;
	}
	
	/**
	 * Retrieves the certificate which contains a public key.
	 * @return Certificate object containing a public key.
	 */
	Certificate getPublicCertificate() {
		return null;
	}
	
}
