/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.auth.saml.util;

import java.io.OutputStream;
import java.security.InvalidAlgorithmParameterException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.util.Collections;
import java.util.LinkedList;

import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dsig.CanonicalizationMethod;
import javax.xml.crypto.dsig.DigestMethod;
import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.SignedInfo;
import javax.xml.crypto.dsig.Transform;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
import javax.xml.crypto.dsig.spec.TransformParameterSpec;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.xml.security.transforms.Transforms;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.adobe.granite.auth.saml.model.xml.SamlXmlConstants;
import com.adobe.granite.auth.saml.model.AbstractRequest;
import com.adobe.granite.auth.saml.model.AuthnRequest;
import com.adobe.granite.auth.saml.model.Issuer;
import com.adobe.granite.auth.saml.model.LogoutRequest;
import com.adobe.granite.auth.saml.model.Message;
import com.adobe.granite.auth.saml.model.NameIdPolicy;

/**
 * SamlWriter is responsible for writing saml data as xml.
 */
public class SamlWriter {

    private final static String DIGEST_METHOD = DigestMethod.SHA256;
    private final static String SIGNATURE_METHOD = SignatureMethod.RSA_SHA1;

    /**
     * DocumentBuilderFactory used to construct DocumentBuilder instances.
     */
    private DocumentBuilderFactory builderFactory;

    /**
     * Creates a new instance of SamlWriter.
     */
    public SamlWriter() {
        super();
        this.builderFactory = DocumentBuilderFactory.newInstance();
        this.builderFactory.setNamespaceAware(true);
    }

    /**
     * Marshals the given AuthnRequest as an xml document and writes it to the given output stream.
     *
     * @param message    AuthnRequest to marshal.
     * @param privateKey The private Key of this SP to sign the authentication request (may be null)
     * @param out        OutputStream to write the xml document to.
     */
    public void write(final Message message, final OutputStream out, final Key privateKey) throws SamlWriterException {
	}

    // -- implementation methods --

    /**
     * Creates a request Document from a given request element.
     *
     * @param request    AuthnRequest to create the document for.
     * @param privateKey The private Key of this SP to sign the authentication request
     * @return Document containing xml data created from the request.
     */
    protected Document createRequestDocument(final AuthnRequest request, final Key privateKey) throws SamlWriterException {
		return null;
	}

    /**
     * Creates a request Document from a given request element.
     *
     * @param request    LogoutRequest to create the document for.
     * @param privateKey The private Key of this SP to sign the authentication request
     * @return Document containing xml data created from the request.
     */
    protected Document createRequestDocument(final LogoutRequest request, final Key privateKey) throws SamlWriterException {
		return null;
	}

    /**
     * Creates an AuthnRequest element from the given AuthnRequest and adds it to the given parent node.
     *
     * @param authnRequest AuthnRequest to create the element for.
     * @param parent       Parent node to add the created element to.
     * @param privateKey The private Key of this SP to sign the authentication request
     */
    protected void createAuthnRequestElement(final AuthnRequest authnRequest, final Node parent, final Key privateKey) throws SamlWriterException {
	}

    /**
     * Creates an xml element for the given Issuer.
     *
     * @param issuer Issuer to create the element for.
     * @param parent Node to add the element to.
     */
    protected void createIssuerElement(final Issuer issuer, final Node parent) {
	}

    /**
     * Creates an xml element for the given NameIdPolicy.
     *
     * @param nameIdPolicy NameIdPolicy to create the xml element for.
     * @param parent       Node the created element is added to.
     * @return The NameIDPolicy Node
     */
    protected Node createNameIdPolicyElement(final NameIdPolicy nameIdPolicy, final Node parent) {
		return null;
	}

    /**
     * Adds attributes and sub-elements common to all AbstractRequest instances to the given request element.
     *
     * @param requestElement Request element to add data to.
     * @param request        AbstractRequest instance which the requestElement should be modified for.
     */
    protected void handleAbstractRequest(final Element requestElement, final AbstractRequest request) {
	}

    protected void createLogoutRequestElement(LogoutRequest request, Node parent, Key privateKey) throws SamlWriterException {
	}

    protected void signDocument(final Document ownerDocument, final Node placeSignatureBefore,
            final Key privateKey, final String messageId) throws SamlWriterException {
			}
}
