/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.granite.translation.api;

public final class TranslationConstants {

    private TranslationConstants() {
        /**
         * This prevents even the native class from calling this constructor as well.
         */
        throw new AssertionError();
    }

    /**
     * Type of content data (plain text, html, etc...)
     */
    public static enum ContentType {

        PLAIN("text/plain"),
        HTML("text/html");

        private String type;

        private ContentType(String type) {
            this.type = type;
        }

        public String getType() {
            return this.type;
        }

    }

    /**
     * Values below represent specific functionality which the translation providers may provide.
     * Used with the TranslationService.isServiceAvailable(String) function.
     */
    public static enum ServiceType {

        TEXT_TRANSLATION, HTML_TRANSLATION, DETECTION, TRANSLATION_MEMORY

    }

    /**
     * Strings below represent the possible status codes for a TranslationService implementation
     */
    public static enum ServiceStatus {

        UNKNOWN_STATUS("Unknown"),
        AUTHENTICATED("Authenticated"),
        AVAILABLE("Available"),
        EXPIRED("Expired"),
        UNAVAILABLE("Unavailable");

        private String status;

        private ServiceStatus(String status) {
            this.status = status;
        }

        public String getStatus() {
            return this.status;
        }

    }

}