/*************************************************************************
*
* ADOBE CONFIDENTIAL
* __________________
*
*  Copyright 2014 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Adobe Systems Incorporated and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Adobe Systems Incorporated and its
* suppliers and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Adobe Systems Incorporated.
**************************************************************************/

package com.adobe.granite.translation.api;

/*
 * The TranslationException class extends the Exception class and adds an ErrorCode enum.  This ErrorCode contains
 * the specific type of failure that occurred within the Translation Framework API call.  Please see the ErrorCode
 * section for details on specific ErrorCode types.
 */
public class TranslationException extends Exception
{

    private ErrorCode errorCode = null;
    private String message = null;

    public TranslationException(String message, ErrorCode errorCode)
    {
        super(message);
        this.message = message;
        this.errorCode = errorCode;
    }

    public TranslationException(String message, Throwable throwable, ErrorCode errorCode)
    {
        super(message, throwable);
        this.message = message;
        this.errorCode = errorCode;
    }

    /*
     * This method overrides the base Exception toString method and outputs the ErrorCode in conjunction with the
     * Exception message.
     */
    @Override
    public String toString() {
		return null;
	}

    @Override
    public String getMessage() {
		return null;
	}

    public ErrorCode getErrorCode(){
		return null;
	}

    /*
     * This enum provides specific error codes for handling and throwing exceptions in the Translation API.  It is
     * expected that Connectors implementing the Translation API leverage these constants when specific exceptions
     * occur.  If an exception is of an unknown type the GENERAL_EXCEPTION should be thrown.
     */
    public enum ErrorCode {

        /*
         * Thrown for all exceptions that do not fit other exception error code types
         */
        GENERAL_EXCEPTION,

        /*
         * An exception occurred during translation
         */
        TRANSLATION_FAILED,

        /*
         * The request to the translation engine failed
         */
        REQUEST_FAILED,

        /*
         * The MT engine specified by this Connector is unavailable/unknown
         */
        NO_ENGINE,

        /*
         * The content format specified is not supported by this MT engine
         */
        NOT_SUPPORTED_FORMAT,

        /*
         * The request has timed out
         */
        REQUEST_TIMEOUT,

        /*
         * Parameters necessary for this functionality are missing
         */
        MISSING_PARAMETER,

        /*
         * The request to the translation engine was interrupted
         */
        REQUEST_INTERRUPT,

        /*
         * The MT engine is temporarily unavailable
         */
        ENGINE_TEMP_UNAVAILABLE,

        /*
         * To be used for exceptions while detecting a language of a string.
         */
        DETECTION_FAILURE,

        /*
         * Thrown if a call to the translation function has the same to and from language.
         */
        TO_FROM_SAME_LANGUAGE,

        /*
         * Returned when a language cannot be determined by the "detect" function
         */
        UNKNOWN_LANGUAGE,

        /*
         * Thrown when the functionality called is not implemented in this connector
         */
        SERVICE_NOT_IMPLEMENTED,

        /*
         * Thrown when the connector doesn't support this language translation direction
         */
        NOT_SUPPORTED_LANG_DIRECTION,

        /*
         * Language code passed in is not in a valid format or not part of the supported list
         */
        INVALID_LANGUAGE,

        /*
         * There are no registered Translation Service Factories in this system
         */
        NO_REGISTERED_FACTORIES,

        /*
         * There are no registered Translation Service Factories for this name
         */
        UNKNOWN_FACTORY_NAME,

        /*
         * The Cloud Configuration credentials are blank or missing
         */
        MISSING_CREDENTIALS,

        /*
         * Language code passed in is "valid" but not supported by this current connector
         */
        UNSUPPORTED_LANGUAGE,

        /*
         * Current credentials are not authorized for this functionality
         */
        NOT_AUTHORIZED

    }
}