/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.translation.api;

import org.apache.sling.api.resource.Resource;

import java.util.Map;

/**
 * The TranslationManager maintains a list of installed TranslationServiceFactory instances within the system.
 * These are registered with the Manager at activation and must contain a unique "FactoryName" identifier.  The
 * TranslationManager is obtained via the normal OSGi service registry.  This service creates TranslationService
 * instances using the respective factories via one of the "createTranslationService()" methods below.
 */
public interface TranslationManager {

    /**
     * Creates a <code>TranslationService</code> instance registered with the given translation factory
     * name.
     *
     * TranslationManager returns the TranslationService instance
     *
     * From within a JSP for example:
     * TranslationManager tm = sling.getService(TranslationManager.class);
     * TranslationService translationService = tm.createTranslationService(resource);
     *
     * Or from within a Component/Service you would import the TranslationManager Object:
     * @Reference
     * private TranslationManager tm;
     *
     * @param factoryName the translation factory name
     * @return the <code>TranslationService</code> instance
     */
    TranslationService createTranslationService(String factoryName) throws TranslationException {
		return null;
	}

    /**
     * Creates a <code>TranslationService</code> instance based upon logic determined by the properties of
     * the passed in resource.  The Translation configuration logic is used to determine which properties to base
     * the decision off of (Content Path, Content Category, Provider Name stored on content).  By default a
     * configured factory name (TranslationConstants.TN_FACTORY_NAME) takes precedence.  If no Connector can be
     * determined the system default TranslationService will be returned.
     *
     * @param resource The resource of a content node containing the content to be translated.
     * @return the <code>TranslationService</code> instance
     */
    TranslationService createTranslationService(Resource resource) throws TranslationException {
		return null;
	}

    /**
     * Returns a Map of the currently available TranslationServiceFactories
     *
     * The service returns both the factory name and a factory label for use in drop downs.
     * This saves us from having to make a second call (per factory) to getLabel();
     * As noted: the factory names (the ID) is the KEY and the factory label is the VALUE.
     * This is the value that can be shown when factory will show up in UI.
     *
     * The return is a MAP<NAME, LABEL> of factories within the system.
     *
     * @return Map<String, String> containing currently installed factories (<Factory Name, Factory Label>)
     */
    Map<String, String> getAvailableFactoryNames() throws TranslationException {
		return null;
	}
}