/*************************************************************************
*
* ADOBE CONFIDENTIAL
* __________________
*
*  Copyright 2014 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Adobe Systems Incorporated and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Adobe Systems Incorporated and its
* suppliers and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Adobe Systems Incorporated.
**************************************************************************/

package com.adobe.granite.translation.core.common;

import com.adobe.granite.translation.api.TranslationConfig;
import com.adobe.granite.translation.api.TranslationException;
import com.adobe.granite.translation.api.TranslationService;
import org.apache.commons.lang.StringUtils;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;

public abstract class AbstractTranslationService implements TranslationService {

    private static final Logger log = LoggerFactory.getLogger(AbstractTranslationService.class);

    protected ResourceResolver resolver;

    protected TranslationServiceInfo translationInfo;

    protected TranslationConfig tc;

    protected String defaultCategory;

    /**
     * Object containing list of supported language codes and their appropriate mappings.
     * This is a one to one mapping list in which the "key" is the accepted language code and corresponds to a node
     * name from /libs/wcm/core/resources/languages.  The "value" is the corresponding mapping for the current
     * connector engine.
     *
     * Example:  map.put("en_us", "eng")  --> "en_us" corresponds to /libs/wcm/core/resources/languages/en_us and
     * "eng corresponds to the expected value in this MT engine for "English"
     */
    protected Map<String, String> availableLanguageMap;
    protected Map<String, String> availableCategoryMap;

	protected AbstractTranslationService(Resource resource,
			Map<String, String> availableLanguageMap,
			Map<String, String> availableCategoryMap, String name,
			String label, String attribution, TranslationConfig tc) {
        log.trace("Starting Constructor for: AbstractTranslationService");

        if(resource != null){
            this.resolver = resource.getResourceResolver();
        }
        else{
            log.debug("Resource passed via constructor call was null.");
            this.resolver = null;
        }
        this.tc = tc;
        this.availableLanguageMap = availableLanguageMap;
        this.availableCategoryMap = availableCategoryMap;
        translationInfo = new TranslationServiceInfoImpl(name, label, attribution);
    }

    @Override
    public TranslationServiceInfo getTranslationServiceInfo(){
		return null;
	}

    @Override
    public String getDefaultCategory(){
		return null;
	}

    @Override
    public void setDefaultCategory(String defaultCategory){
	}

    protected boolean validateLanguageCode(String langCode) throws TranslationException{
		return false;
	}

    public class TranslationServiceInfoImpl implements TranslationServiceInfo{

        private String attribution;
        private String translationServiceName;
        private String translationServiceLabel;

        TranslationServiceInfoImpl(String name, String label, String attribution){
            this.attribution = attribution;
            this.translationServiceName = name;
            this.translationServiceLabel = label;
        }

        @Override
        public String getTranslationServiceAttribution(){
			return null;
		}

        @Override
        public String getTranslationServiceLabel(){
			return null;
		}

        @Override
        public String getTranslationServiceName(){
			return null;
		}
    }



}