/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.ui.components;

import java.io.IOException;
import java.util.Map.Entry;

import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.jsp.PageContext;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.request.RequestDispatcherOptions;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.scripting.SlingBindings;
import org.apache.sling.api.servlets.ServletResolver;
import org.apache.sling.jcr.resource.JcrResourceConstants;
import org.apache.sling.scripting.jsp.util.JspSlingHttpServletResponseWrapper;

import com.adobe.granite.ui.components.ds.DataSource;
import com.adobe.granite.ui.components.ds.EmptyDataSource;
import com.adobe.granite.ui.components.ds.ResourceDataSource;
import com.adobe.granite.ui.components.rendercondition.RenderCondition;
import com.adobe.granite.ui.components.rendercondition.SimpleRenderCondition;
import com.adobe.granite.xss.XSSAPI;
import com.day.cq.i18n.I18n;

/**
 * A convenient helper for development of Granite UI component.
 */
public class ComponentHelper {
    private static final String DEFAULT_LAYOUT_RT = "granite/ui/components/foundation/layouts/container";
    
    private PageContext pageContext;
    
    private SlingBindings bindings;

    private SlingHttpServletRequest request;

    private I18n i18n;

    private XSSAPI xss;

    private Config config;
    
    private Value value;

    private ExpressionHelper ex;
    
    private State state;

    private OptionsHolder optionsHolder;

    public ComponentHelper(PageContext pageContext) {
        this.pageContext = pageContext;

        this.bindings = (SlingBindings) pageContext.getRequest().getAttribute(SlingBindings.class.getName());

        request = bindings.getRequest();
        i18n = new I18n(request);
        xss = bindings.getSling().getService(XSSAPI.class).getRequestSpecificAPI(request);
        config = new Config(request.getResource());
        value = new Value(request, config);
        ex = new ExpressionHelper(bindings.getSling().getService(ExpressionResolver.class), pageContext);
        state = new State(request);

        optionsHolder = (OptionsHolder) request.getAttribute(OptionsHolder.class.getName());
        request.removeAttribute(OptionsHolder.class.getName());
    }

    /**
     * Returns I18n appropriate for the current page.
     */
    public I18n getI18n() {
		return null;
	}

    /**
     * Returns XSSAPI based on the current request.
     */
    public XSSAPI getXss() {
		return null;
	}

    /**
     * Returns the config of current resource of the page.
     */
    public Config getConfig() {
		return null;
	}
    
    /**
     * Returns the values that is applicable for the current page.
     */
    public Value getValue() {
		return null;
	}

    /**
     * Returns the ExpressionHelper appropriate for the current page.
     */
    public ExpressionHelper getExpressionHelper() {
		return null;
	}
    
    /**
     * Returns the client state.
     */
    public State getState() {
		return null;
	}

    /**
     * Consumes the current available tag for current page. If the request
     * doesn't have the tag applicable to the page, a new tag is created.
     * <p>
     * There is a mechanism such that a tag can be passed to another page when
     * including that page. This method is intended as a way to consume the tag
     * passed by other page. Component developer can make use this method to get
     * the main tag of the component regardless if there is a tag passed by
     * other page or not.
     * 
     * @see #include(Resource, Tag)
     * @see #include(Resource, String, Tag)
     */
    public Tag consumeTag() {
		return null;
	}
    
    /**
     * Consumes the current available layout resource for current page. This
     * method first attempts to return the layout resource from the options,
     * second it will attempt to return {@link Config#LAYOUT} child node,
     * otherwise <code>null</code> is returned.
     * <p>
     * This method is usually called by layout implementation to get the layout
     * resource that can be passed by the caller.
     * 
     * @see #includeForLayout(Resource, Options)
     * @see #includeForLayout(Resource, Resource, Options)
     */
    public Resource consumeLayoutResource() {
		return null;
	}

    public void populateCommonAttrs(AttrBuilder attrs) {
	}
    
    /**
     * Returns the options passed from another page. If no options is passed,
     * empty options is returned.
     * <p>
     * There is a mechanism such that options can be passed to another page when
     * including that page.
     * 
     * @see #include(Resource, Options)
     * @see #include(Resource, String, Options)
     */
    public Options getOptions() {
		return null;
	}

    /**
     * Returns the layout config of current resource of the page. This method is
     * setting the default resource type of the layout.
     * 
     * @see LayoutBuilder#getResourceType()
     */
    public LayoutBuilder getLayout() {
		return null;
	}
    
    /**
     * Returns the associated resource type of current resource for the purpose rendering read only version.
     * First the granite:readOnlyResourceType property of the resource type of the current resource (the RT) is used.
     * Otherwise it is defaulted to <code>readonly</code> child resource of the RT.
     */
    public String getReadOnlyResourceType() {
		return null;
	}
    
    /**
     * Returns the associated resource type of the given content resource for the purpose rendering read only version.
     * First the granite:readOnlyResourceType property of the resource type of the content resource (the RT) is used.
     * Otherwise it is defaulted to <code>readonly</code> child resource of the RT.
     */
    public String getReadOnlyResourceType(Resource resource) {
		return null;
	}

    /**
     * Returns the datasource for items of the current resource.
     */
    public DataSource getItemDataSource() throws ServletException, IOException {
		return null;
	}
    
    /**
     * Returns the datasource for items of the given resource.
     */
    public DataSource getItemDataSource(Resource resource) throws ServletException, IOException {
		return null;
	}

    /**
     * Returns the render condition of the current resource.
     */
    public RenderCondition getRenderCondition() throws ServletException, IOException {
		return null;
	}
    
    /**
     * Returns the render condition of the given resource.
     */
    public RenderCondition getRenderCondition(Resource resource) throws ServletException, IOException {
		return null;
	}

    /**
     * Fetches data via include of the given type.
     */
    @SuppressWarnings("unchecked")
    private <T> T fetchData(Resource resource, String resourceType, Class<T> type) throws ServletException, IOException {
		return null;
	}
    
    /**
     * Returns the icon class(es) for the given icon string from the content property.
     */
    public String getIconClass(String icon) {
		return null;
	}

    private String toCamel(String s) {
		return null;
	}
    
    private String getResourceType(Resource resource) {
		return null;
	}

    /**
     * Includes the given resource and passes the given tag to its renderer.
     * This method performs similarly to &lt;sling:include resource="" /&gt;.
     */
    public void include(Resource resource, Tag tag) throws ServletException, IOException {
	}

    /**
     * Includes the given resource and passes the given options to its renderer.
     * This method performs similarly to &lt;sling:include resource="" /&gt;.
     */
    public void include(Resource resource, Options options) throws ServletException, IOException {
	}

    /**
     * Includes the given resource with the given resourceType and passes the
     * given tag to its renderer. This method performs similarly to
     * &lt;sling:include resource="" resourceType="" /&gt;.
     */
    public void include(Resource resource, String resourceType, Tag tag) throws ServletException, IOException {
	}

    /**
     * Includes the given resource with the given resourceType and passes the
     * given options to its renderer. This method performs similarly to
     * &lt;sling:include resource="" resourceType="" /&gt;.
     */
    public void include(Resource resource, String resourceType, Options options) throws ServletException, IOException {
	}
    
    /**
     * A convenient overload to
     * {@link #includeForLayout(Resource, Resource, Options)}, with
     * layoutResource as <code>null</code>.
     */
    public void includeForLayout(Resource resource, Options options) throws ServletException, IOException {
	}
    
    /**
     * Includes the given resource to be rendered by the given layoutResource.
     * This method is used by a component to delegate the rendering process to a
     * layout.
     * <p>
     * If layoutResource is not <code>null</code>, the
     * {@link Options#layoutResource(Resource)} is set.
     * <p>
     * This method will attempt to derive the resourceType to be passed to
     * {@link #include(Resource, String, Options)} based the following
     * priorities:
     * <ol>
     * <li>layoutResource is not null, the resourceType is layoutResource's RT</li>
     * <li>layoutResource is null, the resourceType is {@link Config#LAYOUT}
     * child node's RT</li>
     * <li>the resourceType is default layout as a catch-all fallback</li>
     * </ol>
     */
    public void includeForLayout(Resource resource, Resource layoutResource, Options options) throws ServletException, IOException {
	}

    /**
     * Calls the given script and passes the given options to its renderer. This
     * method performs similarly to &lt;sling:call script="" /&gt;.
     */
    public void call(String script, Options options) throws ServletException, IOException {
	}

    private class OptionsHolder {
        private Options options;

        private String path;

        public OptionsHolder(Options options, String path) {
            this.options = options;
            this.path = path;
        }

        public Options getOptions() {
			return null;
		}

        public String getPath() {
			return null;
		}
    }

    /**
     * An options to be passed to the included resource's renderer.
     */
    public static class Options {
        private Tag tag;
        private boolean rootField = true;
        private Resource layout;

        /**
         * Creates a new instance.
         */
        public Options() {
        }

        /**
         * Returns the tag.
         */
        public Tag tag() {
			return null;
		}

        /**
         * Sets the tag.
         */
        public Options tag(Tag tag) {
			return null;
		}

        /**
         * Returns <code>true</code> is the renderer (the field) should render
         * itself as root field. See {@link #rootField(boolean)} for details.
         */
        public boolean rootField() {
			return false;
		}

        /**
         * Sets <code>true</code> to make the renderer (the field) should render
         * itself as root field; <code>false</code> otherwise. A root field is a
         * field that acts in its own context, instead of as part of a composite
         * field. For example, sizing field consists of weight and height
         * fields. So sizing field is a composite field and wants to leverage
         * the existing number field for width and height. In this case when
         * sizing field is including (
         * {@link ComponentHelper#include(Resource, Options)}) the number field,
         * it should set this option as <code>false</code>.
         * 
         * <p>
         * The field implementation is free to interpret the exact behaviour of
         * root/nont-root field. Most likely scenario, the root field will
         * handle it own sizing state (e.g. inline-block/block state), while non
         * root field will not, where the parent composite field is managing it.
         */
        public Options rootField(boolean flag) {
			return null;
		}
        
        /**
         * Returns the layout resource.
         */
        public Resource layoutResource() {
			return null;
		}
        
        /**
         * Sets the layout resource.
         */
        public Options layoutResource(Resource r) {
			return null;
		}
    }
}
