/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.granite.ui.components;

import java.util.Collections;
import java.util.Iterator;

import org.apache.jackrabbit.JcrConstants;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.api.wrappers.CompositeValueMap;

/**
 * Read config properties of a resource
 */
public class Config {

    /**
     * The resource
     */
    private Resource resource;

    /**
     * The parent of the resource
     */
    private Resource parentResource;

    /**
     * The config properties of the resource
     */
    private ValueMap properties;

    /**
     * The properties inherited from the parent resource
     */
    private ValueMap parentProperties;

    /**
     * The path of the defaults relative to the path of the parent ("default")
     */
    public static String DEFAULTS = "defaults";

    /**
     * The default name of the resource's container containing the primary child resources ("items")
     */
    public static String ITEMS = "items";

    /**
     * The name of the resource's node that is determining its layout ("layout")
     */
    public static String LAYOUT = "layout";
    
    /**
     * The name of the resource's node that is a config for datasource ("datasource")
     */
    public static String DATASOURCE = "datasource";
    
    /**
     * The name of the resource's node that is a config for render condition ("rendercondition")
     */
    public static String RENDERCONDITION = "rendercondition";

    /**
     * Create a new Config object for the given resource.
     * @param resource The resource
     */
    public Config(Resource resource) {
        this.resource = resource;
        properties = ResourceUtil.getValueMap(resource);
    }

    /**
     * Create a new Config object for the given resource. If <code>buildAll</code> is <code>true</code>
     * {@link #parentProperties} are provided.
     * @param resource The resource
     * @param buildAll <code>true</code> to provide inherited properties
     */
    public Config(Resource resource, boolean buildAll) {
        this.resource = resource;
        properties = ResourceUtil.getValueMap(resource);
        if (buildAll) {
            parentResource = getParentResource();
            if (parentResource != null) {
                parentProperties = ResourceUtil.getValueMap(parentResource);
                properties = new CompositeValueMap(properties, ResourceUtil.getValueMap(parentResource.getChild(DEFAULTS)));
            }
        }
    }

    /**
     * Create a new Config object for the given resource.
     * @param resource The resource
     * @param defaultProperties The default properties. Pass <code>null</code> if these are not required later on.
     * @param parentProperties The properties of the parent resource. Pass <code>null</code> if these are not required later on.
     */
    public Config(Resource resource, ValueMap defaultProperties, ValueMap parentProperties) {
        this.resource = resource;
        properties = new CompositeValueMap(ResourceUtil.getValueMap(resource), defaultProperties);
        this.parentProperties = parentProperties;
    }


    /**
     * Get a named config property of the resource converted to a string.
     * @param name The name of the property
     * @return Return named value converted to a string or an empty string if not existing.
     */
    public String get(String name) {
		return null;
	}

    /**
     * Get a named config property of the resource and convert it into the given type.
     * @param name The name of the property
     * @param defaultValue The default value to use if the named property does not exist or cannot be converted to the
     *        requested type. The default value is also used to define the type to convert the value to. If this is <code>null</code>
     *         any existing property is not converted.
     * @return Return named value converted to type T or the default value if non existing or can't be converted.
     */
    public <T> T get(String name, T defaultValue) {
		return null;
	}

    /**
     * Get a named config property of the resource and convert it into the given type.
     * @param name The name of the property
     * @param type The class of the type
     * @return Return named value converted to type T or <code>null</code> if non existing or can't be converted.
     */
    public <T> T get(String name, Class<T> type) {
		return null;
	}


    // getInherited

    /**
     * Get a named config property of the resource converted to a string. If the property does not exist the config
     * property of the parent resource is taken.
     * @param name The name of the property
     * @return Return named value of the resource or its parent converted to a string or an empty string if not existing.
     */
    public String getInherited(String name) {
		return null;
	}

    /**
     * Get a named config property of the resource converted into the given type. If the property does not exist the
     * config property of the parent resource is taken.
     * @param name The name of the property
     * @param defaultValue The default value to use if the named property does not exist or cannot be converted to the
     *        requested type. The default value is also used to define the type to convert the value to. If this is <code>null</code>
     *         any existing property is not converted.
     * @return Return named value of the resource or its parent converted to type T or the default value if non existing
     *         or can't be converted.
     */
    public <T> T getInherited(String name, T defaultValue) {
		return null;
	}

    /**
     * Get a named config property of the resource converted into the given type. If the property does not exist the
     * config property of the parent resource is taken.
     * @param name The name of the property
     * @param type The class of the type
     * @return Return named value of the resource or its parent converted to type T or <code>null</code> if non existing or can't be
     *         converted.
     */
    public <T> T getInherited(String name, Class<T> type) {
		return null;
	}


    // getInheritedDefault

    /**
     * Get a named config property of the resource converted to a string. If the property does not exist it is taken
     * from the defaults of the parent.
     * @param name The name of the property
     * @return Return named value of the resource or its parent's defaults converted to a string or an empty string if
     *         not existing.
     */
    public String getInheritedDefault(String name) {
		return null;
	}

    /**
     * Get a named config property of the resource converted into the given type. If the property does not exist it
     * is taken from the defaults of the parent.
     * @param name The name of the property
     * @param defaultValue The default value to use if the named property does not exist or cannot be converted to the
     *        requested type. The default value is also used to define the type to convert the value to. If this is <code>null</code>
     *         any existing property is not converted.
     * @return Return named value of the resource or its parent's defaults converted to type T or the default value if
     *         non existing or can't be converted.
     */
    public <T> T getInheritedDefault(String name, T defaultValue) {
		return null;
	}

    /**
     * Get a named config property of the resource converted into the given type. If the property does not exist it
     * is taken from the defaults of the parent.
     * @param name The name of the property
     * @param type The class of the type
     * @return Return named value of the resource or its parent's defaults converted to type T or <code>null</code> if non existing
     *         or can't be converted.
     */
    public <T> T getInheritedDefault(String name, Class<T> type) {
		return null;
	}

    /**
     * Get the default properties that will be applied to child elements. Be aware that this are not the defaults that
     * are applied to the resource itself.
     * @return The default properties
     */
    public ValueMap getDefaultProperties() {
		return null;
	}

    /**
     * Get the properties of the resource merged with the default properties.
     * @return The properties
     */
    public ValueMap getProperties() {
		return null;
	}


    /**
     * Get the child resources at node {@link #ITEMS} of the resource.
     * @return The items
     */
    public Iterator<Resource> getItems() {
		return null;
	}

    /**
     * Get the child resources at the given node of the resource.
     * @param name The name of the node
     * @return The items
     */
    public Iterator<Resource> getItems(String name) {
		return null;
	}

    /**
     * Get the child resources at node {@link #ITEMS} of the given resource.
     * @param resource The resource
     * @return The items
     */
    public Iterator<Resource> getItems(Resource resource) {
		return null;
	}

    /**
     * Get the child resources at the given node of the given resource.
     * @param resource The resource
     * @param name The name of the node
     * @return The items
     */
    public Iterator<Resource> getItems(Resource resource, String name) {
		return null;
	}

    /**
     * Get the named child of this config.
     * @param name The name
     * @return The child resource
     */
    public Resource getChild(String name) {
		return null;
	}

    /**
     * Get the parent of the resource. If the parent is of type <code>{@link JcrConstants#NT_UNSTRUCTURED}</code> the
     * direct parent is a container (e.g. "items") and its parent is returned.
     * @return The parent resource
     */
    public Resource getParentResource() {
		return null;
	}

    /**
     * Returns the type of the default value
     * @param defaultValue The default value
     * @return The type of the default value
     */
    @SuppressWarnings("unchecked")
    private <T> Class<T> getType(T defaultValue) {
		return null;
	}
}
