/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.granite.ui.components;

import java.util.Arrays;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;

/**
 * A helper service to access the values for form. The values are represented as
 * a map of key-value pair (a {@link ValueMap}). The values are set at request
 * scope. Usually the values are set by form component and they are read by
 * field components.
 * 
 * <p>
 * To set the values, one can set them to the request attribute either using
 * {@link #FORM_VALUESS_ATTRIBUTE} or {@link #CONTENTPATH_ATTRIBUTE}.
 * </p>
 */
public class Value {

    public static final String FORM_VALUESS_ATTRIBUTE = "granite.ui.form.values";
    public static final String CONTENTPATH_ATTRIBUTE = "granite.ui.form.contentpath";

    private Config config;
    private ValueMap values = null;

    /**
     * Create a new Value.
     * 
     * @param request
     *            The sling request
     * @param config
     *            The config of the form field component
     */
    public Value(SlingHttpServletRequest request, Config config) {
        this.config = config;

        ValueMap formValues = (ValueMap) request.getAttribute(FORM_VALUESS_ATTRIBUTE);

        if (formValues != null) {
            values = formValues;
        } else {
            String contentPath = (String) request.getAttribute(CONTENTPATH_ATTRIBUTE);

            if (contentPath != null) {
                Resource contentResource = request.getResourceResolver().getResource(contentPath);

                if (contentResource != null && !ResourceUtil.isNonExistingResource(contentResource)) {
                    values = ResourceUtil.getValueMap(contentResource);
                }
            }
        }
    }

    /**
     * A shortcut to {@link #get(String, Object)}, with empty string as default
     * value.
     */
    public String get(String name) {
		return null;
	}

    /**
     * Returns a value for the given name. The value is read from the resource
     * set to this object, unless the resource doesn't exist or
     * <code>ignoreData</code> property of the config is <code>true</code>,
     * which will return <code>value</code> property of the config in that case.
     * 
     * @param name
     *            The name of the value to return
     * @param defaultValue
     *            The default value
     * @return Returns the named value converted to type T or the default value
     *         if non existing or can't be converted.
     */
    public <T> T get(String name, T defaultValue) {
		return null;
	}

    /**
     * Returns a value for the given name. The value is read from the resource
     * set to this object, unless the resource doesn't exist or
     * <code>ignoreData</code> property of the config is <code>true</code>,
     * which will return <code>value</code> property of the config in that case.
     * 
     * @param name
     *            The name of the value to return
     * @param type
     *            The return type
     * @return Returns the named value converted to type T or <code>null</code>
     *         if non existing or can't be converted.
     */
    public <T> T get(String name, Class<T> type) {
		return null;
	}

    /**
     * A shortcut of {@link #val(String, Object)}, with name is taken from
     * <code>name<code> property of the config.
     */
    public <T> T val(T defaultValue) {
		return null;
	}

    /**
     * Returns a value for the given name. The value is read from the resource
     * set to this object, unless the resource doesn't exist or
     * <code>ignoreData</code> property of the config is <code>true</code>,
     * which will return the given defaultValue in that case.
     * 
     * The key difference compared to {@link #get(String, Object)} is that this
     * method will return the given default value instead of value from
     * <code>value</code> property of the config.
     * 
     * @param name
     *            The name of the value to return
     * @param defaultValue
     *            The default value
     * @return Returns the named value converted to type T or the default value
     *         if non existing or can't be converted.
     */
    public <T> T val(String name, T defaultValue) {
		return null;
	}

    /**
     * A shortcut of {@link #isSelected(String, String, boolean)}, with name is
     * taken from <code>name<code> property of the config.
     */
    public boolean isSelected(String value, boolean defaultReturn) {
		return false;
	}

    /**
     * Returns <code>true</code> if the given value is selected;
     * <code>false</code> otherwise.
     * 
     * If the resource doesn't exist or <code>ignoreData</code> property of the
     * config is <code>true</code>, the given defaultReturn is returned.
     * Otherwise, the given value is compared against the value for the given
     * name.
     */
    public boolean isSelected(String name, String value, boolean defaultReturn) {
		return false;
	}

    /**
     * A shortcut to {@link #getContentValue(String, Object)}, with empty string
     * a default value.
     */
    public String getContentValue(String name) {
		return null;
	}

    /**
     * Returns value from the given name, converted to the type of given default
     * value.
     * 
     * @param name
     *            The name of the value property
     * @param defaultValue
     *            The default value
     * @return Return named value converted to type T or the default value if
     *         non existing or can't be converted.
     */
    public <T> T getContentValue(String name, T defaultValue) {
		return null;
	}

    /**
     * Returns value from the given name, converted to the given type.
     * 
     * @param name
     *            The name of the value to return
     * @param type
     *            The return type
     * @return Returns the named value converted to type T or <code>null</code>
     *         if non existing or can't be converted.
     */
    public <T> T getContentValue(String name, Class<T> type) {
		return null;
	}
}
