/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.analytics.testandtarget;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;

import aQute.bnd.annotation.ProviderType;

import com.day.cq.analytics.testandtarget.impl.TestandtargetCampaign;
import com.day.cq.wcm.webservicesupport.Configuration;

/**
 * The <tt>TestandtargetService</tt> is the entry point for all the operations which interact with the
 * Adobe Target REST API.
 * 
 *  @see TestandtargetCampaignMediator
 */
@ProviderType
public interface TestandtargetService {
    
    /** folder list operation */
    static final String OPERATION_FOLDER_LIST = "folderList";
	
	/** campaign list operation */
	static final String OPERATION_CAMPAIGN_LIST = "campaignList";
	
	/** HTML offer save operation */
	static final String OPERATION_SAVE_HTML_OFFER = "saveHtmlOfferContent";
	
	/** HTML offer get operation */
	static final String OPERATION_GET_HTML_OFFER = "getHtmlOfferContent";
	
	/** HTML offer list operation */
	static final String OPERATION_HTML_OFFER_LIST = "offerList";
	
	/** Widget offer deletion operation */
	static final String OPERATION_DELETE = "deleteWidgetOffer";
	
	/** Widget offer save operation */
	static final String OPERATION_SAVE = "saveWidgetOffer";
    
    /** save operation */
    static final String OPERATION_SET_CAMPAIGN_STATE = "setCampaignState";
	
    /** Campaign create operation */
    static final String OPERATION_SAVE_CAMPAIGN = "saveCampaign";
    
    static final String OPERATION_REPORT = "report";

	/** Property name client code */
    static final String PN_CLIENTCODE = "clientcode";

    /** Property name E-Mail */
    static final String PN_EMAIL = "email";

    /** Property name password */
    static final String PN_PASSWORD = "password";
    
    public static final String CAMPAIGN_STATE_APPROVED = "Approved";
    
    public static final String CAMPAIGN_STATE_DEACTIVATED = "Deactivated";
    
    /**
     * Returns a {@link Tree} of the available folders on Adobe Target.
     * 
     * @param configuration Service {@link Configuration}
     * @return A {@link Tree} of available folders
     * @throws TestandtargetException
     */
    Folder listFolders(Configuration configuration) throws TestandtargetException {
		return null;
	}

    /**
     * Creates a widget offer on Adobe Target.
     * 
     * @param configuration
     *            Service {@link Configuration}
     * @param name
     *            Name of the widget offer
     * @param url
     *            The encoded URL of the third-party server that hosts the
     *            dynamically generated response
     * @param id
     *            The third-party ID of the widget offer
     * @return A string representation of the third-party id
     * @throws TestandtargetException
     */
    @Deprecated
    String publishOffer(Configuration configuration, String name, String url, String id) throws TestandtargetException {
		return null;
	}

    /**
     * Deletes a widget offer on Adobe Target.
     * 
     * @param configuration
     *            Service {@link Configuration}
     * @param name
     *            Name of the widget offer
     * @param url
     *            The encoded URL of the third-party server that hosts the
     *            dynamically generated response
     * @param id
     *            The third-party ID of the widget offer
     * @return A string representation of the third-party id
     * @throws TestandtargetException
     */ 
    @Deprecated
    String unpublishOffer(Configuration configuration, String name, String url, String id) throws TestandtargetException {
		return null;
	}    

    /**
     * Creates or updates a campaign 
     * 
     * @param configuration
     *  		Service {@link Configuration}
     * @param thirdPartyId
     * 			The campaign third party id, used for tracking updates
     * @param campaign TODO
     * @throws TestandtargetException
     */
    void saveCampaign( Configuration configuration, String thirdPartyId, TestandtargetCampaign campaign ) throws TestandtargetException {
	}
    
    /**
     * Creates a widget offer.
     * 
     * @param configuration Service {@link Configuration}
     * @param name Name of the widget offer
     * @param url The encoded URL of the third-party server that hosts the
     *            dynamically generated response
     * @param id The third-party ID of the widget offer
     * @return A string representation of the third-party id
     * @throws TestandtargetException
     */
    String createWidgetOffer(Configuration configuration, String name, String url, String id) throws TestandtargetException {
		return null;
	}
    
    /**
     * Creates an HTML offer.
     * 
     * @param configuration Service {@link Configuration}
     * @param offerName Name of the offer
     * @param folderId Folder id
     * @param content Offer content as plain HTML.
     * @throws TestandtargetException
     */
    void createHTMLOffer(Configuration configuration, String offerName, String folderId, String content) throws TestandtargetException {
	}
    
    /**
     * Gets a HTML offer.
     * 
     * @param configuration Service {@link Configuration}
     * @param offerName Name of the offer
     * @param folderId Folder id
     * @return An HTMLOffer
     * @throws TestandtargetException
     */
    HTMLOffer getHTMLOffer(Configuration configuration, String offerName) throws TestandtargetException {
		return null;
	}
    
    /**
     * Returns a JSON representation of offers.
     * 
     * @param configuration Service {@link Configuration}
     * @param folderId Folder id
     * @return An {@link Collection} of {@link Offers}
     * @throws TestandtargetException
     */    
    Collection<Offer> listOffers(Configuration configuration, String folderId) throws TestandtargetException {
		return null;
	}
    
	/**
	 * Returns a {@link Map} with campaigns. The Map key contains the campaign
	 * ID and the Map value the campaign name.
	 * 
	 * @param configuration
	 *            Service {@link Configuration}
	 * @param before
	 *            A date value. Includes campaigns that were active at least
	 *            once before the specified date. By default, the before
	 *            parameter is 2100-01-01T00:00.
	 * @param after
	 *            A date value. Includes campaigns that were active at least
	 *            once after the specified date. By default, the after parameter
	 *            is 1969-00-00T00:00.
	 * @param environment
	 *            A URL-encoded host group name, as defined in the T&T Tool. By
	 *            default, the environment value is Production.
	 * @param name
	 *            The campaign name, or portion of the campaign name, that you
	 *            want to match.
	 * @param state
	 *            Comma-separated list of states to match. Supported values
	 *            include: saved, activated, library. By default, the filtered
	 *            results include all states.
	 * @param label
	 *            Comma-separated list of labels to match. By default, the
	 *            filtered results include all labels.
	 * @return
	 * @throws TestandtargetException
	 */
	Map<Integer, String> listCampaigns(Configuration configuration, Date before, Date after, String environment, String name, String state, String label) throws TestandtargetException {
		return null;
	}

   /**
     * Set the state of a campaign.
     * 
     * @param configuration
     *            Service {@link Configuration}
     * @param state
     *            The campaign state , one of {@link #CAMPAIGN_STATE_APPROVED} or {@link #CAMPAIGN_STATE_DEACTIVATED}
     * @param testAndTargetCampaignId
     *            The campaign identifier. You must provide either testAndTargetCampaignId or thirdPartyCampaignId 
     * @param thirdPartyCampaignId
     *            The third-party campaign identifier. You must provide either testAndTargetCampaignId or thirdPartyCampaignId 
     * @return true if the operation succeeded
     * @throws TestandtargetException
     */
	boolean setCampaignState(Configuration configuration, String campaignState, String testAndTargetCampaignId, String thirdPartyCampaignId) throws TestandtargetException {
	return false;
}
	
    /**
     * Checks the provided <code>email</code>, <code>password</code> and
     * <code>clientcode</code> by issuing an API call on the method campaignList
     * with a dummy search string.
     * 
     * @param email
     * @param password
     * @param clientcode
     * @throws TestandtargetException
     */
	void checkCredentials(String clientcode, String email, String password) throws TestandtargetException {
	}

    /**
     * Retrieves the performance report data for a specific campaign
     * 
     * <p>
     * All parameters are mandatory, except for <tt>campaignId</tt> and <tt>thirdPartyId</tt> , out of which exactly one
     * must be specified.
     * </p>
     * 
     * @param configuration
     * @param reportType
     * @param campaignId
     * @param thirdPartyCampaignId
     * @param start
     * @param end
     * @param resolution
     * @throws TestandtargetException
     * 
     * @see <a href="https://developer.omniture.com/en_US/documentation/test-target/r-performance-report">Adobe Target
     *      performance report API</a>
     */
    Reports getPerformanceReport(Configuration configuration, ReportType reportType, String thirdPartyCampaignId,
            String campaignId, Date start, Date end, Resolution resolution) throws TestandtargetException {
		return null;
	}

    /**
     * Retrieves a list of segments for the specified <tt>configuration</tt>
     * 
     * <p>
     * Optional filtering parameters may be set in the <tt>request</tt> parameter.
     * </p>
     * 
     * @param configuration Service {@link Configuration}
     * @param request optional request parameters. May be null.
     * @throws TestandtargetException
     * @return a list of segments, possibly empty, never <code>null</code>
     * 
     */
    List<Segment> listSegments(Configuration configuration, ListSegmentsRequest request)
            throws TestandtargetException {
		return null;
	}
}
