package com.day.cq.apns;

import javax.jcr.RepositoryException;

import org.apache.sling.api.resource.PersistenceException;

import com.day.cq.security.Authorizable;


/**
 * The <code>PushNotificationService</code> provides functionality to interact with
 * Apple's Push Notification service.
 */
public interface PushNotificationService {
	/**
	 * Stores the provided device token in the user's profile for later use.
	 * @param token		The device token in hex format
	 * @param session	The user's session
	 * @throws RepositoryException
	 * @throws IllegalArgumentException
	 * @throws PersistenceException
	 */
	public void registerDeviceToken(String token, String userId) throws NotificationException, IllegalArgumentException {
	}

	/**
	 * Checks if authorizable is registered for push notifications.
	 * @param authorizable	The authorizable to check
	 * @return				<code>true</code> if registered. <code>false</code> otherwise.
	 */
	public boolean isRegistered(Authorizable authorizable) {
		return false;
	}
	
	/**
	 * Checks if the given string is a valid device token, i.e. has a length
	 * of 64 characters and contains hex characters only.
	 * @param token		The device token to check
	 * @return			<code>true</code> if token is valid. <code>false</code> otherwise.
	 */
	public boolean isValidDeviceToken(String token) {
		return false;
	}
	
	/**
	 * Checks if the given token has been blacklisted.
	 * @param token		The device token to check
	 * @return			<code>true</code> if token is blacklisted. <code>false</code> otherwise.
	 */
	public boolean isBlacklistedToken(String token) {
		return false;
	}
	
	/**
	 * Sends a request for notification to the configured, central
	 * APNs gateway server, which forwards it to the actual APNs.
	 * @param token		The device token
	 * @param message	The alert message
	 * @param badge		The badge number to display on app icon
	 * @throws NotificationException If sending notification request failed.
	 */
	public void sendNotificationRequest(String token, String message, int badge) throws NotificationException {
	}
	
	/**
	 * Sends a request for notification to the configured, central
	 * APNs gateway server, which forwards it to the actual APNs.
	 * The notification request is only send if the authorizable
	 * has registered for push notifications.
	 * @param authorizable	The authorizable to notify
	 * @param message		The alert message
	 * @param badge			The badge number to display on app icon
	 * @throws NotificationException If sending notification request failed.
	 */
	public void sendNotificationRequest(Authorizable authorizable, String message, int badge) throws NotificationException {
	}
	
	/**
	 * Sends the actual notification to APNs. In general, you should
	 * not call this method directly, but instead use
	 * <code>{@link PushNotificationService#sendNotification(Authorizable, String, int)}</code>.
	 * @param token		The device token
	 * @param message	The alert message
	 * @param badge		The badge number to display on app icon
	 */
	public void sendNotification(String token, String message, int badge) {
	}
	
	/**
	 * Updates the list of invalid device tokens reported by the
	 * APNS feedback service.
	 */
	public void updateBlacklist() throws NotificationException {
	}
}
