/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.auth.pin;

import java.util.Iterator;

/**
 * The <code>Pin</code> interface represents a token which may be used to access
 * the system without a password. Each such token belongs to a user on whose
 * behalf access is granted.
 * <p>
 * The use of a PIN may further be restricted by the {@link #getMaxUse() number
 * of times} it may be used and an {@link #getExpiryTime() expiry time}.
 * Additionally the PIN may be {@link #isEnabled() enabled or disabled}.
 * <p>
 * This interface is not intended to be implemented by clients. Instances of
 * this interface are returned by the methods of the {@link PinManager}.
 *
 * @deprecated CQ 5.6 For security reaons the PIN authentication has been
 * deprecated. Please use token based authentication instead.
 */
public interface Pin {

    /**
     * Returns the ID of this pin. The PIN ID is used as the value of the
     * <code>:cqpin</code> request parameter or cookie to get access to the
     * system.
     */
    String getId() {
		return null;
	}

    /**
     * Returns ID of the user to which this PIN belongs. Access to the system
     * using this PIN is granted for this user.
     */
    String getUserId() {
		return null;
	}

    /**
     * Return the total number times this PIN has already been used. Note, that
     * this includes successful and failed uses of this PIN.
     */
    int getUseCounter() {
		return 0;
	}

    /**
     * Returns <code>true</code> if this PIN is currently enabled.
     */
    boolean isEnabled() {
		return false;
	}

    /**
     * Returns the maximum number of times this PIN may be used. If this value
     * is negative the use of the PIN is not limited by number of uses.
     */
    int getMaxUse() {
		return 0;
	}

    /**
     * Returns the expiry time in milliseconds since the epoch. This is the time
     * on the server at which the PIN expires. If no expiry time has been
     * configured for this PIN a negative number is returned.
     */
    long getExpiryTime() {
		return 0L;
	}

    /**
     * Enables or disables this PIN.
     * <p>
     * This method may only be called the PIN owner or by PIN administrators.
     *
     * @param enabled <code>true</code> to enable the PIN and <code>false</code>
     *            to disable this PIN.
     */
    void setEnabled(boolean enabled) {
	}

    /**
     * Sets the maximum number of number of time this PIN may be used.
     * <p>
     * This method does not reset the {@link #getUseCounter() use counter}. Thus
     * if this number is less than or equal to the current use counter, the PIN
     * becomes invalid immediately.
     * <p>
     * This method may only be called the PIN owner or by PIN administrators.
     *
     * @param maxUse The maximum number of uses of this PIN. If this value is
     *            negative, the use of the PIN is not limited by the use
     *            counter.
     */
    void setMaxUse(int maxUse) {
	}

    /**
     * Sets the time in milliseconds since the epoch of the expiry of this PIN.
     *
     * @param expiryTime The expiry time in milliseconds since the epoch. If
     *            this time is negative, the PIN will never expire. If this time
     *            is in the past, the PIN becomes invalid immediately.
     */
    void setExpiryTime(long expiryTime) {
	}

    /**
     * Sets the maximum age in seconds of this PIN. That is the PIN will be
     * valid for the given number of seconds from the moment the age is set.
     * <p>
     * The age must be positive number. Specifying a negative number will have
     * no effect. To switch off the PIN expiry the {@link #setExpiryTime(long)}
     * must be called with a negative value.
     *
     * @param age The maximum age in seconds of this PIN. This must be a
     *            positive number. If this is called with a negative number,
     *            this method has no effect.
     */
    void setAge(int age) {
	}

    /**
     * Returns an iterator {@link PinUse} objects of the recorded uses of this
     * PIN. If the PIN has never been used or if this method is not called by
     * the owner or a PIN administrator an empty iterator is returned.
     */
    Iterator<PinUse> getUses() {
		return null;
	}
}