/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.auth.pin;

import java.util.Iterator;

/**
 * The <code>PinManager</code> represents a PIN manager which may be used to
 * access and managed {@link Pin PINs}.
 * <p>
 * This interface is not intended to be implemented by clients. Instances of
 * this interface are returned by the
 * {@link PinManagerFactory#getPinManager(org.apache.sling.api.resource.ResourceResolver)}
 * method.
 *
 * @deprecated CQ 5.6 For security reaons the PIN authentication has been
 * deprecated. Please use token based authentication instead.
 */
public interface PinManager {

    /**
     * Creates and returns a new PIN for the named user.
     * <p>
     * This method may only be called by the named user or a PIN administrator.
     * If this method is called by another non-administrator user no PIN is
     * created and <code>null</code> is returned.
     *
     * @param userId The name of the user for which a PIN is to be returned.
     * @return The created {@link Pin} or <code>null</code> if not called by a
     *         PIN administrator or the owner of the new PIN.
     * @throws NullPointerException if <code>userId</code> is <code>null</code>.
     */
    Pin createPin(final String userId) {
		return null;
	}

    /**
     * Returns the {@link Pin} identified by the given {@link Pin#getId() PIN
     * Id} or <code>null</code> if no such PIN is defined in the system.
     * <p>
     * This method will also return the actual existing {@link Pin} if called by
     * the PIN owner or by a PIN administrator.
     *
     * @param pinId The identification of the {@link Pin} to return.
     * @return The named {@link Pin} or <code>null</code>.
     * @throws NullPointerException if <code>pinId</code> is <code>null</code>.
     */
    Pin getPin(final String pinId) {
		return null;
	}

    /**
     * Returns an iterator of all PINS present in the system.
     * <p>
     * This method only returns all defined PINs if called by a PIN
     * administrator. For regular users this method only returns the PINs of the
     * user itself.
     */
    Iterator<Pin> listPins() {
		return null;
	}

    /**
     * Returns an iterator of {@link Pin}s present in the system belonging to
     * the given user.
     * <p>
     * This method only returns the {@link Pin}s if called by a PIN
     * administrator. For regular users this method only returns the PINs of the
     * user itself. If a regular user requests the PINs of another user, an
     * empty iterator is returned.
     *
     * @param userId The name of the user whose {@link Pin}s have to be
     *            returned. This may be <code>null</code> in which case this
     *            method returns the same as calling the {@link #listPins()}
     *            method.
     */
    Iterator<Pin> listPins(String userId) {
		return null;
	}

    /**
     * Deletes the {@link Pin} with the given ID. If this method is not called
     * by the {@link Pin} owner it must be called by a PIN administrator.
     * Otherwise this method has no effect. That is non-administrators cannot
     * delete PINs of other users.
     * <p>
     * Likewise trying to delete a non-existing PIN has no effect.
     *
     * @param pinId The identification of the {@link Pin} to delete.
     * @return <code>true</code> if the named PIN has been deleted.
     *         <code>false</code> is returned if not called by the owner or a
     *         PIN administrator or if the PIN does not exist at all.
     * @throws NullPointerException if <code>pinId</code> is <code>null</code>.
     */
    boolean deletePin(final String pinId) {
		return false;
	}
}