/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.auth.pin;

/**
 * The <code>PinUse</code> interface represents one use of a PIN. These uses are
 * recorded everytime a PIN is used. Instances of this interface are returned by
 * the {@link Pin#getUses()} method.
 * <p>
 * This in interface is not intended to be implemented by clients.
 *
 * @deprecated CQ 5.6 For security reaons the PIN authentication has been
 * deprecated. Please use token based authentication instead.
 */
public interface PinUse {

    /**
     * Returns the server time as reported by
     * <code>System.currentTimeMillis()</code> of the time of validating the
     * PIN.
     */
    long getTime() {
		return 0L;
	}

    /**
     * Returns the HTTP method used on the request authenticated by the PIN.
     * This is the value reported by <code>HttpServletRequest.getMethod()</code>
     * .
     */
    String getRequestMethod() {
		return null;
	}

    /**
     * Returns the request URL used on the request authenticated by the PIN.
     * This is the value reported by
     * <code>HttpServletRequest.getRequestURI()</code>.
     */
    String getRequestURI() {
		return null;
	}

    /**
     * Returns the client's user agent as reported in the
     * <code>User-Agent</code> header of the request.
     */
    String getUserAgent() {
		return null;
	}

    /**
     * Returns the remote IP address and port number as reported by
     * <code>ServletRequest.getRemoteAddr()</code> and
     * <code>ServletRequest.getRemotePort()</code> methods.
     */
    String getRemoteHost() {
		return null;
	}

    /**
     * Returns <code>true</code> if access was granted using the PIN. Otherwise
     * the PIN was not valid and access was not granted.
     */
    boolean granted() {
		return false;
	}
}
