/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.commons;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <code>LanguageUtil</code>...
 */
public class LanguageUtil {

    /**
     * Set of all valid language iso codes.
     * @see Locale#getISOLanguages()
     */
    public static final Set<String> ISO_LANGUAGES =
            Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(Locale.getISOLanguages())));

    /**
     * Set of all valid country iso codes.
     * @see Locale#getISOCountries()
     */
    public static final Set<String> ISO_COUNTRIES =
            Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(Locale.getISOCountries())));

    // pseudo language for i18n testing
    public static final String PSEUDO_LANGUAGE = "zz";

    /**
     * Regexp to find the Language Label in a Path. eg "en" in "/content/geometrixx/en"
     */
    private static final Pattern SIMPLE_PATTERN = Pattern.compile("/([a-zA-Z]{2})(/|$)");

    /**
     * Regexp to find the Language-Country Code in a Path. eg "en_GB" in "/content/geometrixx/en_GB"
     */
    private static final Pattern DISTINCT_PATTERN = Pattern.compile("/([a-zA-Z]{2,3}_[a-zA-Z]{2,3})(/|$)");

    /**
     * Returns the language locale for the given language/country abbreviation.
     * @param code the language/country code. eg de_CH
     * @return the locale or null if not valid.
     */
    public static Locale getLocale(String code) {
		return null;
	}

    /**
     * Returns the language for the given language/country abbreviation.
     * @param code the language/country code. eg de_CH or de-CH
     * @return the locale or null if not valid.
     *
     * @since 5.4
     */
    public static Language getLanguage(String code) {
		return null;
	}

    /**
     * Returns the language root for the given path by only analyzing the path
     * names starting at the root.
     *
     * @param path path
     * @return the language root or <code>null</code> if not found
     */
    public static String getLanguageRoot(String path) {
		return null;
	}
}