/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.api.metadata.xmp;

import static com.day.cq.commons.jcr.JcrConstants.JCR_TITLE;
import static com.day.cq.dam.api.DamConstants.DC_CONTRIBUTOR;
import static com.day.cq.dam.api.DamConstants.DC_COVERAGE;
import static com.day.cq.dam.api.DamConstants.DC_CREATOR;
import static com.day.cq.dam.api.DamConstants.DC_DATE;
import static com.day.cq.dam.api.DamConstants.DC_DESCRIPTION;
import static com.day.cq.dam.api.DamConstants.DC_FORMAT;
import static com.day.cq.dam.api.DamConstants.DC_IDENTIFIER;
import static com.day.cq.dam.api.DamConstants.DC_LANGUAGE;
import static com.day.cq.dam.api.DamConstants.DC_PUBLISHER;
import static com.day.cq.dam.api.DamConstants.DC_RELATION;
import static com.day.cq.dam.api.DamConstants.DC_RIGHTS;
import static com.day.cq.dam.api.DamConstants.DC_SUBJECT;
import static com.day.cq.dam.api.DamConstants.DC_TITLE;
import static com.day.cq.dam.api.DamConstants.TIFF_IMAGELENGTH;
import static com.day.cq.dam.api.DamConstants.TIFF_IMAGEWIDTH;


import java.util.HashMap;
import java.util.Map;

/**
 * The <code>XmpMappings</code> class ...
 */
public interface XmpMappings {
    static final Map<String , String> defaultSimpleXmpMappings =
           new HashMap<String , String>() {{
        // dc
        put("identifier", DC_IDENTIFIER);
        put("format", DC_FORMAT);
        put("coverage", DC_COVERAGE);
        put("rights", DC_RIGHTS);
        put("title", DC_TITLE + "," + JCR_TITLE);
        put("Title", DC_TITLE + "," + JCR_TITLE);
        put("dc:title", DC_TITLE + "," + JCR_TITLE);
        put("Object Name", DC_TITLE + "," + JCR_TITLE)   ;

        // xmp
        put("CreationDate", "xmp:CreateDate");
        put("createdate", "xmp:CreateDate");
        put("creatortool", "xmp:CreatorTool");
        put("modifydate", "xmp:ModifyDate");
        //commons imaginge has changed modifydate to DateTime, hence adjusting
        //not removing older
        put("DateTime", "xmp:ModifyDate");
        put("modificationdate", "xmp:ModifyDate");
        put("ModificationDate", "xmp:ModifyDate");

        // exif tiff
        put("Compression", "tiff:Compression");
        put("CreateDate", "tiff:DateTime,exif:DateTimeOriginal");
        put("Make", "tiff:Make");
        put("Model", "tiff:Model");
        /* changed for bug 39584, not related to bug as such but
        it was wrongly mapped to exif:PixelXDimension and exif:PixelYDimension
        exif:PixelXDimension and exif:PixelYDimension is already mapped to XResolution and YResolution */
        put("ExifImageWidth", "exif:ExifImageWidth");
        put("ExifImageLength", "exif:ExifImageLength");
        put("ResolutionUnit", "tiff:ResolutionUnit");
        put("YCbCrPositioning", "tiff:YCbCrPositioning");
        put("ImageDescription", "tiff:ImageDescription");
        put("Orientation", "tiff:Orientation");
        put("XResolution", "tiff:XResolution,exif:PixelXDimension");
        put("YResolution", "tiff:YResolution,exif:PixelYDimension");
        put("ImageWidth", TIFF_IMAGEWIDTH);
        put("ImageLength", TIFF_IMAGELENGTH);
        put("Image Width", TIFF_IMAGEWIDTH);
        put("Image Length", TIFF_IMAGELENGTH);
        put("PhotometricInterpretation", "tiff:PhotometricInterpretation");
        put("BitsPerSample", "tiff:BitsPerSample");
        put("SamplesPerPixel", "tiff:SamplesPerPixel");
        put("PlanarConfiguration", "tiff:PlanarConfiguration");
        put("YCbCrSubSampling", "tiff:YCbCrSubSampling");
        put("TransferFunction", "tiff:TransferFunction");
        put("WhitePoint", "tiff:WhitePoint");
        put("PrimaryChromaticities", "tiff:PrimaryChromaticities");
        put("YCbCrCoefficients", "tiff:YCbCrCoefficients");
        put("ReferenceBlackWhite", "tiff:ReferenceBlackWhite");
        put("Software", "tiff:Software");
        put("Artist", "tiff:Artist");
        put("Copyright", "tiff:Copyright");
        put("Copyright Notice", "tiff:Copyright");

        // exif
        put("Contrast", "exif:Contrast");
        put("CompressedBitsPerPixel", "exif:CompressedBitsPerPixel");
        put("PhotographicSensitivity", "psAux:ISO"); // Bug 29556
        put("MaxApertureValue", "exif:MaxApertureValue");
        put("FNumber", "exif:FNumber");
        put("Color Space", "exif:ColorSpace");
        put("Sharpness", "exif:Sharpness");
        put("FileSource", "exif:FileSource");
        put("FocalLength", "exif:FocalLength");
        put("ExposureMode", "exif:ExposureMode");
        put("Saturation", "exif:Saturation");
        put("FlashpixVersion", "exif:FlashpixVersion");
        put("ExposureTime", "exif:ExposureTime");
        put("LightSource", "exif:LightSource");
        put("SceneCaptureType", "exif:SceneCaptureType");
        put("ExposureProgram", "exif:ExposureProgram");
        put("MeteringMode", "exif:MeteringMode");
        put("DateTimeOriginal", "exif:DateTimeOriginal");
        put("SceneType", "exif:SceneType");
        put("WhiteBalance", "exif:WhiteBalance");
        put("Flash", "exif:Flash");
        put("CustomRendered", "exif:CustomRendered");
        put("ExifVersion", "exif:ExifVersion");
    }};

    static final Map<String , String> defaultBagXmpMappings =
           new HashMap<String , String>() {{
        // dc
        put("language", DC_LANGUAGE);
        put("publisher", DC_PUBLISHER);
        put("relation", DC_RELATION);
        put("contributor", DC_CONTRIBUTOR);
        put("subject", DC_SUBJECT);
        put("Subject", DC_SUBJECT);
        put("Category", DC_SUBJECT);
    }};

    static final Map<String , String> defaultSeqXmpMappings =
           new HashMap<String , String>() {{
        // dc
        put("creator", DC_CREATOR);
        put("Creator", DC_CREATOR);
        put("Originating Program", DC_CREATOR);
        put("date", DC_DATE);
        put("Date Created", DC_DATE);

        // exif
        put("ComponentsConfiguration", "exif:ComponentsConfiguration");
    }};

    static final Map<String , String> defaultAltXmpMappings =
           new HashMap<String , String>() {{
        // dc
        put("description", DC_DESCRIPTION);
        put("Caption/Abstract", DC_DESCRIPTION);
    }};
}
