/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.core.process;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.Property;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.day.cq.dam.api.Asset;
import com.day.cq.dam.api.renditions.RenditionMaker;
import com.day.cq.dam.api.renditions.RenditionTemplate;
import com.day.cq.dam.api.thumbnail.ThumbnailConfig;
import com.day.cq.dam.commons.process.AbstractAssetWorkflowProcess;
import com.day.cq.dam.commons.thumbnail.ThumbnailConfigImpl;
import com.day.cq.workflow.WorkflowException;
import com.day.cq.workflow.WorkflowSession;
import com.day.cq.workflow.exec.WorkItem;
import com.day.cq.workflow.metadata.MetaDataMap;

/**
 * The <code>CreateThumbnailProcess</code> is called in a Workflow process step.
 * This process will create one or more Thumbnails for the Asset to be procesed.
 * The Thumbnail generation is delegated to the appropriate
 * {@link com.day.cq.dam.api.handler.AssetHandler AssetHandler}. The thumbails
 * Dimensions are given as the Workflow
 * {@link com.day.cq.workflow.exec.WorkflowProcess#execute(WorkItem, WorkflowSession, MetaDataMap)
 * Process arguments}<br>
 * Each argument is considered as one thumbnail configuration. The configuration
 * has to be enclosed in squared brackets (<i>[]</i>) The configuration consists
 * of a number for the width, the height in pixel and an optional flag that
 * indicates if the image should be centered.<br>
 * These values are seperated by a colon character (<i>:</i>). <br>
 * Dimension:<br>
 * The thumbnail from an image will be created by resizeing according the
 * thumbnail configuration. The ascpect ratio is respected and the image will
 * have at most the dimension given by the configuration.<br>
 * Centering:<br>
 * If the center flag is set to <i>true</i>, the thumbnail image will have
 * exactly the size given by the configuraiton. If the resized image is smaller
 * it will be centered within the thumbnail.
 * <p/>
 * <p/>
 * Example with the following Workflow step arguments:
 *
 * <pre>
 *    [70:70],
 *    [80:100:true]
 * </pre>
 * <p/>
 * The Process creates two PNG images one of the size of 70x70 pixel and one
 * 80x100 where the result is centred
 */
@Component
@Service
@Property(name = "process.label", value = "Create Thumbnail")
public class CreateThumbnailProcess extends AbstractAssetWorkflowProcess {
	
    /**
     * Logger instance for this class.
     */
    private static final Logger log = LoggerFactory.getLogger(CreateThumbnailProcess.class);

    /**
     * The available arguments to this process implementation.
     */
    public enum Arguments {
        PROCESS_ARGS, CONFIGS, SKIP_MIME_TYPES
    }

    /** The parsed configuration for this wf process */
    public static class Config {
        public String[] skipMimeTypes;
        public ThumbnailConfig[] thumbnails;

        public Config() {}

        public Config(String[] skipMimeTypes, ThumbnailConfig[] thumbnails) {
            this.skipMimeTypes = skipMimeTypes;
            this.thumbnails = thumbnails;
        }
    }

    @Reference
    private RenditionMaker renditionMaker;

    public void execute(WorkItem workItem, WorkflowSession workflowSession, MetaDataMap metaData)
            throws WorkflowException {
			}

    public static RenditionTemplate[] createRenditionTemplates(Asset asset, ThumbnailConfig[] thumbnails, RenditionMaker renditionMaker) {
		return null;
	}

    public Config parseConfig(MetaDataMap metaData) {
		return null;
	}

    public static ThumbnailConfig[] parseThumbnailArguments(String[] args) {
		return null;
	}

    public static ThumbnailConfig parseThumbnailArguments(final String arg) {
		return null;
	}

    protected boolean handleAsset(Asset asset, Config config) {
		return false;
	}
}
