/*
 * Copyright 1997-2010 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.mailer.commons;

import com.adobe.granite.security.user.UserProperties;
import com.day.cq.security.profile.Profile;
import org.apache.commons.lang.text.StrLookup;
import org.apache.jackrabbit.api.security.user.Authorizable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import static java.beans.Introspector.IGNORE_ALL_BEANINFO;
import static java.beans.Introspector.getBeanInfo;

/**
 * Implements a {@link org.apache.commons.lang.text.StrLookup StrLookup} that is
 * looking up variables from a
 * {@link com.day.cq.security.profile.Profile Profile}.
 * <br>
 * The variable name is matched to a getter Method on the Profile by the Bean
 * convention. Eg.: primaryMail -> Profile.getPrimaryMail();<br>
 * In addition to the Profile's variables the
 * {@link com.day.cq.security.profile.Profile#getAuthorizable() Profile'a Authorizable's ID}
 * can be accessed via the key <code>authorizableID</code>
 * <br>
 * Additional variables can bee added via a {@link #put(String, String)}.
 * If the key exists in the Profile, the value is overwritten with the newly set
 * @see StrLookup
 * @since 5.4
 */
public class ProfileVariableLookup extends StrLookup {

    public final static String AUTHORIZABLE_ID = "authorizableID";

    private final static Logger log = LoggerFactory.getLogger(ProfileVariableLookup.class);

    private final Map<String, String> overlay;

    private Profile profile;

    private final Map<String, Method> map;

    private Authorizable authorizable;

    private UserProperties userProperties;

    public ProfileVariableLookup(Profile profile) throws IntrospectionException {
        this(profile, new HashMap<String, String>());
    }

    /**
     *Use {@link #ProfileVariableLookup(org.apache.jackrabbit.api.security.user.Authorizable, com.adobe.granite.security.user.UserProperties, java.util.Map)}
     */
    @Deprecated
    public ProfileVariableLookup(Profile profile, Map<String, String> overlays) throws IntrospectionException {
        this.profile = profile;
        this.map = new HashMap<String, Method>();
        //note this lines release the dependency to a commons.bean
        for (PropertyDescriptor propDesc : getBeanInfo(profile.getClass(), IGNORE_ALL_BEANINFO).getPropertyDescriptors()) {
            if (propDesc.getPropertyType().isAssignableFrom(String.class)) {
                map.put(propDesc.getName(), propDesc.getReadMethod());
            }
        }
        if ( profile.getAuthorizable() != null ) {
            final Node authNode = profile.getAuthorizable().adaptTo(Node.class);
            try {
                if ( authNode != null && authNode.getIdentifier() != null ) {
                    overlays.put("userUUID", authNode.getIdentifier());
                }
            } catch (final RepositoryException re) {
                // ignore
            }
        }
        this.overlay = overlays;
    }

    public ProfileVariableLookup(Authorizable authorizable, UserProperties userProperties, Map<String, String> overlays) throws IntrospectionException {
        this.authorizable = authorizable;
        this.map = new HashMap<String, Method>();
        this.userProperties = userProperties;
        //note this lines release the dependency to a commons.bean
        for (PropertyDescriptor propDesc : getBeanInfo(authorizable.getClass(), IGNORE_ALL_BEANINFO).getPropertyDescriptors()) {
            if (propDesc.getPropertyType().isAssignableFrom(String.class)) {
                map.put(propDesc.getName(), propDesc.getReadMethod());
            }
        }
        this.overlay = overlays;

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String lookup(String variable) {
		return null;
	}

    /**
     * Adds or overwrites a mapping of this replacer.
     * @param var name of the variable to map
     * @param replace the value of the variable
     */
    public void put(String var, String replace) {
	}
}