/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.mcm.landingpage.parser.taghandlers.cta;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.day.cq.wcm.designimporter.util.TagUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.sling.api.wrappers.ValueMapDecorator;
import org.ccil.cowan.tagsoup.AttributesImpl;
import org.xml.sax.Attributes;

import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.dam.indd.PageComponent;
import com.day.cq.wcm.designimporter.DesignImportException;
import com.day.cq.wcm.designimporter.parser.HTMLContentType;
import com.day.cq.wcm.designimporter.parser.taghandlers.AbstractTagHandler;

/**
 * The <code>LeadFormCTAComponentTagHandler</code> handles the CTA Lead Form component. Maps various lead form fields to corresponding lead
 * components and other generic components with corresponding attributes/properties provided in imported zip.
 * Supported components  are:
 * "mcm/components/cta-form/start"
 "mcm/components/cta-form/end"
 "mcm/components/cta-form/first-name"
 "mcm/components/cta-form/last-name"
 "mcm/components/cta-form/emailId"
 "mcm/components/cta-form/address"
 "mcm/components/cta-form/gender"
 "mcm/components/cta-form/dateOfBirth"
 "mcm/components/cta-form/about"
 "mcm/components/cta-form/userId"
 "mcm/components/cta-form/submit"
 "foundation/components/form/text"
 "foundation/components/form/checkbox"
 "foundation/components/form/radio"
 "foundation/components/form/password"
 "foundation/components/form/hidden"
 "foundation/components/parsys"

 Custom component can be included inside cta-form by providing full path inside data-cq-component.
 *
 * @author nidhgupt
 */
public class LeadFormCTAComponentTagHandler extends AbstractTagHandler {

    static public final String RESOURCE_TYPE_CTA_LEADFORM_START = "mcm/components/cta-form/start";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_END = "mcm/components/cta-form/end";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_FIRSTNAME = "mcm/components/cta-form/first-name";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_LASTNAME = "mcm/components/cta-form/last-name";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_EMAILID = "mcm/components/cta-form/emailId";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_ADDRESS = "mcm/components/cta-form/address";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_GENDER = "mcm/components/cta-form/gender";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_DOB = "mcm/components/cta-form/dateOfBirth";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_ABOUT = "mcm/components/cta-form/about";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_USERID = "mcm/components/cta-form/userId";

    static public final String RESOURCE_TYPE_CTA_LEADFORM_SUBMIT = "mcm/components/cta-form/submit";

    static public final String RESOURCE_TYPE_TEXT = "foundation/components/form/text";

    static public final String RESOURCE_TYPE_CHECKBOX = "foundation/components/form/checkbox";

    static public final String RESOURCE_TYPE_RADIO = "foundation/components/form/radio";

    static public final String RESOURCE_TYPE_PASSWORD = "foundation/components/form/password";

    static public final String RESOURCE_TYPE_HIDDEN = "foundation/components/form/hidden";

    static public final String RESOURCE_TYPE_PARSYS = "foundation/components/parsys";

    static private final String FORM_START = "start";

    static private final String FORM_END = "form_end";

    private String labelBuffer;

    private String labelForParameter;

    private String redirectUrl;

    private String groupName;

    private String parsysHintName;

    static private Map<String, String> supportedSubComponents = new HashMap<String, String>();

    private Map<Integer, String> orderedMapForComponents = new HashMap<Integer, String>();

    private int componentCounter = 0;

    static {
        supportedSubComponents.put("firstName", RESOURCE_TYPE_CTA_LEADFORM_FIRSTNAME);
        supportedSubComponents.put("lastName", RESOURCE_TYPE_CTA_LEADFORM_LASTNAME);
        supportedSubComponents.put("emailId", RESOURCE_TYPE_CTA_LEADFORM_EMAILID);
        supportedSubComponents.put("address", RESOURCE_TYPE_CTA_LEADFORM_ADDRESS);
        supportedSubComponents.put("dob", RESOURCE_TYPE_CTA_LEADFORM_DOB);
        supportedSubComponents.put("userId", RESOURCE_TYPE_CTA_LEADFORM_USERID);
        supportedSubComponents.put("about", RESOURCE_TYPE_CTA_LEADFORM_ABOUT);
        supportedSubComponents.put("gender", RESOURCE_TYPE_CTA_LEADFORM_GENDER);
        supportedSubComponents.put("submit", RESOURCE_TYPE_CTA_LEADFORM_SUBMIT);

        supportedSubComponents.put("text", RESOURCE_TYPE_TEXT);
        supportedSubComponents.put("checkbox", RESOURCE_TYPE_CHECKBOX);
        supportedSubComponents.put("radio", RESOURCE_TYPE_RADIO);
        supportedSubComponents.put("hidden", RESOURCE_TYPE_HIDDEN);
        supportedSubComponents.put("password", RESOURCE_TYPE_PASSWORD);
    }

    private Map<String, ComponentDetails> ctaFormSubComponents = new HashMap<String, ComponentDetails>();

    @Override
    public void beginHandling(String uri, String localName, String qName, Attributes atts) throws DesignImportException {
	}

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws DesignImportException {
	}

    @Override
    public void characters(char[] ch, int start, int length) throws DesignImportException {
	}

    @Override
    public void endHandling(String uri, String localName, String qName) throws DesignImportException {
	}

    @Override
    public void endElement(String uri, String localName, String qName) throws DesignImportException {
	}

    @Override
    public boolean supportsContent(HTMLContentType htmlContentType) {
		return false;
	}

    private String getNameHint(String componentType) {
		return null;
	}

    @Override
    public Object getContent(HTMLContentType htmlContentType) {
		return null;
	}

    public class ComponentDetails {
        private String hintName;

        private Map<String, Object> properties;

        private String resourceType;

        public String getHintName() {
			return null;
		}

        public void setHintName(String hintName) {
		}

        public Map<String, Object> getProperties() {
			return null;
		}

        public void setProperties(Map<String, Object> properties) {
		}

        public ComponentDetails(String hintName) {
            this.hintName = hintName;
            this.properties = new HashMap<String, Object>();
        }

        public String getResourceType() {
			return null;
		}

        public void setResourceType(String resourceType) {
		}

    }
}
