/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.personalization;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * The <tt>Segment</tt> class is a server-side representation of a segment which may be assigned to a user.
 * 
 * <p>
 * Since AEM segments are resolved client-side, this class makes a best-effort approach to represent the segments on the
 * server side, but some precision may be lost. For instance, script segments will not have their scripts loaded by this
 * segment, but will rather be referenced as {@link Kind#ClientOnly} segments.
 * </p>
 * 
 * <p>
 * This class is intended to be used, but not extended by consumers.
 * </p>
 * 
 */
public class Segment {

    public static final String NAME_PERCENTILE = "percentile";

    public enum Kind {

        /**
         * Composite segment kind which holds references to child segments in the <tt>children</tt> property.
         * 
         * <p>
         * The top-level segment is evaluated to true only if all child segments evaluate to true.
         * </p>
         */
        And,

        /**
         * Composite segment kind which holds references to child segments in the <tt>children</tt> property.
         * 
         * <p>
         * The top-level segment is evaluated to true only if any of its child segments evaluate to true.
         * </p>
         */
        Or,

        /**
         * Segment kind which should be directly evaluated based on its name/operator/value combination.
         */
        Direct,

        /**
         * Segment kind which is a reference to a segment defined in an external system
         */
        ExternalReference,

        /**
         * Segment kind which signals that the segment could not be properly represented on the server side.
         * 
         * <p>
         * Instead of being defined by its name/operator/value combination, this segment contains its JCR repository
         * path in the <tt>value</tt> property. Typical example of this kind of segment is a segment which contains a
         * script trait.
         * </p>
         */
        ClientOnly
    }
	
    public static Segment newLogicSegment(Kind kind) {
		return null;
	}

    public static Segment newDirectSegment(String name, String operator, String value) {
		return null;
	}

    public static Segment newDirectSegment(String name, String operator, List<String> values) {
		return null;
	}

    public static Segment newExternalReferenceSegment(String name, String operator, String externalId) {
		return null;
	}

    /**
     * Creates a new <tt>ClientOnly</tt> segment instance
     * 
     * @param path the repository location of the segment
     * @return a new segment instance
     */
    public static Segment newByPathSegment(String path) {
		return null;
	}

	private final String name;
    private final String operator;
    private final List<String> values;
	private final Kind kind;
	
	private final List<Segment> children = new ArrayList<Segment>();
	
    private Segment(String name, String operator, List<String> values, Kind kind) {

        this.name = name;
        this.operator = operator;
        this.values = values;
        this.kind = kind;
    }

	public Kind getKind() {
		return null;
	}
	
	public String getName() {
		return null;
	}
	
	public String getOperator() {
		return null;
	}
	
    public List<String> getValue() {
		return null;
	}
	
	public List<Segment> getChildren() {
		return null;
	}

	public void addChild(Segment resolvedSegment) {
	}

	@Override
	public String toString() {
		return null;
	}

}
