/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.personalization;

import java.util.List;

import javax.jcr.RepositoryException;

import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.commons.json.JSONObject;

import com.day.cq.wcm.api.Page;

/**
 * A service which provides access to campaigns, experiences and targeted content.
 */
public interface TargetedContentManager {

    /**
     * Returns a JSON array of active teasers for a particular campaignPath (legacy teasers)
     * or location.
     *
     * The JSON for each teaser includes:
     * <ul>
     *     <li>path</li>
     *     <li>name</li>
     *     <li>title</li>
     *     <li>campainName (misspelled for legacy reasons)</li>
     *     <li>thumbnail</li>
     *     <li>id</li>
     *     <li>segments[]</li>
     *     <li>tags[]
     *         <ul>
     *             <li>name</li>
     *             <li>title</li>
     *             <li>titlePath</li>
     *             <li>path</li>
     *             <li>tagID</li>
     *         </ul>
     *     </li>
     * </ul>
     *
     * Teasers are ordered first by campaign priority and then by experience page order.
     * This allows the "first" strategy to be used for priority selection.
     *
     * While it might be nice to support more sophisticated strategies (for instance, a
     * more specific segment in a lower-priority campaign ahead of a more generic segment
     * in a higher-priority campaign), there is currently no support for this.
     *
     * @param resolver A resourceResolver from the request (for access control).
     * @param campaignPath A campaign path for a legacy teaser (or null).
     * @param location A named destination (or null for a legacy teaser).
     */
    public JSONObject getTeaserInfo(ResourceResolver resolver, String campaignPath, String location) {
		return null;
	}

    /**
     * Returns a list of campaigns sorted by priority.
     * 
     * @param resourceResolver A resourceResolver from the request (for access control).
     * 
     * @return returns a list of campaigns sorted by priority.
     */
    List<Page> getCampaigns(ResourceResolver resourceResolver) throws RepositoryException {
		return null;
	}

}
