/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.search.eval;

import java.math.BigDecimal;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.jcr.query.Row;

import org.apache.felix.scr.annotations.Component;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.day.cq.search.Predicate;

/**
 * <code>RangePropertyPredicateEvaluator</code> is a generic evaluator for
 * checking a jcr property against an interval. This applies to properties with
 * linear types such as LONG, DOUBLE and DATE (for date there is a specific
 * {@link DateRangePredicateEvaluator} that can handle the specific format).
 * 
 * <p>
 * You can define a lower bound and an upper bound or only one of them. The
 * operation (eg. "lesser than" or "lesser or equals") can also be specified for
 * lower and upper bound individually.
 *
 * <h3>Name:</h3>
 * rangeproperty
 *
 * <h3>Properties:</h3>
 * <dl>
 * <dt>property</dt><dd>relative path to property</dd>
 * <dt>lowerBound</dt><dd>lower bound to check property for</dd>
 * <dt>lowerOperation</dt><dd>">" (default) or ">=", applies to the lowerValue</dd>
 * <dt>upperBound</dt><dd>upper bound to check property for</dd>
 * <dt>upperOperation</dt><dd>"<" (default) or "<=", applies to the lowerValue</dd>
 * <dt>decimal</dt><dd>(boolean) if the checked property is of type Decimal</dd>
 * </dl>
 *
 * @since 5.2
 */
@Component(metatype = false, factory = "com.day.cq.search.eval.PredicateEvaluator/rangeproperty")
public class RangePropertyPredicateEvaluator extends AbstractPredicateEvaluator {
    private static final Logger log = LoggerFactory.getLogger(RangePropertyPredicateEvaluator.class);

    public static final String PROPERTY = "property";
    public static final String LOWER_BOUND = "lowerBound";
    public static final String LOWER_OPERATION = "lowerOperation";
    public static final String UPPER_BOUND = "upperBound";
    public static final String UPPER_OPERATION = "upperOperation";
    public static final String PROPERTY_DECIMAL = "decimal";

    @Override
    public String getXPathExpression(Predicate p, EvaluationContext context) {
		return null;
	}
    
    protected String getXPathExpression(String property, String lowerValue, String lowerOperation, String upperValue,
            String upperOperation) {
				return null;
			}

    @Override
    public String[] getOrderByProperties(Predicate p, EvaluationContext context) {
		return null;
	}

    @Override
    public boolean canXpath(Predicate p, EvaluationContext context) {
		return false;
	}
    
    @Override
    public boolean canFilter(Predicate p, EvaluationContext context) {
		return false;
	}
    
    @Override
    public boolean includes(final Predicate p, final Row row, final EvaluationContext context) {
		return false;
	}
    
    protected boolean includes(final Node node, final String property, final String lowerValue,
            final String lowerOperation, final String upperValue, final String upperOperation, final boolean decimal) {
				return false;
			}
    
}
