/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.search.facets.extractors;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;

import com.day.cq.search.Predicate;
import com.day.cq.search.facets.Facet;
import com.day.cq.search.facets.buckets.SimpleBucket;

/**
 * <code>DistinctBucketsFacetExtractor</code> automatically extracts buckets
 * based on distinct values for given properties (or node paths) in the result.
 * 
 * @since 5.2
 */
public class DistinctValuesFacetExtractor extends PropertyFacetExtractor {
    
    private final String nodePathFilter;

    /**
     * The compiled expression pattern.
     */
    private final Pattern nodePathPattern;
    
    private Predicate predicateTemplate;
    
    private String valueParameterName;
    
    private Map<String, SimpleBucket> bucketMap = new HashMap<String, SimpleBucket>();

//    /**
//     * Creates a new facet extractor with the given <code>nodePathFilter</code>.
//     * The values for this facet are based on the paths of the result nodes.
//     * 
//     * @param filter
//     *            a regular expression to filter the path of result nodes.
//     */
//    public DistinctValuesFacetExtractor(String nodePathFilter) {
//        this(".", nodePathFilter);
//    }

    /**
     * Creates a new facet extractor with the given <code>nodePathFilter</code>.
     * The values for this facet are based on the property values referenced by
     * <code>propertyRelPath</code>. A <code>predicateTemplate</code> must be
     * given which for each detected bucket will be cloned and filled with the
     * specific value of the bucket; the value will be placed in the parameter
     * of the predicate given by <code>valueParameterName</code>.
     * 
     * @param propertyRelPath
     *            a relative path that points to a property. The relative path
     *            is based on the path of the result nodes.
     * @param nodePathFilter
     *            a regular expression to filter the property value or
     *            <code>null</code> to use the property value as is.
     * @param predicateTemplate
     *            the predicate template, which will be cloned and filled with
     *            the specific value for each bucket
     * @param valueParameterName
     *            name of the paramter in the <code>predicateTemplate</code> to
     *            set with the value for the bucket
     */
    public DistinctValuesFacetExtractor(String propertyRelPath, String nodePathFilter, Predicate predicateTemplate, String valueParameterName) {
        super(propertyRelPath);
        
        this.predicateTemplate = predicateTemplate;
        this.valueParameterName = valueParameterName;
        
        if (nodePathFilter == null) {
            // match all
            this.nodePathFilter = "(.*)";
        } else {
            this.nodePathFilter = nodePathFilter;
        }
        
        this.nodePathPattern = Pattern.compile(this.nodePathFilter);
    }

    public Facet getFacet() {
		return null;
	}

    protected void handleValue(Value value) throws RepositoryException {
	}

    /**
     * This allows subclasses to filter the bucket value. If this method returns
     * <code>null</code>, the bucket will be ignored. This original
     * implementation simply returns the given value.
     */
    protected String getBucketValue(String value) {
		return null;
	}

    /**
     * Filters the <code>values</code> by applying the filter of the definition
     * associated with this facet.
     *
     * @param values the values to filter.
     * @param vf the value factory.
     * @return the filtered values.
     * @throws RepositoryException if an error occurs while reading the values.
     */
    protected List<Value> filter(List<Value> values, ValueFactory vf) throws RepositoryException {
		return null;
	}
    
    @Override
    public boolean equals(Object obj) {
		return false;
	}

    @Override
    public int hashCode() {
		return 0;
	}
}
