/*
 * Copyright 1997-2009 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.security;

import org.apache.sling.api.request.RequestParameter;

import javax.jcr.AccessDeniedException;
import javax.jcr.RepositoryException;
import java.net.URL;
import java.util.Map;

/**
 * Simple Utility for User-Account Selfservices.
 * Allows basicly for creation and password handling.
 * The Profile can be read and stored via its API
 */
public interface AccountManager {

    /**
     * @param fragment of the id / name
     * @return User or {@link null null} if no User is related to the fragment
     */
    User findAccount(String fragment) {
		return null;
	}

    /**
     * Convenience to create a new {@link com.day.cq.security.User User} together
     * with setting properties in one method call.
     * Where the keys are the propertie names and the value the 
     *
     * @param uid to create account for
     * @param pwd to set on the new user account
     * @param group to assign membership to the new group
     * @param properties to set
     * @return User or <code>null</code> if no User is related to the fragment
     * @throws javax.jcr.RepositoryException in case of error creating the user
     * @throws javax.jcr.AccessDeniedException in case of missing privileges to create accounts
     * @see UserManager#createUser(String, String, String, String)
     */
    User createAccount(String uid, String pwd, String group,  Map<String, RequestParameter[]> properties) throws AccessDeniedException, RepositoryException {
		return null;
	}

    /**
     * Convenience update an Account with a set of properties
     * with setting properties in one method call.
     * 
     * @param user to update
     * @param properties to set
     * @throws RepositoryException in case of error updateing the account
     */
    void updateAccount(User user,  Map<String, RequestParameter[]> properties) throws RepositoryException {
	}

    /**
     * Convenenience to either search and update an existing account or create it
     *
     * @param id of the account to update or to use for creation
     * @param pwd   in case the acoount should be created
     * @param group to assign to a newly created user
     * @param properties to set to the user
     * @return account modified
     * @throws javax.jcr.RepositoryException in case of error creating the user
     * @throws javax.jcr.AccessDeniedException in case of missing privileges to create accounts
     * @see #findAccount(String)
     * @see #createAccount(String, String, String, Map)
     * @see #updateAccount(User, Map)
     */
    User getOrCreateAccount(String id, String pwd, String group, Map<String, RequestParameter[]> properties) throws AccessDeniedException, RepositoryException {
		return null;
	}

    /**
     * Reset the Password of the Account to a randomly generated
     *
     * @param user to reset the password for
     * @return the newly generated password
     * @throws javax.jcr.RepositoryException in case of error creating the user
     * @throws javax.jcr.AccessDeniedException in case of missing privileges to create accounts
     */
    String resetPassword(User user) throws AccessDeniedException, RepositoryException {
		return null;
	}

    /**
     * Reset the Password of the Account to a randomly generated, requires a key
     * in order to verify, that the reset has been requested by the key owner.
     * The key is the result of the call to request o password reset
     * {@link #requestPasswordReset(User, URL) }
     *
     * @param user to reset the password for
     * @param key required to reset the password
     * @return the newly generated password or null on failure
     * @throws javax.jcr.RepositoryException in case of error creating the user
     * @throws javax.jcr.AccessDeniedException in case of missing privileges to create accounts
     * @see #requestPasswordReset(User, URL)
     */
    String resetPassword(User user, String key) throws AccessDeniedException, RepositoryException {
		return null;
	}

    /**
     * Set the Password of the Account to a given password. Requires a key
     * in order to verify, that the reset has been requested by the key owner.
     * The key is the result of the call to request o password reset
     * {@link #requestPasswordReset(User, URL) }
     *
     * @param user to reset the password for
     * @param key required to set the password
     * @param pwd new password
     * @return true on success, false on failure
     * @throws javax.jcr.RepositoryException in case of error creating the user
     * @throws javax.jcr.AccessDeniedException in case of missing privileges to create accounts
     * @see #requestPasswordReset(User, URL)
     */
    boolean setPassword(User user, String key, String pwd) throws AccessDeniedException, RepositoryException {
		return false;
	}

    /**
     * Request a password rest. Results in a key that has to be presented in order
     * to finally set the Password to some new {@link #resetPassword(User, String) value}.
     * 
     * @param user to reset the password for
     * @param requestUrl the url called to request the reset
     * @throws javax.jcr.RepositoryException in case of error creating the user
     * @throws javax.jcr.AccessDeniedException in case of missing privileges to create accounts
     * @see #resetPassword(User, String)
     * @return a key that allows to reset the password
     */
    boolean requestPasswordReset(User user, URL requestUrl) throws RepositoryException {
		return false;
	}

    /**
     * Request for creating a new account. Results in a mail being sent to the requester which
     * contains further information how to complete account generation.
     *
     * @param email  email address of requester
     * @param requestUrl the url called to request the reset
     * @return  true if account request succeeded, false otherwise
     */
    boolean requestAccount(String email, URL requestUrl) throws RepositoryException {
		return false;
	}

    /**
     * Simple utility to send a mail to a given user account.
     * Adds the ability to replace variables within the mail body with properties
     * from the User.
     * This Properties can be addressed by useing their name as variabl name.
     * 
     * @param user to receive the mail
     * @param from the from e-mail address
     * @param subject of the mail to send
     * @param body of the mail to send
     * @param replace map of replacements for variables, with variable name as key
     *        and value as value
     * @return true if the mail could be send
     * @throws RepositoryException in case of error accessing the repository
     */
    boolean sendMail(User user, String from, String subject, String body, Map<String, String> replace) throws RepositoryException {
		return false;
	}
}
