/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.security.util;

import com.day.cq.commons.JSONWriterUtil;
import com.day.cq.replication.ReplicationQueue;
import com.day.cq.replication.ReplicationStatus;
import com.day.cq.security.Authorizable;
import com.day.cq.security.Group;
import com.day.cq.security.User;
import com.day.cq.security.impl.CQAuthorizableImpl;
import com.day.cq.security.impl.profile.CqProfile;
import com.day.cq.security.profile.Profile;
import com.day.cq.security.profile.ProfileManager;
import com.day.cq.xss.XSSProtectionService;

import org.apache.sling.api.SlingException;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.commons.json.JSONException;
import org.apache.sling.commons.json.io.JSONWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import java.io.Writer;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import static com.day.cq.commons.jcr.JcrConstants.JCR_CONTENT;
import static com.day.cq.commons.jcr.JcrConstants.JCR_CREATED;
import static com.day.cq.commons.jcr.JcrConstants.JCR_CREATED_BY;
import static com.day.cq.commons.jcr.JcrConstants.JCR_LASTMODIFIED;

/**
 * Little writer knowing some of the Authorizables special cases that do not
 * compile a Bean-convention
 *
 * @deprecated cq 5.5 Use platform functionality instead.
 */
public class AuthorizableJSONWriter {

    /**
     * Maps user property names to profile properties. This is only needed
     * for backward compatibility reasons. See bug# 26797.
     */
    private static final Map<String, String> USER_TO_PROFILE_PROPS;

    static {
        Map<String, String> tmp = new HashMap<String, String>();
        tmp.put(Authorizable.PROPERTY_DESCRIPTION, CqProfile.NAME_ABOUT_ME);
        tmp.put(Authorizable.PROPERTY_EMAIL, CqProfile.NAME_E_MAIL);
        tmp.put(Authorizable.PROPERTY_FIRST_NAME, CqProfile.NAME_GIVEN_NAME);
        tmp.put(Authorizable.PROPERTY_LAST_NAME, CqProfile.NAME_FAMILY_NAME);
        USER_TO_PROFILE_PROPS = tmp;
    }

    private final static Logger log = LoggerFactory.getLogger(AuthorizableJSONWriter.class);

    public static final String GROUP = "group";

    public static final String USER = "user";

    public static final String KEY_HOME_PATH = "home";

    private final Session session;

    private final ProfileManager pMgr;

    private Set<String> outputProps;

    private XSSProtectionService xss;

    private boolean basic;

    private int membersLimit = -1;

    /**
     * Creates a default authorizable writer.
     */
    public AuthorizableJSONWriter() {
        this(null, null, null, null);
    }
    
    /**
     * todo
     * @param pMgr the profile manager
     * @param session the session
     * @param outputProps the output properties
     * @param xss the XSS protection service
     * @since 5.4
     */
    public AuthorizableJSONWriter(ProfileManager pMgr, Session session, String[] outputProps, XSSProtectionService xss) {
        this.session = session;
        this.pMgr = pMgr;
        setOutputProps(outputProps);
        this.xss = xss;
    }

    /**
     * Sets the members limit.
     * @param membersLimit the limit
     */
    public void setMembersLimit(int membersLimit) {
	}

    /**
     * Set the properties that will be present in the output.
     * @param outputProps the output properties
     * @since 5.4
     */
    public void setOutputProps(String[] outputProps) {
	}

    /**
     * Define if only basic information should be written
     * @param basic <code>true</code> to only dump basic information
     */
    public void setBasic(boolean basic) {
	}

    /**
     * Set XSS protection service to use to prevent from XSS attacks
     *
     * @param xss
     * @since 5.4
     */
    public void setXss(XSSProtectionService xss) {
	}

    public void write(JSONWriter writer, Authorizable authorizable) throws JSONException {
	}

    public void write(Writer out, Authorizable authorizable) throws JSONException {
	}

    public void writeTable(Writer out, Authorizable authorizable) throws JSONException, RepositoryException {
	}

    private void write(JSONWriter writer, String key, String value) throws JSONException {
	}

    private void write(JSONWriter writer, String key, long value) throws JSONException {
	}

    private void writeRows(JSONWriter json, Authorizable authorizable) throws JSONException, RepositoryException {
	}

    private static void writeColumnHeader(JSONWriter json, Authorizable authorizable) throws JSONException {
	}

    private void writeAuthorizables(JSONWriter writer, String key, Iterator<? extends Authorizable> auths, int limit, JSONWriterUtil.WriteMode writeMode)
            throws JSONException {
			}

    private void writeModificationDates(JSONWriter writer, Authorizable authorizable) throws JSONException {
	}

    private void writeReplication(JSONWriter writer, Authorizable authorizable) throws JSONException {
	}

    private String getFormattedName(String id) {
		return null;
	}

    private boolean includeKey(String key) {
		return false;
	}

}
