/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.wcm.designimporter;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;

import com.day.cq.wcm.api.Page;
import com.day.cq.wcm.designimporter.DesignImportException;

/**
 * The <code>CanvasPageManager</code> defines the interface to provide methods on created Canvas Page.
 * Interface provides both mobile and desktop APIs. 
 * 
 *
 */
@Deprecated
public interface CanvasPageManager {

    /**
     * Deletes the Canvas Page along with its artifacts
     * 
     * @param page The page to be deleted
     * @throws DesignImportException is thrown if the deletion fails
     */
    public void deleteCanvasPage(Page page) throws DesignImportException {
	}

    /**
     * Deletes the Canvas Page along with its artifacts
     * 
     * @param resource The resource to be deleted
     * @throws DesignImportException is thrown if the deletion fails
     */
    public void deleteCanvasPage(Resource resource) throws DesignImportException {
	}

    /**
     * Deletes the Canvas Page artifacts i.e. Canvas and Design components
     * 
     * @param page The page whose artifacts are to be deleted
     * @throws DesignImportException is thrown if the deletion fails
     */
    public void deleteCanvasPageArtifacts(Page page) throws DesignImportException {
	}

    /**
     * Deletes the Canvas Page artifacts i.e. Canvas and Design components
     * 
     * @param resource The resource whose artifacts are to be deleted
     * @throws DesignImportException is thrown if the deletion fails
     */
    public void deleteCanvasPageArtifacts(Resource resource) throws DesignImportException {
	}

    /**
     * Retrieves the mobile page of this canvas page
     * 
     * @param canvasPageResource The resource whose mobile version is to be retrieved
     * @return Page or <code>null</code>
     */
    public Page getMobileVersion(Resource canvasPageResource) {
		return null;
	}

    /**
     * Retrieves the mobile page of this canvas page
     * 
     * @param canvasPage The canvas page whose mobile version is to be retrieved
     * @return Page or <code>null</code>
     */
    public Page getMobileVersion(Page canvasPage) {
		return null;
	}

    /**
     * Retrieves the mobile page of this canvas page
     * 
     * @param resolver The resource resolver
     * @param canvasPagePath The path of the canvas page whose mobile version is to be retrieved
     * @return Page or <code>null</code>
     */
    public Page getMobileVersion(ResourceResolver resolver, String canvasPagePath) {
		return null;
	}

    /**
     * Retrieves the desktop canvas page of this mobile canvas page
     * 
     * @param mobileCanvasPageResource The mobile canvas page resource whose desktop version is to be retrieved
     * @return Page or <code>null</code>
     */
    public Page getDesktopVersion(Resource mobileCanvasPageResource) {
		return null;
	}

    /**
     * Retrieves the desktop canvas page of this mobile canvas page
     * 
     * @param mobileCanvasPage The Mobile canvasPage whose desktop version is to be retrieved
     * @return Page or <code>null</code>
     */
    public Page getDesktopVersion(Page mobileCanvasPage) {
		return null;
	}

    /**
     * Retrieves the desktop canvas page of this mobile canvas page
     * 
     * @param resolver The resource resolver
     * @param mobileCanvasPagePath The path of the mobile canvas page whose desktop version is to be retrieved
     * @return Page or <code>null</code>
     */
    public Page getDesktopVersion(ResourceResolver resolver, String mobileCanvasPagePath) {
		return null;
	}

    /**
     * Retrieves the canvas page if the passed path points to a canvas page else returns null
     * 
     * @param resolver The resource resolver
     * @param path The path to the page
     * @return Page or <code>null</code>
     */
    public Page getCanvasPage(ResourceResolver resolver, String path) {
		return null;
	}

    /**
     * Determines whether the passed path points to a canvas page or not
     * 
     * @param resolver The resource resolver
     * @param path The path to the page
     * @return <code>true</code> if path points to canvas page else returns <code>false</code>
     */
    public boolean isCanvasPage(ResourceResolver resolver, String path) {
		return false;
	}

    /**
     * Determines whether the passed page points to a canvas page or not
     * 
     * @param resolver The resource resolver
     * @param page The page to be tested
     * @return <code>true</code> if page points to canvas page else returns <code>false</code>
     */
    public boolean isCanvasPage(Page page) {
		return false;
	}

    /**
     * Determines whether the passed resource points to a canvas page or not
     * 
     * @param resolver The resource resolver
     * @param resource The resource to be tested
     * @return <code>true</code> if resource points to canvas page else returns <code>false</code>
     */
    public boolean isCanvasPage(Resource resource) {
		return false;
	}

    /**
     * Determines whether the passed path points to a mobile canvas page or not
     * 
     * @param resolver The resource resolver
     * @param canvasPagePath The path to the page
     * @return <code>true</code> if canvasPagePath points to mobile canvas page else returns <code>false</code>
     */
    public boolean isMobileCanvasPage(ResourceResolver resolver, String canvasPagePath) {
		return false;
	}

    /**
     * Determines whether the passed page points to a mobile canvas page or not
     * 
     * @param resolver The resource resolver
     * @param canvasPage The page to be tested
     * @return <code>true</code> if canvasPage points to mobile canvas page else returns <code>false</code>
     */
    public boolean isMobileCanvasPage(Page canvasPage) {
		return false;
	}

    /**
     * Determines whether the passed resource points to a mobile canvas page or not
     * 
     * @param resolver The resource resolver
     * @param canvasPageResource The resource to be tested
     * @return <code>true</code> if canvasPageResource points to mobile canvas page else returns <code>false</code>
     */
    public boolean isMobileCanvasPage(Resource canvasPageResource) {
		return false;
	}

    /**
     * Determines whether the passed canvasPagePath points to a desktop canvas page or not
     * 
     * @param resolver The resource resolver
     * @param canvasPagePath The path to the page
     * @return <code>true</code> if canvasPagePath points to desktop canvas page else returns <code>false</code>
     */
    public boolean isDesktopCanvasPage(ResourceResolver resolver, String canvasPagePath) {
		return false;
	}

    /**
     * Determines whether the passed page points to a desktop canvas page or not
     * 
     * @param resolver The resource resolver
     * @param canvasPage The page to be tested
     * @return <code>true</code> if canvasPage points to desktop canvas page else returns <code>false</code>
     */
    public boolean isDesktopCanvasPage(Page canvasPage) {
		return false;
	}

    /**
     * Determines whether the passed canvasPageResource points to a desktop canvas page or not
     * 
     * @param resolver The resource resolver
     * @param canvasPageResource The resource to be tested
     * @return <code>true</code> if canvasPageResource points to desktop canvas page else returns <code>false</code>
     */
    public boolean isDesktopCanvasPage(Resource canvasPageResource) {
		return false;
	}
}
