/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.foundation;

import java.awt.Color;
import java.awt.Rectangle;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ValueMap;

import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.graphics.Graph;
import com.day.cq.graphics.chart.Axis;
import com.day.cq.graphics.chart.BarChart;
import com.day.cq.graphics.chart.Data;
import com.day.cq.graphics.chart.Grid;
import com.day.cq.graphics.chart.LineChart;
import com.day.cq.graphics.chart.PieChart;
import com.day.cq.wcm.api.designer.Style;
import com.day.cq.wcm.foundation.Table.Cell;
import com.day.cq.wcm.foundation.Table.Row;
import com.day.image.Font;
import com.day.image.Layer;
import com.day.image.LineStyle;

/**
 * Provides convenience methods for displaying charts.
 */
public class Chart {
    /**
     * value of the pie chart type
     */
    public static final String PIE_CHART_TYPE = "pie";

    /**
     * value of the bar chart type
     */
    public static final String BAR_CHART_TYPE = "bar";

    /**
     * value of the line chart type
     */
    public static final String LINE_CHART_TYPE = "line";

    /**
     * name of the title property
     */
    public static final String PN_TITLE = JcrConstants.JCR_TITLE;

    /**
     * name of the chart type property
     */
    public static final String PN_CHART_TYPE = "chartType";

    /**
     * name of the alt name property
     */
    public static final String PN_ALT = "chartAlt";

    /**
     * name of the data property
     */
    public static final String PN_DATA = "chartData";

    /**
     * name of the width property
     */
    public static final String PN_WIDTH = "chartWidth";

    /**
     * name of the width property
     */
    public static final String PN_HEIGHT = "chartHeight";

    /**
     * name of the start angle property for pie charts
     */
    public static final String PN_PC_START_ANGLE = "pieStartAngle";

    /**
     * name of the line width property for line charts
     */
    public static final String PN_LC_LINE_WIDTH = "lineLineWidth";

    /**
     * name of the bar width property for bar charts
     */
    public static final String PN_BC_BAR_WIDTH = "barBarWidth";

    /**
     * name of the bar spacing property for bar charts
     */
    public static final String PN_BC_BAR_SPACING = "barBarSpacing";

    /**
     * name of the background color property
     */
    public static final String PN_BG_COLOR = "bgColor";

    /**
     * name of the label color property
     */
    public static final String PN_LABEL_COLOR = "labelColor";

    /**
     * name of the line color property
     */
    public static final String PN_LINE_COLOR = "lineColor";

    /**
     * name of the data color property
     */
    public static final String PN_DATA_COLORS = "dataColors";
    
    /** Default data color */
    public static final Color DEFAULT_DATA_COLOR = Color.BLUE;

    /**
     * internal properties
     */
    private final ValueMap properties;

    /**
     * map of overlaid properties
     */
    private Map<String, Object> overlaid = new HashMap<String, Object>();

    /**
     * map of default properties
     */
    private Map<String, Object> defaults = new HashMap<String, Object>();

    /**
     * internal data table
     */
    private Table table;

    /**
     * Creates a new chart based on the given resource.
     *
     * @param resource resource of the chart
     * @throws IllegalArgumentException if the given resource is not adaptable to node.
     */
    public Chart(Resource resource) {
        this.properties = resource == null ? ValueMap.EMPTY : resource
                .adaptTo(ValueMap.class);

        defaults.put(PN_WIDTH, 400);
        defaults.put(PN_HEIGHT, 200);
        defaults.put(PN_LINE_COLOR, "808080");
        defaults.put(PN_LABEL_COLOR, "808080");
        defaults.put(PN_CHART_TYPE, PIE_CHART_TYPE);
        defaults.put(PN_PC_START_ANGLE, 0.0);
        defaults.put(PN_LC_LINE_WIDTH, 2.0f);
        defaults.put(PN_BC_BAR_WIDTH, 20);
        defaults.put(PN_BC_BAR_SPACING, 0);

        initChartData();
    }

    /**
     * Checks if this chart has content
     *
     * @return <code>true</code> if this chart has content
     */
    public boolean hasData() {
		return false;
	}

    /**
     * Returns the chart alt name as defined by {@value #PN_ALT}
     *
     * @return the alt name
     * @see #PN_ALT
     */
    public String getAlt() {
		return null;
	}

    /**
     * Get a property either from the overlaid map or the underlying properties.
     *
     * @param name name of the property
     * @return value of the property or null
     */
    public Object get(String name) {
		return null;
	}

    /**
     * Get a string property either from the overlaid map or the underlying
     * properties.
     *
     * @param name name of the property
     * @return string value of the property or null
     */
    public String getString(String name) {
		return null;
	}

    /**
     * Get a integer property either from the overlaid map or the underlying
     * properties.
     *
     * @param name name of the property
     * @return integer value of the property or null
     */
    public Integer getInt(String name) {
		return null;
	}

    /**
     * Get a double property either from the overlaid map or the underlying
     * properties.
     *
     * @param name name of the property
     * @return double value of the property or null
     */
    public Double getDouble(String name) {
		return null;
	}

    /**
     * Get a float property either from the overlaid map or the underlying
     * properties.
     *
     * @param name name of the property
     * @return double value of the property or null
     */
    public Float getFloat(String name) {
		return null;
	}

    /**
     * Get a color either from the overlaid map or the underlying properties.
     *
     * @param name name of the property
     * @return Color color of the property or null
     */
    public Color getColor(String name) {
		return null;
	}

    /**
     * Loads several definitions from style.
     *
     * @param style style to load definitions from
     */
    public void loadStyleData(Style style) {
	}

    /**
     * Returns the chart layer of this chart.
     *
     * @return the layer
     * @throws IOException         if an I/O error occurs.
     * @throws RepositoryException if a repository error occurs.
     */
    public Layer getChartLayer() throws IOException, RepositoryException {
		return null;
	}

    /**
     * Returns the legend layer of this chart.
     *
     * @return the layer
     * @throws IOException         if an I/O error occurs.
     * @throws RepositoryException if a repository error occurs.
     */
    public Layer getLegendLayer() throws IOException, RepositoryException {
		return null;
	}

    private Color bgColor;
    private Color labelColor;
    private Color lineColor;

    private Font labelFont;
    private Font legendFont;
    private Font legendFontBold;

    private LineStyle lineStyle;
    private LineStyle gridLineStyle;

    private Double labelLsDef;
    private int labelLs;
    private double labelCs;

    private Double legendLsDef;
    private int legendLs;
    private double legendCs;

    ArrayList<String> dataColors = new ArrayList<String>();

    private void initStyle() {
	}

    private Color getDataColor(int index) {
		return null;
	}

    private void setDataColors(Graph graph) {
	}

    private Graph getGraph() {
		return null;
	}

    private int initXAxis(Graph graph, int yaWidth, int xaHeight) {
		return 0;
	}

    private int initYAxis(Graph graph, int yaWidth, int xaHeight) {
		return 0;
	}

    private String[] labels;
    private String[] legends;
    private double[][] samples;
    private double maxValue = 0;

    private void initChartData() {
	}
}