/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */

package com.day.cq.wcm.foundation;

import java.io.PrintWriter;
import java.util.Map;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;

import com.day.cq.commons.ImageResource;
import com.day.cq.wcm.api.components.DropTarget;
import com.day.cq.wcm.api.components.Component;
import com.day.cq.wcm.api.designer.Style;
import com.day.cq.wcm.commons.WCMUtils;
import com.day.cq.dam.api.Asset;
import com.day.cq.dam.api.Rendition;
import com.day.text.Text;

/**
 * Provides convenience methods for displaying images.
 */
public class Image extends ImageResource {

    /**
     * name of the image map property
     */
    public static final String PN_IMAGE_MAP = "imageMap";

    /**
     * id of the image map
     */
    private String imageMapId;

    /**
     * deserialized image map
     */
    private ImageMap imageMap;

    /**
     * flag controlling if a placeholder image should be rendered if this image
     * has no content.
     */
    private boolean noPlaceholder;

    /**
     * Creates a new image based on the given resource. the image properties are
     * considered to 'on' the given resource.
     *
     * @param resource resource of the image
     * @throws IllegalArgumentException if the given resource is not adaptable to node.
     */
    public Image(Resource resource) {
        super(resource);

        // init default params
        imageMap = null;
        if (properties.containsKey(PN_IMAGE_MAP)) {
            try {
                String mapDefinition = properties.get(PN_IMAGE_MAP, "");
                if (mapDefinition.length() > 0) {
                    imageMap = ImageMap.fromString(mapDefinition);
                    imageMapId = "map_"
                            + Math.round(Math.random() * Integer.MAX_VALUE) + "_"
                            + System.currentTimeMillis();
                }
            } catch (IllegalArgumentException iae) {
                // ignore wrong map definition
                imageMap = null;
                imageMapId = null;
            }
        }
    }

    /**
     * Creates a new image based on the given resource. the image properties are
     * considered to 'on' the given resource unless <code>imageName</code>
     * is specified. then the respective child resource holds the image
     * properties.
     *
     * @param resource  current resource
     * @param imageName name of the image resource
     * @throws IllegalArgumentException if the given resource is not adaptable to node.
     */
    public Image(Resource resource, String imageName) {
        this(getRelativeResource(resource, imageName));
    }

    /**
     * Sets the drop target id for this image. the id is added as css class
     * to the image attribute. and has the format:
     * "{@value DropTarget#CSS_CLASS_PREFIX}{id}-{classifier}"
     *
     * @param id the drop target id as configured in edit config.
     * @param classifier optional classifier
     */
    public void setDropTargetId(String id, String classifier) {
	}

    /**
     *
     * Sets the drop target id for this image, using the name of the resources
     * as path.
     *
     * @param id the drop target id as configured in edit config.
     */
    public void setDropTargetId(String id) {
	}

    /**
     * Returns the placeholder flag.
     * @return <code>true</code> if no placeholder for empty content should be
     *         drawn.
     */
    public boolean hasNoPlaceholder() {
		return false;
	}

    /**
     * Sets the placeholder flag.
     * @param noPlaceholder if <code>true</code> no placeholder for empty content
     *        is used.
     */
    public void setNoPlaceholder(boolean noPlaceholder) {
	}

    /**
     * Loads several definitions from style.
     * <p>
     * Currently, the minimum/maximum width and height are transferred from the given style.
     *
     * @param style style to load definitions from
     */
    public void loadStyleData(Style style) {
	}

    /**
     * {@inheritDoc}
     *
     * @return <code>true</code> if {@link #hasNoPlaceholder()} is <code>false</code>
     *         or the image has content.
     */
    @Override
    protected boolean canDraw() {
		return false;
	}

    @Override
    protected Map<String, String> getImageTagAttributes() {
		return null;
	}

    /**
     * {@inheritDoc}
     */
    @Override
    protected void doDraw(PrintWriter w)  {
	}

    /**
     * {@inheritDoc}
     *
     * Checks if the component of the resource provides an icon for the
     * respective type.
     */
    @Override
    public String getIconPath() {
		return null;
	}


    /**
     * {@inheritDoc}
     *
     * Checks if the resource is an asset and returns the correct rendition.
     */
    @Override
    protected Resource getReferencedResource(String path) {
		return null;
	}

}