/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */

package com.day.cq.wcm.foundation;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.text.FieldPosition;
import java.text.Format;
import java.text.ParsePosition;

/**
 * The <code>AutoFormatter</code> class implements the automatic conversion
 * of line endings to <code>&lt;br></code> HTML lists.
 * <p/>
 * This implementation only supports automatically converting bullet and
 * numbered lists as well as line breaking.
 */
public class TextFormat extends Format {

    /**
     * The default line breaking tag
     */
    private static final String DEFAULT_BR = "<br>";

    /**
     * The default opening tag for ordered lists
     */
    private static final String DEFAULT_TAG_OL_OPEN = "<ol start=\"%s\">";

    /**
     * The default closing tag for ordered lists
     */
    private static final String DEFAULT_TAG_OL_CLOSE = "</ol>";

    /**
     * The default opening tag for items in ordered lists
     */
    private static final String DEFAULT_TAG_OL_ITEM_OPEN = "<li>";

    /**
     * The default closing tag for items in ordered lists
     */
    private static final String DEFAULT_TAG_OL_ITEM_CLOSE = "</li>";

    /**
     * The default opening tag for unordered lists
     */
    private static final String DEFAULT_TAG_UL_OPEN = "<ul>";

    /**
     * The default closing tag for unordered lists
     */
    private static final String DEFAULT_TAG_UL_CLOSE = "</ul>";

    /**
     * The default opening tag for items in unordered lists
     */
    private static final String DEFAULT_TAG_UL_ITEM_OPEN = "<li>";

    /**
     * The default closing tag for items in unordered lists
     */
    private static final String DEFAULT_TAG_UL_ITEM_CLOSE = "</li>";

    /**
     * The normal text formatting state - no list in action
     */
    private static final int STATE_NORMAL = 1;

    /**
     * State where a <em>&lt;ol&gt;</em> tag is currently open
     */
    private static final int STATE_OL = 3;

    /**
     * State where a <em>&lt;ul&gt;</em> tag is currently open
     */
    private static final int STATE_UL = 4;

    /**
     * The tag used for line breaks, default {@link #DEFAULT_BR}
     */
    private String tagBr = DEFAULT_BR;

    /**
     * The tag used to start ordered lists, default {@link #DEFAULT_TAG_OL_OPEN}
     */
    private String tagOlOpen = DEFAULT_TAG_OL_OPEN;

    /**
     * The tag used to end ordered lists, default {@link #DEFAULT_TAG_OL_CLOSE}
     */
    private String tagOlClose = DEFAULT_TAG_OL_CLOSE;

    /**
     * The tag used to start items in ordered lists, default {@link #DEFAULT_TAG_OL_ITEM_OPEN}
     */
    private String tagOlItemOpen = DEFAULT_TAG_OL_ITEM_OPEN;

    /**
     * The tag used to close items in ordered lists, default {@link #DEFAULT_TAG_OL_ITEM_CLOSE}
     */
    private String tagOlItemClose = DEFAULT_TAG_OL_ITEM_CLOSE;

    /**
     * The tag used to start unordered lists, default {@link #DEFAULT_TAG_UL_OPEN}
     */
    private String tagUlOpen = DEFAULT_TAG_UL_OPEN;

    /**
     * The tag used to end ordered lists, default {@link #DEFAULT_TAG_UL_OPEN}
     */
    private String tagUlClose = DEFAULT_TAG_UL_CLOSE;

    /**
     * The tag used to start items in unordered lists, default {@link #DEFAULT_TAG_UL_ITEM_OPEN}
     */
    private String tagUlItemOpen = DEFAULT_TAG_UL_ITEM_OPEN;

    /**
     * The tag used to close items in unordered lists, default {@link #DEFAULT_TAG_UL_ITEM_CLOSE}
     */
    private String tagUlItemClose = DEFAULT_TAG_UL_ITEM_CLOSE;

    private boolean autoBr = true;

    private boolean autoList = true;

    private boolean escapeXML = true;

    public String getTagBr() {
		return null;
	}

    public void setTagBr(String tagBr) {
	}

    public String getTagOlOpen() {
		return null;
	}

    public void setTagOlOpen(String tagOlOpen) {
	}

    public String getTagOlClose() {
		return null;
	}

    public void setTagOlClose(String tagOlClose) {
	}

    public String getTagOlItemOpen() {
		return null;
	}

    public void setTagOlItemOpen(String tagOlItemOpen) {
	}

    public String getTagOlItemClose() {
		return null;
	}

    public void setTagOlItemClose(String tagOlItemClose) {
	}

    public String getTagUlOpen() {
		return null;
	}

    public void setTagUlOpen(String tagUlOpen) {
	}

    public String getTagUlClose() {
		return null;
	}

    public void setTagUlClose(String tagUlClose) {
	}

    public String getTagUlItemOpen() {
		return null;
	}

    public void setTagUlItemOpen(String tagUlItemOpen) {
	}

    public String getTagUlItemClose() {
		return null;
	}

    public void setTagUlItemClose(String tagUlItemClose) {
	}

    public boolean isAutoBr() {
		return false;
	}

    public void setAutoBr(boolean autoBr) {
	}

    public boolean isAutoList() {
		return false;
	}

    public void setAutoList(boolean autoList) {
	}

    public boolean isEscapeXML() {
		return false;
	}

    public void setEscapeXML(boolean escapeXML) {
	}

    /**
     * The <code>TextFormat</code> class does not support parsing, so an
     * <code>UnsupportedOperationException</code> is thrown when trying to
     * parse.
     *
     * @param source The source string to parse. Ignored.
     * @param status The position to define parsing. Ignored.
     * @throws UnsupportedOperationException as it is not yet implemented.
     */
    public Object parseObject(String source, ParsePosition status) {
		return null;
	}

    /**
     * Formats the object according to the standard modifiers, which are
     * automatic line breaks and list formatting. This implementation only
     * supports strings and completely ignores the pos parameter.
     *
     * @param obj        The object to format, which must be a String or a
     *                   <code>ClassCastException</code> will be thrown.
     * @param toAppendTo Where to append the formatted data. If <code>null</code>,
     *                   a new string buffer is allocated.
     * @param pos        Formatting position information. Not used.
     * @return a <code>StringBuffer</code> containing the formatted string. This
     *         is either the same as <code>toAppendTo</code> or a newly
     *         allocated <code>StringBuffer</code> if the parameter is
     *         <code>null</code>.
     */
    public StringBuffer format(Object obj, StringBuffer toAppendTo,
                               FieldPosition pos) {
								return null;
							}

    private CharSequence escape(String string) {
		return null;
	}
}
