/*
 * Copyright 1997-2010 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.wcm.foundation.forms;

import com.adobe.granite.xss.XSSAPI;
import com.day.cq.commons.jcr.JcrConstants;
import com.day.cq.wcm.api.LanguageManager;
import com.day.cq.wcm.api.WCMMode;
import com.day.cq.wcm.api.components.ComponentContext;
import com.day.cq.wcm.foundation.ELEvaluator;
import com.day.cq.wcm.foundation.forms.impl.FormChooserServlet;
import com.day.cq.wcm.foundation.forms.impl.FormsManagerImpl;
import com.day.cq.wcm.foundation.forms.impl.FormsUtil;
import com.day.cq.wcm.foundation.forms.impl.JspSlingHttpServletResponseWrapper;
import com.day.cq.wcm.foundation.forms.impl.ResourceWrapper;
import com.day.cq.widget.HtmlLibraryManager;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.request.RequestDispatcherOptions;
import org.apache.sling.api.request.RequestParameter;
import org.apache.sling.api.request.RequestParameterMap;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.api.scripting.SlingBindings;
import org.apache.sling.api.scripting.SlingScriptHelper;
import org.apache.sling.commons.json.JSONException;
import org.apache.sling.commons.json.jcr.JsonItemWriter;
import org.apache.sling.jcr.resource.JcrResourceConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

/**
 * Helper class for the forms components.
 */
public class FormsHelper {

    /** The logger. */
    private static final Logger LOGGER = LoggerFactory.getLogger(FormsHelper.class.getName());

    private FormsHelper() {
        // no instances
    }

    public static final String REQ_ATTR_GLOBAL_LOAD_MAP      = "cq.form.loadmap";
    public static final String REQ_ATTR_GLOBAL_LOAD_RESOURCE = "cq.form.loadresource";
    public static final String REQ_ATTR_EDIT_RESOURCES       = FormResourceEdit.RESOURCES_ATTRIBUTE;
    public static final String REQ_ATTR_CLIENT_VALIDATION    = "cq.form.clientvalidation";
    public static final String REQ_ATTR_FORMID               = "cq.form.id";
    public static final String REQ_ATTR_WRITTEN_JAVASCRIPT   = "cq.form.javascript";
    public static final String REQ_ATTR_ACTION_SUFFIX        = "cq.form.action.suffix";
    public static final String REQ_ATTR_FORWARD_PATH         = "cq.form.forward.path";
    public static final String REQ_ATTR_FORWARD_OPTIONS      = "cq.form.forward.options";
    public static final String REQ_ATTR_IS_INIT              = "cq.form.init";
    public static final String REQ_ATTR_READ_ONLY            = "cq.form.readonly";
    public static final String REQ_ATTR_REDIRECT             = "cq.form.redirect";
    public static final String REQ_ATTR_REDIRECT_TO_REFERRER = FormsConstants.REQUEST_ATTR_REDIRECT_TO_REFERRER;
    public static final String REQ_ATTR_PROP_WHITELIST         = "cq.form.prop.whitelist";

    /**
     * Signal the start of the form.
     * Prepare the request object, write out the client javascript (if the form
     * is configured accordingly) and write out the start form tag and hidden fields:
     * {@link FormsConstants#REQUEST_PROPERTY_FORMID} with the value of the form id.
     * {@link FormsConstants#REQUEST_PROPERTY_FORM_START} with the relative path to the form start par
     * and <code>_charset_</code> with the value <code>UTF-8</code>
     *
     * @param request The current request.
     * @param response The current response.
     * @param out The jsp writer.
     * @deprecated Use {@link #startForm(SlingHttpServletRequest, SlingHttpServletResponse)}
     */
    @Deprecated
    public static void startForm(final SlingHttpServletRequest request,
                                 final SlingHttpServletResponse response,
                                 final JspWriter out)
    throws IOException, ServletException {
	}

    /**
     * Signal the start of the form.
     * Prepare the request object, write out the client javascript (if the form
     * is configured accordingly) and write out the start form tag and hidden fields:
     * {@link FormsConstants#REQUEST_PROPERTY_FORMID} with the value of the form id.
     * {@link FormsConstants#REQUEST_PROPERTY_FORM_START} with the relative path to the form start par
     * and <code>_charset_</code> with the value <code>UTF-8</code>
     *
     * @param request The current request.
     * @param response The current response.
     * @since 5.3
     */
    public static void startForm(final SlingHttpServletRequest request,
                                 final SlingHttpServletResponse response)
    throws IOException, ServletException {
	}

    /**
     * Initialize the current request object.
     * Provide the required environment for form elements.
     */
    private static void initialize(final SlingHttpServletRequest request,
                                   final Resource formResource,
                                   final SlingHttpServletResponse response)
    throws IOException, ServletException {
	}

    /**
     * Runs/includes a script of the form action.
     */
    public static void runAction(final String actionType,
                                 final String scriptName,
                                 final Resource formResource,
                                 final SlingHttpServletRequest request,
                                 final SlingHttpServletResponse response) throws IOException, ServletException {
								}

    /**
     * Check if the request is initialized.
     * This method checks if a form start has already been processed during this
     * request. The check is usually only required in author mode where a form
     * element might be requested directly (and not by a complete page reload).
     * @param request The current request.
     */
    private static void checkInit(final SlingHttpServletRequest request) {
	}

    /**
     * Search the form start for the current form element.
     */
    private static Resource searchFormStart(final Resource resource) {
		return null;
	}

    /**
     * Check the type of the resource.
     * @return <code>true</code> if the type or super type is the given type.
     * @since 5.2
     */
    private static boolean checkResourceType(final Resource resource, final String type) {
		return false;
	}

    /**
     * Signal the end of the form.
     * @param req Request
     */
    public static void endForm(final SlingHttpServletRequest req) {
	}

    /**
     * Set the forward path for processing the form.
     * @param req The current request.
     * @param path The forward path.
     */
    public static void setForwardPath(final SlingHttpServletRequest req, final String path) {
	}

    /**
     * Set the forward path for processing the form and makes sure the "form"
     * selector from the form request is cleared upon forwarding. This is
     * usually required if the forward should go to the sling POST servlet, to
     * avoid conflicts with the form chooser.
     * 
     * <p>
     * This is not required if the forward path already contains an extension
     * and possible selectors, in which case these overwrite the form request
     * selector anyway.
     * 
     * <p>
     * * This uses
     * {@link #setForwardOptions(ServletRequest, RequestDispatcherOptions)
     * setForwardOptions} with
     * {@link RequestDispatcherOptions#setReplaceSelectors(String)
     * RequestDispatcherOptions.setReplaceSelectors("")}.
     * 
     * @since 5.5
     * 
     * @param req
     *            The current request.
     * @param path
     *            The forward path.
     * @param clearFormSelector
     *            if the "form" selector should be cleared
     */
    public static void setForwardPath(final SlingHttpServletRequest req, final String path, final boolean clearFormSelector) {
	}
    
    /**
     * Get the forward path for processing the form.
     * @param req The current request.
     * @return The forward path or null.
     */
    public static String getForwardPath(final SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Sets the Sling {@link RequestDispatcherOptions} to be used when
     * forwarding to {@link #getForwardPath(SlingHttpServletRequest)}.
     * 
     * @since 5.5
     * 
     * @param req
     *            The current request
     * @param options
     *            options to use with forward path
     */
    public static void setForwardOptions(final ServletRequest req, final RequestDispatcherOptions options) {
	}

    /**
     * Gets the Sling {@link RequestDispatcherOptions} to be used when
     * forwarding to {@link #getForwardPath(SlingHttpServletRequest)}.
     * 
     * @since 5.5
     *
     * @param req
     *            The current request
     * @return options to use with forward path or <code>null</code>
     */
    public static RequestDispatcherOptions getForwardOptions(final ServletRequest req) {
		return null;
	}

    /**
     * Sets the Sling POST serlvet's ":redirect" parameter dynamically during
     * the form POST execution, when
     * {@link #setForwardPath(SlingHttpServletRequest, String)} is used.
     * 
     * @since 5.5
     * 
     * @param request
     *            the current request
     * @param redirect
     *            a redirect path/url
     */
    public static void setForwardRedirect(final ServletRequest request, String redirect) {
	}

    /**
     * Returns the redirect to inject as ":redirect" parameter for the Sling
     * POST servlet, when a form
     * {@link #setForwardPath(SlingHttpServletRequest, String) forward} is done.
     * 
     * @since 5.5
     * 
     * @param request
     *            the current request
     * @returna redirect path/url
     */
    public static String getForwardRedirect(final ServletRequest request) {
		return null;
	}
    
    /**
     * Set a request suffix to be added to the form action's URI.
     * @param req The current request.
     * @param suffix The suffix or null.
     */
    public static void setActionSuffix(final SlingHttpServletRequest req, final String suffix) {
	}

    /**
     * Get the request suffix currently set for the form action's URI.
     * @param req The current Request.
     * @return THe suffix or null.
     */
    public static String getActionSuffix(final SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Set the load resource for the form.
     * @param req The current request.
     * @param rsrc The load resource
     */
    public static void setFormLoadResource(final SlingHttpServletRequest req, final Resource rsrc) {
	}

    /**
     * Get the load resource for the form.
     * @param req current request
     * @return load resource or <code>null</code> if not set
     */
    public static Resource getFormLoadResource(final SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Return the form values to load.
     * @param req The request
     * @return The values or null
     */
    public static ValueMap getGlobalFormValues(final SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Sets the list of resources to be handled by the "edit" resources form action.
     * @param req current request
     * @param resources the list of resources
     */
    public static void setFormEditResources(final SlingHttpServletRequest req, final List<Resource> resources) {
	}

    /**
     * Get the list of resources to be handled by the "edit" resources form action.
     * @param req current request
     * @return the list of resources (or <code>null</code> if not set)
     */
    public static List<Resource> getFormEditResources(final SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Return the name of the check method.
     * Method name is based on from id with any hyphens converted to underscores.  If the form can't be found, it will default to defaultForm.
     * @param req The current request.
     * @return The method name
     */
    public static String getFormsPreCheckMethodName(final SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Generate the java script
     */
    private static void writeJavaScript(final SlingHttpServletRequest req,
                                        final SlingHttpServletResponse response,
                                        final Resource formResource)
    throws IOException, ServletException {
	}

    /**
     * Include the resource with the given selector with method GET
     * @param request The current request.
     * @param response The current response.
     * @param resource The resource to include.
     * @param selectorString The selector string to use for inclusion.
     * @throws IOException
     * @throws ServletException
     */
    public static void includeResource(final SlingHttpServletRequest request,
                                       final SlingHttpServletResponse response,
                                       final Resource resource,
                                       final String selectorString)
    throws IOException, ServletException {
	}

    /**
     * Are we generating client validation?
     * @param req Request
     * @return true or false
     */
    public static boolean doClientValidation(final SlingHttpServletRequest req) {
		return false;
	}

    /**
     * Return the formid
     * @param req Request
     * @return The form id or null
     */
    public static String getFormId(final SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Return the parameter name for the field
     * @param rsrc The resource
     * @return The parameter name.
     */
    public static String getParameterName(final Resource rsrc) {
		return null;
	}

    /**
     * Return the id for the field
     * @param req The current request.
     * @param rsrc The resource.
     * @return The id.
     */
    public static String getFieldId(final SlingHttpServletRequest req, final Resource rsrc) {
		return null;
	}

    /**
     * Return all form elements for this form.
     * @param formResource The form resource-
     * @return An iterator for all form elements.
     */
    public static Iterator<Resource> getFormElements(final Resource formResource) {
		return null;
	}

    /**
     * Return a list of content fields.
     * This method returns all field names (= request parameter names) that contain actually content
     * by filtering out special request parameters like those starting with a ":", "_charset_" etc.
     * @return Iterator for the field names.
     */
    public static Iterator<String> getContentRequestParameterNames(final SlingHttpServletRequest req) {
		return null;
	}

    /**
     * Helper method to collect all form elements.
     * @param list
     * @param iter
     * @return
     */
    private static boolean collectFormElements(final List<Resource> list, final Iterator<Resource> iter) {
		return false;
	}

    /**
     * Returns a resource identified by a path relative to the
     * <code>elementResource</code>. This method also considers form edit
     * resources.
     *
     * @param request         the current request.
     * @param elementResource the element resource.
     * @param relPath         a path relative to <code>elementResource</code>.
     * @return the resource or <code>null</code> if none exists at the given
     *         <code>relPath</code>.
     */
    public static Resource getResource(SlingHttpServletRequest request,
                                       Resource elementResource,
                                       String relPath) {
										return null;
									}

    /**
     * Return the value for the element.
     * This method invokes {@link #getValues(SlingHttpServletRequest, Resource)} and returns
     * if available the first value from the array.
     * @param request The current request.
     * @param elementResource The element resource.
     * @return The value for the form element or null.
     */
    public static String getValue(final SlingHttpServletRequest request, final Resource elementResource) {
		return null;
	}

    /**
     * Return the value for the element.
     * This method invokes {@link #getValues(SlingHttpServletRequest, Resource)} and returns
     * if available the first value from the array.
     * @param request The current request.
     * @param elementResource The element resource.
     * @param nameParam The name of the name parameter (defaults to "name")
     * @return The value for the form element or null.
     */
    public static String getValue(final SlingHttpServletRequest request, final Resource elementResource, final String nameParam) {
		return null;
	}

    /**
     * Return the values for the element.
     * This method
     * @param request The current request.
     * @param elementResource The element resource.
     * @return The values for the form element or null.
     */
    public static String[] getValues(final SlingHttpServletRequest request, final Resource elementResource) {
		return null;
	}

    /**
     * Return the values for the element.
     * This method
     * @param request The current request.
     * @param elementResource The element resource.
     * @param nameParam The name of the name parameter (defaults to "name")
     * @return The values for the form element or null.
     */
    public static String[] getValues(final SlingHttpServletRequest request, final Resource elementResource, final String nameParam) {
		return null;
	}

    /**
     * Return the values for the element as a list
     * This method
     * @param request The current request.
     * @param elementResource The element resource.
     * @return The values for the form element or an empty list.
     */
    public static List<String> getValuesAsList(final SlingHttpServletRequest request, final Resource elementResource) {
		return null;
	}

    /**
     * Returns the value for the given name (property).
     * @param request The current request
     * @param name name of the property (or property path)
     * @param defaultValue default value to return if property is not present
     * @return The value for the given property
     */
    public static String getValue(final SlingHttpServletRequest request, final String name, final String defaultValue) {
		return null;
	}

    /**
     * Returns the values for the given name (property).
     * @param request The current request
     * @param name name of the property (or property path)
     * @param defaultValues default values to return if property is not present
     * @return The values for the given property
     */
    public static String[] getValues(final SlingHttpServletRequest request, final String name, final String[] defaultValues) {
		return null;
	}

    /**
     * Writes the given form load resource as JSON into the given writer. This
     * will dump the full tree; use
     * {@link #inlineValuesAsJson(SlingHttpServletRequest, Writer, String, int)}
     * for controlling the node depth.
     *
     * <p>
     * Can be used in JSPs to inline JSON for javascript code, for example:
     *
     * <pre>
     * var data = &lt;% FormsHelper.inlineValuesAsJson(slingRequest, out, &quot;.&quot;); %&gt;;
     * </pre>
     *
     * which might result in:
     *
     * <pre>
     * var data = { &quot;jcr:primaryType&quot;: &quot;nt:unstructured&quot;, &quot;property&quot; : &quot;value&quot; };
     * </pre>
     *
     * <p>
     * If the path cannot be found, an empty object "<code>{}</code>" will be
     * written. Any exception will be passed to the caller.
     *
     * <p>
     * The underlying form load resource must be based on a JCR Node. The path
     * is relative and allows to specify subnodes. It cannot point to JCR
     * properties, please use
     * {@link #getValue(SlingHttpServletRequest, String, String)} or
     * {@link #getValues(SlingHttpServletRequest, String, String[])} for them.
     *
     * @param request
     *            the current request
     * @param out
     *            a writer, such as a {@link JspWriter}, to write the JSON into.
     *            Will automatically be flushed before and after.
     * @param path
     *            an absolute node path or a node path relativ to the current
     *            form load resource. Use "." for the resource node itself.
     * @throws RepositoryException
     *             if some jcr error happened
     * @throws JSONException
     *             if writing the json failed
     * @throws IOException
     *             if there was a problem with the writer
     */
    public static void inlineValuesAsJson(final SlingHttpServletRequest request, Writer out, String path) throws IOException, RepositoryException, JSONException {
	}

    /**
     * Writes the given form load resource as JSON into the given writer. Can be
     * used in JSPs to inline JSON for javascript code, for example:
     *
     * <pre>
     * var data = &lt;% FormsHelper.inlineValuesAsJson(slingRequest, out, &quot;.&quot;); %&gt;;
     * </pre>
     *
     * which might result in:
     *
     * <pre>
     * var data = { &quot;jcr:primaryType&quot;: &quot;nt:unstructured&quot;, &quot;property&quot; : &quot;value&quot; };
     * </pre>
     *
     * <p>
     * If the path cannot be found, an empty object "<code>{}</code>" will be
     * written. Any exception will be passed to the caller.
     *
     * <p>
     * The underlying form load resource must be based on a JCR Node. The path
     * is relative and allows to specify subnodes. It cannot point to JCR
     * properties, please use
     * {@link #getValue(SlingHttpServletRequest, String, String)} or
     * {@link #getValues(SlingHttpServletRequest, String, String[])} for them.
     *
     * @param request
     *            the current request
     * @param out
     *            a writer, such as a {@link JspWriter}, to write the JSON into.
     *            Will automatically be flushed before and after.
     * @param path
     *            an absolute node path or a node path relativ to the current
     *            form load resource. Use "." for the resource node itself.
     * @param nodeDepth
     *            until which depth the tree should be written; 0 means the
     *            current node and its properties only; -1 means the whole tree.
     * @throws RepositoryException
     *             if some jcr error happened
     * @throws JSONException
     *             if writing the json failed
     * @throws IOException
     *             if there was a problem with the writer
     */
    public static void inlineValuesAsJson(final SlingHttpServletRequest request, Writer out, String path, int nodeDepth) throws IOException, RepositoryException, JSONException {
	}

    /**
     * Recurse down a subtree collecting all showHideExpressions into a map keyed by element name.
     *
     * @param node the node to check for showHideExpressions.
     * @param map a collection of node names, and their associated showHideExpression.
     */
    private static void accumulateShowHideExpressions(final Node node, Map<String, String> map) throws RepositoryException {
	}

    /**
     * Return a flattened map of all showHideExpressions that are descendants of a resource.
     *
     * @param resource the node to check to see if there's any showHideExpressions.
     * @return A map of showHideExpressions keyed by element name.
     */
    public static Map<String, String> getShowHideExpressions(final Resource resource) throws RepositoryException {
		return null;
	}

    /**
     * Return the options for a form element
     * @param request
     * @param elementResource
     * @return A map of options (key-value) or null.
     */
    public static Map<String, String> getOptions(final SlingHttpServletRequest request, final Resource elementResource) {
		return null;
	}

    /**
     * Is this field required?
     * @param formElement The form element.
     * @return true if the field is required, false otherwise.
     */
    public static boolean isRequired(final Resource formElement) {
		return false;
	}

    /**
     * Marks the current form rendering to produce an read-only form. Form field
     * renderings must use
     * {@link #isReadOnly(SlingHttpServletRequest, Resource)} to decide between
     * edit or read-only representations.
     *
     * @param request
     *            the current request
     */
    public static void setFormReadOnly(final SlingHttpServletRequest request) {
	}

    /**
     * Returns true if either the passed form field is configured as read-only
     * or if the entire form is to be displayed in a read-only way. The latter
     * is the case when {@link #setFormReadOnly(SlingHttpServletRequest)} was
     * called, for example if the "view" selector of the
     * {@link FormChooserServlet} is used.
     *
     * @param request
     *            the current request
     * @param formElement
     *            the form field resource
     * @return <code>true</code> if this field is to be rendered as read-only,
     *         <code>false</code> otherwise.
     */
    public static boolean isReadOnly(final SlingHttpServletRequest request, Resource formElement) {
		return false;
	}

    /**
     * Returns true if the entire form has to be displayed in a read-only way.
     * @param request
     *            the current request
     * @return <code>true</code> if a form element has to be rendered as read-only,
     *         <code>false</code> otherwise.
     * @since 5.5
     */
    public static boolean isReadOnly(final SlingHttpServletRequest request) {
		return false;
	}

    /**
     * Returns true if the passed form field is configured as read-only. This is
     * determined solely by looking at the field's "readOnly" property.
     *
     * @deprecated
     * To always support the global read-only flag, set for example by the
     * "view" selector of the {@link FormChooserServlet}, use
     * {@link #isReadOnly(SlingHttpServletRequest, Resource)} instead.
     *
     * @param formElement
     *            the form field resource
     * @return <code>true</code> if this field is read-only, <code>false</code>
     *         otherwise.
     */
    @Deprecated
    public static boolean isReadOnly(Resource formElement) {
		return false;
	}

    /**
     * <p>Checks the rule specified by the given form property.</p>
     * <p>Rules may be used to enable/disable form elements according to some serverside
     * conditions.</p>
     * <p>Currently, only rules depending on access rights are available.</p>
     */
    public static boolean checkRule(final Resource resource,
                                    final SlingHttpServletRequest req,
                                    final PageContext pageContext, String propName) {
										return false;
									}

    /**
     * Return the title for the field.
     * @param formElement The form element.
     * @param defaultTitle The default title.
     * @return The title to display.
     */
    public static String getTitle(final Resource formElement, String defaultTitle) {
		return null;
	}

    /**
     * Return the description for the field.
     * @param formElement The form element.
     * @param defaultDescription The default description.
     * @return The description to display.
     */
    public static String getDescription(final Resource formElement, String defaultDescription) {
		return null;
	}

    /**
     * Is this a field with multi selection?
     * @param formElement The form element
     * 
     * @since 5.5
     */
    public static boolean hasMultiSelection(final Resource formElement) {
		return false;
	}

    /**
     * Redirect to the referrer.
     * This method redirects to the referer and adds optional request parameters.
     * @param req The current request
     * @param res The current response
     */
    public static void redirectToReferrer(final SlingHttpServletRequest req,
                                          final SlingHttpServletResponse res,
                                          final Map<String, String[]> params)
    throws IOException {
	}

    /**
     * Redirect to the referrer. This method redirects to the referrerand copies
     * the request parameters.
     *
     * @param request The current request
     * @param res The current response
     * @since 5.2
     */
    public static void redirectToReferrer(final SlingHttpServletRequest request,
                                          final SlingHttpServletResponse res)
    throws IOException {
	}

    /**
     * Fix the form.
     * When a form start is added, add automatically a forms end (if missing)
     * When only a form end is on the page remove it
     * @since 5.2
     */
    public static Resource checkFormStructure(final Resource rsrc) {
		return null;
	}

    /**
     * Url encode the value.
     * The value is encoded with character set UTF-8.
     * @param value The value
     * @return The encoded value.
     * @since 5.2
     */
    public static String encodeValue(String value) {
		return null;
	}

    /**
     * Url decode the value.
     * The value is decoded with character set UTF-8.
     * @param value The value
     * @return The decoded value.
     * @since 5.2
     */
    public static String decodeValue(String value) {
		return null;
	}


    /**
     * Return the css classes for the field.
     * If the field has a property {@link FormsConstants#ELEMENT_PROPERTY_CSS}
     * the value of this property is appended to the default css. If not, only the
     * default css is returned.
     * @param props The field properties.
     * @param defaultCss The default css for this field.
     * @return The css classes
     *
     * @since 5.4
     */
    public static String getCss(final ValueMap props, final String defaultCss) {
		return null;
	}

    /**
     * Returns the HTTP "referrer" header from the request, and also looks out
     * for the common "referer" misspelling.
     * 
     * @since 5.5
     * 
     * @param request
     *            current request
     * @return value of the referrer header or <code>null</code> if not present
     */
    public static String getReferrer(HttpServletRequest request) {
		return null;
	}

    /**
     * Sets a flag to redirect to the HTTP referrer after the forward of a form
     * POST request. This will usually only be used if no explicit redirect is
     * already given in the ":redirect" parameter used by the Sling POST servlet.
     * 
     * @since 5.5
     * 
     * @param request
     *            current request
     * @param redirectToReferrer
     *            <code>true</code> to enable the redirect to the referrer
     */
    public static void setRedirectToReferrer(ServletRequest request, boolean redirectToReferrer) {
	}

    /**
     * Returns whether there should be a redirect to the HTTP referrer after the
     * forward of a form POST request.
     * 
     * @since 5.5
     * 
     * @param request
     *            current request
     * @return <code>true</code> if there should be a redirect to the referrer
     */
    public static boolean isRedirectToReferrer(ServletRequest request) {
		return false;
	}

    /**
     * Returns the locale.
     * If the request originates from a jsp, it returns the locale as defined
     * by the {@link com.day.cq.wcm.api.LanguageManager}, otherwise it returns
     * <code>null</code>
     * @param request The current request
     * @return the locale or <code>null</code> if not determinable
     */
    public static Locale getLocale(SlingHttpServletRequest request) {
		return null;
	}

    /**
     * Returns the localized message.
     * If the request originates from a jsp, it returns the localized message
     * based on the {@link java.util.ResourceBundle} of the request and the
     * locale defined by the {@link com.day.cq.wcm.api.LanguageManager}.
     * Otherwise it returns the original message.
     * @param msg The message to be localized
     * @param request The current request
     * @return the localized message or the original message if it cannot be localized
     */
    public static String getLocalizedMessage(String msg, SlingHttpServletRequest request) {
		return null;
	}

    /**
     * Get the list of white listed data name patterns.
     * @param req The current request.
     * @return The array of white listed data name patterns
     */
    public static String[] getWhitelistPatterns(final SlingHttpServletRequest req) {
		return null;
	}

}
