/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.wcm.msm.commons;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ValueMap;

import com.day.cq.wcm.api.WCMException;
import com.day.cq.wcm.msm.api.LiveAction;
import com.day.cq.wcm.msm.api.LiveActionFactory;

/**
 * Base implementation of a {@code LiveActionFactory} service
 *
 * @param <ActionType>
 */
public abstract class BaseActionFactory<ActionType extends LiveAction> implements LiveActionFactory<ActionType> {
    
    /**
     * Creates a new ActionType instance.
     * 
     * @param config the {@code ValueMap} containing the action's configuration. It may be null.
     * @return a new action instance
     * @throws WCMException
     */
    protected abstract ActionType newActionInstance(ValueMap config) throws WCMException {
		return null;
	}
    

    /**
     * Creates an action using a {@code ValueMap} object as the source of the
     * action's configuration
     * <br/>
     * Returns a new action instance as returned by the {@link BaseActionFactory#newActionInstance(ValueMap) newActionInstance} method
     */
    public ActionType createAction(Resource resource) throws WCMException {
		return null;
	}
}
