/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.crx.sc.qom;

import static javax.jcr.query.qom.QueryObjectModelConstants.JCR_JOIN_TYPE_LEFT_OUTER;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jcr.RepositoryException;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.query.QueryResult;
import javax.jcr.query.Row;
import javax.jcr.query.RowIterator;
import javax.jcr.query.qom.ChildNodeJoinCondition;
import javax.jcr.query.qom.Constraint;
import javax.jcr.query.qom.DescendantNodeJoinCondition;
import javax.jcr.query.qom.EquiJoinCondition;
import javax.jcr.query.qom.Join;
import javax.jcr.query.qom.JoinCondition;
import javax.jcr.query.qom.PropertyValue;
import javax.jcr.query.qom.QueryObjectModelFactory;
import javax.jcr.query.qom.SameNodeJoinCondition;
import javax.jcr.query.qom.Selector;
import javax.jcr.query.qom.Source;

import org.apache.jackrabbit.commons.iterator.RowIterable;
import org.apache.jackrabbit.commons.iterator.RowIteratorAdapter;
import org.apache.jackrabbit.core.query.lucene.join.QueryEngine;

/**
 * A join merger is used by the {@link QueryEngine} class to efficiently merge together two parts of a join query.
 * <p>
 * Each join condition type ({@link EquiJoinCondition equi-} and {@link SameNodeJoinCondition same},
 * {@link ChildNodeJoinCondition child} or {@link DescendantJoinCondition descendant} node joins) has it's own merger
 * class that extends the functionality of this abstract base class with functionality specific to that join
 * condition.
 */
public abstract class JoinMerger {

    /**
     * Static factory method for creating a merger for the given join.
     * @param join join
     * @param columns columns of the query
     * @param evaluator operand evaluator
     * @param factory QOM factory
     * @return join merger
     * @throws RepositoryException if the merger can not be created
     */
    public static JoinMerger getJoinMerger(final Join join, final Map<String, PropertyValue> columns,
        final OperandEvaluator evaluator, final QueryObjectModelFactory factory) throws RepositoryException {
			return null;
		}

    private final String type;

    protected final Set<String> leftSelectors;

    protected final Set<String> rightSelectors;

    private final String[] selectorNames;

    private final Map<String, PropertyValue> columns;

    private final String[] columnNames;

    protected final OperandEvaluator evaluator;

    protected final QueryObjectModelFactory factory;

    protected JoinMerger(final Join join, final Map<String, PropertyValue> columns, final OperandEvaluator evaluator,
        final QueryObjectModelFactory factory) throws RepositoryException {
        this.type = join.getJoinType();

        this.leftSelectors = getSelectorNames(join.getLeft());
        this.rightSelectors = getSelectorNames(join.getRight());

        final Set<String> selectors = new LinkedHashSet<String>();
        selectors.addAll(leftSelectors);
        selectors.addAll(rightSelectors);
        this.selectorNames = selectors.toArray(new String[selectors.size()]);

        this.columns = columns;
        this.columnNames = columns.keySet().toArray(new String[columns.size()]);

        this.evaluator = evaluator;
        this.factory = factory;
    }

    public Set<String> getLeftSelectors() {
		return null;
	}

    public Set<String> getRightSelectors() {
		return null;
	}

    private Set<String> getSelectorNames(final Source source) throws RepositoryException {
		return null;
	}

    public QueryResult merge(final RowIterator leftRows, final RowIterator rightRows) throws RepositoryException {
		return null;
	}

    /**
     * Merges the given left and right rows to a single joined row.
     * @param left left row, possibly <code>null</code> in a right outer join
     * @param right right row, possibly <code>null</code> in a left outer join
     * @return joined row
     */
    private Row mergeRow(final Row left, final Row right) {
		return null;
	}

    public abstract Set<String> getLeftValues(Row row) throws RepositoryException {
		return null;
	}

    public abstract Set<String> getRightValues(Row row) throws RepositoryException {
		return null;
	}

    public abstract List<Constraint> getRightJoinConstraints(List<Row> leftRows) throws RepositoryException {
		return null;
	}

}
