/*
 * $URL$
 * $Id$
 *
 * Copyright 1997-2006 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.util.diff;

import java.io.IOException;
import java.io.StringWriter;

/**
 * Implements a tree-way diff between a base document and 2 dervied ones.
 * The result of the diff operation is a list of {@link Hunk3} that can
 * record the changes and conflicts.
 *
 * @author tripod
 * @version $Rev$, $Date$
 */
public class DocumentDiff3 {

    /**
     * The CVS/SVN id
     */
    static final String CVS_ID = "$URL$ $Rev$ $Date$";

    /**
     * the base document
     */
    private final Document base;

    /**
     * the left document
     */
    private final Document left;

    /**
     * the right documents
     */
    private final Document right;

    /**
     * the script of changes from the base to the left document
     */
    private final Diff.Change leftChanges;

    /**
     * the script of changes from the base to the right document
     */
    private final Diff.Change rightChanges;

    /**
     * the chain of {@link Hunk3}s of this diff. this hunk here is just a
     * sentinel so that the chain is never empty.
     */
    private final Hunk3 hunks = new Hunk3(null, null, null, null);

    /**
     * flag that indicates that this diff has conflicts
     */
    private boolean hasConflicts;

    /**
     * Creates a new document diff 3 object.
     * @param base the base document
     * @param left the left document
     * @param right the right document
     */
    public DocumentDiff3(Document base, Document left, Document right) {
        this.base = base;
        this.left = left;
        this.right = right;
        Document.Element[] baseElems = base.getElements();
        Document.Element[] leftElems = left.getElements();
        Document.Element[] rightElems = right.getElements();
        leftChanges = new Diff(baseElems, leftElems).diff_2(false);
        rightChanges = new Diff(baseElems, rightElems).diff_2(false);
        initHunks();
    }

    /**
     * Returns a chain of {@link Hunk3}s that contain the modifications.
     * @return a chain of {@link Hunk3}s.
     */
    public Hunk3 getHunks() {
		return null;
	}

    /**
     * Indicates if any of the hunks has a conflict.
     * @return <code>true</code> if any of the hunks has a conflict.
     */
    public boolean hasConflicts() {
		return false;
	}

    /**
     * Writes the resulting document to the given string buffer. this may include
     * conflicting regions.
     *
     * @param buf the string buffer to write to
     * @param lineSeparator the line separator to use
     * @param showBase if set to <code>true</code> the base section of a conflict
     *        is also included in the output.
     */
    public void write(StringBuffer buf, String lineSeparator, boolean showBase) {
	}
    
    /**
     * Writes the resulting document to the given write. this may include
     * conflicting regions.
     *
     * @param w the writer to write to
     * @param showBase if set to <code>true</code> the base section of a conflict
     *        is also included in the output.
     * @throws IOException if an I/O error occurs
     */
    public void write(DiffWriter w, boolean showBase) throws IOException {
	}

    /**
     * initializes the hunks
     */
    private void initHunks() {
	}

    /**
     * Wraps a chain of {@link Diff.Change}s with a list of {@link MyChange}s.
     * this is rather a convencience wrapping so that the algorithm above is
     * easier to understand.
     *
     * @param df the chain of changes
     * @return the wrapped chain of changes
     */
    private MyChange wrap(Diff.Change df) {
		return null;
	}

    /**
     * Dumps a chain of my changes.
     * @param c the change
     * @param left the left doc
     * @param right the right doc
     */
    private void dump(MyChange c, Document left, Document right) {
	}

    /**
     * prints a element line
     * @param b the base line number
     * @param l the left line number
     * @param r the right line number
     * @param prefix the prefix
     * @param i the index
     * @param doc the document
     */
    private void dump(int b, int l, int r, String prefix, int i, Document doc) {
	}

    /**
     * Wrapper class for the {@link Diff.Change}. This is mainly used to
     * make the merge algorithm easier, but might result in a performace
     * drop if the change list is huge.
     */
    private static class MyChange {

        /**
         * the low line in the left document
         */
        private final int low0;

        /**
         * the low line in the right document
         */
        private final int low1;

        /**
         * the high line in the left document
         */
        private final int high0;

        /**
         * the high line in the right document
         */
        private final int high1;

        /**
         * the next change
         */
        private MyChange next;

        /**
         * Constructs a new change and adds it the the previous one.
         * @param low0 the low line of the left document
         * @param low1 the low line of the right document
         * @param len0 the length of the change in the left document
         * @param len1 the length of the change in the right document
         * @param prev the previous change
         */
        public MyChange(int low0, int low1, int len0, int len1, MyChange prev) {
            this.low0 = low0;
            this.low1 = low1;
            this.high0 = low0 + len0;
            this.high1 = low1 + len1;
            if (prev != null) {
                prev.next = this;
            }
        }

        /**
         * Checks if this is a deletion.
         * @return <code>true</code> if this is a deletion.
         */
        public boolean isDelete() {
			return false;
		}

        /**
         * Checks if this is an insertion
         * @return <code>false</code> if this is an insertion.
         */
        public boolean isInsert() {
			return false;
		}

        /**
         * Returns a debug string for this change.
         * @return a debug string for this change.
         */
        public String toString() {
			return null;
		}
    }

}