/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.jackrabbit.oak.plugins.memory;

import static com.google.common.base.Preconditions.checkArgument;

import java.math.BigDecimal;
import java.util.List;

import javax.annotation.Nonnull;
import javax.jcr.PropertyType;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import org.apache.jackrabbit.oak.api.Blob;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.plugins.value.Conversions.Converter;

/**
 * Abstract base class for multi valued {@code PropertyState} implementations.
 */
abstract class MultiPropertyState<T> extends EmptyPropertyState {
    protected final List<T> values;

    /**
     * Create a new property state with the given {@code name}
     * and {@code values}
     * @param name  The name of the property state.
     * @param values  The values of the property state.
     */
    protected MultiPropertyState(String name, Iterable<T> values) {
        super(name);
        this.values = Lists.newArrayList(values);
    }

    /**
     * Create a converter for converting a value to other types.
     * @param  value  The value to convert
     * @return  A converter for the value of this property
     */
    public abstract Converter getConverter(T value) {
		return null;
	}

    @SuppressWarnings("unchecked")
    private <S> S  convertTo(Type<S> type) {
		return null;
	}

    /**
     * @throws IllegalArgumentException if {@code type} is not one of the
     * values defined in {@link Type} or if {@code type.isArray()} is {@code false}.
     */
    @SuppressWarnings("unchecked")
    @Nonnull
    @Override
    public <S> S getValue(Type<S> type) {
		return null;
	}

    @SuppressWarnings("unchecked")
    private <S> S  convertTo(Type<S> type, int index) {
		return null;
	}

    /**
     * @throws IllegalArgumentException if {@code type} is not one of the
     * values defined in {@link Type} or if {@code type.isArray()} is {@code true}
     * @throws IndexOutOfBoundsException if {@code index >= count()}.
     */
    @SuppressWarnings("unchecked")
    @Nonnull
    @Override
    public <S> S getValue(Type<S> type, int index) {
		return null;
	}

    @Override
    public final int count() {
		return 0;
	}

    @Override
    public long size(int index) {
		return 0L;
	}

}
