/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.jackrabbit.oak.spi.security;

import java.security.Principal;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.annotation.Nonnull;

import com.google.common.base.Function;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;

import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Tree;
import org.apache.jackrabbit.oak.spi.commit.CommitHook;
import org.apache.jackrabbit.oak.spi.commit.MoveTracker;
import org.apache.jackrabbit.oak.spi.commit.ValidatorProvider;
import org.apache.jackrabbit.oak.spi.lifecycle.CompositeInitializer;
import org.apache.jackrabbit.oak.spi.lifecycle.CompositeWorkspaceInitializer;
import org.apache.jackrabbit.oak.spi.lifecycle.RepositoryInitializer;
import org.apache.jackrabbit.oak.spi.lifecycle.WorkspaceInitializer;
import org.apache.jackrabbit.oak.spi.xml.ProtectedItemImporter;
import org.apache.jackrabbit.oak.plugins.tree.TreeLocation;

/**
 * Abstract base implementation for {@link SecurityConfiguration}s that can
 * combine different implementations.
 */
public abstract class CompositeConfiguration<T extends SecurityConfiguration> implements SecurityConfiguration {

    private final List<T> configurations = new CopyOnWriteArrayList<T>();

    private final String name;
    private final SecurityProvider securityProvider;

    private T defaultConfig;

    public CompositeConfiguration(@Nonnull String name, @Nonnull SecurityProvider securityProvider) {
        this.name = name;
        this.securityProvider = securityProvider;
    }

    public void setDefaultConfig(@Nonnull T defaultConfig) {
	}

    public void addConfiguration(@Nonnull T configuration) {
	}

    public void removeConfiguration(@Nonnull T configuration) {
	}

    protected List<T> getConfigurations() {
		return null;
	}

    protected SecurityProvider getSecurityProvider() {
		return null;
	}

    //----------------------------------------------< SecurityConfiguration >---
    @Nonnull
    @Override
    public String getName() {
		return null;
	}

    @Nonnull
    @Override
    public ConfigurationParameters getParameters() {
		return null;
	}

    @Nonnull
    @Override
    public WorkspaceInitializer getWorkspaceInitializer() {
		return null;
	}

    @Nonnull
    @Override
    public RepositoryInitializer getRepositoryInitializer() {
		return null;
	}

    @Nonnull
    @Override
    public List<? extends CommitHook> getCommitHooks(final String workspaceName) {
		return null;
	}

    @Nonnull
    @Override
    public List<? extends ValidatorProvider> getValidators(final String workspaceName, final Set<Principal> principals, final MoveTracker moveTracker) {
		return null;
	}

    @Nonnull
    @Override
    public List<ProtectedItemImporter> getProtectedItemImporters() {
		return null;
	}

    @Override
    public Context getContext() {
		return null;
	}
}
