/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.spi.state;

import static java.util.Collections.emptyList;
import static org.apache.jackrabbit.oak.api.Type.BOOLEAN;
import static org.apache.jackrabbit.oak.api.Type.LONG;
import static org.apache.jackrabbit.oak.api.Type.NAME;
import static org.apache.jackrabbit.oak.api.Type.NAMES;
import static org.apache.jackrabbit.oak.api.Type.STRING;
import static org.apache.jackrabbit.oak.api.Type.STRINGS;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;

import org.apache.jackrabbit.oak.api.PropertyState;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;

/**
 * Abstract base class for {@link NodeState} implementations.
 * This base class contains default implementations of the
 * {@link #equals(Object)} and {@link #hashCode()} methods based on
 * the implemented interface.
 * <p>
 * This class also implements trivial (and potentially very slow) versions of
 * the {@link #getProperty(String)} and {@link #getPropertyCount()} methods
 * based on {@link #getProperties()}. The {@link #getChildNodeCount(long)} method
 * is similarly implemented based on {@link #getChildNodeEntries()}.
 * Subclasses should normally override these method with a more efficient
 * alternatives.
 */
public abstract class AbstractNodeState implements NodeState {

    public static boolean isValidName(String name) {
		return false;
	}

    public static void checkValidName(String name)
            throws IllegalArgumentException {
			}

    public static boolean getBoolean(NodeState state, String name) {
		return false;
	}

    public static long getLong(NodeState state, String name) {
		return 0L;
	}

    public static String getString(NodeState state, String name) {
		return null;
	}

    public static Iterable<String> getStrings(NodeState state, String name) {
		return null;
	}

    public static String getName(NodeState state, String name) {
		return null;
	}

    public static Iterable<String> getNames(NodeState state, String name) {
		return null;
	}

    /**
     * Generic default comparison algorithm that simply walks through the
     * property and child node lists of the given base state and compares
     * the entries one by one with corresponding ones (if any) in this state.
     */
    public static boolean compareAgainstBaseState(
            NodeState state, NodeState base, NodeStateDiff diff) {
				return false;
			}

    /**
     * Compares the properties of {@code base} state with {@code this}
     * state.
     *
     * @param state the head node state.
     * @param base the base node state.
     * @param diff the node state diff.
     * @return {@code true} to continue the comparison, {@code false} to stop
     */
    public static boolean comparePropertiesAgainstBaseState(
            NodeState state, NodeState base, NodeStateDiff diff) {
				return false;
			}

    public static String toString(NodeState state) {
		return null;
	}

    @Override
    public boolean hasProperty(String name) {
		return false;
	}

    @Override
    public boolean getBoolean(String name) {
		return false;
	}

    @Override
    public long getLong(String name) {
		return 0L;
	}

    @Override
    public String getString(String name) {
		return null;
	}

    @Override
    public Iterable<String> getStrings(String name) {
		return null;
	}

    @Override @CheckForNull
    public String getName(@Nonnull String name) {
		return null;
	}

    @Override @Nonnull
    public Iterable<String> getNames(@Nonnull String name) {
		return null;
	}

    @Override
    public PropertyState getProperty(String name) {
		return null;
	}

    @Override
    public long getPropertyCount() {
		return 0L;
	}

    @Override
    public long getChildNodeCount(long max) {
		return 0L;
	}

    @Override
    public Iterable<String> getChildNodeNames() {
		return null;
	}

    /**
     * Generic default comparison algorithm that simply walks through the
     * property and child node lists of the given base state and compares
     * the entries one by one with corresponding ones (if any) in this state.
     */
    @Override
    public boolean compareAgainstBaseState(NodeState base, NodeStateDiff diff) {
		return false;
	}

    /**
     * Returns a string representation of this node state.
     *
     * @return string representation
     */
    public String toString() {
		return null;
	}

    /**
     * Checks whether the given object is equal to this one. Two node states
     * are considered equal if all their properties and child nodes match,
     * regardless of ordering. Subclasses may override this method with a
     * more efficient equality check if one is available.
     *
     * @param that target of the comparison
     * @return {@code true} if the objects are equal,
     *         {@code false} otherwise
     */
    @Override
    public boolean equals(Object that) {
		return false;
	}

    public static boolean equals(NodeState a, NodeState b) {
		return false;
	}

    /**
     * Returns a hash code that's compatible with how the
     * {@link #equals(Object)} method is implemented. The current
     * implementation simply returns zero for everything since
     * {@link NodeState} instances are not intended for use as hash keys.
     *
     * @return hash code
     */
    @Override
    public int hashCode() {
		return 0;
	}

    //-----------------------------------------------------------< private >--

    protected static long count(Iterable<?> iterable) {
		return 0L;
	}

}
