/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.spi.state;

import static com.google.common.base.Preconditions.checkNotNull;

import java.io.IOException;
import java.io.InputStream;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;

import org.apache.jackrabbit.oak.api.Blob;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Type;

/**
 * A node builder that throws an {@link UnsupportedOperationException} on
 * all attempts to modify the given base state.
 */
public class ReadOnlyBuilder implements NodeBuilder {

    @Nonnull
    private final NodeState state;

    public ReadOnlyBuilder(@Nonnull NodeState state) {
        this.state = checkNotNull(state);
    }

    protected RuntimeException unsupported() {
		return null;
	}

    @Override
    public boolean exists() {
		return false;
	}

    @Override
    public boolean isNew() {
		return false;
	}

    @Override
    public boolean isNew(String name) {
		return false;
	}

    @Override
    public boolean isModified() {
		return false;
	}

    @Override
    public boolean isReplaced() {
		return false;
	}

    @Override
    public boolean isReplaced(String name) {
		return false;
	}

    @Override @Nonnull
    public NodeState getNodeState() {
		return null;
	}

    @Override @Nonnull
    public NodeState getBaseState() {
		return null;
	}

    @Override
    public long getChildNodeCount(long max) {
		return 0L;
	}

    @Override
    public boolean hasChildNode(String name) {
		return false;
	}

    @Override
    public Iterable<String> getChildNodeNames() {
		return null;
	}

    @Override @Nonnull
    public NodeBuilder setChildNode(String name, NodeState nodeState) {
		return null;
	}

    @Override
    public boolean remove() {
		return false;
	}

    @Override
    public boolean moveTo(NodeBuilder newParent, String newName) {
		return false;
	}

    @Override
    public long getPropertyCount() {
		return 0L;
	}

    @Override
    public Iterable<? extends PropertyState> getProperties() {
		return null;
	}

    @Override
    public boolean hasProperty(String name) {
		return false;
	}

    @Override
    public PropertyState getProperty(String name) {
		return null;
	}

    @Override
    public boolean getBoolean(String name) {
		return false;
	}

    @Override @CheckForNull
    public String getString(@Nonnull String name) {
		return null;
	}

    @Override @CheckForNull
    public String getName(@Nonnull String name) {
		return null;
	}

    @Override @Nonnull
    public Iterable<String> getNames(@Nonnull String name) {
		return null;
	}

    @Override @Nonnull
    public NodeBuilder removeProperty(String name) {
		return null;
	}

    @Override
    public NodeBuilder setProperty(PropertyState property) {
		return null;
	}

    @Override
    public <T> NodeBuilder setProperty(String name, T value) {
		return null;
	}

    @Override
    public <T> NodeBuilder setProperty(String name, T value, Type<T> type) {
		return null;
	}

    @Override
    public ReadOnlyBuilder child(String name) {
		return null;
	}

    @Override @Nonnull
    public NodeBuilder getChildNode(@Nonnull String name) {
		return null;
	}

    @Override @Nonnull
    public NodeBuilder setChildNode(@Nonnull String name) {
		return null;
	}

    @Override
    public Blob createBlob(InputStream stream) throws IOException {
		return null;
	}

}
