/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.client;

import java.rmi.RemoteException;

import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

import javax.jcr.Repository;

import org.apache.jackrabbit.rmi.remote.RemoteXASession;
import org.apache.jackrabbit.rmi.remote.SerializableXid;

/**
 * Local adapter for the JCR-RMI
 * {@link org.apache.jackrabbit.rmi.remote.RemoteXASession RemoteXASession}
 * interface.
 *
 * @since 1.4
 */
public class ClientXASession extends ClientSession implements XAResource {

    /**
     * The adapted remote transaction enabled session.
     */
    private RemoteXASession remote;

    /**
     * Creates a client adapter for the given remote session which is
     * transaction enabled.
     */
    public ClientXASession(
            Repository repository, RemoteXASession remote,
            LocalAdapterFactory factory) {
        super(repository, remote, factory);
        this.remote = remote;
    }

    /**
     * Returns <code>true</code> if the given object is a local
     * adapter that refers to the same remote XA resource.
     *
     * @see http://blogs.sun.com/fkieviet/entry/j2ee_jca_resource_adapters_the
     */
    public boolean isSameRM(XAResource xares) throws XAException {
		return false;
	}

    private XAException getXAException(RemoteException e) {
		return null;
	}

    public void commit(Xid xid, boolean onePhase) throws XAException {
	}

    public void end(Xid xid, int flags) throws XAException {
	}

    public void forget(Xid xid) throws XAException {
	}

    public int getTransactionTimeout() throws XAException {
		return 0;
	}

    public int prepare(Xid xid) throws XAException {
		return 0;
	}

    public Xid[] recover(int flag) throws XAException {
		return null;
	}

    public void rollback(Xid xid) throws XAException {
	}

    public boolean setTransactionTimeout(int seconds) throws XAException {
		return false;
	}

    public void start(Xid xid, int flags) throws XAException {
	}

}
