/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.server;

import java.rmi.RemoteException;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.lock.Lock;
import javax.jcr.version.Version;

import org.apache.jackrabbit.rmi.remote.RemoteItem;
import org.apache.jackrabbit.rmi.remote.RemoteIterator;
import org.apache.jackrabbit.rmi.remote.RemoteLock;
import org.apache.jackrabbit.rmi.remote.RemoteNode;
import org.apache.jackrabbit.rmi.remote.RemoteNodeDefinition;
import org.apache.jackrabbit.rmi.remote.RemoteNodeType;
import org.apache.jackrabbit.rmi.remote.RemoteProperty;
import org.apache.jackrabbit.rmi.remote.RemoteVersion;
import org.apache.jackrabbit.rmi.remote.RemoteVersionHistory;

/**
 * Remote adapter for the JCR {@link javax.jcr.Node Node} interface.
 * This class makes a local node available as an RMI service using
 * the {@link org.apache.jackrabbit.rmi.remote.RemoteNode RemoteNode}
 * interface.
 *
 * @see javax.jcr.Node
 * @see org.apache.jackrabbit.rmi.remote.RemoteNode
 */
public class ServerNode extends ServerItem implements RemoteNode {

    /** The adapted local node. */
    private Node node;

    /**
     * Creates a remote adapter for the given local node.
     *
     * @param node local node
     * @param factory remote adapter factory
     * @throws RemoteException on RMI errors
     */
    public ServerNode(Node node, RemoteAdapterFactory factory)
            throws RemoteException {
        super(node, factory);
        this.node = node;
    }

    /** {@inheritDoc} */
    public RemoteNode addNode(String path)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteNode addNode(String path, String type)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteProperty getProperty(String path)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteIterator getProperties()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteItem getPrimaryItem()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteIterator getProperties(String pattern)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteIterator getProperties(String[] globs)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteIterator getReferences()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteIterator getReferences(String name)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public String getIdentifier() throws RepositoryException, RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    @SuppressWarnings("deprecation")
    public String getUUID() throws RepositoryException, RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public boolean hasNodes() throws RepositoryException, RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean hasProperties() throws RepositoryException, RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean hasProperty(String path)
            throws RepositoryException, RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public RemoteNodeType[] getMixinNodeTypes()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteNodeType getPrimaryNodeType()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public boolean isNodeType(String type)
            throws RepositoryException, RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public RemoteIterator getNodes() throws RepositoryException, RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public RemoteIterator getNodes(String pattern)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteIterator getNodes(String[] globs)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteNode getNode(String path)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public boolean hasNode(String path)
            throws RepositoryException, RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public RemoteProperty setProperty(String name, Value value)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteProperty setProperty(String name, Value value, int type)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public void addMixin(String name)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public boolean canAddMixin(String name)
            throws RepositoryException, RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public void removeMixin(String name)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void orderBefore(String src, String dst)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public RemoteProperty setProperty(String name, Value[] values)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteNodeDefinition getDefinition()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteVersion checkin() throws RepositoryException, RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public void checkout() throws RepositoryException, RemoteException {
	}

    /** {@inheritDoc} */
    public String getCorrespondingNodePath(String workspace)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public int getIndex() throws RepositoryException, RemoteException {
		return 0;
	}

    /** {@inheritDoc} */
    public RemoteIterator merge(String workspace, boolean bestEffort)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public void cancelMerge(String versionUUID)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void doneMerge(String versionUUID)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void restore(String version, boolean removeExisting)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void restoreByUUID(String versionUUID, boolean removeExisting)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void restore(String versionUUID, String path, boolean removeExisting)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void restoreByLabel(String label, boolean removeExisting)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void update(String workspace)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public boolean holdsLock() throws RepositoryException, RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean isCheckedOut() throws RepositoryException, RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public RemoteVersionHistory getVersionHistory()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteVersion getBaseVersion()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public boolean isLocked() throws RepositoryException, RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public RemoteProperty setProperty(String name, Value[] values, int type)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public void unlock() throws RepositoryException, RemoteException {
	}

    /** {@inheritDoc} */
    public RemoteLock getLock() throws RepositoryException, RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public RemoteLock lock(boolean isDeep, boolean isSessionScoped)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteIterator getSharedSet() 
    		throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public void followLifecycleTransition(String transition)
    		throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
	public String[] getAllowedLifecycleTransistions()
			throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
	public RemoteIterator getWeakReferences() 
			throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
	public RemoteIterator getWeakReferences(String name)
			throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
	public void removeShare() throws RepositoryException, RemoteException {
	}

    /** {@inheritDoc} */
	public void removeSharedSet() throws RepositoryException, RemoteException {
	}

    /** {@inheritDoc} */
	public void setPrimaryType(String nodeTypeName) 
			throws RepositoryException, RemoteException {
			}

    //---------- Implementation helper -----------------------------------------

    /**
     * Returns the {@link Version} instance for the given UUID.
     *
     * @param versionUUID The UUID of the version.
     *
     * @return The version node.
     *
     * @throws RepositoryException if an error occurrs accessing the version
     *      node or if the UUID does not denote a version.
     */
    protected Version getVersionByUUID(String versionUUID)
            throws RepositoryException {
				return null;
			}

}
