/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.server;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.rmi.RemoteException;

import javax.jcr.Credentials;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.UnsupportedRepositoryOperationException;

import org.apache.jackrabbit.rmi.remote.RemoteItem;
import org.apache.jackrabbit.rmi.remote.RemoteNode;
import org.apache.jackrabbit.rmi.remote.RemoteProperty;
import org.apache.jackrabbit.rmi.remote.RemoteSession;
import org.apache.jackrabbit.rmi.remote.RemoteWorkspace;
import org.apache.jackrabbit.rmi.remote.security.RemoteAccessControlManager;

/**
 * Remote adapter for the JCR {@link javax.jcr.Session Session} interface.
 * This class makes a local session available as an RMI service using the
 * {@link org.apache.jackrabbit.rmi.remote.RemoteSession RemoteSession}
 * interface.
 *
 * @see javax.jcr.Session
 * @see org.apache.jackrabbit.rmi.remote.RemoteSession
 */
public class ServerSession extends ServerObject implements RemoteSession {

    /** The adapted local session. */
    private Session session;

    /**
     * The server workspace for this session. This field is assigned on demand
     * by the first call to {@link #getWorkspace()}. The assumption is that
     * there is only one workspace instance per session and that each call to
     * the <code>Session.getWorkspace()</code> method of a single session will
     * allways return the same object.
     */
    private RemoteWorkspace remoteWorkspace;

    /**
     * Creates a remote adapter for the given local session.
     *
     * @param session local session
     * @param factory remote adapter factory
     * @throws RemoteException on RMI errors
     */
    public ServerSession(Session session, RemoteAdapterFactory factory)
            throws RemoteException {
        super(factory);
        this.session = session;
    }

    /** {@inheritDoc} */
    public String getUserID() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public Object getAttribute(String name) throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public String[] getAttributeNames() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public RemoteSession impersonate(Credentials credentials)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteWorkspace getWorkspace() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public boolean hasPermission(String path, String actions)
            throws RepositoryException, RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public String getNamespacePrefix(String uri)
             throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public String[] getNamespacePrefixes()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public String getNamespaceURI(String prefix)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public void setNamespacePrefix(String prefix, String uri)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public boolean itemExists(String path) throws RepositoryException, RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean nodeExists(String path) throws RepositoryException, RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public boolean propertyExists(String path) throws RepositoryException, RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public RemoteNode getNodeByIdentifier(String id)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    @SuppressWarnings("deprecation")
    public RemoteNode getNodeByUUID(String uuid)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteNode getRootNode()
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteItem getItem(String path)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteNode getNode(String path)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteProperty getProperty(String path)
            throws RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public boolean hasPendingChanges()
            throws RepositoryException, RemoteException {
				return false;
			}

    /** {@inheritDoc} */
    public void removeItem(String path)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void move(String from, String to)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void save() throws RepositoryException, RemoteException {
	}

    /** {@inheritDoc} */
    public void refresh(boolean keepChanges)
            throws RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void logout() throws RemoteException {
	}

    /** {@inheritDoc} */
    public boolean isLive() throws RemoteException {
		return false;
	}

    /** {@inheritDoc} */
    public void importXML(String path, byte[] xml, int mode)
            throws IOException, RepositoryException, RemoteException {
			}

    /** {@inheritDoc} */
    public void addLockToken(String token) throws RemoteException {
	}

    /** {@inheritDoc} */
    public String[] getLockTokens() throws RemoteException {
		return null;
	}

    /** {@inheritDoc} */
    public void removeLockToken(String token) throws RemoteException {
	}

    /** {@inheritDoc} */
    public byte[] exportDocumentView(
            String path, boolean binaryAsLink, boolean noRecurse)
            throws IOException, RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public byte[] exportSystemView(
            String path, boolean binaryAsLink, boolean noRecurse)
            throws IOException, RepositoryException, RemoteException {
				return null;
			}

    /** {@inheritDoc} */
    public RemoteAccessControlManager getAccessControlManager()
            throws UnsupportedRepositoryOperationException,
            RepositoryException, RemoteException {
				return null;
			}
}
