/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.util;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

/**
 * This Class implements an InputStream that provides the same functionality
 * as a <code>FileInputStream</code> but opens the file by the first file access.
 */
public class LazyFileInputStream extends InputStream {

    /**
     * the underlying input stream
     */
    private FileInputStream in;

    /**
     * FileDescriptor to use
     */
    private FileDescriptor fd;

    /**
     * File to use
     */
    private File file;

    /**
     * Creates a new <code>LazyFileInputStream</code> for the given file. If the
     * file is unreadably, a FileNotFoundException is thrown.
     *
     * @param file
     * @throws java.io.FileNotFoundException
     */
    public LazyFileInputStream(File file)
            throws FileNotFoundException {
        // check if we can read from the file
        if (!file.canRead()) {
            throw new FileNotFoundException(file.getPath());
        }
        this.file = file;
    }

    /**
     * Creates a new <code>LazyFileInputStream</code> for the given file
     * desciptor.
     *
     * @param fdObj
     */
    public LazyFileInputStream(FileDescriptor fdObj) {
        this.fd = fdObj;
    }

    /**
     * Creates a new <code>LazyFileInputStream</code> for the given file. If the
     * file is unreadably, a FileNotFoundException is thrown.
     *
     * @param name
     * @throws java.io.FileNotFoundException
     */
    public LazyFileInputStream(String name) throws FileNotFoundException {
        this(new File(name));
    }

    /**
     * Opens the underlying file input stream in neccessairy.
     * @throws java.io.IOException
     */
    public void open() throws IOException {
	}

    /**
     * {@inheritDoc}
     */
    public int read() throws IOException {
		return 0;
	}

    /**
     * {@inheritDoc}
     */
    public int available() throws IOException {
		return 0;
	}

    /**
     * {@inheritDoc}
     */
    public void close() throws IOException {
	}

    /**
     * {@inheritDoc}
     */
    public synchronized void reset() throws IOException {
	}

    /**
     * {@inheritDoc}
     */
    public boolean markSupported() {
		return false;
	}

    /**
     * {@inheritDoc}
     */
    public synchronized void mark(int readlimit) {
	}

    /**
     * {@inheritDoc}
     */
    public long skip(long n) throws IOException {
		return 0L;
	}

    /**
     * {@inheritDoc}
     */
    public int read(byte[] b) throws IOException {
		return 0;
	}

    /**
     * {@inheritDoc}
     */
    public int read(byte[] b, int off, int len) throws IOException {
		return 0;
	}

}
