/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.fs.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.Map;
import java.util.Properties;

import javax.jcr.NamespaceException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.jackrabbit.spi.commons.privilege.ParseException;
import org.apache.jackrabbit.spi.commons.privilege.PrivilegeDefinitionReader;
import org.apache.jackrabbit.vault.fs.api.VaultFsConfig;
import org.apache.jackrabbit.vault.fs.api.WorkspaceFilter;
import org.apache.jackrabbit.vault.fs.spi.CNDReader;
import org.apache.jackrabbit.vault.fs.spi.NodeTypeSet;
import org.apache.jackrabbit.vault.fs.spi.PrivilegeDefinitions;
import org.apache.jackrabbit.vault.fs.spi.ServiceProviderFactory;
import org.apache.jackrabbit.vault.util.Constants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Abstracts the way of accessing the vault specific meta-info of a checkout.
 */
public class DefaultMetaInf implements MetaInf {

    private static Logger log = LoggerFactory.getLogger(DefaultMetaInf.class);

    private VaultSettings settings;

    private WorkspaceFilter filter;

    private VaultFsConfig config;

    private Properties properties;

    private Collection<NodeTypeSet> cnds = new LinkedList<NodeTypeSet>();

    private PrivilegeDefinitions privileges = new PrivilegeDefinitions();

    private boolean hasDefinition;

    /**
     * Returns the package format version of this package. If the package
     * lacks this information, {@link #FORMAT_VERSION_2} is returned, since this
     * feature was implemented recently.
     *
     * @return the package format version
     * @since 2.0
     */
    public int getPackageFormatVersion() {
		return 0;
	}

    public void loadFilter(InputStream in, String systemId)
            throws ConfigurationException, IOException {
			}

    public void loadConfig(InputStream in, String systemId)
            throws ConfigurationException, IOException {
			}

    public void loadSettings(InputStream in, String systemId)
            throws ConfigurationException, IOException {
			}

    public void loadProperties(InputStream in, String systemId)
            throws IOException {
			}

    public void loadPrivileges(InputStream in, String systemId)
            throws IOException {
			}

    public void save(File metaDir) throws IOException {
	}

    public VaultSettings getSettings() {
		return null;
	}

    public void setSettings(VaultSettings settings) {
	}

    public WorkspaceFilter getFilter() {
		return null;
	}

    public void setFilter(WorkspaceFilter filter) {
	}

    public VaultFsConfig getConfig() {
		return null;
	}

    public void setConfig(VaultFsConfig config) {
	}

    public Properties getProperties() {
		return null;
	}

    public void setProperties(Properties properties) {
	}

    public Collection<NodeTypeSet> getNodeTypes() {
		return null;
	}

    public PrivilegeDefinitions getPrivileges() {
		return null;
	}

    public void setCNDs(Collection<NodeTypeSet> cnds) {
	}

    public boolean hasDefinition() {
		return false;
	}

    public void setHasDefinition(boolean hasDefinition) {
	}

    protected void loadSettings(File metaDir)
            throws ConfigurationException, IOException {
			}

    protected void saveSettings(File metaDir) throws IOException {
	}

    protected void loadConfig(File metaDir)
            throws ConfigurationException, IOException {
			}

    protected void saveConfig(File metaDir)
            throws IOException {
			}

    protected void loadFilter(File metaDir, boolean vltMode)
            throws ConfigurationException, IOException {
			}

    protected void saveFilter(File metaDir)
            throws IOException {
			}

    protected void loadProperties(File metaDir) throws IOException {
	}

    protected void saveProperties(File metaDir) throws IOException {
	}

    protected void loadPrivileges(File metaDir) throws IOException {
	}

    protected void loadCNDs(File metaDir) throws IOException {
	}
}