/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.fs.io;

import java.io.File;
import java.io.IOException;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.apache.jackrabbit.vault.fs.api.ImportMode;
import org.apache.jackrabbit.vault.fs.api.ProgressTrackerListener;
import org.apache.jackrabbit.vault.fs.api.WorkspaceFilter;

/**
 * Option that control the package import.
 */
public class ImportOptions {

    private boolean strict;

    private ProgressTrackerListener listener;

    private String patchParentPath = "/var/crxpatches/";

    private File patchDirectory;

    private boolean patchKeepInRepo = true;

    private boolean nonRecursive = false;

    private boolean dryRun;

    private int autoSave = -1;

    private AccessControlHandling acHandling = null;

    private ImportMode importMode;

    private Pattern cndPattern = Pattern.compile("^/(apps|libs)/([^/]+/){1,2}nodetypes/.+\\.cnd$");

    private WorkspaceFilter filter = null;

    private ClassLoader hookClassLoader;

    public ImportOptions() {
        // default constructor.
    }

    /**
     * @deprecated use {@link #copy()} instead.
     * @param base base options
     */
    @Deprecated
    public ImportOptions(ImportOptions base) {
        if (base != null) {
            strict = base.strict;
            listener = base.listener;
            patchParentPath = base.patchParentPath;
            patchDirectory = base.patchDirectory;
            patchKeepInRepo = base.patchKeepInRepo;
            nonRecursive = base.nonRecursive;
            dryRun = base.dryRun;
            autoSave = base.autoSave;
            acHandling = base.acHandling;
            importMode = base.importMode;
            cndPattern = base.cndPattern;
            filter = base.filter;
            hookClassLoader = base.hookClassLoader;
        }
    }

    public ImportOptions copy() {
		return null;
	}

    public boolean isStrict() {
		return false;
	}

    public void setStrict(boolean strict) {
	}

    public ProgressTrackerListener getListener() {
		return null;
	}

    public void setListener(ProgressTrackerListener listener) {
	}

    public String getPatchParentPath() {
		return null;
	}

    public void setPatchParentPath(String patchParentPath) {
	}

    public File getPatchDirectory() {
		return null;
	}

    public void setPatchDirectory(File patchDirectory) throws IOException {
	}

    public boolean isPatchKeepInRepo() {
		return false;
	}

    public void setPatchKeepInRepo(boolean patchKeepInRepo) {
	}

    public AccessControlHandling getAccessControlHandling() {
		return null;
	}

    public void setAccessControlHandling(AccessControlHandling acHandling) {
	}

    /**
     * Defines the package installation should recursively install sub packages. Note that if this flag is enabled,
     * the {@link org.apache.jackrabbit.vault.packaging.SubPackageHandling} configuration has no effect, as sub packages
     * are not evaluated at all.
     *
     * @return {@code true} if package installation should not install sub packages.
     */
    public boolean isNonRecursive() {
		return false;
	}

    /**
     * @see #isNonRecursive()
     */
    public void setNonRecursive(boolean nonRecursive) {
	}

    public Pattern getCndPattern() {
		return null;
	}

    public void setCndPattern(String cndPattern) throws PatternSyntaxException {
	}

    /**
     * @since 2.2.14
     */
    public boolean isDryRun() {
		return false;
	}

    /**
     * @since 2.2.14
     */
    public void setDryRun(boolean dryRun) {
	}

    /**
     * @since 2.2.16
     */
    public void setAutoSaveThreshold(int threshold) {
	}

    /**
     * @since 2.2.16
     */
    public int getAutoSaveThreshold() {
		return 0;
	}

    /**
     * @since 2.3
     */
    public ImportMode getImportMode() {
		return null;
	}

    /**
     * @since 2.3
     */
    public void setImportMode(ImportMode importMode) {
	}

    /**
     * @since 2.3.20
     */
    public WorkspaceFilter getFilter() {
		return null;
	}

    /**
     * @since 2.3.20
     */
    public void setFilter(WorkspaceFilter filter) {
	}

    /**
     * @since 2.3.22
     */
    public ClassLoader getHookClassLoader() {
		return null;
	}

    /**
     * @since 2.3.22
     */
    public void setHookClassLoader(ClassLoader hookClassLoader) {
	}
}