/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.fs.io;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.jackrabbit.vault.fs.api.VaultInputSource;
import org.apache.jackrabbit.vault.fs.config.MetaInf;
import org.apache.jackrabbit.vault.util.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Implements an archive based on a zip file, but deflates the entries first
 * in a tmp file. this is only used to circumvent a bug in ZipFile of jdk1.5,
 * that has problems with large zip files.
 */
class ZipStreamArchive extends AbstractArchive {

    /**
     * default logger
     */
    private static final Logger log = LoggerFactory.getLogger(ZipStreamArchive.class);

    private final File zipFile;

    private File tmpFile;

    private RandomAccessFile raf;

    private JarEntry root;

    public ZipStreamArchive(File zipFile) {
        this.zipFile = zipFile;
    }

    public void open(boolean strict) throws IOException {
	}

    private long copy(InputStream in) throws IOException {
		return 0L;
	}

    public InputStream openInputStream(Entry entry) throws IOException {
		return null;
	}

    public VaultInputSource getInputSource(Entry entry) throws IOException {
		return null;
	}

    public MetaInf getMetaInf() {
		return null;
	}

    public void close() {
	}

    public Entry getRoot() throws IOException {
		return null;
	}

    private class RafInputSource extends VaultInputSource {

        private final JarEntry entry;

        private RafInputSource(JarEntry entry) {
            this.entry = entry;
        }

        @Override
        public InputStream getByteStream() {
			return null;
		}

        public long getContentLength() {
			return 0L;
		}

        public long getLastModified() {
			return 0L;
		}
    }

    private class RafInputStream extends InputStream {

        private long pos;

        private long end;

        private long mark;

        private RafInputStream(JarEntry entry) {
            pos = entry.pos;
            end = pos + entry.len;
        }

        public int read() throws IOException {
			return 0;
		}

        @Override
        public int read(byte[] b, int off, int len) throws IOException {
			return 0;
		}

        @Override
        public int read(byte[] b) throws IOException {
			return 0;
		}

        @Override
        public long skip(long n) throws IOException {
			return 0L;
		}

        @Override
        public int available() throws IOException {
			return 0;
		}

        @Override
        public void close() throws IOException {
		}

        @Override
        public void mark(int readlimit) {
		}

        @Override
        public void reset() throws IOException {
		}

        @Override
        public boolean markSupported() {
			return false;
		}
    }

    private static class JarEntry implements Entry {

        public final String name;

        public final long date;

        public final long pos;

        public final long len;

        public Map<String, JarEntry> children;

        private JarEntry(String name) {
            this.name = name;
            this.date = 0;
            pos = -1;
            len = 0;
        }

        private JarEntry(String name, long date, long pos, long len) {
            this.name = name;
            this.date = date;
            this.pos = pos;
            this.len = len;
        }

        public String getName() {
			return null;
		}

        public boolean isDirectory() {
			return false;
		}

        public JarEntry add(JarEntry e) {
			return null;
		}

        public JarEntry add(String name) {
			return null;
		}

        public Collection<? extends Entry> getChildren() {
			return null;
		}

        public Entry getChild(String name) {
			return null;
		}

    }
}