/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.util;

import java.io.IOException;
import java.io.InputStream;

/**
 * Provides an input stream wrapper that detects line feed sequences and 
 * replaces them by new ones.
 *
 */
public class LineInputStream extends InputStream {

    public static final byte[] LS_UNIX = new byte[]{0x0a};

    public static final byte[] LS_WINDOWS = new byte[]{0x0d, 0x0a};

    public static final byte[] LS_NATIVE = System.getProperty("line.separator").getBytes();

    private byte[] buffer = new byte[8192];

    private byte[] lineFeed = LS_NATIVE;

    private byte[] lineSpool;

    private int pos = 0;

    private int end = 0;

    private static final char STATE_INIT = ' ';
    private static final char STATE_CR = 'c';
    private static final char STATE_LF = 'l';
    private static final char STATE_CRLF = 'f';

    private char state = STATE_INIT;

    boolean isEof = false;

    private byte[] spool;

    private int spoolPos = 0;

    private final InputStream in;

    public LineInputStream(InputStream in, byte[] ls) {
        this.in = in;
        if (ls != null) {
            lineFeed = ls;
        }
        lineSpool = new byte[lineFeed.length + 1];
        System.arraycopy(lineFeed, 0, lineSpool, 0, lineFeed.length);
    }

    private int fillBuffer() throws IOException {
		return 0;
	}

    public int read() throws IOException {
		return 0;
	}

    public int read(byte b[], int off, int len) throws IOException {
		return 0;
	}

    public void close() throws IOException {
	}

}