/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.util.xml.xerces.xni.parser;

import org.apache.jackrabbit.vault.util.xml.xerces.xni.XMLLocator;
import org.apache.jackrabbit.vault.util.xml.xerces.xni.XNIException;

/**
 * A parsing exception. This exception is different from the standard
 * XNI exception in that it stores the location in the document (or
 * its entities) where the exception occurred.
 *
 * @author Andy Clark, IBM
 * @version $Id$
 */
public class XMLParseException
        extends XNIException {

    //
    // Data
    //

    /**
     * Public identifier.
     */
    protected String fPublicId;

    /**
     * literal System identifier.
     */
    protected String fLiteralSystemId;

    /**
     * expanded System identifier.
     */
    protected String fExpandedSystemId;

    /**
     * Base system identifier.
     */
    protected String fBaseSystemId;

    /**
     * Line number.
     */
    protected int fLineNumber = -1;

    /**
     * Column number.
     */
    protected int fColumnNumber = -1;

    //
    // Constructors
    //

    /**
     * Constructs a parse exception.
     */
    public XMLParseException(XMLLocator locator, String message) {
        super(message);
        if (locator != null) {
            fPublicId = locator.getPublicId();
            fLiteralSystemId = locator.getLiteralSystemId();
            fExpandedSystemId = locator.getExpandedSystemId();
            fBaseSystemId = locator.getBaseSystemId();
            fLineNumber = locator.getLineNumber();
            fColumnNumber = locator.getColumnNumber();
        }
    } // <init>(XMLLocator,String)

    /**
     * Constructs a parse exception.
     */
    public XMLParseException(XMLLocator locator,
                             String message, Exception exception) {
        super(message, exception);
        fPublicId = locator.getPublicId();
        fLiteralSystemId = locator.getLiteralSystemId();
        fExpandedSystemId = locator.getExpandedSystemId();
        fBaseSystemId = locator.getBaseSystemId();
        fLineNumber = locator.getLineNumber();
        fColumnNumber = locator.getColumnNumber();
    } // <init>(XMLLocator,String,Exception)

    //
    // Public methods
    //

    /**
     * Returns the public identifier.
     */
    public String getPublicId() {
		return null;
	} // getPublicId():String

    /**
     * Returns the expanded system identifier.
     */
    public String getExpandedSystemId() {
		return null;
	} // getExpandedSystemId():String

    /**
     * Returns the literal system identifier.
     */
    public String getLiteralSystemId() {
		return null;
	} // getLiteralSystemId():String

    /**
     * Returns the base system identifier.
     */
    public String getBaseSystemId() {
		return null;
	} // getBaseSystemId():String

    /**
     * Returns the line number.
     */
    public int getLineNumber() {
		return 0;
	} // getLineNumber():int

    /**
     * Returns the row number.
     */
    public int getColumnNumber() {
		return 0;
	} // getRowNumber():int

    //
    // Object methods
    //

    /**
     * Returns a string representation of this object.
     */
    public String toString() {
		return null;
	} // toString():String

} // XMLParseException
